#include <Types.h>
#include <Events.h>
#include <Dialogs.h>
#include <Lists.h>
#include <Devices.h>
#include <QuickDraw.h>
#include <Dialogs.h>
#include <Resources.h>
#include <Packages.h>
#include <EPPC.h>
#include <Errors.h>
#include <Fonts.h>
#include <Processes.h>
#include <Folders.h>
#include <AppleEvents.h>
#include <ToolUtils.h>
#include <Finder.h>
#include <OSEvents.h>

#include <myUtils.h>

#include "IconFamily.h"

#include "list.h"
#include "inetd.conf.h"

pascal CDEV* 
ENTRY(short message, short item, short numItems, short , EventRecord *theEvent, CDEV* storage, DialogPtr CPDialog)
{
	if (message == macDev) {
		return (CDEV*) 1;
	}
	else if (storage != nil) {
		switch (message) {
			case initDev:									/* initialize cdev */
				storage = new CDEV;
				if (storage == nil)
					return cdevMemErr;
				
				storage->fDialog = CPDialog;
				storage->fNumItems = numItems;
				
				storage->DrawEverything();
				storage->SetInitialState();
				storage->OpenPrepFile();
				storage->SetUpList();
				
				break;
			case nulDev:
				break;
			case hitDev:									/* handle hit on item */
				switch (numItems + item) {
					case kStartBtn:
						storage->TurnOnOrOff();
						break;
					case kNewBtn:
					case kChangeBtn:
						storage->DoChooseProgram();
						break;
					case kDeleteBtn:
						storage->DeleteItem();
						break;
					case kListBox:
						storage->SelectItem(theEvent->where, theEvent->modifiers);
						break;
				}
				break;
			case closeDev:									/* clean up and dispose */
				if (storage) delete storage;
				break;
			case updateDev:									/* handle any update drawing */
				storage->DrawEverything();
				
				if (storage->fList)
					LUpdate(((GrafPtr) CPDialog)->visRgn, storage->fList);
				break;
			case activDev:									/* activate any needed items */
			case deactivDev:								/* deactivate any needed items */
				if (storage->fList)
					LActivate((message == activDev), storage->fList);
				break;
			case keyEvtDev:									/* respond to keydown */
				break;
			case macDev:
			case undoDev:
				break;
			case cutDev:
			case copyDev:
			case pasteDev:
			case clearDev:
				break;
		}

		return storage;
	}  
	
	return nil;
}

CDEV::CDEV()
{
	fCurStatus = nil;
	fList = nil;
	
	fMadeChanges = false;
	
	fPrepFile = 0;
	
	fSelected = false;
}

CDEV::~CDEV()
{
	if (fPrepFile)
		CloseResFile(fPrepFile);

	if (fMadeChanges)
		ResetDaemon();

	if (fList)
		LDispose(fList);

	if (fCurStatus) {
		HUnlock(fCurStatus);
		ReleaseResource(fCurStatus);
	}
}

void
CDEV::DeleteItem()
{
	Handle		resHandle;
	short		curResFile;
	ListData	data;
	short		size;
	Cell		cell;
	
	curResFile = CurResFile();
	UseResFile(fPrepFile);
	
	cell.v = 0;
	cell.h = 0;
	while (LGetSelect(true, &cell, fList)) {
		size = sizeof(data);
		LGetCell(&data, &size, cell, fList);
		
		resHandle = GetResource(data.resType, data.resID);	
		if (resHandle) {
			RmveResource(resHandle);
			DisposeHandle(resHandle);
			UpdateResFile(fPrepFile);
			LDelRow(1, cell.v, fList);
		}
	}
	
	UseResFile(curResFile);
	
	fMadeChanges = true;

	HiliteControl(SnatchHandle(fDialog, fNumItems + kChangeBtn), 255);
	HiliteControl(SnatchHandle(fDialog, fNumItems + kDeleteBtn), 255);
	
	fSelected = false;
}

void
CDEV::SelectItem(Point where, short modifiers)
{
	Boolean	dClick;
	Cell	cell;
	Boolean	inCell	= false;
	
//	this->UnloadConfig();
	
	GlobalToLocal(&where);
	dClick = LClick(where, modifiers, fList);
	
	cell.v = 0;
	cell.h = 0;
	inCell = LGetSelect(true, &cell, fList);
	
	HiliteControl(SnatchHandle(fDialog, fNumItems + kChangeBtn), (inCell ? 0 : 255));
	HiliteControl(SnatchHandle(fDialog, fNumItems + kDeleteBtn), (inCell ? 0 : 255));
	
	fSelected = inCell;
	
//	this->LoadConfig();
	
	if (dClick) {
		DoHilite(SnatchHandle(fDialog, fNumItems + kChangeBtn));
		this->DoChooseProgram();
	}
}

void
CDEV::UnloadConfig()
{
	if (!fSelected)
		return;
		
	if (fConfigDITL) {
	
		ReleaseResource(fConfigDITL);
	}
	
	if (fConfigCDEV) {
	
		ReleaseResource(fConfigCDEV);
	}
}

void
CDEV::LoadConfig()
{
	ListData		save;
	short			saveSize;
	Cell			cell;
	FSSpecHandle	progHandle;
	short			resFile;
	short			curResFile;
	
	Handle			ditlHandle;
	Handle			cdevHandle;
	
	if (!fSelected)
		return;
	
	saveSize = sizeof(save);
	LGetSelect(true, &cell, fList);
	LGetCell(&save, &saveSize, cell, fList);
	progHandle = (FSSpecHandle) GetResource(save.resType, save.resID);
	HLock((Handle) progHandle);
	
	resFile = FSpOpenResFile(*progHandle, fsRdPerm);
	curResFile = SwitchResFile(resFile);

	ditlHandle = GetResource('DITL', 1000);
	cdevHandle = GetResource('cdev', 1000);
	if (ditlHandle && cdevHandle) {
		fConfigDITL = ditlHandle;
		fConfigCDEV = cdevHandle;
	}

	SwitchResFile(curResFile);
	CloseResFile(resFile);

	HUnlock((Handle) progHandle);
	ReleaseResource((Handle) progHandle);
}

void
CDEV::DrawEverything()
{
	DrawIcon(fDialog, kICON1);
	DrawIcon(fDialog, kICON2);
	DrawMyString(fDialog, kServerItm);
	DrawMyString(fDialog, kConfigItm);
	Outline(fDialog, kDivider);
	Outline(fDialog, kListBox);
	Outline(fDialog, kStatusBox);
	Outline(fDialog, kInfoBox);
}

void
CDEV::SetInitialState()
{
	Handle				status;
	Handle				info;

	ProcessSerialNumber	psn;
	ProcessInfoRec		prc;
	Boolean				found = false;
	
	psn.highLongOfPSN = 0;
	psn.lowLongOfPSN = kNoProcess;
	
	prc.processInfoLength = sizeof(ProcessInfoRec);
	prc.processName = nil;
	prc.processAppSpec = nil;
	
	while (!found && (GetNextProcess(&psn) == noErr)) {
		if (GetProcessInformation(&psn, &prc) == noErr) {
			if ((prc.processType == 'appe') && (prc.processSignature == 'inet'))
				found = true;
		}
	}
	
	fIsRunning = found;
	if (fIsRunning) {
		SetCTitle(SnatchHandle(fDialog, fNumItems + kStartBtn), "\pStop");
		status = GetResource('STR ', kIsOnStr);
	}
	else {
		SetCTitle(SnatchHandle(fDialog, fNumItems + kStartBtn), "\pStart");
		status = GetResource('STR ', kIsOffStr);
	}

	HLock(status);
	fCurStatus = status;
	SetIText((Handle) SnatchHandle(fDialog, fNumItems + kStatusStr), (Str255) *status);
	
	HiliteControl(SnatchHandle(fDialog, fNumItems + kChangeBtn), 255);
	HiliteControl(SnatchHandle(fDialog, fNumItems + kDeleteBtn), 255);
}

void
CDEV::OpenPrepFile()
{
	short	vRefNum;
	long	dirID;
	short	resFile;
	FSSpec	resSpec;
	Handle	prepName;
	
	prepName = GetResource('STR ', kPrepName);
	HLock(prepName);

	FindFolder(kOnSystemDisk, kPreferencesFolderType, false, &vRefNum, &dirID);
	
	if (FSMakeFSSpec(vRefNum, dirID, (Str255) *prepName, &resSpec) == fnfErr) 
		FSpCreateResFile(&resSpec, 'inet', 'prep', smSystemScript);
		
	resFile = FSpOpenResFile(&resSpec, fsRdWrPerm);
	DebugIfErr(ResError());
	
	HUnlock(prepName);
	ReleaseResource(prepName);
	
	fPrepFile = resFile;
}

void
CDEV::SetUpList()
{
	ListHandle	lHandle;
	Handle		iHandle;
	Rect		iRect;
	short		iType;
	Point		cellSize;
	Rect		dataRect;

	GetDItem(fDialog, fNumItems + kListBox, &iType, &iHandle, &iRect);
	InsetRect(&iRect, 1, 1);
	SetPt(&cellSize, iRect.right - iRect.left - 15, cListCellHeight);
	SetRect(&dataRect, 0, 0, 1, 0);
	iRect.right -= 15;

	cellSize.v = 0;
	cellSize.h = 0;
	lHandle = LNew(&iRect, &dataRect, cellSize, 128, (WindowPtr) fDialog, false, false, false, true);
	if (lHandle) {
		int			i;
		Handle		resHandle;
		ListData	data;
		short		numItems;
		
		fList = lHandle;
		
		(**lHandle).selFlags = 0;
	
		numItems = CountResources('TCP ');
		for (i = 1; i <= numItems; i++) {
			resHandle = GetIndResource('TCP ', i);
			GetResInfo(resHandle, &data.resID, &data.resType, data.resName);
			ReleaseResource(resHandle);
			
			this->AddCell(&data);
		}

		numItems = CountResources('UDP ');
		for (i = 1; i <= numItems; i++) {
			resHandle = GetIndResource('UDP ', i);
			GetResInfo(resHandle, &data.resID, &data.resType, data.resName);
			ReleaseResource(resHandle);

			this->AddCell(&data);
		}

		LDoDraw(true, lHandle);
	}
}

void
CDEV::AddCell(ListData* nu)
{
	Cell		cell;
	ListData	data;
	
	cell.v = 0;
	cell.h = 0;
	
	LSearch(&data, sizeof(data), nil, &cell, fList);
	
	LAddRow(1, cell.v, fList);
	LSetCell(nu, sizeof(ListData), cell, fList);
	LDraw(cell, fList);
}

void
CDEV::TurnOnOrOff()
{
	ControlHandle	button;
	Handle			status;
	short			id;
	
	if (fCurStatus) {
		HUnlock(fCurStatus);
		ReleaseResource(fCurStatus);
	}

	button = SnatchHandle(fDialog, fNumItems + kStartBtn);
	
	if (fIsRunning) {
		if (this->StopDaemon()) {
			fIsRunning = false;
			SetCTitle(button, "\pStart");
			id = kIsOffStr;
		}
		else {
			SysBeep(1);
			id = kIsOnStr;
		}
	}
	else {
		if (this->StartDaemon()) {
			fIsRunning = true;
			SetCTitle(button, "\pStop");
			id = kIsOnStr;
		}
		else {
			SysBeep(1);
			id = kIsOffStr;
		}
	}

	status = GetResource('STR ', id);
	HLock(status);
	fCurStatus = status;
	SetIText((Handle) SnatchHandle(fDialog, fNumItems + kStatusStr), (Str255) *status);
}

Boolean
CDEV::StartDaemon()
{
	Boolean				result = false;
	LaunchParamBlockRec	lpb;
	FSSpec				process;
	Handle				name;
	short				vRefNum;
	long				dirID;
	
	name = GetResource('STR ', kDaemonName);
	HLock(name);
	
	FindFolder(kOnSystemDisk, kExtensionFolderType, false, &vRefNum, &dirID);
	FSMakeFSSpec(vRefNum, dirID, (Str255) *name, &process);

	lpb.launchAppParameters = nil;
	lpb.launchBlockID = extendedBlock;
	lpb.launchEPBLength = extendedBlockLen;
	lpb.launchFileFlags = 0;
	lpb.launchControlFlags = launchContinue + launchNoFileFlags;
	lpb.launchAppSpec = &process;
	
	result = (LaunchApplication(&lpb) == noErr);
	
	HUnlock(name);
	ReleaseResource(name);
	
	return result;
}

Boolean
CDEV::StopDaemon()
{
	AppleEvent	theEvent;
	AEDesc		theAddress;
	long		theType = 'inet';
	OSErr		theErr;

	AECreateDesc(typeApplSignature, (Ptr) &theType, sizeof(theType), &theAddress);
	AECreateAppleEvent(kCoreEventClass, kAEQuitApplication,  &theAddress, kAutoGenerateReturnID, kAnyTransactionID, &theEvent);

	theErr = AESend(&theEvent, nil, kAENoReply + kAECanInteract + kAECanSwitchLayer, kAEHighPriority, 120, nil, nil);

	AEDisposeDesc(&theEvent);
	AEDisposeDesc(&theAddress);
	
	return (theErr == noErr);
}

void
CDEV::ResetDaemon()
{
	AppleEvent	theEvent;
	AEDesc		theAddress;
	long		theType = 'inet';
	OSErr		theErr;

	AECreateDesc(typeApplSignature, (Ptr) &theType, sizeof(theType), &theAddress);
	AECreateAppleEvent('INET', 'RSET',  &theAddress, kAutoGenerateReturnID, kAnyTransactionID, &theEvent);

	theErr = AESend(&theEvent, nil, kAENoReply + kAECanInteract + kAECanSwitchLayer, kAEHighPriority, 120, nil, nil);

	AEDisposeDesc(&theEvent);
	AEDisposeDesc(&theAddress);
}

void
CDEV::DoChooseProgram()
{
	Point				where;
	StandardFileReply	reply;
	SFTypeList			types;
	short				active[3];
	ListData			data;
	ListData			save;
	short				saveSize;
	Handle				resHandle;
	Cell				cell;
	short				curResFile;
	Boolean				changing = false;
	Handle				dead;

	where.v = -1;
	where.h = -1;
	
	active[0] = 2;
	active[1] = 12;
	active[2] = 7;
	
	types[0] = 'APPL';
	types[1] = 'appe';
	types[2] = kApplicationAliasType;
	
	cell.v = 0;
	cell.h = 0;

	if (fSelected) {
		saveSize = sizeof(save);
		LGetSelect(true, &cell, fList);
		LGetCell(&save, &saveSize, cell, fList);
		changing = true;
		fProtocol = save.resType;
		fPortNum = save.resID;
	}
	else {
		fProtocol = 'TCP ';
		fPortNum = 1;
	}
	
	CustomGetFile(	(FileFilterYDProcPtr) nil, 
					3, types, &reply, 128, where, 
					(DlgHookYDProcPtr) &MyDialogHook, 
					(ModalFilterYDProcPtr) &MyDialogFilter, 
					active, (ActivateYDProcPtr) &MyActiveProc, 
					this);

	if (reply.sfGood) {
		curResFile = CurResFile();
		UseResFile(fPrepFile);
		
		if (changing) {
			dead = GetResource(save.resType, save.resID);
			RmveResource(dead);
			UpdateResFile(fPrepFile);
			DisposeHandle(dead);
		}
	
		PtrToHand(&reply.sfFile, &resHandle, sizeof(FSSpec));
		AddResource(resHandle, fProtocol, fPortNum, reply.sfFile.name);
		WriteResource(resHandle);
		
		GetResInfo(resHandle, &data.resID, &data.resType, data.resName);
		ReleaseResource(resHandle);

		if (changing) {
			LSetCell(&data, sizeof(data), cell, fList);
			LDraw(cell, fList);
		}
		else {
			this->AddCell(&data);
		}
	
		UpdateResFile(fPrepFile);
		UseResFile(curResFile);

		fMadeChanges = true;
	} 
}

pascal void
MyActiveProc(DialogPtr dlog, short item, Boolean activating, CDEV* usrData)
{
	if (item == 12) {
		usrData->fOnText = activating;
		
		if (!activating) {
			Str255	pText;
			long	pNum;
			
			GetIText((Handle) SnatchHandle(dlog, 12), pText);
			StringToNum(pText, &pNum);
			
			if ((pNum < 1) || (pNum > 0xFFFF)) {
				SysBeep(1);
				SetIText((Handle) SnatchHandle(dlog, 12), usrData->fPortText);
			}
			else {
				NumToString(pNum, pText);
				SetIText((Handle) SnatchHandle(dlog, 12), pText);
			}
		}
		else {
			GetIText((Handle) SnatchHandle(dlog, 12), usrData->fPortText);
		}
	}
}

pascal Boolean
MyDialogFilter(DialogPtr , EventRecord* event, short* , CDEV* usrData)
{
	char	key;

	if (usrData->fOnText) {
		if ((event->what == keyDown) || (event->what == autoKey)) {
			key = (char) (event->message & charCodeMask);
			
			if (event->modifiers & cmdKey)
				return false;
			else if (((key <= '9') && (key >= '0')) || 
				(key == '\t') || 
				(key == 0x08) ||
				(key == 0x1B))
				return false;
			else {
				SysBeep(1);
				return true;
			}
		}
	}
	else
		return false;
}

pascal Boolean
MyFileFilter(CInfoPBPtr pb, CDEV* )
{
	Boolean result = true;
	
	if ((pb->hFileInfo.ioFlAttrib & (1<<4)) || 
		(pb->hFileInfo.ioFlFndrInfo.fdType == 'APPL') ||
		(pb->hFileInfo.ioFlFndrInfo.fdType == kApplicationAliasType))
		result = false;

	return result;
}

pascal short
MyDialogHook(short item, DialogPtr dlog, CDEV* usrData)
{
	long	port;
	Str255	str;

	switch (item) {
		case 14:
		case 15:
			SetCtlValue(SnatchHandle(dlog, 14), (item == 14));
			SetCtlValue(SnatchHandle(dlog, 15), (item == 15));
			
			usrData->fProtocol = ((item == 14) ? 'TCP ' : 'UDP ');
			
			return sfHookNullEvent;
		case sfHookFirstCall:
			SetCtlValue(SnatchHandle(dlog, 14), (usrData->fProtocol == 'TCP '));
			SetCtlValue(SnatchHandle(dlog, 15), (usrData->fProtocol == 'UDP '));
			
			NumToString((long) usrData->fPortNum, str);
			
			SetIText((Handle) SnatchHandle(dlog, 12), str);
			SelIText(dlog, 12, 0, 32767);
			
			usrData->fOnText = true;
			
			return sfHookNullEvent;
		case sfHookLastCall:
			GetIText((Handle) SnatchHandle(dlog, 12), usrData->fPortText);
			StringToNum(usrData->fPortText, &port);
			
			usrData->fPortNum = (short) port;
			usrData->fProtocol = (GetCtlValue(SnatchHandle(dlog, 14)) ? 'TCP ' : 'UDP ');
			
			return item;
		default:
			return item;
	}
}

void
Outline(DialogPtr dlog, short item)
{
	Rect		iRect;
	short		iType;
	Handle		iHandle;
	PenState	pen;
	QDRecPtr	myQD;
	
	Style		saveFace;
	short		saveFont;
	short		saveSize;
	StringPtr	string;
	
	FontInfo	fontInfo;
	Rect		tmpRect;
	
	GetDItem(dlog, item, &iType, &iHandle, &iRect);

	myQD = GetQDVars();
	
	GetPenState(&pen);
	PenNormal();
	
	if ((item == kStatusBox) || (item == kInfoBox)) {
		PenPat(&myQD->gray);
		
		iRect.top += 10;
		FrameRect(&iRect);
		
		saveFace = ((GrafPtr) dlog)->txFace;
		saveFont = ((GrafPtr) dlog)->txFont;
		saveSize = ((GrafPtr) dlog)->txSize;
		
		TextFont(applFont);
		TextFace(bold);
		TextSize(9);
		
		if (item == kStatusBox)
			string = "\pStatus";
		else
			string = "\p";
		
		GetFontInfo(&fontInfo);
		tmpRect.left = iRect.left + 9;
		tmpRect.bottom = iRect.top + 2;
		tmpRect.top = tmpRect.bottom - fontInfo.ascent;
		tmpRect.right = tmpRect.left + StringWidth(string) + 1;
		EraseRect(&tmpRect);
		
		MoveTo(iRect.left + 10, iRect.top + 2);
		DrawString(string);
		
		TextFont(saveFont);
		TextFace(saveFace);
		TextSize(saveSize);
	}
	else
		FrameRect(&iRect);

	SetPenState(&pen);
}

void
DrawIcon(DialogPtr dlog, short item)
{
	Rect	iRect;
	short	iType;
	Handle	iHandle;
	Handle	sHandle;
	
	GetDItem(dlog, item, &iType, &iHandle, &iRect);
	GetIconSuite(&sHandle, 129, 0xFFFFFFFF);
	PlotIconSuite(&iRect, 0, 0, sHandle);
	DisposeIconSuite(sHandle, true);
}

void
DrawMyString(DialogPtr dlog, short item)
{
	Rect	iRect;
	short	iType;
	Handle	iHandle;
	short	saveFace;
	short	saveSize;
	short	saveFont;
	
	GetDItem(dlog, item, &iType, &iHandle, &iRect);
	MoveTo(iRect.left, iRect.bottom);
	
	saveFace = ((GrafPtr) dlog)->txFace;
	saveSize = ((GrafPtr) dlog)->txSize;
	saveFont = ((GrafPtr) dlog)->txFont;
	
	TextFont(systemFont);
	TextFace(normal);
	TextSize(12);

	if (item == kServerItm) 
		DrawString("\pInternet Server");
	else 
		DrawString("\pServices Directory");
	
	TextFace(saveFace);
	TextSize(saveSize);
	TextFont(saveFont);
}

ControlHandle 
SnatchHandle(DialogPtr thebox, short theGetItem)
{
    short	itemtype;
    Rect 	itemrect;
    Handle 	thandle;
	
    GetDItem(thebox, theGetItem, &itemtype, &thandle, &itemrect);
	
    return (ControlHandle) thandle;
}

QDRecPtr
GetQDVars()
{
	return (QDRecPtr) (*((long*) SetCurrentA5()) - sizeof(QDRec) - sizeof(GrafPtr) + 8);
}

pascal void
DoHilite(ControlHandle control)
{
	long	ticks = 8;

	HiliteControl(control, 1);
	Delay(ticks, &ticks);
	HiliteControl(control, 0);
}
