//---------------------------------------------------------------------
//
//	Copyright  1992 David Peterson.
//	All rights reserved.
//
//	Permission to use, copy, modify, and distribute this software for
//	any purpose and without fee is hereby granted, provided that the
//	above copyright notice appear in all copies and that both that
//	copyright notice and this permission notice appear in supporting
//	documentation.
//
//---------------------------------------------------------------------

#include "RshD.h"

#include <UFailure.h>

#include <TCP.h>
#include <CType.h>

#include <String.h>
#include <Strings.h>
#include <Desk.h>
#include <Sysequ.h>


void main();

void
main()
{
	RshD* 		aDaemon;
	FailInfo	fi;
	
	VOLATILE(aDaemon);
	
	InitUDaemonApp(*((long*) DefltStack));

	if (fi.Try()) {
		aDaemon = new RshD;
		FailNIL(aDaemon);
	
		aDaemon->Initialize();
		aDaemon->Run();
		
		delete aDaemon;
		
		fi.Success();
	}
	else if (aDaemon) {
		FailInfo	tmp;
		
		if (tmp.Try()) {
			delete aDaemon;
			tmp.Success();
		}
	}
}

void 
RshD::InstallAEHandlers()
{
	Inherited::InstallAEHandlers();

	FailOSErr(AEInstallEventHandler('INET', 'TSTR', 
				(EventHandlerProcPtr) &AEStreamHandler, (long) this, false));

	FailOSErr(AEInstallEventHandler('MPS ', 'outp', 
				(EventHandlerProcPtr) &AEStreamHandler, (long) this, false));

	FailOSErr(AEInstallEventHandler('MPS ', 'diag', 
				(EventHandlerProcPtr) &AEStreamHandler, (long) this, false));

	FailOSErr(AEInstallEventHandler(kCoreEventClass, kAEAnswer, 
				(EventHandlerProcPtr) &AEStreamHandler, (long) this, false));
}

pascal OSErr 
AEStreamHandler(AppleEvent* messagein, AppleEvent* /*reply*/, long refIn)
{
	AEDesc		streamDesc;
	FailInfo	fi;

	if (fi.Try()) {
		FailOSErr(AEGetParamDesc(messagein, 'STRM', typeLongInteger, &streamDesc));

		HLock(streamDesc.dataHandle);
		((RshD*) refIn)->HandleNewStream(*((StreamPtr*) *(streamDesc.dataHandle)));
		HUnlock(streamDesc.dataHandle);

		AEDisposeDesc(&streamDesc);
		
		fi.Success();
	}

	return fi.error;
}

pascal OSErr 
AEToolOutput(AppleEvent* messagein, AppleEvent* /*reply*/, long /*refIn*/)
{
	Command*	command;
	Size		size;
	AEDesc		diag;
	DescType	type;
	Size		dummy;
	char*		cp;
	char*		end;
	FailInfo	fi;
	
	if (fi.Try()) {
		FailOSErr(AEGetAttributePtr(messagein, keyTransactionIDAttr, typeWildCard, &type,
									(Ptr) &command, sizeof(command), &dummy));
									
		FailOSErr(AEGetParamDesc(messagein, keyDirectObject, typeChar, &diag));
		FailOSErr(AESizeOfParam(messagein, keyDirectObject, &type, &size));
		FailOSErr(AEGetAttributePtr(messagein, keyEventIDAttr, typeType, &type, nil, 0, &dummy));
		
		HLock(diag.dataHandle);
				
		cp = *diag.dataHandle;
		end = cp + size;
		while (cp <= end) {
			if (*cp == '\n')
				*cp = '\r';
				
			cp++;
		}
		
		FailOSErr(tcpWrite(command->fStdErr, *diag.dataHandle, (short) size));
	
		HUnlock(diag.dataHandle);
		
		AEDisposeDesc(&diag);
	
		fi.Success();
	}
	
	return fi.error;
}

pascal OSErr 
AEAnswerHandler(AppleEvent* messagein, AppleEvent* /*reply*/, long refIn)
{
	Command*	command;
	DescType	type;
	Size		dummy;
	FailInfo	fi;
	
	if (fi.Try()) {
		FailOSErr(AEGetAttributePtr(messagein, keyTransactionIDAttr, typeWildCard, &type,
									(Ptr) &command, sizeof(command), &dummy));
									
		command->fRelease = true;
		((RshD*) refIn)->LoseIt(command);
	
		fi.Success();
	}

	return fi.error;
}

void
RshD::HandleNewStream(StreamPtr stream)
{
	Command*	nu;
	
	nu = new Command(stream, this);
	if (nu) {
		if (nu->Initialize()) 
			fCommandList.PutOn(nu);
		else
			delete nu;
	}
	
	if (fCommandList.First() == nil)
		this->DoQuit();
}

void
RshD::LoseIt(Command* command)
{
	fCommandList.TakeOff(command);
	delete command;
	
	if (fCommandList.First() == nil)
		this->DoQuit();
}

Command::Command(StreamPtr stream, RshD* shell)
{
	fStdOut = stream;
	fStdErr = nil;
	
	fShell = shell;
	fRelease = false;
}

Command::~Command()
{
	if (fRelease) {
		if (fStdOut) tcpClose(fStdOut);
		if (fStdErr) tcpClose(fStdErr);
	}
}

/*	The return value here tells rshd whether or not to hang around.
	If true, we handed off to ToolServer and need to wait for a reply.
*/
Boolean
Command::Initialize()
{
	FSSpec	cmdSpec;
	char	cliname[16];
	char	servuname[16];
	char	cmdString[256];
	char	cmdName[16];
	char	port2[8];
	short	len;
	PSN		psn;
	
	len = 1;
	tcpRead(fStdOut, &port2[0], &len);
	if (port2[0] != 0) {
		len = sizeof(port2) - 1;
		tcpRead(fStdOut, &port2[1], &len);
		
//		MBPrintf("rshd: remote sent port #%s", port2);
		
		if (!this->OpenSecond(port2)) {
			fRelease = true;
			
			return false;
		}
	}

	this->GetStr(cliname, sizeof(cliname));
	this->GetStr(servuname, sizeof(servuname));
	this->GetStr(cmdString, sizeof(cmdString));
	
//	MBPrintf("rshd: got: %s, %s, %s", cliname, servuname, cmdString);

	this->ExtractCommandName(cmdString, cmdName, sizeof(cmdName));
	
	if (this->FindCommand(cmdName, &cmdSpec)) {
		this->LaunchProcess(&cmdSpec, cmdString);
		this->SendOK();
		
		return false;
	}
	else if (this->FindToolServer(&psn)) {
		this->GiveToToolServer(&psn, cmdString);
		this->SendOK();
		
		return true;
	}
	else {
		this->SendError("rshd: nope");
		fRelease = true;
		
		return false;
	}
}

Boolean
Command::OpenSecond(char* portstring)
{
	TCPiopb		status;
	OSErr		theErr;
	tcp_port	port = 0;
	
	while (isdigit(*portstring)) 
		port = port * 10 + (*portstring++ - '0');
		
	theErr = tcpStatus(fStdOut, &status);
	
	if (theErr == noErr)
		fStdErr = tcpCreate(4 * 1024, nil, nil);
		
	if (fStdErr > 0)
		theErr = tcpOpenReserved(fStdErr, status.csParam.status.remoteHost, port);
	else {
		theErr = (OSErr) fStdErr;
		fStdErr = nil;
	}
		
	return (theErr == noErr);
}

void
Command::SendOK()
{
	tcpWrite(fStdOut, "", 1);
}

void
Command::SendError(char* msg)
{
	WDS(3)	wds;
	char	no		= 1;

	wds.zero			= 0;
	wds.block[0].ptr	= &no;
	wds.block[0].length	= 1;
	wds.block[1].ptr	= msg;
	wds.block[1].length	= strlen(msg) + 1;
	wds.block[2].ptr	= "\r";
	wds.block[2].length	= 1;
	
	tcpWriteWDS(fStdOut, (wdsEntry*) &wds);
}

void
Command::GetStr(char* buff, int cnt)
{
	char	c;
	short	len;

	do {
		len = sizeof(c);
		tcpRead(fStdOut, &c, &len);
		
		if (len != 0) 
			*buff++ = c;
			
		if (--cnt == 0) 
			break;
		
	} while (c != 0);
	
	*buff = 0;
}

void
Command::ExtractCommandName(char* instr, char* cmdName, int len)
{
	while (*instr != ' ') {
		*cmdName++ = *instr++;
		
		if (--len == 0)
			break;
	}
	
	*cmdName = 0;
}

Boolean
Command::FindCommand(char* cmd, FSSpec* cmdSpec)
{
	return (FSMakeFSSpec(0, 0, c2pstr(cmd), cmdSpec) == noErr);
}

void
Command::LaunchProcess(FSSpec* cmd, char* cmdString)
{	
	LaunchParamBlockRec	lpb;
	AppleEvent			theEvent;
	AEAddressDesc		myAddress;
	AEDesc				launchDesc;
	OSErr				theErr;
	
	AECreateDesc(typeProcessSerialNumber, (Ptr) &(fShell->fPSN), sizeof(PSN), &myAddress);
	AECreateAppleEvent('INET', 'TSTR', &myAddress, kAutoGenerateReturnID, kAnyTransactionID, &theEvent);

	AEPutParamPtr(&theEvent, 'STRM', typeLongInteger, (Ptr) &(fStdOut), sizeof(StreamPtr));
	AEPutParamPtr(&theEvent, 'CMND', typeChar, cmdString, strlen(cmdString) + 1);

	AECoerceDesc(&theEvent, typeAppParameters, &launchDesc);

	HLock(launchDesc.dataHandle);

	lpb.launchAppParameters = (AppParameters*) *(launchDesc.dataHandle);
	lpb.launchBlockID = extendedBlock;
	lpb.launchEPBLength = extendedBlockLen;
	lpb.launchFileFlags = 0;
	lpb.launchControlFlags = launchContinue + launchNoFileFlags;
	lpb.launchAppSpec = cmd;
	
	theErr = LaunchApplication(&lpb);
	
	HUnlock(launchDesc.dataHandle);
	
	AEDisposeDesc(&launchDesc);
	AEDisposeDesc(&theEvent);
	AEDisposeDesc(&myAddress);
	
	fRelease = (theErr != noErr);
}

Boolean
Command::FindToolServer(PSNPtr ts)
{
	ProcessSerialNumber	psn;
	ProcessInfoRec		prc;
	Boolean				found = false;
	
	psn.highLongOfPSN = 0;
	psn.lowLongOfPSN = kNoProcess;
	
	prc.processInfoLength = sizeof(ProcessInfoRec);
	prc.processName = nil;
	prc.processAppSpec = nil;

	while (!found && (GetNextProcess(&psn) == noErr)) {
		if (GetProcessInformation(&psn, &prc) == noErr) {
			if ((prc.processType == 'APPL') && (prc.processSignature == 'MPSX')) {
				ts->highLongOfPSN = psn.highLongOfPSN;
				ts->lowLongOfPSN = psn.lowLongOfPSN;			
				found = true;
			}
		}
	}

	return found;
}

void
Command::GiveToToolServer(PSNPtr ts, char* cmdString)
{
	AppleEvent		theEvent;
	AEAddressDesc	toolserver;
	AppleEvent		reply;
	
	DebugIfErr(AECreateDesc(typeProcessSerialNumber, (Ptr) ts, sizeof(PSN), &toolserver));
	DebugIfErr(AECreateAppleEvent('MPS ', 'scpt', &toolserver, kAutoGenerateReturnID, (long) this, &theEvent));
	DebugIfErr(AEPutParamPtr(&theEvent, keyDirectObject, typeChar, cmdString, strlen(cmdString) + 1));
	
	DebugIfErr(AESend(&theEvent, &reply, kAEQueueReply | kAENeverInteract, kAENormalPriority, kAEDefaultTimeout, nil, nil));
	
	AEDisposeDesc(&theEvent);
	AEDisposeDesc(&toolserver);
}
