//
//                                    WiniEdit
//                       Example Windows application program
//                      S. Chernicoff         15 January 1995
// 

//         Simple program to display and manipulate a window on the screen


	#include <windows.h>
	#include "WiniEdit.h"
	#include "WiniEdit Resources.h"

//--------------------------------------------------------------------------------------------------------------

INT CALLBACK WinMain (HINSTANCE instHandle, HINSTANCE prevHandle, CHAR *commandLine, INT showState)
	
	// Main function
	
	{
		
		Initialize (instHandle, prevHandle, commandLine, showState);	// Do one-time-only initialization
		
		
		do
			MainLoop ();											// Execute one pass of main loop
		while ( ContinueFlag );									// Continue until time to quit
		
		
		Finalize ();												// Do one-time-only finalization
		
		
		return NO_ERROR;											// Signal successful completion
		
		
	}	/* end WinMain */

//--------------------------------------------------------------------------------------------------------------

VOID PASCAL Initialize (HANDLE instHandle, HANDLE prevHandle, CHAR *commandLine, INT showState)
	
	// Do one-time-only initialization.
	
	{
		ThisInstance = instHandle;								// Save global instance handle
		PrevInstance = prevHandle;								// Save previous instance handle
		
		InitProgram ();											// Do global program initialization
		InitInstance (commandLine, showState);				// Initialize this instance of program
		
	}	/* end Initialize */

//--------------------------------------------------------------------------------------------------------------

VOID InitProgram (VOID)
	
	// Do global program initialization.
	
	{
		WNDCLASS  windowClass;												// Window class
		CHAR     *resourceID;												// Resource ID in string form
		HICON     progIcon;													// Program's screen icon
		HCURSOR   arrowCursor;												// Default cursor
		HBRUSH    bkBrush;													// Brush for painting window background
		
				
		if ( PrevInstance == NULL )										// Is this the first instance of program?
			{
				LoadString (ThisInstance, ProgName_Str,				// Get program name from resource
				            ProgName, ClassNameMax);
				windowClass.lpszClassName = ProgName;					// Use as class name
				
				windowClass.lpfnWndProc = WNDPROC(DoMessage);		// Set window procedure
				windowClass.hInstance   = ThisInstance;				// Current instance is the owner
				
				windowClass.style = CS_HREDRAW | CS_VREDRAW;			// Redraw on horizontal or vertical resize
				
				resourceID = MAKEINTRESOURCE(Main_Menu);				// Convert resource ID
				windowClass.lpszMenuName = resourceID;					// Set menu
				
				resourceID = MAKEINTRESOURCE(ProgIcon_ID);			// Convert resource ID
				progIcon   = LoadIcon(ThisInstance, resourceID);	// Load icon
				windowClass.hIcon = progIcon;								// Set icon
				
				arrowCursor = LoadCursor(NULL, IDC_ARROW);			// Load standard arrow cursor
				windowClass.hCursor = arrowCursor;						// Set cursor
				
				bkBrush = HBRUSH(COLOR_WINDOW + 1);						// Create brush for system background color
				windowClass.hbrBackground = bkBrush;					// Set window background brush
				
				windowClass.cbClsExtra = 0;								// No extra class data
				windowClass.cbWndExtra = 0;								// No extra window data
				
				RegisterClass (&windowClass);								// Register the class
				
			}	/* end if ( PrevInstance == NULL ) */
		
	}	/* end InitProgram */

//--------------------------------------------------------------------------------------------------------------

VOID InitInstance (CHAR *commandLine, INT showState)
	
	// Initialize this instance of program.
	
	{
		InitWindow (commandLine, showState);				// Initialize main window
		
		InitFileParams  ();										// Initialize parameters for file dialogs
		InitTextFormat  ();										// Initialize text format
		InitColorParams ();										// Initialize parameters for color dialog
	
		InitAccelerators ();										// Initialize accelerator table
		
	}	/* end InitInstance */

//--------------------------------------------------------------------------------------------------------------

VOID InitWindow (CHAR *commandLine, INT showState)
	
	// Initialize main window.
	
	{
		DWORD	 windowStyle;										// Style options for main window
		
		
		LoadString (ThisInstance, NoTitle_Str, NoNameTitle, TitleMax);	// Get default title from resource

		
		windowStyle = WS_OVERLAPPEDWINDOW |					// Use standard window style
						  WS_CLIPCHILDREN;						// Don't overwrite children
		
		TheWindow = CreateWindow (ProgName,					// Use program name for class name
										  NoNameTitle,				// Use default title for window with no associated file
										  windowStyle,				// Standard window with children clipped out
										  CW_USEDEFAULT,			// Let Windows choose initial x
										  CW_USEDEFAULT,			//   and y position
										  CW_USEDEFAULT,			// Let Windows choose initial width
										  CW_USEDEFAULT,			//   and height
										  NULL,						// No parent window
										  NULL,						// Use menu from window class
										  ThisInstance,			// Current program instance is the owner
										  NULL);						// No special creation parameters
		
		
		if ( commandLine[0] != '\0' )							// Is there an initial file to open?
			ReadDoc (commandLine);								// Read it into the window
		
		
		ShowWindow   (TheWindow, showState);				// Display window on screen
		UpdateWindow (TheWindow);								// Force update of client area
		
		
	}	/* end InitWindow */

//--------------------------------------------------------------------------------------------------------------

VOID InitFileParams (VOID)
	
	// Initialize parameters for file dialogs.
	
	{
		INT   filterLength;										// Length of filter string
		INT   charIndex;											// Index into filter string
		CHAR  dummyChar;											// Character used as placeholder in filter string
		
		
		FileParams.lStructSize = sizeof(OPENFILENAME);	// Size of data structure
		
		FileParams.hwndOwner = TheWindow;					// Owning window
		FileParams.hInstance = ThisInstance;				// Owning instance of program
		
		FileParams.lpstrTitle = NULL;							// Use standard title for dialog window
		FileParams.Flags      = OFN_CREATEPROMPT |		// Prompt if file doesn't exist on open
		                        OFN_OVERWRITEPROMPT;		//    or already exists on save
		
		filterLength = LoadString(ThisInstance, FileFilter_Str,					// Get filter string
		                          FilterString, FilterMax);
		dummyChar = FilterString[filterLength - 1];									// Last character is placeholder
		for ( (charIndex = 0); (charIndex < filterLength); (charIndex++) )	// Step through filter string
			if ( FilterString[charIndex] == dummyChar )								// Find placeholders and
				FilterString[charIndex] = '\0';											//    replace them with nulls
		
		FileParams.lpstrFilter  = FilterString;			// Point to filter string
		FileParams.nFilterIndex = 1;							// Select first filter in string
		
		FileParams.lpstrInitialDir = NULL;					// Start with current directory
		
		LoadString (ThisInstance, FileExt_Str,				// Get default extension from resource
		            FileExt, FileExtMax);
		FileParams.lpstrDefExt = FileExt;					// Set default extension
		
		NewPath[0] = '\0';										// Start with null path name
		FileParams.lpstrFile = NewPath;						// Pointer to path buffer
		FileParams.nMaxFile  = PathMax;						// Length of path buffer
		
		NewTitle[0] = '\0';										// Start with null file title
		FileParams.lpstrFileTitle = NewTitle;				// Pointer to title buffer
		FileParams.nMaxFileTitle  = TitleMax;				// Length of title buffer
		
		FileParams.nFileOffset    = 0;						// File and extension offsets
		FileParams.nFileExtension = 0;						//    will be returned at time of call
		
		FileParams.lpTemplateName    = NULL;				// No custom dialog template
		FileParams.lpfnHook          = NULL;				// No custom hook function
		FileParams.lCustData         = 0;					// Data for custom hook not needed
		FileParams.lpstrCustomFilter = NULL;				// No custom filter buffer
		FileParams.nMaxCustFilter    = 0;					// Bufferlength is zero
		
	}	/* end InitFileParams */

//--------------------------------------------------------------------------------------------------------------

VOID InitTextFormat (VOID)
	
	// Initialize text format.
	
	{
		
		DeviceFont = GetStockObject(DEVICE_DEFAULT_FONT);			// Get standard device font
		GetObject (DeviceFont, sizeof(TextFormat), &TextFormat);	// Initialize logical font characteristics
		TheFont = CreateFontIndirect(&TextFormat);					// Save as current font
		
		
		FontParams.lStructSize = sizeof(CHOOSEFONT);					// Size of data structure
		
		FontParams.hwndOwner = TheWindow;								// Owning window
		FontParams.hInstance = ThisInstance;							// Owning instance of program
		
		FontParams.Flags = CF_INITTOLOGFONTSTRUCT |					// Initialize dialog from logical font
		                   CF_SCREENFONTS |								// List screen fonts only
		                   CF_EFFECTS;									// Enable effects
		
		FontParams.lpLogFont = &TextFormat;								// Use for initial font
		FontParams.nFontType = 0;											// Font type will be returned at time of call
		
		FontParams.iPointSize = 0;											// Point size will be returned at time of call
		FontParams.nSizeMin   = 0;											// No minimum point size
		FontParams.nSizeMax   = 0;											// No maximum point size
		
		TextColor = 0;															// Default to system text color
		FontParams.rgbColors = GetSysColor(COLOR_WINDOWTEXT);		// Start with system text color
		
		FontParams.hDC            = NULL;								// No printer context
		FontParams.lpszStyle      = NULL;								// No style buffer
		FontParams.lpTemplateName = NULL;								// No custom dialog template
		FontParams.lpfnHook       = NULL;								// No custom hook function
		FontParams.lCustData      = 0;									// Data for custom hook not needed
		
		
	}	/* end InitTextFormat */

//--------------------------------------------------------------------------------------------------------------

VOID InitColorParams (VOID)
	
	// Initialize parameters for color dialog.
	
	{
		ColorParams.lStructSize = sizeof(CHOOSECOLOR);	// Size of data structure
		
		ColorParams.hwndOwner = TheWindow;					// Owning window
		ColorParams.hInstance = ThisInstance;				// Owning instance of program
		
		ColorParams.Flags = CC_RGBINIT;						// Use initial color setting
		
		BackgroundColor = 0;												// Default to system background color
		ColorParams.rgbResult    = GetSysColor(COLOR_WINDOW);	// Start with system background color
		ColorParams.lpCustColors = UserColors;						// Pointer to custom color array
		
		ColorParams.lpTemplateName = NULL;					// No custom dialog template
		ColorParams.lpfnHook       = NULL;					// No custom hook function
		ColorParams.lCustData      = 0;						// Data for custom hook not needed
		
	}	/* end InitColorParams */

//--------------------------------------------------------------------------------------------------------------

VOID InitAccelerators (VOID)
	
	// Initialize accelerator table.
	
	{
		CHAR *resourceID;														// Resource ID in string form
		
		
		resourceID = MAKEINTRESOURCE(Accel_ID);						// Convert resource ID
		AccelTable = LoadAccelerators(ThisInstance, resourceID);	// Load accelerator table
		
	}	/* end InitAccelerators */

//--------------------------------------------------------------------------------------------------------------

VOID MainLoop (VOID)
	
	// Execute one pass of main program loop.
	
	{
		MSG   theMessage;											// Next message to process
		BOOL  translated;											// Was message translated as a keyboard accelerator?
		
		
		ContinueFlag = GetMessage(&theMessage, NULL, 0, 0);	// Get next message
		
		translated = TranslateAccelerator (TheWindow, AccelTable, &theMessage);	// Check for keyboard accelerator
		
		if ( !translated )										// Was the message an accelerator?
			{
				TranslateMessage (&theMessage);				// Convert virtual keys to characters
				DispatchMessage  (&theMessage);				// Send message to window procedure
			
			}  /* end if ( !translated ) */
		
	}	/* end MainLoop */

//--------------------------------------------------------------------------------------------------------------

LONG CALLBACK DoMessage (HWND thisWindow, UINT msgCode, WPARAM wParam, LPARAM lParam)
	
	// Get and process one message.
	
	{

		LONG  result = 0;														// Function result
		
		
		ErrorFlag = FALSE;													// Clear I/O error flag
		
		
		switch ( msgCode )													// Dispatch on message code
			{
				case WM_CREATE:
					DoCreate (thisWindow, wParam, lParam);				// Handle WM_CREATE message
					break;
					
				case WM_SIZE:
					DoSize (thisWindow, wParam, lParam);				// Handle WM_SIZE message
					break;
					
//				case WM_PAINT:
//					DoPaint (thisWindow, wParam, lParam);				// Handle WM_PAINT message
//					break;
//					
				case WM_CTLCOLOREDIT:
					result = DoCtlColorEdit (thisWindow, wParam, lParam);		// Handle WM_CTLCOLOREDIT message
					break;
					
				case WM_SYSCOLORCHANGE:
					DoSysColorChange (thisWindow, wParam, lParam);	// Handle WM_SYSCOLORCHANGE message
					break;
					
				case WM_SETFOCUS:
					DoSetFocus (thisWindow, wParam, lParam);			// Handle WM_SETFOCUS message
					break;
					
				case WM_INITMENUPOPUP:
					DoInitMenuPopup (thisWindow, wParam, lParam);	// Handle WM_INITMENUPOPUP message
					break;
					
				case WM_COMMAND:
					DoCommand (thisWindow, wParam, lParam);			// Handle WM_COMMAND message
					break;
					
				case WM_QUERYENDSESSION:
					result = DoQuery (thisWindow, wParam, lParam);	// Handle WM_QUERYENDSESSION message
					break;
					
				case WM_CLOSE:
					DoClose ();													// Handle WM_CLOSE message
					break;
					
				case WM_DESTROY:
					DoDestroy (thisWindow, wParam, lParam);			// Handle WM_DESTROY message
					break;
					
				default:
					result = DefWindowProc (thisWindow, msgCode,		// Pass message to Windows
													wParam, lParam);			//    for default processing
					break;
					
			}  /* end switch ( msgCode ) */
		
		
		return result;
		
		
	}	/* end DoMessage */

//--------------------------------------------------------------------------------------------------------------

VOID DoCreate (HWND thisWindow, WPARAM wParam, LPARAM lParam)
	
	// Handle WM_CREATE message.
	
	{
		DWORD     editStyle;										// Style options for edit control
		COLORREF  bkColor;										// Color for window background
		
		
		editStyle = WS_CHILD |									// Child window
						WS_VISIBLE |								//    visible on screen
						WS_VSCROLL |								//    vertical scroll bar
						ES_AUTOVSCROLL |							//    vertical autoscroll
						ES_MULTILINE |								//    multiple lines of text
						ES_LEFT;										//    flush-left alignment
		
		TheEditor = CreateWindow ("EDIT",					// Standard edit control
										  NULL,						// No title
										  editStyle,				// Style options as above
										  0, 0, 0, 0,				// Position and size will be set later
										  thisWindow,				// Main window is the parent
										  HMENU(Edit_Control),	// Child identifier for notification messages
										  ThisInstance,			// Current program instance is the owner
										  NULL);						// No special creation parameters
		
		
		lstrcpy (FileTitle, NoNameTitle);					// Set default file title
		
		
		SendMessage (TheEditor, WM_SETFONT,					// Set to current text font
		             WPARAM(TheFont), 0);
		
		if ( BackgroundColor != 0 )							// Is there an explicit background color?
			bkColor = BackgroundColor;							// Use it
		else
			bkColor = GetSysColor(COLOR_WINDOW);			// Otherwise use system background color
		BackgroundBrush = CreateSolidBrush(bkColor);		// Create background brush
		
		
	}	/* end DoCreate */

//--------------------------------------------------------------------------------------------------------------

VOID DoSize (HWND thisWindow, WPARAM wParam, LPARAM lParam)
	
	// Handle WM_SIZE message.
	
	{
		INT     newWidth;												// New width of client area
		INT     newHeight;											// New height of client area
		RECT    textRect;												// Formatting rectangle for wrapping text
		LPARAM  rectParam;											// Pointer to rectangle as long-word parameter
		
		
		newWidth  = LOWORD(lParam);								// Extract new dimensions
		newHeight = HIWORD(lParam);								//    from message parameter
		
		MoveWindow (TheEditor,										// Resize edit control to fit
		            0, 0,
		            newWidth, newHeight,
		            TRUE);
		
		rectParam = LPARAM(&textRect);							// Convert to long integer
		SendMessage (TheEditor, EM_GETRECT, 0, rectParam);	// Get formatting rectangle
		InflateRect (&textRect, -TextMargin, -TextMargin);	// Inset by text margin
		SendMessage (TheEditor, EM_SETRECT, 0, rectParam);	// Set new rectangle
		
	}	/* end DoSize */

//--------------------------------------------------------------------------------------------------------------
//
//VOID DoPaint (HWND thisWindow, WPARAM wParam, LPARAM lParam)
//	
//	// Handle WM_PAINT message.
//	
//	{
//		HDC          theContext;									// Device context for window
//		PAINTSTRUCT  paintInfo;										// Information structure for painting window
//		RECT         clientRect;									// Window's client rectangle
//		
//		
//		theContext = BeginPaint (thisWindow, &paintInfo);	// Open device context
//			
//			GetClientRect (thisWindow, &clientRect);			// Get client rectangle
//			/* . . . */;												// Draw contents of client rectangle
//			
//		EndPaint (thisWindow, &paintInfo);						// Close device context
//		
//	}	/* end DoPaint */
//
//--------------------------------------------------------------------------------------------------------------

LONG DoCtlColorEdit (HWND thisWindow, WPARAM wParam, LPARAM lParam)
	
	// Handle WM_CTLCOLOREDIT message.
	
	{
		HDC       theContext;									// Handle to device context
		COLORREF  txColor;										// Color for displaying text
		COLORREF  bkColor;										// Color for window background
		
		
		theContext = HDC(wParam);								// Get device context from parameter
		
		
		if ( TextColor != 0 )									// Is there an explicit text color?
			txColor = TextColor;									// Use it
		else
			txColor = GetSysColor(COLOR_WINDOWTEXT);		// Otherwise use system text color
		
		if ( BackgroundColor != 0 )							// Is there an explicit background color?
			bkColor = BackgroundColor;							// Use it
		else
			bkColor = GetSysColor(COLOR_WINDOW);			// Otherwise use system background color
		
		
		SetTextColor (theContext, txColor);					// Set text color
		SetBkColor   (theContext, bkColor);					// Set background color
		
		
		return LONG(BackgroundBrush);							// Return window's background brush
		
		
	}	/* end DoCtlColorEdit */

//--------------------------------------------------------------------------------------------------------------

VOID DoSysColorChange (HWND thisWindow, WPARAM wParam, LPARAM lParam)
	
	// Handle WM_SYSCOLORCHANGE message.
	
	{
		COLORREF  txColor;										// New system text color
		COLORREF  bkColor;										// New system background color
		
		
		txColor = GetSysColor(COLOR_WINDOWTEXT);			// Get new text color
		bkColor = GetSysColor(COLOR_WINDOW);				// Get new background color
		
		
		if ( TextColor == 0 )									// Is text defaulted to system color?
			FontParams.rgbColors = txColor;					// Set to new system text color
		else if ( txColor == TextColor )						// Does new color match existing setting?
			TextColor = 0;											// Default back to system color
		
		
		if ( BackgroundColor == 0 )							// Is background defaulted to system color?
			{
				ColorParams.rgbResult = bkColor;				// Set to new system background color
		
				DeleteObject (BackgroundBrush);					// Destroy old background brush
				BackgroundBrush = CreateSolidBrush(bkColor);	// Create new brush
				
			}  /* end if ( BackgroundColor == 0 ) */
		else if ( bkColor == BackgroundColor )				// Does new color match existing setting?
			BackgroundColor = 0;									// Default back to system color
		
		
		InvalidateRect (TheWindow, NULL, TRUE);			// Force repaint
		
		
	}	/* end DoSysColorChange */

//--------------------------------------------------------------------------------------------------------------

VOID DoSetFocus (HWND thisWindow, WPARAM wParam, LPARAM lParam)
	
	// Handle WM_SETFOCUS message.
	
	{
		SetFocus (TheEditor);									// Pass focus to the edit control
		
	}	/* end DoSetFocus */

//--------------------------------------------------------------------------------------------------------------

VOID DoInitMenuPopup (HWND thisWindow, WPARAM wParam, LPARAM lParam)
	
	// Handle WM_INITMENUPOPUP message.
	
	{
		HMENU  theMenu   = HMENU(wParam);					// Handle to menu to be adjusted
		UINT   menuIndex = LOWORD(lParam);					// Relative position of menu in menu bar
		BOOL   isSystem  = HIWORD(lParam);					// Is it the system menu?
		
		
//		theMenu   = HMENU(wParam);								// Convert parameter to menu handle
//		menuIndex = LOWORD(lParam);							// Get menu index
//		isSystem  = HIWORD(lParam);							// Get system flag
		
		if ( isSystem )											// Is it the system menu?
			FixSystemMenu ();										// Enable/disable system menu commands
		else
			switch ( menuIndex )
				{
					case File_Menu:
						FixFileMenu (theMenu);					// Enable/disable File menu commands
						break;
					
					case Edit_Menu:
						FixEditMenu (theMenu);					// Enable/disable Edit menu commands
						break;
					
					case Format_Menu:
						FixFormatMenu (theMenu);				// Enable/disable Format menu commands
						break;
					
					case Help_Menu:
						FixHelpMenu (theMenu);					// Enable/disable Help menu commands
						break;
					
				}  /* end switch ( menuIndex ) */
		
	}	/* end DoInitMenuPopup */

//--------------------------------------------------------------------------------------------------------------

VOID FixSystemMenu (VOID)
	
	// Enable/disable system menu commands.
	
	{
		/* Nothing to do */
		
	}	/* end FixSystemMenu */

//--------------------------------------------------------------------------------------------------------------

VOID FixFileMenu (HMENU theMenu)
	
	// Enable/disable File menu commands.
	
	{
		BOOL  dirty;												// Have window's contents been changed since last save?
		
		
		EnableMenuItem (theMenu, New_Item,    MF_ENABLED);				// These commands are always available
		EnableMenuItem (theMenu, Open_Item,   MF_ENABLED);
		EnableMenuItem (theMenu, Close_Item,  MF_ENABLED);
		EnableMenuItem (theMenu, SaveAs_Item, MF_ENABLED);
		EnableMenuItem (theMenu, Exit_Item,   MF_ENABLED);
		
		EnableMenuItem (theMenu, Setup_Item, MF_GRAYED);				// Printing not implemented
		EnableMenuItem (theMenu, Print_Item, MF_GRAYED);
		
		
		dirty = SendMessage(TheEditor, EM_GETMODIFY, 0, 0);			// Ask if text has been edited
		if ( dirty )																// Has it been?
			{
				EnableMenuItem (theMenu, Save_Item, MF_ENABLED);		// Enable Save command

				if ( TheFile != NULL )											// Is window associated with a file?
					EnableMenuItem (theMenu, Revert_Item, MF_ENABLED);	// Enable Revert to Saved... command
				else
					EnableMenuItem (theMenu, Revert_Item, MF_GRAYED);	// If no file, gray out Revert
				
			}  /* end if ( dirty ) */
		else
			{
				EnableMenuItem (theMenu, Save_Item,   MF_GRAYED);		// If text not dirty,
				EnableMenuItem (theMenu, Revert_Item, MF_GRAYED);		//    gray out Save and Revert
				
			}	/* end else */
		
		
	}	/* end FixFileMenu */

//--------------------------------------------------------------------------------------------------------------

VOID FixEditMenu (HMENU theMenu)
	
	// Enable/disable Edit menu commands.
	
	{
		BOOL  canUndo;												// Can editor support Undo command?
		LONG  selStart;											// Character position at start of selection
		LONG  selEnd;												// Character position at end of selection
		BOOL  canPaste;											// Is there text on the clipboard?
		LONG  textLength;											// Number of characters in document
		
		
		canUndo = SendMessage(TheEditor, EM_CANUNDO, 0, 0);		// Is Undo available?
		if ( canUndo )
			EnableMenuItem (theMenu, Undo_Item, MF_ENABLED);		// Enable Undo command
		else
			EnableMenuItem (theMenu, Undo_Item, MF_GRAYED);			// Gray out Undo command
		
		
		SendMessage ( TheEditor, EM_GETSEL,								// Get selection range
		              WPARAM(&selStart), LPARAM(&selEnd) );
		if ( selStart != selEnd )											// Is there a selection?
			{
				EnableMenuItem (theMenu, Cut_Item,    MF_ENABLED);	// Enable Cut command
				EnableMenuItem (theMenu, Copy_Item,   MF_ENABLED);	// Enable Copy command
				EnableMenuItem (theMenu, Delete_Item, MF_ENABLED);	// Enable Delete command
				
			}  /* end if ( selStart == selEnd ) */
		else
			{
				EnableMenuItem (theMenu, Cut_Item,    MF_GRAYED);	// Gray out Cut command
				EnableMenuItem (theMenu, Copy_Item,   MF_GRAYED);	// Gray out Copy command
				EnableMenuItem (theMenu, Delete_Item, MF_GRAYED);	// Gray out Delete command
				
			}  /* end else */
		
		
		canPaste = IsClipboardFormatAvailable(CF_TEXT);				// Does clipboard contain text?
		if ( canPaste )
			EnableMenuItem (theMenu, Paste_Item, MF_ENABLED);		// Enable Paste command
		else
			EnableMenuItem (theMenu, Paste_Item, MF_GRAYED);		// Gray out Paste command
		
		
		textLength = GetWindowTextLength(TheEditor);					// Get length of text
		if ( textLength > 0 )												// Any text in document?
			EnableMenuItem (theMenu, SelectAll_Item, MF_ENABLED);	// Enable Select All command
		else
			EnableMenuItem (theMenu, SelectAll_Item, MF_GRAYED);	// Gray out Select All command
		
		
	}	/* end FixEditMenu */

//--------------------------------------------------------------------------------------------------------------

VOID FixFormatMenu (HMENU theMenu)
	
	// Enable/disable Format menu commands.
	
	{
		EnableMenuItem (theMenu, Format_Item,     MF_ENABLED);	// Commands always enabled
		EnableMenuItem (theMenu, Default_Item,    MF_ENABLED);
		EnableMenuItem (theMenu, Background_Item, MF_ENABLED);
		
	}	/* end FixFormatMenu */

//--------------------------------------------------------------------------------------------------------------

VOID FixHelpMenu (HMENU theMenu)
	
	// Enable/disable Help menu commands.
	
	{
		EnableMenuItem (theMenu, Help_Item,  MF_GRAYED);	// Gray out Help... command
		EnableMenuItem (theMenu, About_Item, MF_ENABLED);	// Enable About WiniEdit... command
		
	}	/* end FixHelpMenu */

//--------------------------------------------------------------------------------------------------------------

VOID DoCommand (HWND thisWindow, WPARAM wParam, LPARAM lParam)
	
	// Handle WM_COMMAND message.
	
	{
		UINT  notifyCode = HIWORD(wParam);					// Notification code from child control
		UINT  itemID     = LOWORD(wParam);					// Item ID of message originator
		HWND  theControl = HWND(lParam);						// Handle to control sending notification
		

		switch ( notifyCode )
			{
				case 0:
				case 1:
					DoMenuCommand (itemID);										// Handle menu command
					break;
					
				default:
					DoNotification (itemID, theControl, notifyCode);	// Handle control notification
					break;
					
			}  /* end switch ( notifyCode ) */
		
	}	/* end DoCommand */

//--------------------------------------------------------------------------------------------------------------

VOID DoMenuCommand (UINT itemID)
	
	// Handle menu command.
	
	{
		switch ( itemID )											
			{
				
				/* File menu */
				
				case New_Item:
					DoNew ();										// Handle New command
					break;
					
				case Open_Item:
					DoOpen ();										// Handle Open... command
					break;
					
				case Close_Item:
					DoClose ();										// Handle Close command
					break;
					
				case Save_Item:
					DoSave ();										// Handle Save command
					break;
					
				case SaveAs_Item:
					DoSaveAs ();									// Handle Save As... command
					break;
					
				case Revert_Item:
					DoRevert ();									// Handle Revert to Saved... command
					break;
					
				case Setup_Item:
					DoSetup ();										// Handle Page Setup... command
					break;
					
				case Print_Item:
					DoPrint ();										// Handle Print... command
					break;
					
				case Exit_Item:
					DoExit ();										// Handle Exit command
					break;
				
				
				/* Edit menu */
				
				case Undo_Item:
					DoUndo ();										// Handle Undo command
					break;
					
				case Cut_Item:
					DoCut ();										// Handle Cut command
					break;
					
				case Copy_Item:
					DoCopy ();										// Handle Copy command
					break;
					
				case Paste_Item:
					DoPaste ();										// Handle Paste command
					break;
					
				case Delete_Item:
					DoDelete ();									// Handle Delete command
					break;
					
				case SelectAll_Item:
					DoSelectAll ();								// Handle Select All command
					break;
				
				
				/* Format menu */
				
				case Format_Item:
					DoFormat ();									// Handle Text Format... command
					break;
					
				case Default_Item:
					DoDefaultFormat ();							// Handle Default Format command
					break;
					
				case Background_Item:
					DoBackground ();								// Handle Background Color... command
					break;
				
				
				/* Help menu */
				
				case Help_Item:
					DoHelp ();										// Handle Help command
					break;
					
				case About_Item:
					DoAbout ();										// Handle About WiniEdit... command
					break;
					
				
				
				default:
					MessageBeep (MB_OK);							// Error: control should never reach this point
					break;
				
					
			}  /* end switch ( itemID ) */
		
		
	}	/* end DoMenuCommand */

//--------------------------------------------------------------------------------------------------------------

VOID DoNew (VOID)
	
	// Handle New command.
	
	{
		BOOL  confirmed;											// Did user confirm operation?
		
		
		confirmed = CloseDoc ();								// Allow user to save document if necessary
		if ( confirmed )											// Did user confirm?
			{
				TheFile     = NULL;								// Clear current file
				PathName[0] = '\0';								// Clear path name
				lstrcpy (FileTitle, NoNameTitle);			// Set default file title
				
				SetWindowText (TheWindow, NoNameTitle);	// Set window title to default
				
			}  /* end if ( confirmed ) */
		
	}	/* end DoNew */

//--------------------------------------------------------------------------------------------------------------

VOID DoOpen (VOID)
	
	// Handle Open... command.
	
	{
		BOOL    confirmed;										// Did user confirm operation?
		
		
		confirmed = GetOpenFileName (&FileParams);		// Get file name from user
		
		if ( confirmed )											// Did user confirm file selection?
			ReadDoc (NewPath);									// Open file and read into window
		
	}	/* end DoOpen */

//--------------------------------------------------------------------------------------------------------------

VOID ReadDoc (CHAR *newPath)
	
	// Read document into window.
	
	{
		BOOL    confirmed;										// Did user confirm operation?
		HANDLE  newFile;											// Handle to new file
		
		
		confirmed = CloseDoc ();								// Allow user to save previous document if necessary
		if ( confirmed )											// Did user confirm?
			{
				newFile = CreateFile (newPath,								// Open new file
				                      GENERIC_READ | GENERIC_WRITE,	// Read-write access
											 FILE_SHARE_READ,						// Share for reading only
											 NULL,									// No security attributes
											 OPEN_ALWAYS,							// Open existing file or create a new one
											 FILE_ATTRIBUTE_NORMAL,				// No special attributes
											 NULL);									// No template file
				
				if ( newFile == INVALID_HANDLE_VALUE )						// Did operation fail?
					IOCheck ();														// Handle error
				else
					SetLastError (NO_ERROR);									// Clear possible "already exists" error
				
				if ( ErrorFlag )													// Was there an error?
					{
						TheFile      = NULL;										// Window is left with no file:
						PathName[0]  = '\0';										//    clear global file info
						FileTitle[0] = '\0';
				
						SetWindowText (TheWindow, NoNameTitle);			// Set window title to default
						
					}  /* end if ( ErrorFlag ) */
				else
					{
						TheFile = newFile;										// Make new file current
						lstrcpy (PathName,  newPath);							// Save path name
						GetFileTitle (PathName, FileTitle, TitleMax);	// Extract local file name
						
						SetWindowText (TheWindow, FileTitle);				// Local file name becomes window title
						DoRevert ();												// Read file into window
						
					}  /* end else */
				
			}  /* end if ( confirmed ) */
		
		
	}	/* end ReadDoc */

//--------------------------------------------------------------------------------------------------------------

VOID DoClose (VOID)
	
	// Handle Close command.
	
	{
		BOOL  confirmed;											// Did user confirm operation?
		
		
		confirmed = CloseDoc ();								// Allow user to save document if necessary
		if ( confirmed )											// Did user confirm?
			DestroyWindow (TheWindow);							// Destroy the window
		
	}	/* end DoClose */

//--------------------------------------------------------------------------------------------------------------

BOOL CloseDoc (VOID)
	
	// Close document displayed in window.
	
	{
		BOOL  dirty;												// Have window's contents been changed since last save?
		INT   msgResult;											// Result value returned by message box
		BOOL  confirmed;											// Did user confirm operation?
		
		
		dirty = SendMessage(TheEditor, EM_GETMODIFY, 0, 0);	// Ask if text has been edited
		if ( dirty )
			{
				msgResult = ShowUserMessage (Save_Msg,				// Ask user whether to save window contents
				                             MB_ICONQUESTION,	// Question-mark icon
				                             MB_YESNOCANCEL,		// Yes, No, and Cancel buttons
													  "???",					// Title for message box
				                             FileTitle);			// Merge file title into message
				
				switch ( msgResult )								// Dispatch on message result
					{
						case IDYES:
							DoSave ();								// Save window contents to disk
							confirmed = !ErrorFlag;				// Confirm if no error
							break;
						
						case IDNO:
							confirmed = TRUE;						// Confirm without saving
							break;
						
						case IDCANCEL:
							confirmed = FALSE;					// Cancel operation
							break;
						
					}  /* end switch ( msgResult ) */
				
			}  /* end if ( dirty ) */

		else
			confirmed = TRUE;										// Confirm if not dirty
		
		
		if ( confirmed )											// Did user confirm operation?
			{
				if ( (TheFile != NULL) )						// Is window associated with a file?
					{
						CloseHandle (TheFile);					// Close file
						IOCheck ();									// Check for I/O error
			
					}  /* end if ( (TheFile != NULL ) */
				
				SetWindowText (TheEditor, "");				// Clear edit control's text
				
			}  /* end if ( confirmed ) */
		
		
		return confirmed;											// Report confirmation or cancellation
		
		
	}	/* end CloseDoc */

//--------------------------------------------------------------------------------------------------------------

VOID DoSave (VOID)
	
	// Handle Save command.
	
	{
		if ( TheFile == NULL )									// Is window associated with a file?
			DoSaveAs ();											// Get file name from user
		else
			WriteDoc ();											// Write to window's file
		
	}	/* end DoSave */

//--------------------------------------------------------------------------------------------------------------

VOID DoSaveAs (VOID)
	
	// Handle Save As... command.
	
	{
		BOOL    confirmed;										// Did user confirm dialog?
		HANDLE  newFile;											// Handle to new file
		
		
		lstrcpy (NewPath,  PathName);							// Start with current file name
		lstrcpy (NewTitle, FileTitle);
		
		
		confirmed = GetSaveFileName (&FileParams);		// Get file name from user
		if ( !confirmed )											// Did user cancel?
			{
				ErrorFlag = TRUE;									// Force exit to main message loop
				return;												// Skip rest of operation
				
			}  /* end if ( !confirmed ) */
		

		if ( lstrcmp(PathName, NewPath) != 0 )								// Changing to a different file?
			{
				
				if ( TheFile != NULL )											// Was there a previous file?
					{
						CloseHandle (TheFile);									// Close old file
						IOCheck ();													// Check for error
						if (ErrorFlag) return;									// On error, exit to main message loop
						
					}  /* end if ( TheFile != NULL ) */
				
				
				newFile = CreateFile (NewPath,								// Open new file
				                      GENERIC_READ | GENERIC_WRITE,	// Read-write access
											 FILE_SHARE_READ,						// Share for reading only
											 NULL,									// No security attributes
											 CREATE_ALWAYS,						// Overwrite existing file or create a new one
											 FILE_ATTRIBUTE_NORMAL,				// No special attributes
											 NULL);									// No template file
				
				if ( newFile == INVALID_HANDLE_VALUE )						// Did operation fail?
					IOCheck ();														// Handle error
				else
					SetLastError (NO_ERROR);									// Clear possible "already exists" error
						
				
				if ( ErrorFlag )													// Was there an error?
					{
						TheFile      = NULL;										// Window is left with no file:
						PathName[0]  = '\0';										//    clear global file info
						FileTitle[0] = '\0';
						
						SetWindowText (TheWindow, NoNameTitle);			// Set window title to default
						
						return;														// Exit to main message loop
						
					}  /* end if ( ErrorFlag ) */
				else
					{
						TheFile = newFile;										// Make new file current
						lstrcpy (PathName,  NewPath);
						lstrcpy (FileTitle, NewTitle);
				
						SetWindowText (TheWindow, NewTitle);				// Local file name becomes window title
						
					}  /* end else */
				
				
			}  /* end if ( lstrcmp(PathName, NewPath) != 0 ) */
		
		
		WriteDoc ();																// Write window's contents to file
		
		
	}	/* end DoSaveAs */


//--------------------------------------------------------------------------------------------------------------

VOID WriteDoc (VOID)
	
	// Write window contents to a file.
	
	{
		CHAR   textBuffer[DocMax];								// Buffer to hold text of document
		INT    textLength;										// Length of text in bytes
		LONG   textAddr;											// Address of text buffer
		ULONG  bytesWritten;										// Number of bytes written to file
		
		
		textLength = GetWindowTextLength(TheEditor);		// Get length of text
		textLength++;												// Adjust for terminating null character
		textAddr = LPARAM(textBuffer);						// Convert to long parameter
		SendMessage (TheEditor, WM_GETTEXT, textLength, textAddr);	// Get text from edit control
		
		SetFilePointer (TheFile, 0, NULL, FILE_BEGIN);	// Reset file pointer to beginning
		IOCheck ();													// Check for error
		if (ErrorFlag) return;									// On error, exit to main message loop
		
		WriteFile (TheFile, textBuffer, --textLength,	// Write text to file
			        &bytesWritten, NULL);
		IOCheck ();													// Check for error
		if (ErrorFlag) return;									// On error, exit to main message loop
		
		SetEndOfFile (TheFile);									// Set length of file
		IOCheck ();													// Check for error
		if (ErrorFlag) return;									// On error, exit to main message loop
		
		FlushFileBuffers (TheFile);							// Flush buffer to disk
		IOCheck ();													// Check for error
		if (ErrorFlag) return;									// On error, exit to main message loop
		
		SendMessage(TheEditor, EM_SETMODIFY, FALSE, 0);	// Mark text as clean
		
	}	/* end WriteDoc */

//--------------------------------------------------------------------------------------------------------------

VOID DoRevert (VOID)
	
	// Handle Revert to Saved... command.
	
	{
		BOOL   dirty;												// Have window's contents been changed since last save?
		INT    msgResult;											// Result value returned by message box
		CHAR   textBuffer[DocMax];								// Buffer to hold text of document
		LONG   textLength;										// Length of text in bytes
		LONG   textAddr;											// Address of text buffer
		ULONG  bytesRead;											// Number of bytes read from file
		
		
		dirty = SendMessage(TheEditor, EM_GETMODIFY, 0, 0);	// Ask if text has been edited
		if ( dirty )														// Has it been?
			{
				msgResult = ShowUserMessage (Revert_Msg,			// Display revert message
				                             MB_ICONQUESTION,	// Question-mark icon
				                             MB_OKCANCEL,			// OK and Cancel buttons
													  "???",					// Title for message box
				                             FileTitle);			// Merge file title into message
				if ( msgResult == IDCANCEL )							// Did user cancel?
					return;													// Skip rest of operation
				
			}  /* end if ( dirty ) */
		
		textLength = GetFileSize(TheFile, NULL);			// Get length of file
		if ( textLength > DocMax )								// File too long?
			{
				ShowUserMessage (TooLong_Msg,					// Display "file too long" message
				                 MB_ICONEXCLAMATION,		// Exclamation-point icon
								     MB_OK,							// OK button
									  NULL,							// Use default title for message box
								     FileTitle);					// Merge file title into message
				
				CloseHandle (TheFile);							// Close file
				ErrorFlag = TRUE;									// Force exit
				
			}  /* end if ( textLength > DocMax ) */
		else
			IOCheck ();												// Check for I/O error
		if (ErrorFlag) return;									// On error, exit to main message loop
		
		SetFilePointer (TheFile, 0, NULL, FILE_BEGIN);	// Move file pointer to beginning
		IOCheck ();													// Check for error
		if (ErrorFlag) return;									// On error, exit to main message loop
		
		ReadFile (TheFile, textBuffer, textLength,		// Read text of file into block
		          &bytesRead, NULL);
		textBuffer[textLength] = '\0';						// Add terminating null character
		IOCheck ();													// Check for error
		if (ErrorFlag) return;									// On error, exit to main message loop
		
		textAddr = LPARAM(textBuffer);							// Convert to long parameter
		SendMessage (TheEditor, WM_SETTEXT, 0, textAddr);	// Copy file contents into edit control
		
	}	/* end DoRevert */

//--------------------------------------------------------------------------------------------------------------

VOID DoSetup (VOID)
	
	// Handle Page Setup... command.
	
	{
		MessageBeep (MB_OK);										// Printing not implemented
		
	}	/* end DoSetup */

//--------------------------------------------------------------------------------------------------------------

VOID DoPrint (VOID)
	
	// Handle Print... command.
	
	{
		MessageBeep (MB_OK);										// Printing not implemented
		
	}	/* end DoPrint */

//--------------------------------------------------------------------------------------------------------------

VOID DoExit (VOID)
	
	// Handle Exit command.
	
	{
		SendMessage (TheWindow, WM_CLOSE, 0, 0);			// Close the window
		
	}	/* end DoExit */

//--------------------------------------------------------------------------------------------------------------

VOID DoUndo (VOID)
	
	// Handle Undo command.
	
	{
		SendMessage (TheEditor, WM_UNDO, 0, 0);			// Relay operation to edit control
		
	}	/* end DoUndo */

//--------------------------------------------------------------------------------------------------------------

VOID DoCut (VOID)
	
	// Handle Cut command.
	
	{
		SendMessage (TheEditor, WM_CUT, 0, 0);				// Relay operation to edit control
		
	}	/* end DoCut */

//--------------------------------------------------------------------------------------------------------------

VOID DoCopy (VOID)
	
	// Handle Copy command.
	
	{
		SendMessage (TheEditor, WM_COPY, 0, 0);			// Relay operation to edit control
		
	}	/* end DoCopy */

//--------------------------------------------------------------------------------------------------------------

VOID DoPaste (VOID)
	
	// Handle Paste command.
	
	{
		SendMessage (TheEditor, WM_PASTE, 0, 0);			// Relay operation to edit control
		
	}	/* end DoPaste */

//--------------------------------------------------------------------------------------------------------------

VOID DoDelete (VOID)
	
	// Handle Delete command.
	
	{
		SendMessage (TheEditor, WM_CLEAR, 0, 0);			// Relay operation to edit control
		
	}	/* end DoDelete */

//--------------------------------------------------------------------------------------------------------------

VOID DoSelectAll (VOID)
	
	// Handle Select All command.
	
	{
		SendMessage (TheEditor, EM_SETSEL, 0, -1);		// Select entire document
		
	}	/* end DoSelectAll */

//--------------------------------------------------------------------------------------------------------------

VOID DoFormat (VOID)
	
	// Handle Format... command.
	
	{
		BOOL  confirmed;											// Did user confirm dialog?
		
		
		confirmed = ChooseFont (&FontParams);				// Get new format from user
		if ( confirmed )											// Did user confirm?
			SetFormat ();											// Set new text format
		
	}	/* end DoFormat */

//--------------------------------------------------------------------------------------------------------------

VOID DoDefaultFormat (VOID)
	
	// Handle Default Format command.
	
	{
		INT       msgResult;										// Result value returned by message box
		COLORREF  txColor;										// System text color
		COLORREF  bkColor;										// System background color
		
		
		msgResult = ShowUserMessage (DefaultFormat_Msg,	// Display default format message
		                             MB_ICONQUESTION,	// Question-mark icon
		                             MB_OKCANCEL,			// OK and Cancel buttons
											  "???",					// Title for message box
		                             FileTitle);			// Merge file title into message
		
		
		if ( msgResult != IDCANCEL )							// Did user confirm?
			{
				GetObject (DeviceFont, sizeof(TextFormat), &TextFormat);	// Get standard font characteristics
				
				txColor = GetSysColor(COLOR_WINDOWTEXT);	// Get system text color
				bkColor = GetSysColor(COLOR_WINDOW);		// Get system background color
				
				FontParams.rgbColors  = txColor;				// Set text to system color
				ColorParams.rgbResult = bkColor;				// Set background to system color
				
				SetFormat ();										// Set new text format
				SetBackground ();									// Set new background color
				
			}  /* end if ( msgResult != IDCANCEL ) */
		
		
	}	/* end DoDefaultFormat */

//--------------------------------------------------------------------------------------------------------------

VOID SetFormat (VOID)
	
	// Set new text format.
	
	{
		RECT      textRect;										// Formatting rectangle for wrapping text
		LPARAM    redrawFlag;									// Redraw parameter for setting font
		COLORREF  newColor;										// Requested text color
		COLORREF  sysColor;										// System text color
		
		
		SendMessage (TheEditor, EM_GETRECT,					// Get formatting rectangle
		             0, LPARAM(&textRect));
			
			DeleteObject (TheFont);									// Delete previous font
			TheFont    = CreateFontIndirect(&TextFormat);	// Create new font
			redrawFlag = MAKELPARAM(TRUE, 0);					// Set flag to redraw text
			SendMessage (TheEditor, WM_SETFONT,					// Set new font
			             WPARAM(TheFont), redrawFlag);
			
			newColor = FontParams.rgbColors;					// Get requested color
			sysColor = GetSysColor(COLOR_WINDOWTEXT);		// Get system text color
			
			if ( newColor == sysColor )						// Did user choose system color?
				TextColor = 0;										// Default back to system color
			else
				TextColor = newColor;							// Save requested color
			
		SendMessage (TheEditor, EM_SETRECT,					// Restore formatting rectangle
		             0, LPARAM(&textRect));
		
	}	/* end SetFormat */

//--------------------------------------------------------------------------------------------------------------

VOID DoBackground (VOID)
	
	// Handle Background Color... command.
	
	{
		BOOL  confirmed;											// Did user confirm dialog?
		
		
		confirmed = ChooseColor (&ColorParams);			// Get new color from user
		if ( confirmed )											// Did user confirm?
			SetBackground ();										// Set new background color
		
	}	/* end DoBackground */

//--------------------------------------------------------------------------------------------------------------

VOID SetBackground (VOID)
	
	// Set new background color.
	
	{
		COLORREF  newColor;										// Requested background color
		COLORREF  sysColor;										// System background color
		
		
		newColor = ColorParams.rgbResult;					// Get requested color
		sysColor = GetSysColor(COLOR_WINDOW);				// Get system background color

		if ( newColor == sysColor )							// Did user choose system color?
			BackgroundColor = 0;									// Default back to system color
		else
			BackgroundColor = newColor;						// Save requested color
		
		DeleteObject (BackgroundBrush);						// Destroy old background brush
		BackgroundBrush = CreateSolidBrush(newColor);	// Create new brush
		
		InvalidateRect (TheEditor, NULL, TRUE);			// Force repaint
		
	}	/* end SetBackground */

//--------------------------------------------------------------------------------------------------------------

VOID DoHelp (VOID)
	
	// Handle Help command.
	
	{
		MessageBeep (MB_OK);										// Help not implemented
		
	}	/* end DoHelp */

//--------------------------------------------------------------------------------------------------------------

VOID DoAbout (VOID)
	
	// Handle About WiniEdit... command.
	
	{
		CHAR  *resourceID;										// Resource ID in string form
		
		
		resourceID = MAKEINTRESOURCE(About_Dialog);		// Convert resource ID
		DialogBox ( ThisInstance, resourceID,				// Invoke dialog
		            TheWindow, DLGPROC(AboutProc) );
		
	}	/* end DoAbout */

//--------------------------------------------------------------------------------------------------------------

BOOL CALLBACK AboutProc (HWND theDialog, UINT msgCode, WPARAM wParam, LPARAM lParam)
	
	// Dialog procedure for About WiniEdit... dialog.
	
	{
		BOOL  result;												// Function result
		
		
		switch ( msgCode )										// Dispatch on message code
			{
				case WM_INITDIALOG:
					result = TRUE;									// Tell Windows to set the input focus
					break;
					
				case WM_COMMAND:
					switch ( wParam )								// Dispatch on item ID
						{
							case IDOK:								// Continue button, space bar, or Enter key
							case IDCANCEL:							// Esc key
								EndDialog (theDialog, TRUE);	// Dismiss the dialog
								result = TRUE;						// Indicate message accepted
								break;
							
							default:
								result = FALSE;					// Indicate message not accepted
								break;
							
						}  /* end switch ( wParam ) */
					break;
					
				default:
					result = FALSE;								// Indicate message not accepted
					break;
					
			}  /* end switch ( msgCode ) */
		
		return result;
		
	}	/* end AboutProc */

//--------------------------------------------------------------------------------------------------------------

VOID DoNotification (UINT itemID, HWND theControl, UINT notifyCode)
	
	// Handle control notification.
	
	{
		/* Nothing to do */
		
	}	/* end DoNotification */

//--------------------------------------------------------------------------------------------------------------

BOOL DoQuery (HWND thisWindow, UINT wParam, LONG lParam)
	
	// Handle WM_QUERYENDSESSION message.
	
	{
		BOOL  confirmed;											// Did user confirm operation?
		
		
		confirmed = CloseDoc ();								// Allow user to save document if necessary
		return confirmed;											// Report confirmation or cancellation
		
	}	/* end DoQuery */

//--------------------------------------------------------------------------------------------------------------

VOID DoDestroy (HWND thisWindow, UINT wParam, LONG lParam)
	
	// Handle WM_DESTROY message.
	
	{
		DeleteObject (BackgroundBrush);						// Destroy background brush
		
		PostQuitMessage (NO_ERROR);							// Signal program completion
		
	}	/* end DoDestroy */

//--------------------------------------------------------------------------------------------------------------

VOID Finalize (VOID)
	
	// Do one-time-only finalization.
	
	{
		/* Nothing to do */
		
	}	/* end Finalize */

//--------------------------------------------------------------------------------------------------------------

INT ShowUserMessage (UINT msgID, UINT iconStyle, UINT buttonStyle, CHAR *msgTitle, CHAR *argString)
	
	// Display user message on screen.
	
	{
		CHAR     msgFormat[MsgMax];							// Format for confirmation message
		CHAR     msgText  [MsgMax];							// Text of confirmation message
		UINT     msgStyle;										// Style options for message box
		INT      msgResult;										// Result value returned by message box
		
		
		MessageBeep (MB_OK);										// Beep for attention
		
		
		LoadString (ThisInstance, msgID,						// Get message text from resource
		            msgFormat, MsgMax );
		wsprintf   (msgText, msgFormat, argString);		// Merge argument string into message
		
		msgStyle = MB_APPLMODAL |								// Application-modal message box
		           iconStyle |									// Merge in icon style
					  buttonStyle |								// Merge in button style
					  MB_SETFOREGROUND;							// Bring to foreground

		msgResult = MessageBox (TheWindow, msgText,		// Display message box
		                        msgTitle, msgStyle);

		
		return msgResult;											// Pass result back to caller
		
		
	}	/* end ShowUserMessage */

//--------------------------------------------------------------------------------------------------------------

VOID IOCheck (VOID)
	
	// Check for I/O error.
	
	{
		LONG    errorCode;										// Code number identifying error
		CHAR    errorString[10];								// Error code in string form
		
		
		errorCode = GetLastError();							// Get error code
		
		if ( errorCode == NO_ERROR )							// Was there an error?
			ErrorFlag = FALSE;									// If not, just continue normal processing

		else
			{
				ultoa (errorCode, errorString, 10);			// Convert to string
				ShowUserMessage (IOError_Msg,					// Display generic I/O error message
				                 MB_ICONEXCLAMATION,		// Exclamation-point icon
				                 MB_OK,							// OK button
									  NULL,							// Use default title for message box
				                 errorString);				// Merge error code into message
	
				ErrorFlag = TRUE;									// Force exit to main message loop
			
			}  /* end else */
		
	}	/* end IOCheck */

