//
//                                   WiniEdit.h
//                       Example Windows application program
//                      S. Chernicoff         15 January 1995
// 

//           Global header file for WiniEdit example application program


//--------------------------------------------------------------------------------------------------------------



// Global Constants and Variables


	const INT  ClassNameMax = 32;								// Maximum length for program name
	CHAR       ProgName[ClassNameMax];						// Name of program
	
	HANDLE  ThisInstance;										// Handle to this instance of program
	HANDLE  PrevInstance;										// Handle to previous instance of program
	
	HWND    TheWindow;											// Handle to main window
	HWND    TheEditor;											// Handle to edit control
	
	HACCEL  AccelTable;											// Handle to accelerator table
	
	const INT   TextMargin = 4;								// Inset from window to text rectangle
	CHOOSEFONT  FontParams;										// Parameters for font dialog
	LOGFONT     TextFormat;										// Current text formatting characteristics
	HFONT       TheFont;											// Handle to current font
	HFONT       DeviceFont;										// Handle to standard font for device
	
	CHOOSECOLOR  ColorParams;									// Parameters for color dialog
	COLORREF     UserColors[16];								// Array of user's custom colors
	COLORREF     TextColor;										// Color for displaying text
	COLORREF     BackgroundColor;								// Color for window background
	HBRUSH       BackgroundBrush;								// Brush for painting window background
	
	const INT  DocMax = 32767;									// Maximum length of text document
	const INT  MsgMax =   256;									// Maximum length of on-screen messages
	
	HANDLE  TheFile = NULL;										// Handle to current file
	
	const INT  PathMax = MAX_PATH;							// Maximum length for file path names
	CHAR       PathName[PathMax];								// Full path name of current file
	CHAR       NewPath [PathMax];								// Full path name of new file
	
	const INT  TitleMax = 64;									// Maximum length for title strings
	CHAR       FileTitle  [TitleMax];						// Local file name of current file
	CHAR       NewTitle   [TitleMax];						// Local file name of new file
	CHAR       NoNameTitle[TitleMax];						// Default title for window with no associated file
	
	const INT  FileExtMax = 4;									// Maximum length for file extension
	CHAR       FileExt[FileExtMax];							// Default file extension
	
	const INT  FilterMax = 256;								// Maximum length for file filter string
	CHAR       FilterString[FilterMax];						// Filter string for Open and Save dialogs
	
	OPENFILENAME FileParams;									// Parameters for file dialogs
	
	BOOL  ContinueFlag = TRUE;									// Not time to quit yet?
	BOOL  ErrorFlag    = FALSE;								// I/O error flag
	
//--------------------------------------------------------------------------------------------------------------



// Function Prototypes


INT CALLBACK WinMain (HINSTANCE instHandle, HINSTANCE prevInst, CHAR *commandLine, INT showState);
		// Main function
	VOID PASCAL Initialize (HANDLE thisInstance, HANDLE prevInstance, CHAR *commandLine, INT showState);
			// Do one-time-only initialization.
		VOID InitProgram (VOID);
				// Do global program initialization.
		VOID InitInstance (CHAR *commandLine, INT showState);
				// Initialize this instance of program.
			VOID InitWindow (CHAR *commandLine, INT showState);
					// Initialize main window.
			VOID InitFileParams (VOID);
					// Initialize parameters for file dialogs.
			VOID InitTextFormat (VOID);
					// Initialize text format.
			VOID InitColorParams (VOID);
					// Initialize parameters for color dialog.
			VOID InitAccelerators (VOID);
					// Initialize accelerator table.
	VOID MainLoop (VOID);
			// Execute one pass of main program loop.
	LONG CALLBACK DoMessage (HWND thisWindow, UINT msgCode, WPARAM wParam, LPARAM lParam);
			// Get and process one message.
		VOID DoCreate (HWND thisWindow, WPARAM wParam, LPARAM lParam);
				// Handle WM_CREATE message.
		VOID DoSize (HWND thisWindow, WPARAM wParam, LPARAM lParam);
				// Handle WM_SIZE message.
//		VOID DoPaint (HWND thisWindow, WPARAM wParam, LPARAM lParam);
//				// Handle WM_PAINT message.
		LONG DoCtlColorEdit (HWND thisWindow, WPARAM wParam, LPARAM lParam);
				// Handle WM_CTLCOLOREDIT message.
		VOID DoSysColorChange (HWND thisWindow, WPARAM wParam, LPARAM lParam);
				// Handle WM_SYSCOLORCHANGE message.
		VOID DoSetFocus (HWND thisWindow, WPARAM wParam, LPARAM lParam);
				// Handle WM_SETFOCUS message.
		VOID DoInitMenuPopup (HWND thisWindow, WPARAM wParam, LPARAM lParam);
				// Handle WM_INITMENUPOPUP message.
			VOID FixSystemMenu (VOID);
				// Enable/disable system menu commands.
			VOID FixFileMenu (HMENU theMenu);
				// Enable/disable File menu commands.
			VOID FixEditMenu (HMENU theMenu);
				// Enable/disable Edit menu commands.
			VOID FixFormatMenu (HMENU theMenu);
				// Enable/disable Format menu commands.
			VOID FixHelpMenu (HMENU theMenu);
				// Enable/disable Help menu commands.
		VOID DoCommand (HWND thisWindow, WPARAM wParam, LPARAM lParam);
				// Handle WM_COMMAND message.
			VOID DoMenuCommand (UINT itemID);
					// Handle menu command.
				VOID DoNew (VOID);
						// Handle New command.
				VOID DoOpen (VOID);
						// Handle Open... command.
					VOID ReadDoc (CHAR *newPath);
							// Read document into window.
				VOID DoClose (VOID);
						// Handle Close command.
					BOOL CloseDoc (VOID);
							// Close document displayed in window.
				VOID DoSave (VOID);
						// Handle Save command.
				VOID DoSaveAs (VOID);
						// Handle Save As... command.
					VOID WriteDoc (VOID);
							// Write window contents to a file.
				VOID DoRevert (VOID);
						// Handle Revert to Saved... command.
				VOID DoSetup (VOID);
						// Handle Page Setup... command.
				VOID DoPrint (VOID);
						// Handle Print... command.
				VOID DoExit (VOID);
						// Handle Exit command.
				VOID DoUndo (VOID);
						// Handle Undo command.
				VOID DoCut (VOID);
						// Handle Cut command.
				VOID DoCopy (VOID);
						// Handle Copy command.
				VOID DoPaste (VOID);
						// Handle Paste command.
				VOID DoDelete (VOID);
						// Handle Delete command.
				VOID DoSelectAll (VOID);
						// Handle Select All command.
				VOID DoFormat (VOID);
						// Handle Format... command.
				VOID DoDefaultFormat (VOID);
						// Handle Default Format command.
					VOID SetFormat (VOID);
							// Set new text format.
				VOID DoBackground (VOID);
						// Handle Background Color... command.
					VOID SetBackground (VOID);
							// Set new background color.
				VOID DoHelp (VOID);
						// Handle Help command.
				VOID DoAbout (VOID);
						// Handle About WiniEdit... command.
					BOOL CALLBACK AboutProc (HWND thisWindow, UINT msgCode, WPARAM wParam, LPARAM lParam);
							// Dialog procedure for About WiniEdit... dialog.
			VOID DoNotification (UINT itemID, HWND theControl, UINT notifyCode);
					// Handle control notification.
		BOOL DoQuery (HWND thisWindow, UINT wParam, LONG lParam);
				// Handle WM_QUERYENDSESSION message.
		VOID DoDestroy (HWND thisWindow, UINT wParam, LONG lParam);
				// Handle WM_DESTROY message.
	VOID Finalize (VOID);
			// Do one-time-only finalization.
		INT ShowUserMessage (UINT msgID, UINT iconStyle, UINT buttonStyle, CHAR *msgTitle, CHAR  *argString);
				// Display user message on screen.
		VOID IOCheck (VOID);
				// Check for I/O error.
