//		TCityMap.cp
// Copyright  1992 Emergent Behavior. All rights reserved.


#ifndef _CITYMAP_
	#include "CityMap.h"
#endif

#ifndef __STDIO__
	#include <StdIO.h>
#endif

#ifndef _GAFAILURE_
	#include "GAFailure.h"
#endif

//-------------------------------------------------------------------------------------------------------
TCityMap::TCityMap( void )
	:	fDistanceTable( NIL ),
		fCityLocations( NIL ),
		fNumberCities( 0 )
{	
}

//-------------------------------------------------------------------------------------------------------
TCityMap::TCityMap( const TCityMap& sourceMap )
	:	fNumberCities( sourceMap.fNumberCities )
{
	AllocateDistanceTable();
	
	*fDistanceTable	= *(sourceMap.fDistanceTable);
	
	fCityLocations	= new Point[ fNumberCities ];
	FailNIL(fCityLocations);
	*fCityLocations	= *(sourceMap.fCityLocations );
}

//-------------------------------------------------------------------------------------------------------
TCityMap::TCityMap( CityNumber numCities )
	:	fNumberCities( numCities )
{
	AllocateDistanceTable();

	fCityLocations	= new Point[ fNumberCities ];
	FailNIL(fCityLocations);
	
	DecideCityLocations();
	GenerateDistanceTable();
}

//-------------------------------------------------------------------------------------------------------
TCityMap::~TCityMap( void )
{
	delete fDistanceTable;
	delete fCityLocations;
}

//-------------------------------------------------------------------------------------------------------
TCityMap*
TCityMap::Clone( void )
{
	TCityMap* copy = new TCityMap(*this);
	FailNIL(copy);
	return copy;
}

//-------------------------------------------------------------------------------------------------------
Distance
TCityMap::CalcRouteDistance( Importance* importanceList ) const
{
	return CalcTotalDistance( SortCities( importanceList ) );
}

//-------------------------------------------------------------------------------------------------------
Route
TCityMap::SortCities( Importance* importanceList ) const
{
	Importance	highestImportance		= -1;
	short		highestImportanceIndex	= 0;
	Route		theRoute				= new City[ fNumberCities ];
	FailNIL(theRoute);
	
	for ( short cityIndex = 0; cityIndex < fNumberCities; ++cityIndex )
	{
			// Find the next highest importance
		for ( short importanceIndex = 0; importanceIndex < fNumberCities; ++importanceIndex )
		{
			if ( importanceList[ importanceIndex ] > highestImportance )
			{
				highestImportanceIndex	= importanceIndex;
				highestImportance		= importanceList[ highestImportanceIndex ];
			} //.............end if..............
		}	//...........end for importanceIndex...............
		theRoute[ cityIndex ]						= highestImportanceIndex;
		importanceList[ highestImportanceIndex ]	= -1;
		highestImportance 							= -1;

	}	//............end for cityIndex............
	
	return theRoute;
}

//-------------------------------------------------------------------------------------------------------
Distance
TCityMap::CalcTotalDistance( Route theRoute ) const
{
	Distance	totalDistance = 0;
	
		// Calculate the distance of the route to the last city in the list
	for ( short cityIndex = 0; cityIndex < fNumberCities -1 ; ++cityIndex )
	{
		totalDistance	+= GetCityDistance( theRoute[ cityIndex ], theRoute[ cityIndex + 1 ] );
	}
	
	const City	kFirstCity	= 0;
	
		// Add the distance back home
	totalDistance	+= GetCityDistance( theRoute[ fNumberCities-1], theRoute[ kFirstCity ] );
	delete theRoute;

	return totalDistance;
}

//-------------------------------------------------------------------------------------------------------
Distance
TCityMap::GetCityDistance( City city1, City city2 ) const
{
	return fDistanceTable[ city1 ][ city2 ];
}


//-------------------------------------------------------------------------------------------------------
void
TCityMap::DecideCityLocations( void )
{
	for ( short cityIndex = 0; cityIndex < fNumberCities; ++cityIndex )
	{
		fCityLocations[ cityIndex ]	= TPoint( Random()%100, Random()%100 );
	}	
}

//-------------------------------------------------------------------------------------------------------
void
TCityMap::GenerateDistanceTable( void )
{
	Point		baseCityLocation;
	Point		otherCityLocation;
	short		baseCityX;
	short		baseCityY;
	short		otherCityX;
	short		otherCityY;
	Distance	distance;
	
	for ( short baseCityIndex = 0; baseCityIndex < fNumberCities; ++baseCityIndex )
	{
		baseCityLocation	= fCityLocations[ baseCityIndex ];
		baseCityX			= baseCityLocation.h;
		baseCityY			= baseCityLocation.v;
		
		for ( short otherCityIndex = 0; otherCityIndex < fNumberCities; ++otherCityIndex )
		{
			otherCityLocation	= fCityLocations[ otherCityIndex ];
			otherCityX			= otherCityLocation.h;
			otherCityY			= otherCityLocation.v;
			
			distance	= sqrt( SQR( baseCityX - otherCityX ) + SQR( baseCityY - otherCityY ) );
			fDistanceTable[ baseCityIndex ][ otherCityIndex ]	= distance;
			//int intDistance = ((distance +.5) / 1);
			//printf ( "%5d", intDistance );
		}
		//printf ("\n");
	}
	
}

//-------------------------------------------------------------------------------------------------------
void
TCityMap::AllocateDistanceTable( void )
{
	fDistanceTable	= new Distance * [ fNumberCities ];
	FailNIL(fDistanceTable);
	for ( short cityIndex = 0; cityIndex < fNumberCities; ++cityIndex )
	{
		fDistanceTable[ cityIndex ]	= new Distance [ fNumberCities ];
		FailNIL( fDistanceTable[ cityIndex ] );
	}
}

//-------------------------------------------------------------------------------------------------------
CityNumber	
TCityMap::GetNumberCities( void ) const
{
	return fNumberCities;
}

//-------------------------------------------------------------------------------------------------------
Point*		
TCityMap::GetCityLocations( void ) const
{
	return fCityLocations;
}

//-------------------------------------------------------------------------------------------------------
void	
TCityMap::SetNumberCities( CityNumber numCities )
{
	fNumberCities = numCities;
}

//-------------------------------------------------------------------------------------------------------
void	
TCityMap::SetCityLocations( Point * cityLocations )
{
	fCityLocations = cityLocations;
}

//=======================================================================================================

//------------------------------------------- T P O I N T -----------------------------------------------

//-------------------------------------------------------------------------------------------------------
TPoint::TPoint( void )
{
	fPoint.h	= 0;
	fPoint.v	= 0;
}

//-------------------------------------------------------------------------------------------------------
TPoint::TPoint( short xCoord, short yCoord )
{
	fPoint.h	= xCoord;
	fPoint.v	= yCoord;
}

//-------------------------------------------------------------------------------------------------------
TPoint::operator Point()
{
	return fPoint;
}






