/*
	Starting with an object ref for an app, find a well 
	known folder, and iterate over the contents to 
	find all the files' objectRefs and names.
	
	Simplifying assumptions:
	-	no error checking, though 'status' is set
	-	array to handle max NUM_STATIONERY_FILES
	-	text objects for file names allocated elsewhere
	
	Start with app's ref
	Find app's enclosing folder (app's container)
	Within the app's container, find the container with
		a particular name
	Create an iterator within this named folder
	Retrieve all the refs and names for the items in
		this folder
	Cleanup
	
	----------------------------------------------------------
	Sample code only - no guarantees except it doesn't compile
	----------------------------------------------------------
	
	Leon Baranovsky
	Mac OS 8 Evangelism
	Apple Computer
	
	June 20, 1996
*/


#define NUM_STATIONERY_FILES	10

struct InfoList{
	UInt32				count;
	FSObjectRef			objectRef[NUM_STATIONERY_FILES];
	FSName				objectName[NUM_STATIONERY_FILES];
};
typedef struct InfoList InfoList;


OSStatus StationeryGetRefs( 
	FSObjectRef			applicationRef,
	InfoList			*stationeryInfo );
{
	OSStatus			status;

	FSObjectRef			appContainerRef;
	Boolean				appContainerRefDispose = false;
	
	FSObjectRef			stationeryFolderRef;
	Boolean				stationeryFolderRefDispose = false;
	
	FSObjectIterator	stationeryFolderIterator;
	Boolean				stationeryFolderIteratorDispose = false;
	
	Boolean				eoi = false;
	
	
	/*
	 *	initialize variables
	 */
	stationeryInfo->count = 0;
	

	/*
	 *  find ref of container the application is in
	 */
	status = FSObjectGetOneProperty(
			applicationRef, 
			&kFSFileManagerObjectContainerObjectRefPropertyValue, 
			sizeof( appContainerRef ), 
			&appContainerRef );
	
	appContainerRefDispose = true;


	/*
	 *  find Stationery folder
	 */
	status = FSObjectCreateRef(
			appContainerRef, 
			stationeryFolderName, 
			&stationeryFolderRef );
			
	stationeryFolderRefDispose = true;
			

	/*
	 *  create iterator
	 */
	status = FSObjectIteratorCreate(
			stationeryFolderRef, 
			kFSIncludeFiles+kFSTraverseEmbeddedContainers, 
			&stationeryFolderIterator );
	
	stationeryFolderIteratorDispose = true;
	

	/*
	 *  iterate through Stationery folder
	 */
	do {
	
		status = FSObjectIterateOnce(
				stationeryFolderIterator, 
				NULL,		//FSInfoVersion infoVersion_i (kFSInfoCurrentReleasedVersion)
				NULL,		// FSObjectInformation *objectInfo_o 
				&(stationeryInfo->objectRef[ stationeryInfo->count ]), 
				&(stationeryInfo->objectName[ stationeryInfo->count ]) );
		
		stationeryInfo->count++;

		if ( status == E_EndOfIteration  ||  
				stationeryInfo->count >= NUM_STATIONERY_FILES )
			eoi = true;
	
	}  while ( !eoi )

	

Cleanup:

	/*
	 *	cleanup refs and iterator on exit
	 */
	if ( appContainerRefDispose )
		FSObjectRefDispose( appContainerRef );
	
	if ( stationeryFolderRefDispose )
		FSObjectRefDispose( stationeryFolderRef );
	
	if ( stationeryFolderIteratorDispose )
		FSObjectIteratorDispose( stationeryFolderIterator );

}
