//
// We override the scroller code to accomplish three things:
//
// - We want to handle updating of the views within us locally (the normal behavior
//   passes the update all the way up to the grafportview, which is too flickery)
//
// - We need to replace the regular and thumb action procs with our own (because
//   we run in an A4 world)
//
// - There really isn't a third thing, but it's a rule in comedy that only three is
//   funny: no more, no less.
//

#include "LMyActiveScroller.h"
#include <A4Stuff.h>
#include <MixedMode.h>

// The regular scroller code installs actionprocs for its scrollbars.
// We need to replace them with ours:
static ControlActionUPP	sMyVertSBarAction = nil;
static ControlActionUPP	sMyHorizSBarAction = nil;


LMyActiveScroller *LMyActiveScroller::CreateMyActiveScrollerStream(LStream *inStream) {
	return new LMyActiveScroller(inStream);
}


LMyActiveScroller::LMyActiveScroller(LStream *inStream) : LActiveScroller(inStream) {
	// Reassign the action procs to ours.
	ReAssignActionProcs();
}

void LMyActiveScroller::UpdatePort() {
	Draw(NULL); // just draw us and our contents
}


void LMyActiveScroller::ReAssignActionProcs()
{
	// For each of the action procs (thumb, regular) for each of the scrollbars
	// (vertical, horizontal), forget the old procs and install our own. (We can
	// call DisposeRoutineDescriptor on any architecture because MixedMode.h #define's
	// them out).
	if (mVerticalBar != nil) {
		// First, dispose of the old thumb action UPP
		if (mVertThumbAction) {
			DisposeRoutineDescriptor(mVertThumbAction);
			mVertThumbAction = nil;
		}
		// Install our own thumb UPP.
		mVertThumbAction = NewThumbActionProc(LMyActiveScroller::ActiveThumbScroll);		
		mVerticalBar->SetThumbFunc(mVertThumbAction);
		
		// If we haven't, initialize our own regular action UPP (it's global)
		if (!sMyVertSBarAction)
			sMyVertSBarAction = NewControlActionProc((ProcPtr) MyVertSBarAction);
		mVerticalBar->SetActionProc(sMyVertSBarAction);
	}
	if (mHorizontalBar != nil) {
		// First, dispose of the old thumb action UPP
		if (mHorizThumbAction) {
			DisposeRoutineDescriptor(mHorizThumbAction);
			mHorizThumbAction = nil;
		}
		// Install our own thumb UPP.
		mHorizThumbAction = NewThumbActionProc(LMyActiveScroller::ActiveThumbScroll);		
		mHorizontalBar->SetThumbFunc(mHorizThumbAction);
		
		// If we haven't, initialize our own regular action UPP (it's global)
		if (!sMyHorizSBarAction)
			sMyHorizSBarAction = NewControlActionProc((ProcPtr) MyHorizSBarAction);
		mHorizontalBar->SetActionProc(sMyHorizSBarAction);
	}	
}


pascal void	LMyActiveScroller::ActiveThumbScroll()
{
	EnterCodeResource();
	LStdControl *theScrollBar = LStdControl::GetTrackingControl();
	if (theScrollBar != nil) {
		LMyActiveScroller	*theScroller = 
				(LMyActiveScroller *)GetControlReference(theScrollBar->GetMacControl());
		theScroller->HandleThumbScroll(theScrollBar);
	}	
	ExitCodeResource();
}


pascal void LMyActiveScroller::MyVertSBarAction(ControlHandle inMacControl, Int16 inPart) {	
	EnterCodeResource();
	LScroller::VertSBarAction(inMacControl, inPart);
	ExitCodeResource();
}


pascal void LMyActiveScroller::MyHorizSBarAction(ControlHandle inMacControl, Int16 inPart) {
	EnterCodeResource();
	LScroller::HorizSBarAction(inMacControl, inPart);
	ExitCodeResource();
}
