//
// MacHack Sample Plugin
// Bryan Stearns - The Thomas Eliot Company

#include "LSamplePlugin.h"

#include <string.h>
#include <stdio.h>
#include <ctype.h>

#include <URegistrar.h>

#include "LSampleStream.h"
#include "LSampleView.h"
#include "LMyActiveScroller.h"
#include "UUseResFile.h"
#include "UCIString.h"


LSamplePlugin *gPlugin = NULL;
GrafPort *gPluginPort = NULL;

// Netscape's wrapper sets this up, but it's not declared in their .h's....
extern short gResFile;

// A handy table of hex digits
const char gHexDigits[] = "0123456789abcdef";

//
// C++ glue that we call explicitly to force static construction & destruction
//
#ifdef __cplusplus
extern "C" {
#endif
	#ifndef powerc
		extern void	__InitCode__(void);
	#else
		extern void __sinit(void);
		#define __InitCode__ __sinit
	#endif
	extern void	__destroy_global_chain(void);
#ifdef __cplusplus
}
#endif


//
// Instantiate our plugin object
// (Every PP Plugin must implement this!)
//
CNetscapePlugin* CreateNetscapePlugin(NPP instance, uint16 mode, CPluginArguments* adoptedArgs, NPSavedData* /*saved*/) {
	// Create our plugin and return it
	LSamplePlugin *samplePlugin = new LSamplePlugin(instance, mode, adoptedArgs);
	return samplePlugin;
}


//
// Initialize the plugin as a whole
// (Every PP Plugin must implement this!)
//
NPError InitializeNetscapePlugin(void)
{
	// Make sure the static objects get constructed
	// (this may need to go away in the future!)
	static int first_time_check = 0;
	if(!first_time_check) {
		__InitCode__(); // initialize static C++ data
		first_time_check = 1; // only once
	} else
		SysBreakStr("\pWhoops! NPP_Initialize called more than once?!?!?!");
	
	NPError err = LSamplePlugin::Initialize(); // initialize the plugin class
	if (err == NPERR_NO_ERROR) {
		err = CMacPluginView::Initialize(); // Initialize the view class
		if (err == NPERR_NO_ERROR) { 
			// Register the other PowerPlant classes that we use
			URegistrar::RegisterClass(LCaption::class_ID,				(ClassCreatorFunc) LCaption::CreateCaptionStream);
			URegistrar::RegisterClass(LTextEdit::class_ID,				(ClassCreatorFunc) LTextEdit::CreateTextEditStream);
			URegistrar::RegisterClass(LStdPopupMenu::class_ID,			(ClassCreatorFunc) LStdPopupMenu::CreateStdPopupMenuStream);
			
			// Register our custom class, too
			URegistrar::RegisterClass(LMyActiveScroller::class_ID,		(ClassCreatorFunc) LMyActiveScroller::CreateMyActiveScrollerStream);
		}
	}
	return err;
}


//
// Deconstruct the plugin as a whole
// (Every PP Plugin must implement this!)
//
void ShutdownNetscapePlugin( void )
{
	LSamplePlugin::Shutdown();
	
	// Called once after the last instance of us has been destroyed.
	__destroy_global_chain(); // make sure our static destructors get called
}


//
// Initialize the plugin class (this is a static member function)
//
NPError LSamplePlugin::Initialize(void) {
	return NPERR_NO_ERROR; 
}

//
// Shutdown the plugin class (this is a static member function)
NPError LSamplePlugin::Shutdown(void) {
	return NPERR_NO_ERROR;
}


//
// Construct the plugin object
//
LSamplePlugin::LSamplePlugin(NPP instance, uint16 mode, CPluginArguments* adoptedArgs)
: mText(NULL), mPopup(NULL), mScroller(NULL), mRawData(NULL), mRawLength(0), 
  CNetscapePlugin(instance, mode, adoptedArgs) {
	instance->pdata = (void*) this;
	
	// Figure out which mode we're in
	mShowMode = eHexASCIIMode; // start out assuming we're going to be in hex+ASCII Mode
	StUseResFile stResFile(gResFile); // Point at our resource file
	TString<Str31> showTag(kShowTagString,0);
	if (showTag.Length()) { // we got the tag string
		showTag[showTag.Length()+1] = '\0'; // Make it a C string.
		const char *showModeString = mArgs->ArgumentValueAsString((char *) &showTag[1]);
		if (showModeString && *showModeString)
			mShowMode = (EShowMode) LookupString((char *) showModeString, kShowOptionsStringList);
	}

	// We keep a pointer to the plugin object, as well as our grafport, in globals
	gPlugin = this;
	gPluginPort = NULL;
}


//
// Destruct the plugin object
//
LSamplePlugin::~LSamplePlugin()
{
	gPlugin = NULL;
	gPluginPort = NULL;
}


//
// Handle an event - we override to force our resfile
// 
Int16 LSamplePlugin::Handle_Event(void* inEvent) {
	CMacPluginView* macView = GetMacPluginView();
	if (!macView) return false; // ignore events until we get our views

	// Before handling the event, point at our resource file
	// (this is useful if someone needs a text-traits resource, for instance!)	
	StUseResFile stResFile(gResFile); // Point at our resource file
			
	// Pass all the usual events to the view to handle
	return macView->Handle_Event(*(EventRecord *) inEvent);
}


//
// Create a stream to handle this MIME type
//
CNetscapeStream* LSamplePlugin::CreateStream(NPMIMEType type, NPStream *stream, NPBool seekable, uint16 stype) {
	return new LSampleStream(type, stream, seekable, stype, this);
}

//
// Create the view hierarchy for our plugin
//
CPluginView* LSamplePlugin::CreatePluginView(void) {
	// Create our view
	LSampleView *newView = new LSampleView(this);
	gPluginPort = newView->GetMacPort(); // note its port for later

	// Find the views we care about
	mText = (LTextEdit*) newView->FindPaneByID(kTextPane);
	mPopup = (LStdPopupMenu*) newView->FindPaneByID(kPopupPane);
	mScroller = (LMyActiveScroller*) newView->FindPaneByID(kScrollerPane);
	
	// Set the popup to match the show-mode setting, then tell it to let us
	// know when someone changes it.
	if (mPopup) {
		mPopup->SetValue(mShowMode);
		mPopup->AddListener(this);
	}

	// If we've gotten data, put it in the view now
	if (mRawData) 
		DumpText();
		
	return newView;
}


//
// Try to find this string in this STR# resource
// If we find it, return the index (1-based); if not, return 0.
int LookupString(char *target, const ResIDT fromList) {
	StUseResFile stResFile(gResFile); // Point at our resource file
	StResource stringListRsrc('STR#', fromList, false /* don't throw */, true /* curresfile only */);
	Handle stringList = stringListRsrc;
	
	if (!stringList) return 0; // no resource? string not found.
	
	StHandleLocker listLock(stringList);
	short *countPtr = (short *) *stringList;
	short count = *countPtr;
	unsigned char *cp = (unsigned char *) (countPtr + 1);
	int targetLen = strlen(target);
	while (count--) {
		int len = *(cp++); // get the length byte
		if (len == targetLen) { // length matches!
			if (!cistrncmp((char *) cp, target, targetLen))
				return (*countPtr - count);
		}
		cp += len;
	}
	return 0; // not found
}


//
// Someone changed the popup
//
void LSamplePlugin::ListenToMessage(MessageT inMessage, void * /*ioParam*/) {
	if (inMessage == kPopupPane) {
		// user changed the popup menu - rebuild the text
		mShowMode = (EShowMode) (mPopup->GetValue() - 1); // remember the new setting
		DumpText(); // rebuild the text
	}
}


//
// Parse part (or, by default, all) of the raw data according to the current mode
// (if len == 0, we'll replace all the text; otherwise, we'll start at "offset" and
// add to the current text (but we may need to replace the last line if it was partial)
//
void LSamplePlugin::DumpText(int32 offset, int32 len) {
	if (!mText) return; // do nothing if we don't have the view yet
	
	// How big will the text be?
	int32 lineShift; 
	int32 lineMask;
	int32 charsPerLine;
	if (mShowMode == eASCIIMode) {
		// ASCII mode: 64 bytes displayed per line; in addition to each source byte,
		// there are 14 extra characters added per line:
		//     addr:  12345678 12345678 12345678 12345678  12345678 12345678 12345678 12345678
		//     123456789abcdef1123456789abcdef2123456789abcdef3123456789abcef4123456789abcdef51
		lineShift = 6; // 2**6 = 64 bytes per line
		lineMask = 0x1f; // 0001 1111 to mask off line remainder
		charsPerLine = 0x51;
	} else { // it's eHexASCIIMode:
		// Hex+ASCII mode: 16 bytes displayed per line; each source byte expands to 
		//   4 characters directly (two hex digits plus one ASCII char, plus a space), 
		//   plus 12 characters more for each 16 bytes:
		//     addr:  00 00 00 00  00 00 00 00  00 00 00 00  00 00 00 00  12345678 12345678
		//     123456789abcdef1123456789abcdef2123456789abcef3123456789abcdef4123456789abcde
		lineShift = 4; // 2**4 = sixteen bytes per line
		lineMask = 0xf; // 0000 1111 to mask off line remainder
		charsPerLine = 0x4e;
	}
	
	// So that's how many raw bytes per line?
	int bytesPerLine = 1 << lineShift;
	
	// How many lines is the text now?
	int32 newLineCount = mRawLength >> lineShift;
	if ((newLineCount << lineShift) != mRawLength) ++newLineCount; // last partial line

	// Adjust the handle size, then start shoving text in
	// (this may be a little large, but we'll adjust back down when we're done.
	Handle textHandle = mText->GetTextHandle();
	SetHandleSize(textHandle, (newLineCount * charsPerLine));
	
	// We'll reparse from the start of a line
	int32 lineOffset = offset & lineMask;
	len += offset - lineOffset; // add in the existing prior characters
	char *inPtr = *mRawData + lineOffset;
	char *outPtr = *textHandle + (lineOffset / charsPerLine);
	char *inEnd = *mRawData + mRawLength;
	for (int theLine = 0; theLine < newLineCount; ++theLine) {
		int theChar;
		
		// Stuff this line's "address"
		sprintf(outPtr, "%04x:", lineOffset + (theLine * bytesPerLine));
		outPtr += 5; // point past the address
		
		// Stuff the right number of hex digits, with appropriate spacing
		char *inPtrCopy = inPtr;
		if (mShowMode == eHexASCIIMode) {
			for (theChar = 0; theChar < bytesPerLine; ++theChar) {
				// Precede each byte with one 
				*(outPtr++) = ' '; // preceded by one space
				
				// Add an extra space before each longword
				if ((theChar & 0x3) == 0)
					*(outPtr++) = ' ';

				if (inPtrCopy < inEnd) {
					// Stuff this byte in hex
					*(outPtr++) = gHexDigits[(*inPtrCopy >> 4) & 0xf];
					*(outPtr++) = gHexDigits[*inPtrCopy & 0xf];
				} else { // Past the end of the data; fill with spaces
					*(outPtr++) = ' ';
					*(outPtr++) = ' ';
				}
				++inPtrCopy;
			}

			// Add an extra space between hex & ASCII
			*(outPtr++) = ' '; 
		}
		
		// Stuff the right number of ASCII digits, with appropriate spacing
		for (theChar = 0; theChar < bytesPerLine; ++theChar) {
			// Add an extra space before each eight characters
			if ((theChar & 0x7) == 0)
				*(outPtr++) = ' ';

			// Stuff this character in ASCII (or a period if it's not printable
			if (inPtr < inEnd)
				*(outPtr++) = isprint(*inPtr) ? *inPtr : '.';
			++inPtr;
		}
		
		// Add a carriage return
		*(outPtr++) = '\r';
	}
	
	// Reduce the size to the amount we actually made drop the last CR and everything after it.
	SetHandleSize(textHandle, --outPtr - *textHandle); 
	
	// Let TextEdit know we've messed with the text.
	::TECalText(mText->GetMacTEH()); // recalibrate the line ends, etc.
	mText->AdjustImageToText();
	StFocusAndClipIfHidden clipper(mScroller);
	Rect frame;
	mScroller->CalcLocalFrameRect(frame);
	::EraseRect(&frame); // Erase in case the text got shorter
	mScroller->UpdatePort();
}


//
// More data has arrived - add it to the handle and parse it into the text
//
void LSamplePlugin::MoreData(int32 offset, int32 len, void *buffer) {
	// Limit ourselves to 8K of data (since data expands about 4-to-1, and
	// TextEdit can only handle 32K of text...)
	if ((offset + len) > 8192) len = 8192 - offset;
	if (len <= 0) return; // no new data? nothing to do.
	
	// Have we got a handle yet? if not, allocate it; if so, extend it.
	if (!mRawData) {
		mRawData = NewHandle(len);
		if (!mRawData) return; // skip out if we didn't get it.
	} else {
		SetHandleSize(mRawData, mRawLength + len);
		if (MemError()) return; // skip out if we couldn't extend it.
	}
	
	// Copy the data over
	BlockMoveData(buffer, (*mRawData) + offset, len);
	
	// Adjust our length
	mRawLength += len;
	
	// Parse the new data
	DumpText(offset, len);
}