//
// MacHack Sample Plugin
//

#pragma once

#include "NetscapePlugin.h"
#include "MacPluginView.h"

#include <LListener.h>

//
// Resource IDs
//

// Our view
const ResIDT kSampleViewID = 2000;

// Our viewspane IDs
const PaneIDT kTextPane = 'text';
const PaneIDT kPopupPane = 'popp';
const PaneIDT kScrollerPane = 'MScr';

// This STR contains the show-mode HTML tag, "SHOW"
const ResIDT kShowTagString = 2000;
// This STR# contains the allowable options for SHOW:
// "HEX", the parameter for "show hex only"
// "ASCII", the parameter for "show ASCII only"
// "HEXASCII", the parameter for "show hex & ascii"
const ResIDT kShowOptionsStringList = 2001; 

//
// Classes we don't include definitions of
//
class LStdPopupMenu;
class LTextEdit;
class LMyActiveScroller;

//
// Various enumerated types
//

// We can be displayed in either of these modes; these correspond to the choices in the popup
enum EShowMode { 
	eHexASCIIMode, eASCIIMode
};

//
// Our factories - these are called by the Netscape wrapper to create or destroy us
//
CNetscapePlugin* CreateNetscapePlugin(NPP instance, uint16 mode, CPluginArguments* adoptedArgs, NPSavedData* saved);
NPError InitializeNetscapePlugin(void);
void ShutdownNetscapePlugin(void);


//
// The plugin object itself
//
class LSamplePlugin : public CNetscapePlugin, public LListener {
public:
	static NPError Initialize(void);
	static NPError Shutdown(void);
	
	LSamplePlugin(NPP instance, uint16 mode, CPluginArguments* adoptedArgs);
	virtual ~LSamplePlugin();

	// Methods 'cuz we're a plugin
	virtual CNetscapeStream *CreateStream(NPMIMEType type, NPStream *stream, NPBool seekable, uint16 stype);
	virtual CPluginView *CreatePluginView();
	virtual int16 Handle_Event(void* event);
	CMacPluginView* GetMacPluginView() { return (CMacPluginView*) mView; };

	// Methods 'cuz we're a listener
	virtual void ListenToMessage(MessageT inMessage, void *ioParam);

	// Methods 'cuz we're the MacHack Sample Plugin
	void DumpText(int32 offset = 0, int32 len = 0); // Process part (or all) of the text according to the current mode
	void MoreData(int32 offset, int32 len, void *buffer); // More data from the stream
	
	// "Command-line" argument state
	EShowMode mShowMode;

	// Our subviews
	LTextEdit *mText; // The textedit view
	LMyActiveScroller *mScroller; // the scroller that contains the text
	LStdPopupMenu *mPopup; // The popup that controls what's displayed

	// The raw data
	Handle mRawData;
	int32 mRawLength;
};

// We keep a global pointer to the plugin object (mostly so that callbacks, 
// like scrollbar thumbs, can find it)
extern LSamplePlugin *gPlugin;
extern GrafPort *gPluginPort;

// Other handy functions
int LookupString(char *target, const ResIDT fromList); // look up a string in this resource list
