//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//
// NetscapePlugin.cpp
//
//
//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
#ifndef NETSCAPEPLUGIN_H
#include "NetscapePlugin.h"
#endif

extern CNetscapePlugin* CreateNetscapePlugin( uint16 mode, CPluginArguments* adoptedArgs, NPSavedData* saved );
extern NPError InitializeNetscapePlugin( void );
extern void ShutdownNetscapePlugin( void );

static const int kVersionSupportsNotify = 7;
//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//
//	Procedural Methods
// 
//	This are the methods called by npmac.cpp. They cast instance->pdata as a CNetscapePlugin,
//	and call the appropriate member function of that object. These functions require three
//	additional external functions, CreateNetscapePlugin, ShutdownNetscapePlugin, and
//	InitializeNetscapePlugin, which are required so that the correct descendent of CNetscapePlugin
//	gets created, initialized, and shut down.
// 
//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::


//------------------------------------------------------------------------------------
// NPP_Initialize:
//------------------------------------------------------------------------------------
NPError NPP_Initialize(void)
{
	NPError error = InitializeNetscapePlugin();
	return error;
}


//------------------------------------------------------------------------------------
// NPP_Shutdown:
//------------------------------------------------------------------------------------
void NPP_Shutdown(void)
{
	ShutdownNetscapePlugin();
}


//------------------------------------------------------------------------------------
// NPP_New:
//------------------------------------------------------------------------------------
NPError NP_LOADDS
NPP_New(NPMIMEType pluginType,
				NPP instance,
				uint16 mode,
				int16 argc,
				char* argn[],
				char* argv[],
				NPSavedData* saved)
{
	if (instance == NULL)
		return NPERR_INVALID_INSTANCE_ERROR;
	
	CPluginArguments* theArgs = new CPluginArguments( argc, argn, argv );
	
	CNetscapePlugin* thePlugin = CreateNetscapePlugin( instance, mode, theArgs, saved );
	
	if( thePlugin == 0 )
		return NPERR_OUT_OF_MEMORY_ERROR;
	
	return NPERR_NO_ERROR;

}




//------------------------------------------------------------------------------------
// NPP_Destroy:
//------------------------------------------------------------------------------------
NPError NP_LOADDS
NPP_Destroy(NPP instance, NPSavedData** save)
{
	if (instance == NULL)
		return NPERR_INVALID_INSTANCE_ERROR;
		
	CNetscapePlugin* thePlugin = (CNetscapePlugin*) instance->pdata;
	if (thePlugin == 0)
		return NPERR_INVALID_PLUGIN_ERROR;
		
	NPError result = thePlugin->SaveData( save );
	delete thePlugin;
	
	return result;
}




//------------------------------------------------------------------------------------
// NPP_SetWindow:
//------------------------------------------------------------------------------------
NPError NP_LOADDS
NPP_SetWindow(NPP instance, NPWindow* window)
{
	if (instance == NULL)
		return NPERR_INVALID_INSTANCE_ERROR;

	CNetscapePlugin* thePlugin = (CNetscapePlugin*) instance->pdata;
	if( thePlugin == 0 )
		return NPERR_INVALID_PLUGIN_ERROR;
	
	return thePlugin->SetWindow( window );
}



//------------------------------------------------------------------------------------
// NPP_NewStream:
//------------------------------------------------------------------------------------
NPError NP_LOADDS
NPP_NewStream(NPP instance,
							NPMIMEType type,
							NPStream *stream, 
							NPBool seekable,
							uint16 *stype)
{
	if (instance == NULL)
		return NPERR_INVALID_INSTANCE_ERROR;
				
	CNetscapePlugin* controller = (CNetscapePlugin*) instance->pdata;
	
	CNetscapeStream* theStream = controller->CreateStream(type, stream, seekable, *stype );
	
	if( theStream == 0 )
		return NPERR_OUT_OF_MEMORY_ERROR;
		
	*stype = theStream->GetStreamType();
	
	return NPERR_NO_ERROR;
}


//------------------------------------------------------------------------------------
// NPP_WriteReady:
//------------------------------------------------------------------------------------
int32 NP_LOADDS
NPP_WriteReady(NPP instance, NPStream *stream)
{
	if (instance == NULL)
		return -1;

	CNetscapeStream* theStream = CNetscapeStream::EvolveStream( stream );	
	if( theStream == 0 )
		return -1;
	
	return theStream->WriteReady(); 
}



//------------------------------------------------------------------------------------
// NPP_Write:
//------------------------------------------------------------------------------------
int32 NP_LOADDS
NPP_Write(NPP instance, NPStream *stream, int32 offset, int32 len, void *buffer)
{
	if (instance == NULL)
		return -1;
	
	CNetscapeStream* theStream = CNetscapeStream::EvolveStream( stream );	
	if( theStream == 0 )
		return -1;
		
	return theStream->Write( offset, len, buffer );
}



//------------------------------------------------------------------------------------
// NPP_DestroyStream:
//------------------------------------------------------------------------------------
NPError NP_LOADDS
NPP_DestroyStream(NPP instance, NPStream *stream, NPError reason)
{
	if (instance == NULL)
		return NPERR_INVALID_INSTANCE_ERROR;
		
	CNetscapeStream* theStream = CNetscapeStream::EvolveStream( stream );
	if( theStream == 0 )
		return NPERR_GENERIC_ERROR;
	
	NPError retErr = theStream->Finish( reason );		
	delete theStream;
	
	return retErr;
}


//------------------------------------------------------------------------------------
// NPP_StreamAsFile:
//------------------------------------------------------------------------------------
void NP_LOADDS
NPP_StreamAsFile(NPP instance, NPStream *stream, const char* fname)
{
	if (instance == NULL)
		return;
		
	CNetscapeStream* theStream = CNetscapeStream::EvolveStream( stream );	
	if( theStream == 0 )
		return;

	theStream->StreamAsFile( fname );
}



//------------------------------------------------------------------------------------
// NPP_Print:
//------------------------------------------------------------------------------------
void NP_LOADDS
NPP_Print(NPP instance, NPPrint* printInfo)
{
    if(printInfo == NULL)   // trap invalid parm
        return;

	if (instance != NULL)
	{
		CNetscapePlugin* This = (CNetscapePlugin*) instance->pdata;
		This->Print( printInfo );
	}

}


//------------------------------------------------------------------------------------
// NPP_HandleEvent:
// Mac-only.
//------------------------------------------------------------------------------------
int16 NPP_HandleEvent(NPP instance, void* event)
{
	NPBool eventHandled = false;
	if (instance == NULL)
		return eventHandled;
		
	CNetscapePlugin* This = (CNetscapePlugin*) instance->pdata;
	if( This )
		eventHandled = This->Handle_Event( event );
	
	return eventHandled;
}

//------------------------------------------------------------------------------------
// NPP_URLNotify:
//------------------------------------------------------------------------------------
void NPP_URLNotify(NPP instance, const char* url, NPReason reason, void* notifyData)
{
	if( instance != NULL )
	{
		//
	}
}

//------------------------------------------------------------------------------------
// NPP_GetJavaClass:
//------------------------------------------------------------------------------------
jref NPP_GetJavaClass(void)
{
	return 0;
}

//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//
//	Object Allocation Methods
//
//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

//======================================================================
//		CNetscapePlugin::CNetscapePlugin()
//======================================================================

CNetscapePlugin::CNetscapePlugin( NPP instance, uint16 mode, CPluginArguments* adoptedArgs )
:	mInstance( instance ),
	mMode( mode ),
	mArgs( adoptedArgs ),
	mView( 0 )
{
	instance->pdata = (void*) this;
	int pluginMajor, pluginMinor, navMajor, navMinor;
	NPN_Version( &pluginMajor, &pluginMinor, &navMajor, &navMinor );
	
	mNavMinorVersion = navMinor;
}

//======================================================================
//		CNetscapePlugin::~CNetscapePlugin()
//======================================================================

CNetscapePlugin::~CNetscapePlugin()
{
	delete mArgs;
	delete mView;
}

//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//
//	Object Interface Methods
//
//	These methods are called by the procedural methods above. They take care of basic housekeeping,
//	such as keeping track of streams and windows, then notify any inherited classes through the
//	virtual Handle_*() member functions.
//
//::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::


//------------------------------------------------------------------------------------
// CNetscapePlugin::SetWindow
//------------------------------------------------------------------------------------
NPError	
CNetscapePlugin::SetWindow( NPWindow* window )
{	
	if( mView == 0 )
	{
		mView = CreatePluginView( );
	}
	
	NPError result = mView->SetWindow( window );

	return result;
}


//------------------------------------------------------------------------------------
// CNetscapePlugin::Print
//------------------------------------------------------------------------------------
void
CNetscapePlugin::Print( NPPrint* printInfo )
{
    if(printInfo == NULL)   // trap invalid parm
        return;

	if (printInfo->mode == NP_FULL)
	{
		NPBool printResult = PrintFull( printInfo->print.fullPrint );
		
		printInfo->print.fullPrint.pluginPrinted = printResult; // Do the default
		
	}
	else	// If not fullscreen, we must be embedded
	{
		PrintEmbeded( printInfo->print.embedPrint );
	}

}


//------------------------------------------------------------------------------------
// CNetscapePlugin::Handle_Event
//------------------------------------------------------------------------------------
int16 
CNetscapePlugin::Handle_Event( void* event)
{
	return false;
}


//------------------------------------------------------------------------------------
// CNetscapePlugin::CreateStream
//------------------------------------------------------------------------------------
CNetscapeStream*
CNetscapePlugin::CreateStream( NPMIMEType type, NPStream *stream, NPBool seekable, uint16 stype )
{
	return new CNetscapeStream( type, stream, seekable, stype );
}

//------------------------------------------------------------------------------------
// CNetscapePlugin::CreatePluginView
//------------------------------------------------------------------------------------
CPluginView*
CNetscapePlugin::CreatePluginView( )
{
	return new CPluginView( this );
}

//------------------------------------------------------------------------------------
// CNetscapePlugin::SetView
//------------------------------------------------------------------------------------
void
CNetscapePlugin::SetView( CPluginView* adoptedView )
{
	delete mView;
	mView = adoptedView;
}


//------------------------------------------------------------------------------------
// CNetscapePlugin::NavGetURL
//------------------------------------------------------------------------------------
NPError
CNetscapePlugin::NavGetURL(const char* url, const char* target, void* notifyData)
{
	if( notifyData && (mNavMinorVersion >= kVersionSupportsNotify ) ) {
		return NPN_GetURLNotify( mInstance, url, target, notifyData );
	}
	return NPN_GetURL( mInstance, url, target );
}

//------------------------------------------------------------------------------------
// CNetscapePlugin::SeNavPostURLtView
//------------------------------------------------------------------------------------
NPError
CNetscapePlugin::NavPostURL(const char* url, const char* target, 
								uint32 len, const char* buf,
								NPBool file, void* notifyData)
{
	if( notifyData && (mNavMinorVersion >= kVersionSupportsNotify ) ) {
		return NPN_PostURLNotify( mInstance, url, target, len, buf, file, notifyData );
	}
	return NPN_PostURL( mInstance, url, target, len, buf, file );

}

								
//------------------------------------------------------------------------------------
// CNetscapePlugin::NavNewStream
//------------------------------------------------------------------------------------
NPError
CNetscapePlugin::NavNewStream( NPMIMEType type, const char* target, NPStream** stream)
{
	return NPN_NewStream( mInstance, type, target, stream );
}

//------------------------------------------------------------------------------------
// CNetscapePlugin::NavWrite
//------------------------------------------------------------------------------------
int32
CNetscapePlugin::NavWrite(NPStream* stream, int32 len, void* buffer)
{
	return NPN_Write( mInstance, stream, len, buffer );
}

//------------------------------------------------------------------------------------
// CNetscapePlugin::NavDestroyStream
//------------------------------------------------------------------------------------
NPError
CNetscapePlugin::NavDestroyStream(NPStream* stream, NPReason reason)
{
	return NPN_DestroyStream( mInstance, stream, reason );
}

		
//------------------------------------------------------------------------------------
// CNetscapePlugin::NavStatus
//------------------------------------------------------------------------------------
void
CNetscapePlugin::NavStatus(const char* message)
{
	NPN_Status( mInstance, message );
}

//------------------------------------------------------------------------------------
// CNetscapePlugin::NavUserAgent
//------------------------------------------------------------------------------------
const char*
CNetscapePlugin::NavUserAgent()
{
	return NPN_UserAgent( mInstance );
}


//------------------------------------------------------------------------------------
// CNetscapePlugin::NavGetJavaPeer
//------------------------------------------------------------------------------------
jref
CNetscapePlugin::NavGetJavaPeer()
{
	return NPN_GetJavaPeer( mInstance );
}


