#pragma once

#ifndef _NPAPI_H_
#include "npapi.h"
#endif

#include "PluginArgs.h"
#include "NetscapeStream.h"
#include "PluginView.h"
#include "NetscapeMemObject.h"

class CPluginView;

class CNetscapePlugin : public CNetscapeMemObject {
	public:
		static NPError		Initialize( void ) { return NPERR_NO_ERROR; };
		static NPError		Shutdown( void ) { return NPERR_NO_ERROR; };
		
							CNetscapePlugin( NPP instance, uint16 mode, CPluginArguments* adoptedArgs );
		virtual				~CNetscapePlugin();

		virtual NPError		SaveData( NPSavedData** ) { return NPERR_NO_ERROR; };
		
		virtual NPError				SetWindow( NPWindow* window );
		virtual void				Print( NPPrint* printInfo );
		virtual int16				Handle_Event( void* event );
		
		CPluginView*				GetView() { return mView; };
		virtual void				SetView( CPluginView* adoptedView );
		
		virtual CNetscapeStream*	CreateStream( NPMIMEType type, NPStream *stream, NPBool seekable, uint16 stype );

		virtual CPluginView* 		CreatePluginView( ) ;

	// Netscape call-backs.
		NPError     	NavGetURL(const char* url, const char* target, void* notifyData = 0);
		NPError    		NavPostURL(const char* url, const char* target, 
								uint32 len, const char* buf,
								NPBool file, void* notifyData = 0);
								
		NPError     	NavNewStream( NPMIMEType type, const char* target, NPStream** stream);
		int32       	NavWrite(NPStream* stream, int32 len, void* buffer);
		NPError    		NavDestroyStream(NPStream* stream, NPReason reason);
		
		void        	NavStatus(const char* message);
		const char* 	NavUserAgent();

		jref		    NavGetJavaPeer();

	// These functions don't really need to be called through the plugin class, but they
	// are here for completeness.
		void*       	NavMemAlloc(uint32 size)
								{ return NPN_MemAlloc( size ); };
		void       		NavMemFree(void* ptr)
								{ NPN_MemFree( ptr ); };
		uint32      	NavMemFlush(uint32 size)
								{ return NPN_MemFlush( size ); };
		void			NavReloadPlugins(NPBool reloadPages)
								{ NPN_ReloadPlugins( reloadPages ); };

		JRIEnv*	    	NavGetJavaEnv(void)
								{ return NPN_GetJavaEnv( ); };

		int32			GetNavMinorVersion() 
								{ return mNavMinorVersion; };
		
	protected:
		virtual NPBool		PrintFull( const NPFullPrint& fullPrintInfo ) { return false; };
		virtual void		PrintEmbeded( const NPEmbedPrint& embedPrintInfo ) {};

		CPluginView*		mView;
		unsigned short		mMode;
		CPluginArguments*	mArgs;
		NPP					mInstance;

		int32				mNavMinorVersion;
};

CNetscapePlugin* CreateNetscapePlugin( NPP instance, uint16 mode, CPluginArguments* adoptedArgs, NPSavedData* saved );
NPError InitializeNetscapePlugin( void );
void ShutdownNetscapePlugin( void );

