/******************************************************************************
 CMoviePano.cp

				CMoviePano Panorama/EditText Class
	
	Copyright  1996 Symantec Corp. and Glenn L. Austin. All rights reserved.

	Originally generated by Visual Architect, 9:37 AM Wed, Jun 12, 1996
	
	Complete rewrite of CMoviePane, found on the Symantec C++ for Power Mac
	CD subscription.

 ******************************************************************************/

#include "CMoviePano.h"
#include <CApplication.h>

extern CApplication*	gApplication;

long					CMoviePano::numMovies = 0;

#include <Commands.h>
//#include "AppCommands.h"		// Uncomment to include VA-defined commands

TCL_DEFINE_CLASS_D1(CMoviePano, x_CMoviePano);

/**** C O N S T R U C T I O N / D E S T R U C T I O N   M E T H O D S ****/

/******************************************************************************
 CMoviePano()

        Default (does nothing) constructor - for TCL
 ******************************************************************************/
CMoviePano::CMoviePano() : x_CMoviePano()
{
	IXCMoviePano();
	
	TCL_END_CONSTRUCTOR
}

/******************************************************************************
 CMoviePano(...)

        Constructor for creating a CMoviePano from code
 ******************************************************************************/
CMoviePano::CMoviePano(CView *anEnclosure, CBureaucrat *aSupervisor,
		short aWidth, short aHeight, short aHEncl, short aVEncl,
		SizingOption aHSizing, SizingOption aVSizing) : x_CMoviePano()
{
	IPanorama(anEnclosure, aSupervisor,
					aWidth, aHeight, aHEncl, aVEncl,
					aHSizing, aVSizing);
	IXCMoviePano();
	
	TCL_END_CONSTRUCTOR
}

/******************************************************************************
 IXCMoviePano

        Initializes the fields of the CMoviePano, and determines whether
        QuickTime
 ******************************************************************************/
void CMoviePano::IXCMoviePano(void)
{
	theMovie = nil;
	theController = nil;
	movieChore = nil;
	
	theMovieFSS.vRefNum = 0;
	theMovieFSS.parID = 0;
	theMovieFSS.name[0] = 0;
	
	if (!numMovies)
	{
		long	result;
		
		FailOSErr(Gestalt(gestaltQuickTime, &result));
	
		SetCanBeGopher(true);			// to overcome bug in TCL
		SetWantsClicks(true);
		
		FailOSErr(EnterMovies());
	}
	
	++numMovies;
}

/******************************************************************************
 ~CMoviePano()

        Destroys the pieces of the CMoviePano, and if there are no more
        CMoviePanos, it de-initializes QuickTime
 ******************************************************************************/
CMoviePano::~CMoviePano()
{
	TCL_START_DESTRUCTOR
	
	if (movieChore)
		delete movieChore;
	
	if (theController)
		DisposeMovieController(theController);
	
	if (theMovie)
		DisposeMovie(theMovie);
	
	if (!numMovies)			// would be zero on an error during initialization
	{
		if (!--numMovies)	// no more movies are being shown, so deinitialize QuickTime
			ExitMovies();	// not really necessary, but OK to call
	}
}



/******************************************************************************
 SetMovie

        Sets the movie pane to display the specified movie
 ******************************************************************************/
void CMoviePano::SetMovie(FSSpecPtr movieFileFSSpec)
{
	short		movieResID = 0;
	Str255		movieName;
	Boolean		wasChanged;
	short		movieResFile = 0;
	LongRect	lr;
	Rect		r, movieR;
	
	theMovieFSS = *movieFileFSSpec;
	
	GetFrame(&lr);
	FrameToQDR(&lr, &r);
	
	try {
		if (OpenMovieFile(movieFileFSSpec, &movieResFile, fsRdPerm))
			throw;
		
		if (NewMovieFromFile(&theMovie, movieResFile,
				&movieResID, movieName, newMovieActive,
				&wasChanged))
			throw;
		
		CloseMovieFile(movieResFile);
		movieResFile = 0;
		
		if ((hSizing == sizELASTIC) && (vSizing == sizELASTIC))	// change the pane size
			GetMovieBox(theMovie, &movieR);		// get the size that the pane should be
		else
			movieR = r;							// scale the movie to the pane
		
		theController = NewMovieController(theMovie, &movieR, mcTopLeftMovie);
		
		if (!theController)
			throw;
		
		if (MCGetControllerBoundsRect(theController, &movieR))
			throw;
		
		if ((hSizing == sizELASTIC) && (vSizing == sizELASTIC))	// change the pane size
		{
			r.top = r.left = 0;
			r.bottom = (movieR.bottom - movieR.top) - r.bottom;
			r.right = (movieR.right - movieR.left) - r.right;
			
			x_CMoviePano::ChangeSize(&r, true);		// call overridden function
		}
		else			// change the movie size
			SetMovieBox(theMovie, &r);
		
		if (MCDoAction(theController, mcActionSetKeysEnabled, (Ptr) true))
			throw;
		
		movieChore = TCL_NEW(CMovieChore, (this));
	}
	catch(...)
	{
		if (movieResFile)
			CloseMovieFile(movieResFile);
		
		theMovie = nil;
		
		throw;
	}
}

/******************************************************************************
 Draw

        Draws the movie and controller
 ******************************************************************************/
void CMoviePano::Draw(Rect* area)
{
	if (theMovie)
		UpdateMovie(theMovie);
	
	if (theController)
		MCDraw(theController, GetMacPort());
}

/******************************************************************************
 DoClick

        Sends the click off to the movie controller for handling
 ******************************************************************************/
void CMoviePano::DoClick(Point hitPt, short modifiers, long when)
{
	if (theController)
	{
		Prepare();
		
		MCClick(theController, GetMacPort(), hitPt, when, modifiers);
	}
}

void CMoviePano::ChangeSize(Rect *delta, Boolean redraw)
{
	// verify that the size of the pane should change or not...
	if ((hSizing == sizELASTIC) && (vSizing == sizELASTIC))
	{
		// the pane has been sized to match the movie -- don't change the size!
	}
	else
	{
		// resize the pane
		x_CMoviePano::ChangeSize(delta, redraw);
		
		LongRect	lr;
		GetFrame(&lr);
		
		Rect		r;
		FrameToQDR(&lr, &r);
		SetMovieBox(theMovie, &r);		// resize the movie to fit the pane
	}
}

                                /** Object I/O **/
/******************************************************************************
 PutTo

        Put the contents of this object to the stream
 ******************************************************************************/
void CMoviePano::PutTo(CStream &aStream)
{
#ifdef TCL_OBJECT_IO
	aStream.PutLong('cmov');			// put a flag on the stream for GetFrom
	aStream << theMovieFSS.vRefNum << theMovieFSS.parID << theMovieFSS.name;

	x_CMoviePano::PutTo(aStream);	// Let base class save
#endif
}

/******************************************************************************
 GetFrom

        Get the contents of this object from the stream and
        initialize the object
 ******************************************************************************/
void CMoviePano::GetFrom(CStream &aStream)
{
#ifdef TCL_OBJECT_IO
	long	flag = aStream.GetLong();
	
	if (flag == 'cmov')
	{
		FSSpec		aMovie;
		
		aStream >> aMovie.vRefNum >> aMovie.parID >> aMovie.name;
		
		SetMovie(&aMovie);
	}
	else	// need to back up the stream, since I just read someone else's data!
		aStream.MoveTo(aStream.Position() - sizeof(long));
	
	x_CMoviePano::GetFrom(aStream);
#endif	
}

/******************************************************************************
 CMovieChore()

        Initialize the movie chore and put it into the list of idle tasks
 ******************************************************************************/
CMovieChore::CMovieChore(CMoviePano* aMoviePano)
{
	theMoviePano = aMoviePano;
	idling = true;
	
	gApplication->AssignIdleChore(this);
}

/******************************************************************************
 ~CMovieChore()

        Remove the chore from the list of idle tasks
 ******************************************************************************/
CMovieChore::~CMovieChore()
{
	if (idling)
		gApplication->CancelIdleChore(this);
}

/******************************************************************************
 Perform

        "Idle" the movie controller, which does all the necessary stuff
        for updating the movie, playing it, etc.
 ******************************************************************************/
void CMovieChore::Perform(long* maxSleep)
{
	*maxSleep = 1;
	
	MovieController	theController = theMoviePano->GetController();
	
	if (theController)
		MCIdle(theController);
}