/******************************************************************************
 CDnDEditText.h

		Interface for the Drag and Drop-enabled EditText Class

		Copyright  1996 Glenn L. Austin and Symantec Corporation.
		All rights reserved.

 ******************************************************************************/

#pragma once

#include "CDnDEditText.h"

#include <Commands.h>
//#include "AppCommands.h"		// Uncomment to include VA-defined commands

static Pattern	sMyGrey = {
		0xAA, 0x55, 0xAA, 0x55,
		0xAA, 0x55, 0xAA, 0x55
	};

TCL_DEFINE_CLASS_D1(CDnDEditText, x_CDnDEditText);

/**** C O N S T R U C T I O N / D E S T R U C T I O N   M E T H O D S ****/

CDnDEditText::CDnDEditText(void) : CDragNDrop(this)
{
	delSource = false;
}

void CDnDEditText::DispatchClick(EventRecord *macEvent)
{
	CDragNDrop::DispatchClick(macEvent);
}

void CDnDEditText::AdjustCursor(Point where, RgnHandle mouseRgn)
{
	static Boolean	inAdjustCursor = false;
	
	if (!inAdjustCursor)
	{
		inAdjustCursor = true;
		CDragNDrop::AdjustCursor(where, mouseRgn);
		inAdjustCursor = false;
	}
	else
		x_CDnDEditText::AdjustCursor(where, mouseRgn);
}

									// Object I/O
void CDnDEditText::PutTo(CStream& stream)		// replaces CEditText::PutTo
{
#ifdef TCL_OBJECT_IO
	Handle			h;				  // Text contents
	SignedByte		state;
	short			txFont, txFace, txMode, txSize, textLength;
	Str255			fontName;

	stream << spacingCmd << alignCmd << (short) 1;	// send a 1 -- this is a CDnDEditText class
	txFace = (**macTE).txFace;
	txMode = (**macTE).txMode;
	txSize = (**macTE).txSize;
	h = GetTextHandle();
	textLength = GetHandleSize(h);
	stream << txFace << txMode << txSize << textLength;

	txFont = (**macTE).txFont;			// Save font name
	GetFontName(txFont, fontName);
	stream.PutStr255(fontName);

	state = HGetState(h);
	if ((state & 0x80) == 0) {			// If not already locked
		MoveHHi(h);
		HLock(h);
	}
	try_
	{
		stream.Put(*h, textLength);
	}
	catch_all_()
	{
		HSetState(h, state);
		throw_same_();
	}
	end_try_
	HSetState(h, state);

	CAbstractText::PutTo(stream);		// Let base class save
	
	CDragNDrop::PutTo(stream);
#endif
}

void CDnDEditText::GetFrom(CStream& stream)
{
#ifdef TCL_OBJECT_IO
	Handle			volatile h = NULL;	// Text if len > 255
	Str255			text;				// Text if len < 256
	short			txFont, txFace, txMode, txSize, textLength;
	short			reserved;
	GrafPtr			port;

	SetDragAndDrop(false);				// default to Drag and Drop off
	
	stream >> spacingCmd >> alignCmd >> reserved;
	stream >> txFace >> txMode >> txSize >> textLength;

	stream.GetStr255(text);		   // Get font id
	GetFNum (text, &txFont);

										// Go out of our way to avoid
										//   creating a useless handle,
										//   since most strings are small
										/*   and MM is *slow*.			  */
	try_
	{
		if (textLength < 256) {
			stream.Get(&text[1], textLength);
			text[0] = textLength;
		} else {
			h = NewHandleCanFail(textLength);
			FailNIL(h);
			HLock(h);
			stream.Get(*h, textLength);
			HUnlock(h);
		}

		CAbstractText::GetFrom(stream);	// Let base class restore and init

		GetPort(&port);
		SetPort(macPort);
		TextFont(txFont);
		TextFace(txFace);
		TextSize(txSize);
		TextMode(txMode);
		SetPort(port);

	   	IEditTextX();					// Set up TE record

		if (textLength < 256) {
			SetTextString(text);
		} else {
			SetTextHandle(h);			// Set text
		}
		SetAlignCmd(alignCmd);			// Set justification
		SetSpacingCmd(spacingCmd);		// Set line spacing
	}
	catch_all_()
	{
		TCLForgetHandle(h);
		throw_same_();
	}
	end_try_
	TCLForgetHandle(h);

	if (reserved == 1)					// saved as CDnDEditText!
		CDragNDrop::GetFrom(this, stream);
	else
		SetDragAndDrop(kDragAndDropEnableDefault);
#endif
}

void CDnDEditText::SelectionChanged(void)
{
	CEditText::SelectionChanged();
	
	TEGetHiliteRgn(GetSelectedRgn(), macTE);
}

OSErr CDnDEditText::SetupDragRegion(void)
{
	return noErr;				// drag region set up by SelectionChanged
}

Boolean CDnDEditText::IsMyTypeAvailable(DragReference theDrag)
{
	unsigned short	items, i;
	FlavorFlags		theFlags;
	ItemReference	theItem;
	OSErr			err;
	
	CountDragItems(theDrag, &items);
	
	for (i = 1; i <= items; ++i)
	{
		GetDragItemReferenceNumber(theDrag, i, &theItem);
		
		err = GetFlavorFlags(theDrag, theItem, 'TEXT', &theFlags);
		if (err == noErr)
			continue;
			
		return false;
	}
	
	return true;
}

OSErr CDnDEditText::AddFlavors(CWindow* theWindow, DragReference theDrag)
{
	FailOSErr(CDragNDrop::AddFlavors(theWindow, theDrag));
	
	long			selStart, selEnd;
	
	GetSelection(&selStart, &selEnd);
	
	Handle	theText = CopyTextRange(selStart, selEnd);
	
	HLock(theText);
		
	OSErr err = AddDragItemFlavor(theDrag, GetItemReference(0), 'TEXT', *theText,
			GetHandleSize(theText), 0);
	
	DisposeHandle(theText);
	
	FailOSErr(err);
	
	return noErr;
}

ItemReference CDnDEditText::GetItemReference(long itemNbr)
{
	return (ItemReference) this;
}

OSErr CDnDEditText::ReceiveData(CWindow* theWindow, DragReference theDrag)
{
	unsigned short	items, i;
	FlavorFlags		theFlags;
	ItemReference	theItem;
	OSErr			err;
	Point			mouse;
	
	CountDragItems(theDrag, &items);
	
	for (i = 1; i <= items; ++i)
	{
		GetDragItemReferenceNumber(theDrag, i, &theItem);
		
		err = GetFlavorFlags(theDrag, theItem, 'TEXT', &theFlags);
		if (err == noErr)
		{
			Size	dataSize;
			
			GetFlavorDataSize(theDrag, theItem, 'TEXT', &dataSize);
			
			Ptr		theData = NewPtr(dataSize);
			GetFlavorData(theDrag, theItem, 'TEXT', theData, &dataSize, 0L);
			
			GetDragMouse(theDrag, nil, &mouse);
			
			CView*	oldView = cPreparedView;
			Prepare();
			
			GlobalToLocal(&mouse);
			
			oldOffset = TEGetOffset(mouse, macTE);
			oldStart = (**macTE).selStart;
			oldEnd = (**macTE).selEnd;
			
			if (delSource)
			{
				if ((oldStart < oldOffset) && (oldOffset < oldEnd))	// inside move, don't do anything!
				{
					DisposePtr(theData);
					
					return noErr;
				}
				
				TEDelete(macTE);
				
				if (oldOffset > oldEnd)
					oldOffset -= dataSize;
				
				oldStart = oldOffset;			// new selection is the moved data
				oldEnd = oldStart;
				
				delSource = false;
			}
			else
			{
				oldStart = oldEnd = oldOffset;
			}
			
			TESetSelect(oldOffset, oldOffset, macTE);
			
			InsertTextPtr(theData, dataSize, true);
			
			if (oldStart > oldOffset)
				oldStart += dataSize;
			
			if (oldEnd >= oldOffset)
				oldEnd += dataSize;
							
			SetSelection(oldStart, oldEnd, true);
			SelectionChanged();
			
			DisposePtr(theData);
		}
			
	}
	
	return noErr;
}

OSErr CDnDEditText::DragEnter(CWindow* theWindow, DragReference theDrag)
{
	oldStart = (**macTE).selStart;		// save off the selection
	oldEnd = (**macTE).selEnd;
	
	TESetSelect(0, 0, macTE);			// select before the first char
	oldOffset = 0;
	EraseCaretLocation(oldOffset, macTE);
	
	CDragNDrop::DragEnter(theWindow, theDrag);
	
	return noErr;
}

OSErr CDnDEditText::DragInside(CWindow* theWindow, DragReference theDrag)
{
	Point	mouse;
	
	GetDragMouse(theDrag, nil, &mouse);	// get the current mouse location
	
	CView*	oldView = cPreparedView;
	
	Prepare();
	
	GlobalToLocal(&mouse);				// convert the location to local
	
	short	offset = TEGetOffset(mouse, macTE);	// find the offset
	
	if (offset != oldOffset)
	{
		EraseCaretLocation(oldOffset, macTE);	// erase the old caret
		DrawCaretLocation(offset, macTE);		// draw the new caret
	}
	
	oldOffset = offset;
	
	oldView->Prepare();					// restore the prepared view
	
	CDragNDrop::DragInside(theWindow, theDrag);
	
	return noErr;
}

OSErr CDnDEditText::DragLeave(CWindow* theWindow, DragReference theDrag)
{
	CDragNDrop::DragLeave(theWindow, theDrag);
	
	EraseCaretLocation(oldOffset, macTE);	// erase the D&D caret
	TESetSelect(oldStart, oldEnd, macTE);	// reselect the old selection
	EraseCaretLocation(0, macTE);
	
	return noErr;
}

OSErr CDnDEditText::DeleteSource(Boolean deleteNow, CWindow* theWindow, DragReference theDrag)
{
	if (deleteNow)
	{
		TEDelete(macTE);
		SetSelection(oldStart, oldStart, true);
		SelectionChanged();
	}
	else
		delSource = true;
	
	return noErr;
}

short CDnDEditText::TEGetLine(short offset, TEHandle theTE)

{	short		line = 0;

	if (offset > (**theTE).teLength)
		return((**theTE).nLines);

	while ((**theTE).lineStarts[line] < offset)
		line++;
	
	return(line);
}

void CDnDEditText::DrawCaretLocation(short offset, TEHandle theTE)
{
	Point	theLoc;
	short	theLine, lineHeight;
	
	// Get the coordinates and the line of the offset to draw the caret
	theLoc = TEGetPoint(offset, theTE);
	theLine = TEGetLine(offset, theTE);
	
	// For some reason, TextEdit doesn't return the proper coordinates of
	// the last offset in the field if the last character in the record is
	// a carriage return.  TEGetPoint returns a point that is one line higher
	// than expected.  The following code fixes this problem.
	
	if ((offset == (**theTE).teLength) &&
			(*((**theTE).hText))[(**theTE).teLength - 1] == 0x0D)
		theLoc.v += TEGetHeight(theLine, theLine, theTE);
	
	// Invert the caret
	PenNormal();
	PenMode(patXor);
	PenPat(&sMyGrey);
	
	// Get the height of the line that the offset points to
	lineHeight = TEGetHeight(theLine, theLine, theTE);
	
	// Draw the approprate caret image
	MoveTo(theLoc.h - 1, theLoc.v - 1);
	Line(0, 1 - lineHeight);
	
	PenNormal();
}

void CDnDEditText::EraseCaretLocation(short offset, TEHandle theTE)
{
	Point	theLoc;
	short	theLine, lineHeight;
	
	// Get the coordinates and the line of the offset to draw the caret
	theLoc = TEGetPoint(offset, theTE);
	theLine = TEGetLine(offset, theTE);
	
	// For some reason, TextEdit doesn't return the proper coordinates of
	// the last offset in the field if the last character in the record is
	// a carriage return.  TEGetPoint returns a point that is one line higher
	// than expected.  The following code fixes this problem.
	
	if ((offset == (**theTE).teLength) &&
			(*((**theTE).hText))[(**theTE).teLength - 1] == 0x0D)
		theLoc.v += TEGetHeight(theLine, theLine, theTE);
	
	// Invert the caret
	PenNormal();
	PenMode(patBic);
	
	// Get the height of the line that the offset points to
	lineHeight = TEGetHeight(theLine, theLine, theTE);
	
	// Draw the approprate caret image
	
	MoveTo(theLoc.h - 1, theLoc.v - 1);
	Line(0, 1 - lineHeight);
	
	PenNormal();
}