/******************************************************************************
 CDragNDrop.cp

							The Drag and Drop Class

	Implements the Macintosh Drag and Drop stuff for a class which uses this
	class as one of its superclasses.

	Hooks are provided for derived classes to provide information to the
	Drag Manager.

	BASE CLASS = none

	Copyright  1996 Glenn L. Austin and Symantec Corporation. All rights reserved.

 ******************************************************************************/

#include "CDragNDrop.h"
#include <CBureaucrat.h>
#include <CView.h>
#include <CPane.h>
#include <Gestalt.h>
#include <CWindow.h>
#include <OSUtils.h>
#include <CChore.h>
#include <CApplication.h>
#include <Folders.h>

TCL_DEFINE_CLASS_M0(CDragNDrop)

#if (1)
pascal OSErr DragTrackingHandlerP(DragTrackingMessage, WindowPtr,
		CDragNDrop*, DragReference);
pascal OSErr DragReceiveHandlerP(WindowPtr, CDragNDrop*,
		DragReference);
pascal OSErr	DragSendDataP(FlavorType theType, CDragNDrop*,
		ItemReference, DragReference);
pascal OSErr	DragInputP(Point*, short*, CDragNDrop*,
		DragReference);
pascal OSErr	DragDrawingP(DragRegionMessage,
		RgnHandle showRegion, Point showOrigin,
		RgnHandle hideRegion, Point hideOrigin,
		CDragNDrop*, DragReference);
#endif

extern CBureaucrat	*gGopher;			// First in line to get commands

Boolean 				CDragNDrop::dragMgrInitialized = false;
DragTrackingHandlerUPP	CDragNDrop::myDragTrackUPP =
							NewDragTrackingHandlerProc(DragTrackingHandlerP);
DragReceiveHandlerUPP	CDragNDrop::myReceiveUPP =
							NewDragReceiveHandlerProc(DragReceiveHandlerP);
DragSendDataUPP			CDragNDrop::mySendUPP =
							NewDragSendDataProc(DragSendDataP);
DragInputUPP			CDragNDrop::myInputUPP =
							NewDragInputProc(DragInputP);
DragDrawingUPP			CDragNDrop::myDrawingUPP =
							NewDragDrawingProc(DragDrawingP);

#if !GENERATINGCFM
long		appA5;
#endif
extern CApplication*	gApplication;

class CDnDInitChore : public CChore
{
public:
	CDnDInitChore(CDragNDrop* aDnDItem)
			{
				dndToInit = aDnDItem;
			}
	
	virtual void	Perform(long *maxSleep)
			{
				if (dndToInit)
					dndToInit->InitHandlers();
			}
	
private:
	CDragNDrop*		dndToInit;
};

CDragNDrop::CDragNDrop(CPane* aPane, Boolean aEnabled)
{
	myPane = aPane;
	theSelectedRgn = nil;
	handlersInstalled = false;
	canAcceptDrag = inContent = false;
	isEnabled = false;
	
	if (!dragMgrInitialized)
	{
		long	result;
		
		if (Gestalt(gestaltDragMgrAttr, &result) ||
				!(result & (1L << gestaltDragMgrPresent)))
			isEnabled = false;	// Drag Manager is not installed!  Don't install!
		else
		{
#if !GENERATINGCFM
			appA5 = (long) LMGetCurrentA5();		// for later...
#endif
		}
		
		dragMgrInitialized = true;
	}
	
	if (myPane)
		SetDragAndDrop(aEnabled);
	
	FailNIL(theSelectedRgn = NewRgn());
	
	//TCL_END_CONSTRUCTOR
}

CDragNDrop::~CDragNDrop(void)
{
	GrafPtr		myPort;
	
	//TCL_START_DESTRUCTOR

	if (theSelectedRgn)
		DisposeRgn(theSelectedRgn);
	
	if (handlersInstalled)
	{
		myPort = myPane->GetMacPort();

		RemoveReceiveHandler(myReceiveUPP, myPort);
		RemoveTrackingHandler(myDragTrackUPP, myPort);
	}
}

void CDragNDrop::InitHandlers(void)
{
	if (isEnabled)
	{
		GrafPtr		myPort = myPane->GetMacPort();
		
		InstallTrackingHandler(myDragTrackUPP, myPort, this);
		InstallReceiveHandler(myReceiveUPP, myPort, this);
		
		handlersInstalled = true;
	}
}

Boolean CDragNDrop::DragAndDropSet(void)
{
	return isEnabled;
}

void CDragNDrop::SetDragAndDrop(Boolean aEnabled)
{
	if (isEnabled == aEnabled)
		return;					// don't need to do anything
	
	if (myPane)
		isEnabled = aEnabled;
	
	GrafPtr		myPort;
		
	if (isEnabled && myPane)
	{
		myPort = myPane->GetMacPort();
		
		if (myPort)
			InitHandlers();
		else
		{
			CDnDInitChore*	initChore = TCL_NEW(CDnDInitChore,(this));
			gApplication->AssignUrgentChore(initChore);
		}
	}
	else if (!isEnabled && handlersInstalled && myPane)
	{
		myPort = myPane->GetMacPort();

		RemoveReceiveHandler(myReceiveUPP, myPort);
		RemoveTrackingHandler(myDragTrackUPP, myPort);
	}
}

void CDragNDrop::PutTo(CStream& stream)
{
#ifdef TCL_OBJECT_IO
	stream << isEnabled;
#endif
}

void CDragNDrop::GetFrom(CPane* aPane, CStream& stream)
{
#ifdef TCL_OBJECT_IO
	if (myPane && (myPane != aPane))
		SetDragAndDrop(false);			// disable drag and drop for the window
	
	myPane = aPane;
	
	Boolean		enableHandlers;
	
	stream >> enableHandlers;
	
	if (myPane)
		SetDragAndDrop(enableHandlers);
	else
		SetDragAndDrop(kDragAndDropEnableDefault);
#endif
}

void CDragNDrop::DispatchClick(
	register EventRecord	*macEvent)	// Mouse down event record
{
	CView			*hitView;			// View in which mouse was clicked
	Point			hitPt;				// In Frame coords of the view hit
	long			hOff;
	long			vOff;
	extern void	CountClicks(CView *hitView, EventRecord *macEvent);
		
	hitPt = macEvent->where;
		
	hitView = myPane->FindSubview(hitPt);
	if (hitView == NULL)
	{
		if (!myPane->usingLongCoord)
			myPane->GetOrigin(&hOff, &vOff);
		else
		{
			hOff = (myPane->GetMacPort())->portRect.left;
			vOff = (myPane->GetMacPort())->portRect.top;
		}
		hitPt.h += hOff;
		hitPt.v += vOff;
		
		if (isEnabled && !EmptyRgn(theSelectedRgn))
		{
			if (PtInRgn(hitPt, theSelectedRgn) && CanDrag() && WaitMouseMoved(macEvent->where))
			{
				StartDrag(macEvent, TCL_MacToTCLWindow(myPane->GetMacPort()));
				return;
			}
		}
		
		CountClicks(myPane, macEvent);

		if (myPane->canBeGopher && (gGopher != myPane))
		{
				// if old gopher refuses to resign, then
				// we can't process this click

			if (!myPane->BecomeGopher(TRUE))
				return;
		}
		myPane->Prepare();
		
		myPane->DoClick(hitPt, macEvent->modifiers, macEvent->when);
		if (isEnabled)
			SetupDragRegion();
	} else {
		hitView->DispatchClick(macEvent);
	}
}

void CDragNDrop::AdjustCursor(Point where, RgnHandle mouseRgn)
{
	// where is in window coordinates, but mouseRgn is in global coordinates
	
	if (isEnabled && !EmptyRgn(theSelectedRgn))		// theSelectedRgn should always be within the mouseRgn
	{
		LongPt	locWhere;
		
		myPane->WindToFrame(where, &locWhere);
		myPane->FrameToQD(&locWhere, &where);
	
		Rect		r = (**theSelectedRgn).rgnBBox;
		LongRect	lr;
		
		myPane->QDToFrameR(&r, &lr);
		myPane->FrameToGlobalR(&lr, &r);
		
		Boolean	inRgn = PtInRgn(where, theSelectedRgn);
		
		OffsetRgn(theSelectedRgn, r.left - lr.left, r.top - lr.top);
		
		if (inRgn)
		{
			SectRgn(mouseRgn, theSelectedRgn, mouseRgn);
			SetCursor(&qd.arrow);
		}
		else
		{
			// add theSelectedRgn to the mouseRgn
			UnionRgn(mouseRgn, theSelectedRgn, mouseRgn);
			// remove entire theSelectedRgn from theMouseRgn
			XorRgn(mouseRgn, theSelectedRgn, mouseRgn);
			myPane->AdjustCursor(where, mouseRgn);
		}
		
		OffsetRgn(theSelectedRgn, lr.left - r.left, lr.top - r.top);
	}
	else
		AdjustCursor(where, mouseRgn);
}

// Calls used for Drag and Drop
OSErr CDragNDrop::StartDrag(EventRecord* macEvent, CWindow* theWindow)
{
	OSErr			err;
	DragReference	theDrag;
	RgnHandle		dragRgn, tempRgn;
	CView*			oldView = myPane->cPreparedView;
	Point			globalPoint;
	
	dragRgn = NewRgn();
	CopyRgn(theSelectedRgn, dragRgn);
	
	LongRect	paneOrigin = {0, 0, 0, 0};
	Rect		globalR;
	myPane->FrameToGlobalR(&paneOrigin, &globalR);
	OffsetRgn(dragRgn, globalR.left, globalR.top);		// move to global coordinates
		
	theWindow->Prepare();
	LocalToGlobal(&macEvent->where);				// Convert mouse location back in event

	FailOSErr(NewDrag(&theDrag));
	
	err = AddFlavors(theWindow, theDrag);
	if (err)
	{
		DisposeDrag(theDrag);
		oldView->Prepare();
		FailOSErr(err);
	}
	
	err = SetDragSendProc(theDrag, mySendUPP, this);
	if (err)
	{
		DisposeRgn(dragRgn);
		DisposeDrag(theDrag);
		oldView->Prepare();
		FailOSErr(err);
	}
	
	Rect		r = (**dragRgn).rgnBBox;
	
	SetDragItemBounds(theDrag, (ItemReference) myPane, &r);
	
	tempRgn = NewRgn();
	CopyRgn(dragRgn, tempRgn);
	InsetRgn(tempRgn, 1, 1);
	DiffRgn(dragRgn, tempRgn, dragRgn);
	DisposeRgn(tempRgn);
		
	err = TrackDrag(theDrag, macEvent, dragRgn);
	
	if (err == userCanceledErr)			// Drop cancelled
		err = noErr;
	else
	{
		DragAttributes	dragAttr;
		GetDragAttributes(theDrag, &dragAttr);
		
		short	mouseDownMod, mouseUpMod;
		GetDragModifiers(theDrag, nil, &mouseDownMod, &mouseUpMod);
		
		Boolean	copyItem = (mouseDownMod & optionKey) || (mouseUpMod & optionKey);	// copy the selection?
		
		if (!(dragAttr & dragInsideSenderApplication))		// check for drag to trash...
		{
			AEDesc		dropLoc;
			
			err = GetDropLocation(theDrag, &dropLoc);
			if (!err)
			{
				if (!copyItem && DropLocationIsFinderTrash(&dropLoc))
				{
					CView*		oldView = myPane->cPreparedView;
					
					myPane->Prepare();
					
					err = DeleteSource(true, theWindow, theDrag);
					
					oldView->Prepare();
				}
			}
		}
	}

	DisposeRgn(dragRgn);
	DisposeDrag(theDrag);
	
	oldView->Prepare();
	FailOSErr(err);
	
	return err;
}

OSErr CDragNDrop::AddFlavors(CWindow* theWindow, DragReference theDrag)
{
	return AddDragItemFlavor(theDrag, GetItemReference(0), 'DATA', this,
			sizeof(*this), flavorSenderOnly);
}

OSErr CDragNDrop::SetupDragRegion(void)
{
	LongRect	lr;
	
	myPane->GetFrame(&lr);
	
	Rect		r;
	myPane->FrameToQDR(&lr, &r);
	
	RectRgn(theSelectedRgn, &r);
	
	return noErr;
}

Boolean CDragNDrop::IsMyTypeAvailable(DragReference theDrag)
{
	unsigned short	items, i;
	FlavorFlags		theFlags;
	ItemReference	theItem;
	OSErr			err;
	
	CountDragItems(theDrag, &items);
	
	for (i = 1; i <= items; ++i)
	{
		GetDragItemReferenceNumber(theDrag, i, &theItem);
		
		err = GetFlavorFlags(theDrag, theItem, 'DATA', &theFlags);
		if ((err == noErr) && (theFlags & flavorSenderOnly))
			continue;
			
		return false;
	}
	
	return true;
}

Boolean CDragNDrop::CanDrag(void)
{
	return true;
}

OSErr CDragNDrop::DeleteSource(Boolean deleteNow, CWindow* theWindow, DragReference theDrag)
{
	return noErr;
}

// Calls made from DragTrackingHandler
OSErr CDragNDrop::DragEnter(CWindow* theWindow, DragReference theDrag)
{
	LongRect	lr;
	Rect		r;
	CPanorama*	myPano = TCL_DYNAMIC_CAST(CPanorama, myPane);
	CView*		oldView = theWindow->cPreparedView;
	Boolean	drawInside = true;
	
	if (myPano)
	{
		myPano->itsScrollPane->GetInterior(&lr);
		myPano->itsScrollPane->FrameToWindR(&lr, &r);
	}
	else
	{
		myPane->GetFrame(&lr);
		myPane->FrameToWindR(&lr, &r);
	}
	
	if (((CPane*) myPano->itsEnclosure)->itsBorder)
	{
		CPane*	myEncl = (CPane*) myPano->itsEnclosure;
		
		Rect	borderMargin;
		
		myEncl->itsBorder->GetMargin(&borderMargin);
		
		myEncl->itsBorder->CalcBorderRect(&r);
		
		// Is there room enough for the drag hilite inside the border?
		if ((borderMargin.top < 2) || (borderMargin.left < 2) ||
				(borderMargin.bottom < 2) || (borderMargin.right < 2))
		{
			InsetRect(&r, -1, -1);			// account for the pen...
			drawInside = false;
		}
	}
	
	theWindow->Prepare();
	
	ClipRect(&theWindow->GetMacPort()->portRect);
	
	RgnHandle	hiliteRgn = NewRgn();
	
	RectRgn(hiliteRgn, &r);
	
	refreshBorder = false;
	
	if (!drawInside)
	{
		RgnHandle	tRgn = NewRgn();
		
		CopyRgn(hiliteRgn, tRgn);
		InsetRgn(tRgn, -2, -2);
		DiffRgn(tRgn, hiliteRgn, tRgn);
		EraseRgn(tRgn);
		InvalRgn(tRgn);
		DisposeRgn(tRgn);
		
		refreshBorder = true;
	}
	
	ShowDragHilite(theDrag, hiliteRgn, drawInside);
	
	DisposeRgn(hiliteRgn);
	
	oldView->Prepare();
	
	return noErr;
}

OSErr CDragNDrop::DragInside(CWindow* theWindow, DragReference theDrag)
{
	return noErr;
}

OSErr CDragNDrop::DragLeave(CWindow* theWindow, DragReference theDrag)
{
	CView*		oldView = theWindow->cPreparedView;
	CPanorama*	myPano = TCL_DYNAMIC_CAST(CPanorama, myPane);
	
	theWindow->Prepare();
	
	ClipRect(&theWindow->GetMacPort()->portRect);
	
	HideDragHilite(theDrag);
	
	if (refreshBorder)
		theWindow->Update();
	
	oldView->Prepare();
	
	return noErr;
}

// Call made from DragReceiveHandler
OSErr CDragNDrop::ReceiveData(CWindow* theWindow, DragReference theDrag)
{
	return noErr;
}

// Call made from DragSendData
OSErr CDragNDrop::SendData(FlavorType, ItemReference, DragReference theDrag)
{
	return noErr;
}

// Call made from DragInput (A5 world set up for routines, but no low memory)
OSErr CDragNDrop::ModifyInput(Point*, short*, DragReference theDrag)
{
	return noErr;
}

// Calls made from DragDrawing (A5 world set up for routines, but no low memory)
OSErr CDragNDrop::RegionBegin(RgnHandle, Point, DragReference theDrag)
{
	return noErr;
}

OSErr CDragNDrop::RegionDraw(RgnHandle showRgn, RgnHandle hideRgn, DragReference theDrag)
{
	return noErr;
}

OSErr CDragNDrop::RegionHide(RgnHandle, DragReference theDrag)
{
	return noErr;
}

OSErr CDragNDrop::RegionIdle(RgnHandle, DragReference theDrag)
{
	return noErr;
}

OSErr CDragNDrop::RegionEnd(DragReference theDrag)
{
	return noErr;
}

/* Protected routines which usually won't be overridden */

OSErr CDragNDrop::DragTrackingHandler(DragTrackingMessage message,
		CWindow* theWindow, DragReference theDragRef)
{
	OSErr			retVal;
	DragAttributes	attributes;
	Point			mouse, localMouse;
	CView*			oldPrepView = myPane->cPreparedView;
	
	retVal = noErr;
	
	GetDragAttributes(theDragRef, &attributes);
	
	switch (message)
	{
		case dragTrackingEnterHandler:			
			break;
		case dragTrackingEnterWindow:
			canAcceptDrag = IsMyTypeAvailable(theDragRef);
			inContent = false;
			
			GetDragMouse(theDragRef, nil, &mouse);
			localMouse = mouse;
			theWindow->Prepare();
			GlobalToLocal(&localMouse);
			
			myPane->Prepare();
			
			if (myPane->Contains(localMouse))
			{
				inContent = true;
				retVal = DragEnter(theWindow, theDragRef);
			}
			
			oldPrepView->Prepare();
			
			break;
		case dragTrackingInWindow:
			if (!canAcceptDrag)
				break;
			
			theWindow->Prepare();
			
			GetDragMouse(theDragRef, nil, &mouse);
			localMouse = mouse;
			GlobalToLocal(&localMouse);
			
			myPane->Prepare();
			
			if (myPane->Contains(localMouse))
			{
				if (!inContent)
				{
					inContent = true;
					retVal = DragEnter(theWindow, theDragRef);
				}
				else
					retVal = DragInside(theWindow, theDragRef);
			}
			else
			{
				if (inContent)
				{
					inContent = false;
					retVal = DragLeave(theWindow, theDragRef);
				}
			}
			
			oldPrepView->Prepare();
			
			break;
		case dragTrackingLeaveWindow:
		case dragTrackingLeaveHandler:
			if (canAcceptDrag && inContent)
			{
				myPane->Prepare();
				
				inContent = false;
				retVal = DragLeave(theWindow, theDragRef);
				
				oldPrepView->Prepare();
			}
			break;
	}
	
	return retVal;
}

OSErr CDragNDrop::DragReceiveHandler(CWindow* theWindow,
		DragReference theDragRef)
{
	CView*	oldView = myPane->cPreparedView;
	OSErr	retVal = noErr;
	Boolean	wasInContent = inContent;
	
	myPane->Prepare();
	
	if (canAcceptDrag && inContent)
	{
		inContent = false;
		retVal = DragLeave(theWindow, theDragRef);
		if (retVal)
		{
			oldView->Prepare();
			
			return retVal;
		}
	}
	
	DragAttributes	dragAttr;
	GetDragAttributes(theDragRef, &dragAttr);
	
	short	mouseDownMod, mouseUpMod;
	GetDragModifiers(theDragRef, nil, &mouseDownMod, &mouseUpMod);
	
	Boolean	copyItem = (mouseDownMod & optionKey) || (mouseUpMod & optionKey);	// copy the selection?
	
	if (dragAttr & dragHasLeftSenderWindow)		// check for drag to trash...
	{
		AEDesc		dropLoc;
		
		if (!(retVal = GetDropLocation(theDragRef, &dropLoc)))
		{
			if (!copyItem && DropLocationIsFinderTrash(&dropLoc))
			{
				retVal = DeleteSource(true, theWindow, theDragRef);
				
				oldView->Prepare();
				
				return retVal;
			}
		}
	}
	else if (!copyItem && wasInContent)
	{
		retVal = DeleteSource(false, theWindow, theDragRef);
		if (retVal)
		{
			oldView->Prepare();
			
			return retVal;
		}
	}
	
	if (wasInContent)
		retVal = ReceiveData(theWindow, theDragRef);
	else
		retVal = dragNotAcceptedErr;
	
	oldView->Prepare();
	
	return retVal;
}

OSErr CDragNDrop::DragSendData(FlavorType theType,
		ItemReference theItemRef, DragReference theDragRef)
{
	return SendData(theType, theItemRef, theDragRef);
}

OSErr CDragNDrop::DragInput(Point* mouse, short* modifiers,
		DragReference theDragRef)
{
#if !GENERATINGCFM
	long	oldA5 = SetA5(appA5);
#endif
	
	OSErr	retVal = ModifyInput(mouse, modifiers, theDragRef);

#if !GENERATINGCFM
	SetA5(oldA5);
#endif

	return retVal;
}

OSErr CDragNDrop::DragDrawing(DragRegionMessage message,
		RgnHandle showRegion, Point showOrigin,
		RgnHandle hideRegion, Point hideOrigin,
		DragReference theDragRef)
{
	OSErr	retVal;
	
#if !GENERATINGCFM
	long	oldA5 = SetA5(appA5);
#endif
	
	switch (message)
	{
		case dragRegionBegin:
			retVal = RegionBegin(showRegion, showOrigin, theDragRef);
			break;
		case dragRegionDraw:
			retVal = RegionDraw(showRegion, hideRegion, theDragRef);
			break;
		case dragRegionHide:
			retVal = RegionHide(hideRegion, theDragRef);
			break;
		case dragRegionIdle:
			retVal = RegionIdle(showRegion, theDragRef);
			break;
		case dragRegionEnd:
			retVal = RegionEnd(theDragRef);
			break;
	}
	
#if !GENERATINGCFM
	SetA5(oldA5);
#endif
	
	return retVal;
}

ItemReference CDragNDrop::GetItemReference(long itemNbr)
{
	return (ItemReference) itemNbr;
}


//
//	DropLocationIsFinderTrash
//
//	Returns true if the given dropLocation AEDesc is a descriptor of the Finder's Trash.
//
Boolean CDragNDrop::DropLocationIsFinderTrash(AEDesc *dropLocation)

{	OSErr			result;
	AEDesc			dropSpec;
	FSSpec			*theSpec;
	CInfoPBRec		thePB;
	short			trashVRefNum;
	long			trashDirID;

	//
	//	Coerce the dropLocation descriptor to an FSSpec. If there's no dropLocation or
	//	it can't be coerced into an FSSpec, then it couldn't have been the Trash.
	//

	if ((dropLocation->descriptorType != typeNull) &&
		(AECoerceDesc(dropLocation, typeFSS, &dropSpec) == noErr)) {

		HLock(dropSpec.dataHandle);
		theSpec = (FSSpec *) *dropSpec.dataHandle;

		//
		//	Get the directory ID of the given dropLocation object.
		//

		thePB.dirInfo.ioCompletion = 0L;
		thePB.dirInfo.ioNamePtr = (StringPtr) &theSpec->name;
		thePB.dirInfo.ioVRefNum = theSpec->vRefNum;
		thePB.dirInfo.ioFDirIndex = 0;
		thePB.dirInfo.ioDrDirID = theSpec->parID;

		result = PBGetCatInfo(&thePB, false);

		HUnlock(dropSpec.dataHandle);
		AEDisposeDesc(&dropSpec);

		if (result != noErr)
			return(false);

		//
		//	If the result is not a directory, it must not be the Trash.
		//

		if (!(thePB.dirInfo.ioFlAttrib & (1 << 4)))
			return(false);

		//
		//	Get information about the Trash folder.
		//

		FindFolder(theSpec->vRefNum, kTrashFolderType, kCreateFolder, &trashVRefNum, &trashDirID);

		//
		//	If the directory ID of the dropLocation object is the same as the directory ID
		//	returned by FindFolder, then the drop must have occurred into the Trash.
		//

		if (thePB.dirInfo.ioDrDirID == trashDirID)
			return(true);
	}

	return(false);
}

/* routines used as glue to call the appropriate CDragNDrop's method */

pascal OSErr DragTrackingHandlerP(DragTrackingMessage message,
		WindowPtr theWindow, CDragNDrop* obj, DragReference theDragRef)
{
	return obj->DragTrackingHandler(message, TCL_MacToTCLWindow(theWindow),
			theDragRef);
}

pascal OSErr DragReceiveHandlerP(WindowPtr theWindow, CDragNDrop* obj,
		DragReference theDragRef)
{
	return obj->DragReceiveHandler(TCL_MacToTCLWindow(theWindow), theDragRef);
}

pascal OSErr	DragSendDataP(FlavorType theType, CDragNDrop* obj,
		ItemReference theItemRef, DragReference theDragRef)
{
	return obj->DragSendData(theType, theItemRef, theDragRef);
}

pascal OSErr	DragInputP(Point* mouse, short* modifiers,
		CDragNDrop* obj, DragReference theDragRef)
{
	return obj->DragInput(mouse, modifiers, theDragRef);
}

pascal OSErr	DragDrawingP(DragRegionMessage message,
		RgnHandle showRegion, Point showOrigin,
		RgnHandle hideRegion, Point hideOrigin,
		CDragNDrop* obj, DragReference theDragRef)
{
	return obj->DragDrawing(message, showRegion, showOrigin,
			hideRegion, hideOrigin, theDragRef);
}

