/******************************************************************************
 CDragNDrop.h

							The Drag and Drop Class

	Implements the Macintosh Drag and Drop stuff for a class which uses this
	class as one of its superclasses.

	Hooks are provided for derived classes to provide information and handle
	Drag Manager data.

	BASE CLASS = none

	Copyright  1996 Glenn L. Austin and Symantec Corporation. All rights reserved.

 ******************************************************************************/

#pragma once

#include <Drag.h>

#define kDragAndDropEnableDefault	true

class CPane;

class CDragNDrop	//TCL_AUTO_DESTRUCT_OBJECT
{
public:
	TCL_DECLARE_CLASS
	
	CDragNDrop(CPane* aPane = nil, Boolean aEnabled = kDragAndDropEnableDefault);
	virtual ~CDragNDrop(void);
	
	// Utility methods
	void			InitHandlers(void);
	Boolean			DragAndDropSet(void);
	void			SetDragAndDrop(Boolean aEnabled);
	
	// Object I/O
	virtual void		PutTo(CStream&);
	virtual void		GetFrom(CPane*, CStream&);

	// Methods from CView which I want to override
	virtual void	DispatchClick(EventRecord *macEvent);
	virtual void	AdjustCursor(Point where, RgnHandle mouseRgn);
	
	// Calls used for Drag and Drop
	virtual OSErr	StartDrag(EventRecord*, CWindow*);
	virtual OSErr	AddFlavors(CWindow*, DragReference);
	virtual OSErr	SetupDragRegion(void);
	virtual Boolean	IsMyTypeAvailable(DragReference);
	virtual Boolean	CanDrag(void);
	virtual OSErr	DeleteSource(Boolean, CWindow*, DragReference);	// Called if source should be deleted
	
	// Calls made from DragTrackingHandler
	virtual OSErr	DragEnter(CWindow*, DragReference);
	virtual OSErr	DragInside(CWindow*, DragReference);
	virtual OSErr	DragLeave(CWindow*, DragReference);
	
	// Call made from DragReceiveHandler
	virtual OSErr	ReceiveData(CWindow*, DragReference);
	
	// Call made from DragSendData
	virtual OSErr	SendData(FlavorType, ItemReference, DragReference);
	
	// Call made from DragInput (A5 world set up for routines, but no low memory)
	virtual OSErr	ModifyInput(Point*, short*, DragReference);
	
	// Calls made from DragDrawing (A5 world set up for routines, but no low memory)
	virtual OSErr	RegionBegin(RgnHandle, Point, DragReference);
	virtual OSErr	RegionDraw(RgnHandle showRgn, RgnHandle hideRgn, DragReference);
	virtual OSErr	RegionHide(RgnHandle, DragReference);
	virtual OSErr	RegionIdle(RgnHandle, DragReference);
	virtual OSErr	RegionEnd(DragReference);
	
	// Accessor functions
	virtual RgnHandle	GetSelectedRgn(void)	{ return theSelectedRgn; }
	virtual ItemReference GetItemReference(long itemNbr);
	
protected:
	// You shouldn't have to override any of these handlers -- in fact,
	// these handlers call the appropriate public routines!
	
	virtual OSErr	DragTrackingHandler(DragTrackingMessage, CWindow*,
							DragReference);
	OSErr 			DragReceiveHandler(CWindow*, DragReference);
	OSErr			DragSendData(FlavorType, ItemReference,
							DragReference);
	OSErr			DragInput(Point*, short*, DragReference);
	OSErr			DragDrawing(DragRegionMessage,
							RgnHandle showRegion, Point showOrigin,
							RgnHandle hideRegion, Point hideOrigin,
							DragReference);
	Boolean			DropLocationIsFinderTrash(AEDesc *);

private:
	CPane*							myPane;
	RgnHandle						theSelectedRgn;
	Boolean							canAcceptDrag;
	Boolean							inContent;
	Boolean							handlersInstalled;
	Boolean							isEnabled;
	Boolean							refreshBorder;
	
	static Boolean					dragMgrInitialized;
	
	static DragTrackingHandlerUPP	myDragTrackUPP;
	static DragReceiveHandlerUPP	myReceiveUPP;
	static DragSendDataUPP			mySendUPP;
	static DragInputUPP				myInputUPP;
	static DragDrawingUPP			myDrawingUPP;

	friend pascal OSErr DragTrackingHandlerP(DragTrackingMessage, WindowPtr,
			CDragNDrop*, DragReference);
	friend pascal OSErr DragReceiveHandlerP(WindowPtr, CDragNDrop*,
			DragReference);
	friend pascal OSErr	DragSendDataP(FlavorType theType, CDragNDrop*,
			ItemReference, DragReference);
	friend pascal OSErr	DragInputP(Point*, short*, CDragNDrop*,
			DragReference);
	friend pascal OSErr	DragDrawingP(DragRegionMessage,
			RgnHandle showRegion, Point showOrigin,
			RgnHandle hideRegion, Point hideOrigin,
			CDragNDrop*, DragReference);
};
