//------------------------------------------------------------------------------------------
// COPYRIGHT  1997, David Slik. All Rights Reserved
//------------------------------------------------------------------------------------------
// Contents		: MP Graphic Color Routines						(MP GrafLib Color.c)
// Version      : 1.0
// Last Updated	: May 25, 1997
// Written By	: David Slik
//------------------------------------------------------------------------------------------

//------------------------------------------------------------------------------------------
// DISCLAIMER - The author is not responsible for the results of using this code
//------------------------------------------------------------------------------------------
// If you find any bugs in this source code, please email me and I will attempt to fix
// them.  If you have any additions, modifications or suggestions, please let me know.
// I can be reached at the following address: dslik@paradata.com
// Updates will be posted to the URL http://www.paradata.com/~dslik/MPGL/MPGL.html
//
// Permission to use and modify this code is freely granted to all users.
//------------------------------------------------------------------------------------------


// HEADER FILES	---------------------------------------------------------------------

#include "MP GrafLib Color.h"
#include "MP GrafLib Internals.h"


// DIRECTIVES	---------------------------------------------------------------------

#pragma export list DS_SetColor, DS_GetForeColor, DS_GetBackColor, DS_SetForeColor, DS_SetBackColor, DS_GetBitDepth
#pragma lib_export list DS_SetColor, DS_GetForeColor, DS_GetBackColor, DS_SetForeColor, DS_SetBackColor, DS_GetBitDepth


// FUNCTIONS	---------------------------------------------------------------------

//------------------------------------------------------------------------------------------
// Thread Safe		: [x]	| Normal Build		: [x]	| Posts Events	: [ ]
// Reentrant		: [ ]	| Debug Build		: [ ]	| Pulls Events	: [ ]
// MP Compatable	: [x]	| Structure Checking: [ ]	+-----------------------------------
//--------------------------+---------------------------+ Performance Critical	: [ ]
// Allocates Memory	: [ ]	| Calls Functions	: [ ]	| Performance : L [x] M [ ] H [ ]
// Moves Memory		: [ ]	| Calls Toolbox		: [ ]	| Normal Overhead	: ???
// Uses TB Memory 	: [ ]	| Calls QuickDraw	: [ ]	| Debug Overhead	: ???
// Uses PAM Memory	: [ ]	| Async Callback	: [ ]	| MP Overhead		: ???
//------------------------------------------------------------------------------------------
// Function Name		: DS_SetColor
// Function Description	: Creates a RGBColor value from a Red, Green and Blue component
// Error Checking		: None Required
// Last Updated			: May 25, 1997
// Written By			: David Slik
// Modifications		:
// Notes				: 
//------------------------------------------------------------------------------------------
DS_RGBColor DS_SetColor(unsigned char Red, unsigned char Green, unsigned char Blue)
{
	// When drawing directly into a frame buffer, color information needs to be
	// packed into a special format:
	
	// 32 Bit Color - RRRRGGGGBBBB0000
	// 16 Bit Color - RRGGBB00
	// 8 Bit Color  - RGB0

	// This routine creates a color in the above 32 bit format. the color is then compacted
	// if required when the color is set.
	
	DS_RGBColor	theColor = 0;
	
	theColor = theColor + (Red << 8);
	theColor = theColor + (Green << 16);
	theColor = theColor + (Blue << 24);

	return(theColor);
}

//------------------------------------------------------------------------------------------
// Thread Safe		: [x]	| Normal Build		: [x]	| Posts Events	: [ ]
// Reentrant		: [ ]	| Debug Build		: [ ]	| Pulls Events	: [ ]
// MP Compatable	: [x]	| Structure Checking: [ ]	+-----------------------------------
//--------------------------+---------------------------+ Performance Critical	: [ ]
// Allocates Memory	: [ ]	| Calls Functions	: [ ]	| Performance : L [x] M [ ] H [ ]
// Moves Memory		: [ ]	| Calls Toolbox		: [ ]	| Normal Overhead	: ???
// Uses TB Memory 	: [ ]	| Calls QuickDraw	: [ ]	| Debug Overhead	: ???
// Uses PAM Memory	: [ ]	| Async Callback	: [ ]	| MP Overhead		: ???
//------------------------------------------------------------------------------------------
// Function Name		: DS_SetForeColor
// Function Description	: Sets the foreground color - Used for Pen based routines
// Error Checking		: None Required
// Date Created			: May 18, 1997
// Written By			: David Slik
// Modifications		:
// Notes				: 
//------------------------------------------------------------------------------------------
void DS_SetForeColor(DS_RGBColor theColor)
{
	if(TRUE == DS_isEnvironmentSetup())
	{
		gDSGlobals.DS_ForeColor = theColor;
	}
}

//------------------------------------------------------------------------------------------
// Thread Safe		: [x]	| Normal Build		: [x]	| Posts Events	: [ ]
// Reentrant		: [ ]	| Debug Build		: [ ]	| Pulls Events	: [ ]
// MP Compatable	: [x]	| Structure Checking: [ ]	+-----------------------------------
//--------------------------+---------------------------+ Performance Critical	: [ ]
// Allocates Memory	: [ ]	| Calls Functions	: [ ]	| Performance : L [x] M [ ] H [ ]
// Moves Memory		: [ ]	| Calls Toolbox		: [ ]	| Normal Overhead	: ???
// Uses TB Memory 	: [ ]	| Calls QuickDraw	: [ ]	| Debug Overhead	: ???
// Uses PAM Memory	: [ ]	| Async Callback	: [ ]	| MP Overhead		: ???
//------------------------------------------------------------------------------------------
// Function Name		: DS_SetBackColor
// Function Description	: Sets the background color - Used for Pen based routines
// Error Checking		: None Required
// Date Created			: May 18, 1997
// Written By			: David Slik
// Modifications		:
// Notes				: 
//------------------------------------------------------------------------------------------
void DS_SetBackColor(DS_RGBColor theColor)
{
	if(TRUE == DS_isEnvironmentSetup())
	{
		gDSGlobals.DS_BackColor = theColor;
	}
}

//------------------------------------------------------------------------------------------
// Thread Safe		: [x]	| Normal Build		: [x]	| Posts Events	: [ ]
// Reentrant		: [ ]	| Debug Build		: [ ]	| Pulls Events	: [ ]
// MP Compatable	: [x]	| Structure Checking: [ ]	+-----------------------------------
//--------------------------+---------------------------+ Performance Critical	: [ ]
// Allocates Memory	: [ ]	| Calls Functions	: [ ]	| Performance : L [x] M [ ] H [ ]
// Moves Memory		: [ ]	| Calls Toolbox		: [ ]	| Normal Overhead	: ???
// Uses TB Memory 	: [ ]	| Calls QuickDraw	: [ ]	| Debug Overhead	: ???
// Uses PAM Memory	: [ ]	| Async Callback	: [ ]	| MP Overhead		: ???
//------------------------------------------------------------------------------------------
// Function Name		: DS_GetForeColor
// Function Description	: Gets the foreground color
// Error Checking		: None Required
// Date Created			: May 18, 1997
// Written By			: David Slik
// Modifications		:
// Notes				: 
//------------------------------------------------------------------------------------------
DS_RGBColor DS_GetForeColor(void)
{
	if(TRUE == DS_isEnvironmentSetup())
	{
		return(gDSGlobals.DS_ForeColor);
	}
	else
	{
		return(0);
	}
}

//------------------------------------------------------------------------------------------
// Thread Safe		: [x]	| Normal Build		: [x]	| Posts Events	: [ ]
// Reentrant		: [ ]	| Debug Build		: [ ]	| Pulls Events	: [ ]
// MP Compatable	: [x]	| Structure Checking: [ ]	+-----------------------------------
//--------------------------+---------------------------+ Performance Critical	: [ ]
// Allocates Memory	: [ ]	| Calls Functions	: [ ]	| Performance : L [x] M [ ] H [ ]
// Moves Memory		: [ ]	| Calls Toolbox		: [ ]	| Normal Overhead	: ???
// Uses TB Memory 	: [ ]	| Calls QuickDraw	: [ ]	| Debug Overhead	: ???
// Uses PAM Memory	: [ ]	| Async Callback	: [ ]	| MP Overhead		: ???
//------------------------------------------------------------------------------------------
// Function Name		: DS_GetBackColor
// Function Description	: Gets the background color
// Error Checking		: None Required
// Date Created			: May 18, 1997
// Written By			: David Slik
// Modifications		:
// Notes				: 
//------------------------------------------------------------------------------------------
DS_RGBColor DS_GetBackColor(void)
{
	if(TRUE == DS_isEnvironmentSetup())
	{
		return(gDSGlobals.DS_BackColor);
	}
	else
	{
		return(0);
	}
}

//------------------------------------------------------------------------------------------
// Thread Safe		: [x]	| Normal Build		: [x]	| Posts Events	: [ ]
// Reentrant		: [ ]	| Debug Build		: [ ]	| Pulls Events	: [ ]
// MP Compatable	: [x]	| Structure Checking: [ ]	+-----------------------------------
//--------------------------+---------------------------+ Performance Critical	: [ ]
// Allocates Memory	: [ ]	| Calls Functions	: [ ]	| Performance : L [x] M [ ] H [ ]
// Moves Memory		: [ ]	| Calls Toolbox		: [ ]	| Normal Overhead	: ???
// Uses TB Memory 	: [ ]	| Calls QuickDraw	: [ ]	| Debug Overhead	: ???
// Uses PAM Memory	: [ ]	| Async Callback	: [ ]	| MP Overhead		: ???
//------------------------------------------------------------------------------------------
// Function Name		: DS_GetBackColor
// Function Description	: Gets the background color
// Error Checking		: None Required
// Date Created			: May 18, 1997
// Written By			: David Slik
// Modifications		:
// Notes				: 
//------------------------------------------------------------------------------------------
DS_RGBColor DS_GetBitDepth(void)
{
	if(TRUE == DS_isEnvironmentSetup())
	{
		return(gDSGlobals.DisplayBitDepth);
	}
	else
	{
		return(0);
	}
}

