//------------------------------------------------------------------------------------------
// COPYRIGHT  1997, David Slik. All Rights Reserved
//------------------------------------------------------------------------------------------
// Contents		: MP Graphic Drawing Routines					(MP GrafLib Drawing.c)
// Version      : 1.0
// Last Updated	: May 25, 1997
// Written By	: David Slik
//------------------------------------------------------------------------------------------

//------------------------------------------------------------------------------------------
// DISCLAIMER - The author is not responsible for the results of using this code
//------------------------------------------------------------------------------------------
// If you find any bugs in this source code, please email me and I will attempt to fix
// them.  If you have any additions, modifications or suggestions, please let me know.
// I can be reached at the following address: dslik@paradata.com
// Updates will be posted to the URL http://www.paradata.com/~dslik/MPGL/MPGL.html
//
// Permission to use and modify this code is freely granted to all users.
//------------------------------------------------------------------------------------------


// HEADER FILES	---------------------------------------------------------------------

#include "MP GrafLib Drawing.h"
#include "MP GrafLib Internals.h"


// DIRECTIVES	---------------------------------------------------------------------

#pragma export list DS_MoveTo, DS_LineTo, DS_Line, DS_FrameRect, DS_FillRect, DS_SetPixel
#pragma lib_export list DS_MoveTo, DS_LineTo, DS_Line, DS_FrameRect, DS_FillRect, DS_SetPixel


// FUNCTIONS	---------------------------------------------------------------------


// -----------------------------------------------------------------------------
//   TRADITIONAL GRAPHIC ROUTINES - Pen Based
// -----------------------------------------------------------------------------


//------------------------------------------------------------------------------------------
// Thread Safe		: [x]	| Normal Build		: [x]	| Posts Events	: [ ]
// Reentrant		: [ ]	| Debug Build		: [ ]	| Pulls Events	: [ ]
// MP Compatable	: [x]	| Structure Checking: [ ]	+-----------------------------------
//--------------------------+---------------------------+ Performance Critical	: [ ]
// Allocates Memory	: [ ]	| Calls Functions	: [ ]	| Performance : L [x] M [ ] H [ ]
// Moves Memory		: [ ]	| Calls Toolbox		: [ ]	| Normal Overhead	: ???
// Uses TB Memory 	: [ ]	| Calls QuickDraw	: [ ]	| Debug Overhead	: ???
// Uses PAM Memory	: [ ]	| Async Callback	: [ ]	| MP Overhead		: ???
//------------------------------------------------------------------------------------------
// Function Name		: DS_MoveTo
// Function Description	: Sets the position of the pen
// Error Checking		: None Required (Null global checking in called functions)
// Date Created			: May 18, 1997
// Written By			: David Slik
// Modifications		:
// Notes				: 
//------------------------------------------------------------------------------------------
void DS_MoveTo(short horiz, short vert)
{
	_SetPenX(horiz);
	_SetPenY(vert);
}


//------------------------------------------------------------------------------------------
// Thread Safe		: [x]	| Normal Build		: [x]	| Posts Events	: [ ]
// Reentrant		: [ ]	| Debug Build		: [ ]	| Pulls Events	: [ ]
// MP Compatable	: [x]	| Structure Checking: [ ]	+-----------------------------------
//--------------------------+---------------------------+ Performance Critical	: [ ]
// Allocates Memory	: [ ]	| Calls Functions	: [ ]	| Performance : L [ ] M [x] H [ ]
// Moves Memory		: [ ]	| Calls Toolbox		: [ ]	| Normal Overhead	: ???
// Uses TB Memory 	: [ ]	| Calls QuickDraw	: [ ]	| Debug Overhead	: ???
// Uses PAM Memory	: [ ]	| Async Callback	: [ ]	| MP Overhead		: ???
//------------------------------------------------------------------------------------------
// Function Name		: DS_LineTo
// Function Description	: Draws a line from the current position of the pen to a specificed
//						  position.
// Error Checking		: Checks for initilization
// Date Created			: May 18, 1997
// Written By			: David Slik
// Modifications		:
// Notes				: Changes the current pen position.
//						  Should be changed to better line drawing function. Anti-aliasing
//						  would also be a good addition.
//
//						  CLIPPING NEEDS TO BE ADDED!!!
//------------------------------------------------------------------------------------------
void DS_LineTo(short horiz, short vert)
{
	// Simple implementation of the incremental line scan conversion
	// algorithm
	
	if(TRUE == DS_isEnvironmentSetup())
	{
		// Initilized, calculate variables
		
		unsigned int	PixelValue = DS_GetForeColor();
		short			PenHoriz = _GetPenX();
		short			PenVert = _GetPenY();
		
		int				x = PenHoriz;
		float			y = PenVert;
		
		float			deltax = horiz - PenHoriz;
		float			deltay = vert - PenVert;
		float			m = deltay/deltax;
		
		// Draw the line
		while(x != horiz)
		{
			DS_SetPixel(x, y, PixelValue);
			
			y = y + m;
			x++;
		}
		
		// Update position
		DS_MoveTo(PenHoriz + horiz, PenVert + vert);
	}
}
	

	
//------------------------------------------------------------------------------------------
// Thread Safe		: [x]	| Normal Build		: [x]	| Posts Events	: [ ]
// Reentrant		: [ ]	| Debug Build		: [ ]	| Pulls Events	: [ ]
// MP Compatable	: [x]	| Structure Checking: [ ]	+-----------------------------------
//--------------------------+---------------------------+ Performance Critical	: [ ]
// Allocates Memory	: [ ]	| Calls Functions	: [ ]	| Performance : L [x] M [ ] H [ ]
// Moves Memory		: [ ]	| Calls Toolbox		: [ ]	| Normal Overhead	: ???
// Uses TB Memory 	: [ ]	| Calls QuickDraw	: [ ]	| Debug Overhead	: ???
// Uses PAM Memory	: [ ]	| Async Callback	: [ ]	| MP Overhead		: ???
//------------------------------------------------------------------------------------------
// Function Name		: DS_Line
// Function Description	: Calls DS_LineTo with slightly different paramaters
// Error Checking		: None Required
// Date Created			: May 18, 1997
// Written By			: David Slik
// Modifications		:
// Notes				: Changes the current pen position.
//------------------------------------------------------------------------------------------
void DS_Line(short distHoriz, short distVert)
{
	short	horiz = _GetPenX() + distHoriz;
	short	vert = _GetPenY() + distVert;
	
	DS_LineTo(horiz, vert);
}

// -----------------------------------------------------------------------------
//   NEWER GRAPHIC ROUTINES - Rect/Region Based
// -----------------------------------------------------------------------------



//------------------------------------------------------------------------------------------
// Thread Safe		: [x]	| Normal Build		: [x]	| Posts Events	: [ ]
// Reentrant		: [ ]	| Debug Build		: [ ]	| Pulls Events	: [ ]
// MP Compatable	: [x]	| Structure Checking: [ ]	+-----------------------------------
//--------------------------+---------------------------+ Performance Critical	: [ ]
// Allocates Memory	: [ ]	| Calls Functions	: [ ]	| Performance : L [ ] M [x] H [ ]
// Moves Memory		: [ ]	| Calls Toolbox		: [ ]	| Normal Overhead	: ???
// Uses TB Memory 	: [ ]	| Calls QuickDraw	: [ ]	| Debug Overhead	: ???
// Uses PAM Memory	: [ ]	| Async Callback	: [ ]	| MP Overhead		: ???
//------------------------------------------------------------------------------------------
// Function Name		: DS_FrameRect
// Function Description	: Draws the border of a rectangle with the foreground colour
// Error Checking		: Checks for initilization
// Date Created			: May 18, 1997
// Written By			: David Slik
// Modifications		:
// Notes				: 
//------------------------------------------------------------------------------------------
void DS_FrameRect(DS_Rect *theRect)
{
	if(TRUE == DS_isEnvironmentSetup())
	{
		// Initilized, calculate variables
		
		unsigned int	PixelValue = DS_GetForeColor();
		
		int				x = theRect->right;
		int				y = theRect->top;

		// Draw horizontal lines
		while(x != theRect->left)
		{
			DS_SetPixel(x, theRect->top, PixelValue);
			DS_SetPixel(x, theRect->bottom, PixelValue);
			x++;
		}

		// Draw vertical lines
		while(y != theRect->bottom)
		{
			DS_SetPixel(theRect->left, y, PixelValue);
			DS_SetPixel(theRect->right, y, PixelValue);
			y++;
		}
	}
}

//------------------------------------------------------------------------------------------
// Thread Safe		: [x]	| Normal Build		: [x]	| Posts Events	: [ ]
// Reentrant		: [ ]	| Debug Build		: [ ]	| Pulls Events	: [ ]
// MP Compatable	: [x]	| Structure Checking: [ ]	+-----------------------------------
//--------------------------+---------------------------+ Performance Critical	: [ ]
// Allocates Memory	: [ ]	| Calls Functions	: [ ]	| Performance : L [ ] M [x] H [ ]
// Moves Memory		: [ ]	| Calls Toolbox		: [ ]	| Normal Overhead	: ???
// Uses TB Memory 	: [ ]	| Calls QuickDraw	: [ ]	| Debug Overhead	: ???
// Uses PAM Memory	: [ ]	| Async Callback	: [ ]	| MP Overhead		: ???
//------------------------------------------------------------------------------------------
// Function Name		: DS_FillRect
// Function Description	: Fills a rectangle with the background colour
// Error Checking		: Checks for initilization
// Date Created			: May 18, 1997
// Written By			: David Slik
// Modifications		:
// Notes				: 
//------------------------------------------------------------------------------------------
void DS_FillRect(DS_Rect *theRect)
{
	if(TRUE == DS_isEnvironmentSetup())
	{
		// Initilized, calculate variables
		
		unsigned int	PixelValue = DS_GetForeColor();
		int				x = theRect->right + 1;
		int				y = theRect->top + 1;
		
		DS_InsetRect(theRect, 1, 1);

		// Draw across horizontally
		while(x != theRect->left)
		{
			// Draw across vertically
			while(y != theRect->bottom)
			{
				DS_SetPixel(x, y, PixelValue);
				y++;
			}
			x++;
		}

		DS_InsetRect(theRect, -1, -1);
	}
}






//------------------------------------------------------------------------------------------
// Thread Safe		: [x]	| Normal Build		: [x]	| Posts Events	: [ ]
// Reentrant		: [ ]	| Debug Build		: [ ]	| Pulls Events	: [ ]
// MP Compatable	: [x]	| Structure Checking: [ ]	+-----------------------------------
//--------------------------+---------------------------+ Performance Critical	: [ ]
// Allocates Memory	: [ ]	| Calls Functions	: [ ]	| Performance : L [ ] M [x] H [ ]
// Moves Memory		: [ ]	| Calls Toolbox		: [ ]	| Normal Overhead	: ???
// Uses TB Memory 	: [ ]	| Calls QuickDraw	: [ ]	| Debug Overhead	: ???
// Uses PAM Memory	: [ ]	| Async Callback	: [ ]	| MP Overhead		: ???
//------------------------------------------------------------------------------------------
// Function Name		: _SetPenX
// Function Description	: Sets the value of the X Pen component
// Error Checking		: None Required
// Date Created			: May 18, 1997
// Written By			: David Slik
// Modifications		:
// Notes				: Here so global structure can be modified if need be
//------------------------------------------------------------------------------------------
void _SetPenX(short horiz)
{
	gDSGlobals.PenX = horiz;
}

//------------------------------------------------------------------------------------------
// Thread Safe		: [x]	| Normal Build		: [x]	| Posts Events	: [ ]
// Reentrant		: [ ]	| Debug Build		: [ ]	| Pulls Events	: [ ]
// MP Compatable	: [x]	| Structure Checking: [ ]	+-----------------------------------
//--------------------------+---------------------------+ Performance Critical	: [ ]
// Allocates Memory	: [ ]	| Calls Functions	: [ ]	| Performance : L [ ] M [x] H [ ]
// Moves Memory		: [ ]	| Calls Toolbox		: [ ]	| Normal Overhead	: ???
// Uses TB Memory 	: [ ]	| Calls QuickDraw	: [ ]	| Debug Overhead	: ???
// Uses PAM Memory	: [ ]	| Async Callback	: [ ]	| MP Overhead		: ???
//------------------------------------------------------------------------------------------
// Function Name		: _SetPenY
// Function Description	: Sets the value of the Y Pen component
// Error Checking		: None Required
// Date Created			: May 18, 1997
// Written By			: David Slik
// Modifications		:
// Notes				: Here so global structure can be modified if need be
//------------------------------------------------------------------------------------------
void _SetPenY(short vert)
{
	gDSGlobals.PenY = vert;
}

//------------------------------------------------------------------------------------------
// Thread Safe		: [x]	| Normal Build		: [x]	| Posts Events	: [ ]
// Reentrant		: [ ]	| Debug Build		: [ ]	| Pulls Events	: [ ]
// MP Compatable	: [x]	| Structure Checking: [ ]	+-----------------------------------
//--------------------------+---------------------------+ Performance Critical	: [x]
// Allocates Memory	: [ ]	| Calls Functions	: [ ]	| Performance : L [x] M [ ] H [ ]
// Moves Memory		: [ ]	| Calls Toolbox		: [ ]	| Normal Overhead	: ???
// Uses TB Memory 	: [ ]	| Calls QuickDraw	: [ ]	| Debug Overhead	: ???
// Uses PAM Memory	: [ ]	| Async Callback	: [ ]	| MP Overhead		: ???
//------------------------------------------------------------------------------------------
// Function Name		: DS_SetPixel
// Function Description	: Sets the value of a pixel
// Error Checking		: Checks to ensure there is no modification of memory outside frame
//						  buffer.
// Last Updated			: June 2, 1997
// Written By			: David Slik
// Modifications		: Updated to support 16 Bit Colour drawing
// Notes				: 
//------------------------------------------------------------------------------------------
void DS_SetPixel(short xCooridinate, short yCooridinate, unsigned int PixelValue)
{
	// Do Screen Clipping
	if(gDSGlobals.HorizontalResolution < xCooridinate) return;
	if(gDSGlobals.VerticalResolution < yCooridinate) return;
	if(0 > xCooridinate) return;
	if(0 > yCooridinate) return;
	
	// Do Window Clipping
	if(gDSGlobals.ClippingRect.left > xCooridinate) return;
	if(gDSGlobals.ClippingRect.bottom < yCooridinate) return;
	if(gDSGlobals.ClippingRect.right < xCooridinate) return;
	if(gDSGlobals.ClippingRect.top > yCooridinate) return;
	
	// Do color Correction ! CURRENTLY ONLY 32 AND 16 BIT COLOR SUPPORTED
	if(gDSGlobals.DisplayBitDepth == 32)
	{
		int*	PixelLocation = NULL;
		
		PixelLocation = (int*) gDSGlobals.DisplayBaseAddress + gDSGlobals.HorizontalResolution * yCooridinate + xCooridinate;
		PixelLocation[0] = PixelValue;
	}

	if(gDSGlobals.DisplayBitDepth == 16)
	{
		// !! Colour Matching not fully tested. 16 Bit Colors do not yet equal 32 bit colors
		
		short*	PixelLocation = NULL;
		
		PixelLocation = (short*) gDSGlobals.DisplayBaseAddress + gDSGlobals.HorizontalResolution * yCooridinate + xCooridinate;
		PixelLocation[0] = PixelValue / 0xFFFF;
	}
}
