//------------------------------------------------------------------------------------------
// COPYRIGHT  1997, David Slik. All Rights Reserved
//------------------------------------------------------------------------------------------
// Contents		: MP GrafLib Utility Routines					(MP GrafLib Utilities.c)
// Version      : 1.0
// Last Updated	: May 25, 1997
// Written By	: David Slik
//------------------------------------------------------------------------------------------

//------------------------------------------------------------------------------------------
// DISCLAIMER - The author is not responsible for the results of using this code
//------------------------------------------------------------------------------------------
// If you find any bugs in this source code, please email me and I will attempt to fix
// them.  If you have any additions, modifications or suggestions, please let me know.
// I can be reached at the following address: dslik@paradata.com
// Updates will be posted to the URL http://www.paradata.com/~dslik/MPGL/MPGL.html
//
// Permission to use and modify this code is freely granted to all users.
//------------------------------------------------------------------------------------------


// HEADER FILES	---------------------------------------------------------------------

#include "MP GrafLib Utilities.h"

// DIRECTIVES	---------------------------------------------------------------------

#pragma export list DS_SetRect, DS_InsetRect, DS_OffsetRect, DS_RectInRect
#pragma lib_export list DS_SetRect, DS_InsetRect, DS_OffsetRect, DS_RectInRect


// FUNCTIONS	---------------------------------------------------------------------

//------------------------------------------------------------------------------------------
// Thread Safe		: [x]	| Normal Build		: [x]	| Posts Events	: [ ]
// Reentrant		: [ ]	| Debug Build		: [ ]	| Pulls Events	: [ ]
// MP Compatable	: [x]	| Structure Checking: [ ]	+-----------------------------------
//--------------------------+---------------------------+ Performance Critical	: [ ]
// Allocates Memory	: [ ]	| Calls Functions	: [ ]	| Performance : L [x] M [ ] H [ ]
// Moves Memory		: [ ]	| Calls Toolbox		: [ ]	| Normal Overhead	: ???
// Uses TB Memory 	: [ ]	| Calls QuickDraw	: [ ]	| Debug Overhead	: ???
// Uses PAM Memory	: [ ]	| Async Callback	: [ ]	| MP Overhead		: ???
//------------------------------------------------------------------------------------------
// Function Name		: DS_SetRect
// Function Description	: Initilizes values of a Rect Structure
// Error Checking		: None
// Last Updated			: May 21, 1997
// Written By			: David Slik
// Modifications		:
// Notes				: Decision to use own rect structure is to discourage useage
//						  of toolbox functions, which will crash the MP task.
//------------------------------------------------------------------------------------------
void DS_SetRect(DS_Rect *theRect, short rLeft, short rTop, short rRight, short rBottom)
{
	theRect->left = rLeft;
	theRect->top = rTop;
	theRect->right = rRight;
	theRect->bottom = rBottom;
}

//------------------------------------------------------------------------------------------
// Thread Safe		: [x]	| Normal Build		: [x]	| Posts Events	: [ ]
// Reentrant		: [ ]	| Debug Build		: [ ]	| Pulls Events	: [ ]
// MP Compatable	: [x]	| Structure Checking: [ ]	+-----------------------------------
//--------------------------+---------------------------+ Performance Critical	: [ ]
// Allocates Memory	: [ ]	| Calls Functions	: [ ]	| Performance : L [x] M [ ] H [ ]
// Moves Memory		: [ ]	| Calls Toolbox		: [ ]	| Normal Overhead	: ???
// Uses TB Memory 	: [ ]	| Calls QuickDraw	: [ ]	| Debug Overhead	: ???
// Uses PAM Memory	: [ ]	| Async Callback	: [ ]	| MP Overhead		: ???
//------------------------------------------------------------------------------------------
// Function Name		: DS_InsetRect
// Function Description	: Symetrically enlarges or reduces a Rect
// Error Checking		: None
// Last Updated			: May 21, 1997
// Written By			: David Slik
// Modifications		:
// Notes				: Decision to use own rect structure is to discourage useage
//						  of toolbox functions, which will crash the MP task.
//------------------------------------------------------------------------------------------
void DS_InsetRect(DS_Rect *theRect, short distHoriz, short distVert)
{
	theRect->left = theRect->left - distHoriz;
	theRect->top = theRect->top + distVert;
	theRect->right = theRect->right + distHoriz;
	theRect->bottom = theRect->bottom - distVert;
}

//------------------------------------------------------------------------------------------
// Thread Safe		: [x]	| Normal Build		: [x]	| Posts Events	: [ ]
// Reentrant		: [ ]	| Debug Build		: [ ]	| Pulls Events	: [ ]
// MP Compatable	: [x]	| Structure Checking: [ ]	+-----------------------------------
//--------------------------+---------------------------+ Performance Critical	: [ ]
// Allocates Memory	: [ ]	| Calls Functions	: [ ]	| Performance : L [x] M [ ] H [ ]
// Moves Memory		: [ ]	| Calls Toolbox		: [ ]	| Normal Overhead	: ???
// Uses TB Memory 	: [ ]	| Calls QuickDraw	: [ ]	| Debug Overhead	: ???
// Uses PAM Memory	: [ ]	| Async Callback	: [ ]	| MP Overhead		: ???
//------------------------------------------------------------------------------------------
// Function Name		: DS_OffsetRect
// Function Description	: Moves a Rect
// Error Checking		: None
// Last Updated			: May 21, 1997
// Written By			: David Slik
// Modifications		:
// Notes				: Decision to use own rect structure is to discourage useage
//						  of toolbox functions, which will crash the MP task.
//------------------------------------------------------------------------------------------
void DS_OffsetRect(DS_Rect *theRect, short distHoriz, short distVert)
{
	theRect->left = theRect->left + distHoriz;
	theRect->top = theRect->top + distVert;
	theRect->right = theRect->right + distHoriz;
	theRect->bottom = theRect->bottom + distVert;
}



//------------------------------------------------------------------------------------------
// Thread Safe		: [x]	| Normal Build		: [x]	| Posts Events	: [ ]
// Reentrant		: [ ]	| Debug Build		: [ ]	| Pulls Events	: [ ]
// MP Compatable	: [x]	| Structure Checking: [ ]	+-----------------------------------
//--------------------------+---------------------------+ Performance Critical	: [ ]
// Allocates Memory	: [ ]	| Calls Functions	: [ ]	| Performance : L [x] M [ ] H [ ]
// Moves Memory		: [ ]	| Calls Toolbox		: [ ]	| Normal Overhead	: ???
// Uses TB Memory 	: [ ]	| Calls QuickDraw	: [ ]	| Debug Overhead	: ???
// Uses PAM Memory	: [ ]	| Async Callback	: [ ]	| MP Overhead		: ???
//------------------------------------------------------------------------------------------
// Function Name		: DS_RectInRect
// Function Description	: Checks if one rect is contained within another
// Error Checking		: None
// Last Updated			: May 25, 1997
// Written By			: David Slik
// Modifications		:
// Notes				: Brute force, must be more elegent algorithm
//------------------------------------------------------------------------------------------
Boolean DS_RectInRect(DS_Rect *theRect, DS_Rect *InsideRect)
{
	if(InsideRect->left < theRect->left) return(FALSE);
	if(InsideRect->left > theRect->right) return(FALSE);
	
	if(InsideRect->right > theRect->right) return(FALSE);
	if(InsideRect->left < theRect->left) return(FALSE);
	
	if(InsideRect->top < theRect->top) return(FALSE);
	if(InsideRect->top > theRect->bottom) return(FALSE);
	
	if(InsideRect->bottom > theRect->bottom) return(FALSE);
	if(InsideRect->bottom < theRect->top) return(FALSE);
	
	return(TRUE);
}
