//------------------------------------------------------------------------------------------
// COPYRIGHT  1997, David Slik. All Rights Reserved
//------------------------------------------------------------------------------------------
// Contents		: MP Graphic Drawing Routines					(MP GrafLib Drawing.c)
// Version      : 1.0
// Last Updated	: May 25, 1997
// Written By	: David Slik
//------------------------------------------------------------------------------------------

//------------------------------------------------------------------------------------------
// DISCLAIMER - The author is not responsible for the results of using this code
//------------------------------------------------------------------------------------------
// If you find any bugs in this source code, please email me and I will attempt to fix
// them.  If you have any additions, modifications or suggestions, please let me know.
// I can be reached at the following address: dslik@paradata.com
// Updates will be posted to the URL http://www.paradata.com/~dslik/MPGL/MPGL.html
//
// Permission to use and modify this code is freely granted to all users.
//------------------------------------------------------------------------------------------


// HEADER FILES	---------------------------------------------------------------------

#include "MP GrafLib.h"
#include "MP GrafLib Globals.h"


// GLOBALS		---------------------------------------------------------------------

DrawingGlobals	gDSGlobals;

#define		INITCHECK_VALUE		0xF3A4

// DIRECTIVES	---------------------------------------------------------------------

#pragma export list InitMPGrafLib, DeInitMPGrafLib, SetClippingRect
#pragma lib_export list InitMPGrafLib, DeInitMPGrafLib, SetClippingRect


// FUNCTIONS	---------------------------------------------------------------------

OSErr __init(void);
OSErr __terminate(void);
OSErr __main(void);

// Code Fragment Functions

// -----------------------------------------------------------------------------
//   SHARED LIBRARY GLOBAL INITILIZATION FUNCTIONS
// -----------------------------------------------------------------------------


OSErr __init(void)
{

	// This function is called as the shared library is initilized.

//	PRIMARY CODE	--------------------------------------------------------------------------------------------------------------------------------------------------

	//DisplayDebugStr("################################INIT");

	// Initilize Globals
	gDSGlobals.Overwrite = 0;
	gDSGlobals.InitCheck = 0;
	gDSGlobals.DisplayBaseAddress = NULL;
	gDSGlobals.DisplayBitDepth = 0;
	gDSGlobals.HorizontalResolution = 0;
	gDSGlobals.VerticalResolution = 0;
	gDSGlobals.ClippingRect.top = 0;
	gDSGlobals.ClippingRect.right = 0;
	gDSGlobals.ClippingRect.bottom = 0;
	gDSGlobals.ClippingRect.left = 0;
	gDSGlobals.DS_ForeColor = 0;
	gDSGlobals.DS_BackColor = 0;
	gDSGlobals.PenX = 0;
	gDSGlobals.PenY = 0;

	return(ERROR_NONE);			// return a successful value to allow loading to continue
}


OSErr __terminate(void)
{

	// This function is called as the shared library is terminated.

//	PRIMARY CODE	--------------------------------------------------------------------------------------------------------------------------------------------------

	// Clear Globals
	gDSGlobals.Overwrite = 0;
	gDSGlobals.InitCheck = 0;
	gDSGlobals.DisplayBaseAddress = NULL;
	gDSGlobals.DisplayBitDepth = 0;
	gDSGlobals.HorizontalResolution = 0;
	gDSGlobals.VerticalResolution = 0;
	gDSGlobals.ClippingRect.top = 0;
	gDSGlobals.ClippingRect.right = 0;
	gDSGlobals.ClippingRect.bottom = 0;
	gDSGlobals.ClippingRect.left = 0;
	gDSGlobals.DS_ForeColor = 0;
	gDSGlobals.DS_BackColor = 0;
	gDSGlobals.PenX = 0;
	gDSGlobals.PenY = 0;

	//DisplayDebugStr("################################TERMINATE");

	return(ERROR_NONE);			// return a successful value to prevent error
}


OSErr __main(void)
{
	// Should never be called
	
	//	PRIMARY CODE	--------------------------------------------------------------------------------------------------------------------------------------------------

	return(ERROR_NONE);
}



// -----------------------------------------------------------------------------
//   LIBRARY INITILIZATION FUNCTIONS
// -----------------------------------------------------------------------------

//------------------------------------------------------------------------------------------
// Thread Safe		: [x]	| Normal Build		: [x]	| Posts Events	: [ ]
// Reentrant		: [ ]	| Debug Build		: [ ]	| Pulls Events	: [ ]
// MP Compatable	: [x]	| Structure Checking: [ ]	+-----------------------------------
//--------------------------+---------------------------+ Performance Critical	: [ ]
// Allocates Memory	: [ ]	| Calls Functions	: [ ]	| Performance : L [x] M [ ] H [ ]
// Moves Memory		: [ ]	| Calls Toolbox		: [ ]	| Normal Overhead	: ???
// Uses TB Memory 	: [ ]	| Calls QuickDraw	: [ ]	| Debug Overhead	: ???
// Uses PAM Memory	: [ ]	| Async Callback	: [ ]	| MP Overhead		: ???
//------------------------------------------------------------------------------------------
// Function Name		: InitMPGrafLib
// Function Description	: Initilizes the MP GrafLib library with paramaters for specified
//						  display card
// Error Checking		: Checks to make sure library is uninitilized.
// Last Updated			: May 25, 1997
// Written By			: David Slik
// Modifications		:
// Notes				: Should be modified so multiple instances can operate concurrently
//------------------------------------------------------------------------------------------
OSErr InitMPGrafLib(Ptr BaseAddress, short BitDepth, short HRes, short VRes)
{
	// Initilization routine
	
	if(gDSGlobals.InitCheck != INITCHECK_VALUE)
	{
		// Clear Variables
		gDSGlobals.Overwrite = 0;
		gDSGlobals.DS_ForeColor = 0;
		gDSGlobals.DS_BackColor = 0;
		gDSGlobals.PenX = 0;
		gDSGlobals.PenY = 0;

		// Set paramaters
		gDSGlobals.DisplayBaseAddress = BaseAddress;
		gDSGlobals.DisplayBitDepth = BitDepth;
		gDSGlobals.HorizontalResolution = HRes;
		gDSGlobals.VerticalResolution = VRes;
		
		gDSGlobals.ClippingRect.top = 0;
		gDSGlobals.ClippingRect.right = HRes;
		gDSGlobals.ClippingRect.bottom = VRes;
		gDSGlobals.ClippingRect.left = 0;

		// Indicate that now initilized
		gDSGlobals.InitCheck = INITCHECK_VALUE;
		
		return(ERROR_NONE);
	}
	else
	{
		// Already Initilized, call terminate
		return(ERROR_IS_INIT);
	}
}

//------------------------------------------------------------------------------------------
// Thread Safe		: [x]	| Normal Build		: [x]	| Posts Events	: [ ]
// Reentrant		: [ ]	| Debug Build		: [ ]	| Pulls Events	: [ ]
// MP Compatable	: [x]	| Structure Checking: [ ]	+-----------------------------------
//--------------------------+---------------------------+ Performance Critical	: [ ]
// Allocates Memory	: [ ]	| Calls Functions	: [ ]	| Performance : L [x] M [ ] H [ ]
// Moves Memory		: [ ]	| Calls Toolbox		: [ ]	| Normal Overhead	: ???
// Uses TB Memory 	: [ ]	| Calls QuickDraw	: [ ]	| Debug Overhead	: ???
// Uses PAM Memory	: [ ]	| Async Callback	: [ ]	| MP Overhead		: ???
//------------------------------------------------------------------------------------------
// Function Name		: DeInitMPGrafLib
// Function Description	: Uninitilizes the MP GrafLib library
// Error Checking		: Checks to make sure library is initilized.
// Last Updated			: May 25, 1997
// Written By			: David Slik
// Modifications		:
// Notes				: Should be modified so multiple instances can operate concurrently
//------------------------------------------------------------------------------------------
OSErr DeInitMPGrafLib(void)
{
	// Deinitilization routine
	
	if(gDSGlobals.InitCheck == INITCHECK_VALUE)
	{
		// Clear Globals
		gDSGlobals.Overwrite = 0;
		gDSGlobals.InitCheck = 0;
		gDSGlobals.DisplayBaseAddress = NULL;
		gDSGlobals.DisplayBitDepth = 0;
		gDSGlobals.HorizontalResolution = 0;
		gDSGlobals.VerticalResolution = 0;
		gDSGlobals.ClippingRect.top = 0;
		gDSGlobals.ClippingRect.right = 0;
		gDSGlobals.ClippingRect.bottom = 0;
		gDSGlobals.ClippingRect.left = 0;
		gDSGlobals.DS_ForeColor = 0;
		gDSGlobals.DS_BackColor = 0;
		gDSGlobals.PenX = 0;
		gDSGlobals.PenY = 0;
				
		return(ERROR_NONE);
	}
	else
	{
		// Is not initilized
		return(ERROR_NOT_INIT);
	}
}

//------------------------------------------------------------------------------------------
// Thread Safe		: [x]	| Normal Build		: [x]	| Posts Events	: [ ]
// Reentrant		: [ ]	| Debug Build		: [ ]	| Pulls Events	: [ ]
// MP Compatable	: [x]	| Structure Checking: [ ]	+-----------------------------------
//--------------------------+---------------------------+ Performance Critical	: [ ]
// Allocates Memory	: [ ]	| Calls Functions	: [ ]	| Performance : L [x] M [ ] H [ ]
// Moves Memory		: [ ]	| Calls Toolbox		: [ ]	| Normal Overhead	: ???
// Uses TB Memory 	: [ ]	| Calls QuickDraw	: [ ]	| Debug Overhead	: ???
// Uses PAM Memory	: [ ]	| Async Callback	: [ ]	| MP Overhead		: ???
//------------------------------------------------------------------------------------------
// Function Name		: SetClippingRect
// Function Description	: Sets up a clipping rectangle
// Error Checking		: Checks to make sure library is initilized.
// Last Updated			: May 25, 1997
// Written By			: David Slik
// Modifications		:
// Notes				: Good for drawing into a window
//------------------------------------------------------------------------------------------
OSErr SetClippingRect(DS_Rect theClippingRect)
{
	if(DS_RectInRect(&gDSGlobals.ClippingRect, &theClippingRect))
	{
		gDSGlobals.ClippingRect = theClippingRect;
		return(ERROR_NONE);
	}
	else
	{
		return(ERROR_INVALID_CLIPPING);
	}
}