#include "MP GrafLib.h"
#include "MP GrafLib Support.h"
#include <MP.h>

// Prototypes
void main(void);
OSStatus DoLineTest(void *parameter);
void DisplayError(Str255 theErrorMessage);


// MacOS MP Main
void main(void)
{
	// Local Variables
	GrafParams			theScreen;
	WindowPtr			theWindow;
	Rect				WindowRect;
	OSStatus			status = 0;
	MPTaskID			task = NULL;
	MPQueueID			terminationQueue = NULL;
	EventRecord			theEvent;
	short				WindowPart;

	// Initilize the MacOS ToolBox
	InitGraf((Ptr) &qd.thePort);
	InitFonts();
	InitWindows();
	InitMenus();
	InitCursor();

	// Check to ensure MP is loaded
	if(!MPLibraryIsLoaded())
	{
		DisplayError("\pMultiprocessing API Library required to run this software.");
  		return;
	}

	// Create ToolBox Window
	SetRect(&WindowRect, 100, 100, 401, 401);
	theWindow = NewWindow(NULL, &WindowRect, "\p", TRUE, 2, (WindowPtr) -1, FALSE, NULL);
	
	// Get Display Information
	theScreen.BaseAddress = GetIndVideoBaseAddress(0);
	RetreiveVideoParameters(theScreen.BaseAddress, &theScreen.HRes, &theScreen.VRes, &theScreen.BitDepth);
	
	// Create a management Queue
	status = MPCreateQueue(&terminationQueue);

	// Create the task
	status = MPCreateTask(DoLineTest, (void *) &theScreen, kMPUseDefaultStackSize, terminationQueue, 0, 0, kMPNormalTaskOptions, &task);
	if(status != noErr)
	{
		DisplayError("\pCreation of MP Task Failed");
		return;
	}
	
	// Event Loop while drawing occures
	while(status == noErr)//Button() == FALSE)
	{
		status = MPWaitOnQueue(terminationQueue, 0, 0, 0, kDurationImmediate);
		if(status == kMPTimeoutErr) status = noErr;
		
		WaitNextEvent(everyEvent, &theEvent, 30, NULL);
		
		if(theEvent.what == keyDown)
		{
			return;
		}
		if(theEvent.what == mouseDown)									// If it was a mousedown event
		{
			WindowPart = FindWindow(theEvent.where, &theWindow);			// Find what window it is in
	 		if(WindowPart == inSysWindow)
	 		{
				SystemClick(&theEvent, theWindow);
			}
		}
	}

	// Clean up the window
	DisposeWindow(theWindow);
	MPDeleteQueue(terminationQueue);
}




// Generic Error Display Routine
void DisplayError(Str255 theErrorMessage)
{
	// Display Error Message
//	__Debugger(theErrorMessage);
}





static OSStatus LineTest(void *parameter) 
{
	// Do MP Stuff
	char*	MemoryReference = (char*) parameter;
	int		LoopCounter = 1024*768*2;
	int		LoopStop = 1024*400*2;
	
	int		ColourValue = 0;
	

	while(ColourValue != 10)
	{
		while(LoopCounter != LoopStop)
		{
			MemoryReference[LoopCounter] = ColourValue*25;//124 + LoopCounter/10000;
			LoopCounter--;
		}
		
		LoopCounter = 1024*768*2;
		ColourValue++;
	}

	return noErr;
}



