#include <stdio.h>
#include <Types.h>
#include <Memory.h>
#include <Quickdraw.h>
#include <QDOffscreen.h>
#include <Fonts.h>
#include <Events.h>
#include <Menus.h>
#include <Windows.h>
#include <TextEdit.h>
#include <Dialogs.h>
#include <OSUtils.h>
#include <ToolUtils.h>


/* Globals */
Rect				windRect;
GWorldPtr		testGWorld;
	
/* Prototypes */
void Initialize(void);

void TestDashedLines(Point *startPoint, Point *endPoint);

void ConvertPatternToPixPat(Pattern *thePat, RGBColor *foreColor, 
				    PixPatHandle newPixPat);
void TestColorXorPenMarquee(Point *startPoint, Point *endPoint, RGBColor *col, Boolean erasing);

void TestSpeedyPoly(void);
void TestPolyPoly(void);
void SetupTestGWorld();
void TestClipping(void);

// 
//	Main body of program QDGems - demonstrate all of the gems in sequence
//

void main(void)
{
	Initialize();
	
	// Load a bitmap in for use during testing.
	SetupTestGWorld();	

	// ---------------------------------------------------------
	// Gem #1 - Dashed line at any angle
	// ---------------------------------------------------------

	// First hint string
	MoveTo (30,30);
	TextSize(9);	
	DrawString("\pClick and drag to see a dotted line at any angle");

	// Wait for a mouse button click
	do {
	} while (!Button());
		
	// Draw a dashed line from the click position to the
	// current mouse position until the mouse is let go.
	Point startPt, endPt, lastEndPt;
	GetMouse(&startPt);
	lastEndPt.h = startPt.h;
	lastEndPt.v = startPt.v;
	PenNormal();
	do {
		GetMouse(&endPt);
		if ((lastEndPt.h != endPt.h) || (lastEndPt.v != endPt.v))
		{
			PenMode(patXor);
			
			// Xor in the new line
			TestDashedLines(&startPt, &endPt);

			// Xor the old line away
			TestDashedLines(&startPt, &lastEndPt);

			lastEndPt.h = endPt.h;
			lastEndPt.v = endPt.v;
		}
		
	} while (Button());

	PenNormal();

	// ---------------------------------------------------------
	// Gem #2 - Color Xor Pen (used in a marquee)
	// ---------------------------------------------------------

	// Second hint string
	Rect textRect = {0, 0, 40, 600};
	EraseRect(&textRect);
	MoveTo (30,30);
	TextSize(9);	
	DrawString("\pClick and drag to see color xor marquee");

	// Wait for a mouse button click
	do {
	} while (!Button());
	

	// First, draw the demo picture
	RGBColor	whiteColor = {0xFFFF, 0xFFFF, 0xFFFF};
	RGBColor	blackColor = {0, 0, 0};
	RGBForeColor (&blackColor);
	RGBBackColor (&whiteColor);
	Rect		pictRect = {100, 100, 350, 450};
	CopyBits ((BitMap *)&testGWorld->portPixMap,&qd.thePort->portBits, 
					  &testGWorld->portRect, &pictRect, srcCopy, NULL);

	GetMouse(&startPt);
	lastEndPt.h = startPt.h;
	lastEndPt.v = startPt.v;

	// Marquee color is pure red
	RGBColor	marqueeCol = {0xFFFF, 0, 0};
	long lastMarquee = TickCount();
	do {
		// Delay a bit between each frame (for better results, synch to VBL here)
		if (TickCount() > (lastMarquee + 1))
		{
			
			GetMouse(&endPt);

			// Xor in the new marquee
			TestColorXorPenMarquee(&startPt, &endPt, &marqueeCol, FALSE);

			// Xor the old marquee away
			TestColorXorPenMarquee(&startPt, &lastEndPt, &marqueeCol, TRUE);

			lastEndPt.h = endPt.h;
			lastEndPt.v = endPt.v;
			
			lastMarquee = TickCount();
		}		
	} while (Button());

	PenNormal();

	// ---------------------------------------------------------
	// Gem #3 - PolyPolygons
	// ---------------------------------------------------------

	// Third hint string
	EraseRect(&textRect);
	MoveTo (30,30);
	TextSize(9);	
	DrawString("\pClick and hold to see a PolyPolygon demonstration");


	// Wait for a mouse button click
	do {
	} while (!Button());
	
	// Get timing information while testing
	unsigned long pre_test_ticks = TickCount();
	unsigned long numPolygons = 0;
	do {
		numPolygons++;
		TestPolyPoly();
	} while (Button());

	unsigned long post_test_ticks = TickCount();

	double average_polys_per_sec = 60 * ((double)numPolygons / 
																 (double)(post_test_ticks - pre_test_ticks));
	
	char outBuf[100];
	sprintf(outBuf, "Average polys per second, Std QD: %f", average_polys_per_sec);
	c2pstr(outBuf);


	// ---------------------------------------------------------
	// Gem #4 - Speedy Polygons
	// ---------------------------------------------------------
	// Third hint string
	EraseRect(&qd.thePort->portRect);
	MoveTo (30,16);
	TextSize(9);	
	RGBForeColor (&blackColor);
	DrawString((StringPtr)outBuf);

	MoveTo (30,30);
	DrawString("\pClick and hold to see a Speedy PolyPolygon demonstration");

	// Wait for a mouse button click
	do {
	} while (!Button());
	
	// Get timing information while testing
	pre_test_ticks = TickCount();
	numPolygons = 0;
	do {
		numPolygons++;
		TestSpeedyPoly();
	} while (Button());

	post_test_ticks = TickCount();

	double average_speedypolys_per_sec = 60 * ((double)numPolygons / 
																 (double)(post_test_ticks - pre_test_ticks));
	
	char outBuf2[100];
	sprintf(outBuf2, "Average polys per second, Fast Polys: %f", average_speedypolys_per_sec);
	c2pstr(outBuf2);

	// ---------------------------------------------------------
	// Gem #5 - XOR Clipping
	// ---------------------------------------------------------

	PenNormal();

	// Fourth hint string
	EraseRect(&qd.thePort->portRect);
	MoveTo (30,16);
	TextSize(9);	
	RGBForeColor (&blackColor);
	DrawString((StringPtr)outBuf2);

	MoveTo (30,30);
	DrawString("\pClick and hold to see XOR clipping demonstration");

	// Wait for a mouse button click
	do {
	} while (!Button());
	

	do {
		TestClipping();
	} while (Button());

	// Fourth hint string
	EraseRect(&textRect);
	MoveTo (30,30);
	TextSize(9);	
	DrawString("\pClick to Quit");

	// Wait
	do {	
	}	while (!Button());	

	FlushEvents(everyEvent, 0);	
}

// 
//	Initialize everything for the program, make sure we can run
//

void Initialize(void)
{
	WindowPtr	mainPtr;
	OSErr		error;
	SysEnvRec	theWorld;
		
	//
	//	Test the computer to be sure we can do color.  
	//	If not we would crash, which would be bad.  
	//	If we cant run, just beep and exit.
	//

	error = SysEnvirons(1, &theWorld);
	if (theWorld.hasColorQD == false) {
		SysBeep(50);
		ExitToShell();					/* If no color QD, we must leave. */
	}
	
	/* Initialize all the needed managers. */
	InitGraf(&qd.thePort);
	InitFonts();
	InitWindows();
	InitMenus();
	TEInit();
	InitDialogs(nil);
	InitCursor();

	//
	//	To make the Random sequences truly random, we need to make the seed start
	//	at a different number.  An easy way to do this is to put the current time
	//	and date into the seed.  Since it is always incrementing the starting seed
	//	will always be different.  Dont for each call of Random, or the sequence
	//	will no longer be random.  Only needed once, here in the init.
	//
	GetDateTime((unsigned long*) &qd.randSeed);

	//
	//	Make a new window for drawing in, and it must be a color window.  
	//	The window is full screen size, made smaller to make it more visible.
	//
	windRect = qd.screenBits.bounds;
	InsetRect(&windRect, 50, 50);
	mainPtr = NewCWindow(nil, &windRect, "\pBob Land", true, documentProc, 
						(WindowPtr) -1, false, 0);
		
	SetPort(mainPtr);						/* set window to current graf port */
}


// Gem #1 - Dashed lines at any angle
void TestDashedLines(Point *startPoint, Point *endPoint)
{
	// A two-pattern dash table
	Pattern dashPatTable[2] = 
	{
		0xf8,0x7c,0x3e,0x1f,0x8f,0xc7,0xe3,0xf1,
		0x1f,0x3e,0x7c,0xf8,0xf1,0xe3,0xc7,0x8f
	};

	/*  Pattern 0			Pattern 1

			1111 1000			0001 1111
			0111 1100			0011 1110
			0011 1110			0111 1100
			0001 1111			1111 1000
			1000 1111			1111 0001
			1100 0111			1110 0011
			1110 0011			1100 0111
			1111 0001			1000 1111		
	*/


	// Select the approprate pattern based on the angle 
	// quadrant.
	if (startPoint->h < endPoint->h)
	{
		if (startPoint->v < endPoint->v)
			PenPat(&dashPatTable[1]);						// Lower Right Quadrant
		else
			PenPat(&dashPatTable[0]);						// Upper Right Quadrant
	}
	else
	{
		if (startPoint->v < endPoint->v)
			PenPat(&dashPatTable[0]);						// Lower Left Quadrant
		else
			PenPat(&dashPatTable[1]);						// Upper Left Quadrant
	}

	// Move to the start point
	MoveTo(startPoint->h, startPoint->v);

	// LineTo the end point	
	LineTo(endPoint->h, endPoint->v);

}

// Gem #2 - Colored XOr Pen
void TestColorXorPenMarquee(Point *startPoint, Point *endPoint, RGBColor *col, Boolean erasing)
{

	// a pattern for the ants
	static Pattern sAntPattern = { 0xf8,0x7c,0x3e,0x1f,0x8f,0xc7,0xe3,0xf1 };

	// If we're not erasing, shuffle the pattern by one
	if (!erasing)
	{
		unsigned char lastPart = sAntPattern.pat[7] ;
	
		for(short index = 7; index > 0; --index ) 
			sAntPattern.pat[index] = sAntPattern.pat[ index-1 ] ;

		sAntPattern.pat[0] = lastPart ;
	}
	else
	{
		// We're erasing, shuffle the pattern back by one (and 
		// later forwards again)
		unsigned char lastPart = sAntPattern.pat[0] ;
	
		for(short index = 0; index < 7; index++ ) 
			sAntPattern.pat[index] = sAntPattern.pat[ index+1 ] ;

		sAntPattern.pat[7] = lastPart ;
	}
	
	// Init the color pix pat
	PixPatHandle colorPixPat = NewPixPat();
	ConvertPatternToPixPat (&sAntPattern, col, colorPixPat);
	
	if (colorPixPat)
	{
		PenPixPat (colorPixPat);
		PenMode (patXor);
		
		MoveTo (startPoint->h,	startPoint->v);
		LineTo (startPoint->h,	endPoint->v);
		LineTo (endPoint->h, 	endPoint->v);
		LineTo (endPoint->h, 	startPoint->v);
		LineTo (startPoint->h, startPoint->v);
		
		PenNormal();
		DisposePixPat(colorPixPat);
	}

	// If we were erasing, shuffle the pattern forwards again
	if (erasing)
	{
		unsigned char lastPart = sAntPattern.pat[7] ;
	
		for(short index = 7; index > 0; --index ) 
			sAntPattern.pat[index] = sAntPattern.pat[ index-1 ] ;

		sAntPattern.pat[0] = lastPart ;
	}

}


// This function converts a classic QuickDraw B&W pattern into a 
// PixPat with the passed color set as the foreground color.  It
// assumes that the CGrafPort we're going to draw into is the current
// port.  You must allocate the new PixPat using QuickDraw's 
// ::NewPixPat() function before calling this routine.
// Note that once you have allocated the PixPat you are responsible for
// disposing of it (after QuickDraw is done using it, of course).

void ConvertPatternToPixPat(Pattern *thePat, RGBColor *foreColor, 
				    PixPatHandle newPixPat)
{
	// Set the B&W pattern as the Pen Pattern.  QuickDraw will create an 
	// equivalent PixPat and store it in the current port's pnPixPat field.
	::PenPat (thePat);

	// Copy QuickDraw's automatically generated PixPat into a new 
	// PixPatHandle.  Calling CopyPixPat does a deep copy on all the 
	// data associated with the PixPat.
	PixPatHandle curPixPat = ((CGrafPort *)qd.thePort)->pnPixPat;
	::CopyPixPat (curPixPat, newPixPat);
	
	// Now, we change the new PixPat's pattern type to make it a full
	// color PixPat instead of a simple B&W pattern.
	newPixPat[0]->patType = 1;

	// Now we modify the PixPat's color table to insert the foreground
	// and background colors.  We take the background color set in the
	// current port.
	CTabHandle theCTab = newPixPat[0]->patMap[0]->pmTable;
	(*theCTab)->ctTable[0].rgb = ((CGrafPort *)qd.thePort)->rgbBkColor;
	(*theCTab)->ctTable[1].rgb = *foreColor;

	// We now report to QuickDraw the fact that weve modified 
	// some of its exposed data structures so that QuickDraw can 
	// invalidate any information it has cached internally
	::CTabChanged (theCTab);
	::PixPatChanged (newPixPat);
}

// Gem #3 - PolyPolygons
void TestPolyPoly(void)
{
	RGBColor	polyColor;
	long int	newLeft, newTop;
	
	// 
	//	Make a random new color for the poly.
	//
	polyColor.red   = Random();
	polyColor.green = Random();
	polyColor.blue  = Random();
	
	// 
	//	Set that color as the new color to use in drawing.
	//
	RGBForeColor (&polyColor);

	//	
	//	Make a Random new location for the poly, that is normalized to the window size.  
	//	This makes the Integer from Random into a number that is 0..windRect.bottom
	//	and 0..windRect.right.  They are normalized so that we don't spend most of our
	//	time drawing in places outside of the window.
	//
	newTop = Random();	newLeft = Random();
	newTop = ((newTop+32767) * windRect.bottom)/65536;
	newLeft = ((newLeft+32767) * windRect.right)/65536;
	
	//
	//	Move pen to the new location, and paint the colored ball.
	//	
	PolyHandle testPoly = OpenPoly();
	
	MoveTo(newLeft, newTop);

	LineTo (newLeft + 100, newTop);
	LineTo (newLeft + 100, newTop + 100);
	LineTo (newLeft, newTop + 100);
	LineTo (newLeft, newTop);
	
	LineTo (newLeft + 10, newTop + 10);

	LineTo (newLeft + 90, newTop + 10);
	LineTo (newLeft + 90, newTop + 90);
	LineTo (newLeft + 10, newTop + 90);
	LineTo (newLeft + 10, newTop + 10);
	
	LineTo (newLeft, newTop);
	
	ClosePoly();
		
	PaintPoly (testPoly);
	KillPoly (testPoly);
	
}

// Gem #4 - Speedygons
void TestSpeedyPoly(void)
{
	RGBColor	polyColor;
	long int	newLeft, newTop;
	
	// 
	//	Make a random new color for the poly.
	//
	polyColor.red   = Random();
	polyColor.green = Random();
	polyColor.blue  = Random();
	
	// 
	//	Set that color as the new color to use in drawing.
	//
	RGBForeColor (&polyColor);

	//	
	//	Make a Random new location for the poly, that is normalized to the window size.  
	//	This makes the Integer from Random into a number that is 0..windRect.bottom
	//	and 0..windRect.right.  They are normalized so that we don't spend most of our
	//	time drawing in places outside of the window.
	//
	newTop = Random();	newLeft = Random();
	newTop = ((newTop+32767) * windRect.bottom)/65536;
	newLeft = ((newLeft+32767) * windRect.right)/65536;
	
	//
	//	Move pen to the new location, and paint the colored ball.
	//	
	static PolyHandle sTestPoly = NULL;
	
	if (!sTestPoly)
	{
		sTestPoly = (PolyHandle) NewHandle(10000);
		if (!sTestPoly)
			throw -1;

		HLock ((Handle)sTestPoly);		
	}
	
	// There are 10 points in the poly.  One is 
	// accounted for in the Polygon structure itself.
	sTestPoly[0]->polySize = sizeof(Polygon) + 9 * sizeof(Point);
	
	// We already know the bounding box. It's easy to calculate
	// if you don't already know it. 
	sTestPoly[0]->polyBBox.top = newTop;
	sTestPoly[0]->polyBBox.bottom = newTop+100;
	sTestPoly[0]->polyBBox.left = newLeft;
	sTestPoly[0]->polyBBox.right = newLeft+100;

	// Now we fill in the poly points, starting with the 
	// initial MoveTo command. Compare to Gem #3
	sTestPoly[0]->polyPoints[0].h = newLeft;
	sTestPoly[0]->polyPoints[0].v = newTop;

	sTestPoly[0]->polyPoints[1].h = newLeft + 100;
	sTestPoly[0]->polyPoints[1].v = newTop;
	sTestPoly[0]->polyPoints[2].h = newLeft + 100;
	sTestPoly[0]->polyPoints[2].v = newTop + 100;
	sTestPoly[0]->polyPoints[3].h = newLeft;
	sTestPoly[0]->polyPoints[3].v = newTop + 100;
	sTestPoly[0]->polyPoints[4].h = newLeft;
	sTestPoly[0]->polyPoints[4].v = newTop;
	
	sTestPoly[0]->polyPoints[5].h = newLeft + 10;
	sTestPoly[0]->polyPoints[5].v = newTop + 10;
	
	sTestPoly[0]->polyPoints[6].h = newLeft + 90;
	sTestPoly[0]->polyPoints[6].v = newTop + 10;
	sTestPoly[0]->polyPoints[7].h = newLeft + 90;
	sTestPoly[0]->polyPoints[7].v = newTop + 90;
	sTestPoly[0]->polyPoints[8].h = newLeft + 10;
	sTestPoly[0]->polyPoints[8].v = newTop + 90;
	sTestPoly[0]->polyPoints[9].h = newLeft + 10;
	sTestPoly[0]->polyPoints[9].v = newTop + 10;

	sTestPoly[0]->polyPoints[10].h = newLeft;
	sTestPoly[0]->polyPoints[10].v = newTop;
		
	PaintPoly (sTestPoly);
	
}


void SetupTestGWorld()
{
	// Create a new GWorld to hold the test picture
	Rect		pictRect = {0, 0, 250, 350};
	if (NewGWorld (&testGWorld, 0, &pictRect, NULL, NULL, 0) != noErr)
		throw -1;
	
	// Save the old GWorld
	GWorldPtr		currGW;
	GDHandle		currGDH;
	
	GetGWorld (&currGW, &currGDH);

	// Set the new GWorld and draw the picture into it
	SetGWorld (testGWorld, NULL);
	
	PicHandle thePict = GetPicture (128);
	DrawPicture (thePict, &pictRect);
	KillPicture (thePict);
	
	// Restore the GWorld
	SetGWorld (currGW, currGDH);	
}

// Gem #5 - Xor Clipping
void TestClipping(void)
{
	RGBColor	whiteColor = {0xFFFF, 0xFFFF, 0xFFFF};
	RGBColor	blackColor = {0, 0, 0};
	long int	newLeft, newTop;
	Rect			pictRect;
	

	//	
	//	Make a Random new location for the poly, that is normalized to the window size.  
	//	This makes the Integer from Random into a number that is 0..windRect.bottom
	//	and 0..windRect.right.  They are normalized so that we don't spend most of our
	//	time drawing in places outside of the window.
	//
	newTop = Random();	newLeft = Random();
	newTop = ((newTop+32767) * windRect.bottom)/65536;
	newLeft = ((newLeft+32767) * windRect.right)/65536;

	// Set a location for the pict to draw.
	SetRect(&pictRect, newLeft, newTop, newLeft+350, newTop+250);
	
	// Copybits the picture onto the screen in srcXor mode
	RGBForeColor (&blackColor);
	RGBBackColor (&whiteColor);
	CopyBits ((BitMap *)&testGWorld->portPixMap,&qd.thePort->portBits, 
					  &testGWorld->portRect, &pictRect, srcXor, NULL);
	
	
	// Move to the origin for the clipping text (halfway up the picture)
	MoveTo(pictRect.left, pictRect.top + 125);

	// Find the xor base color (assume we're always on the main device - a real 
	// implementation needs to cycle through all monitor's GDevices)
	GDHandle	curGDH = GetMainDevice();

	// Default to white (xor base color for truecolor devices)	
	RGBColor xorbaseColor = {0xffff, 0xffff, 0xffff};

	// Check if the pixmap is paletted
	if (curGDH[0]->gdPMap[0]->pixelSize <= 8)
	{
		// This is a paletted pixmap - use color table index zero as the 
		// xor base color.  (Wow - count the dereferences!)
		xorbaseColor.red = curGDH[0]->gdPMap[0]->pmTable[0]->ctTable[0].rgb.red;
		xorbaseColor.green = curGDH[0]->gdPMap[0]->pmTable[0]->ctTable[0].rgb.green;
		xorbaseColor.blue = curGDH[0]->gdPMap[0]->pmTable[0]->ctTable[0].rgb.blue;
	}	
	
	// Set the xor base color as the fg color and draw in patCopy mode
	RGBForeColor(&xorbaseColor);
	PenNormal();

	TextSize (72);
	DrawString("\pClip Me");

	// Draw the picture again.
	RGBForeColor (&blackColor);
	RGBBackColor (&whiteColor);
	CopyBits ((BitMap *)&testGWorld->portPixMap,&qd.thePort->portBits, 
					  &testGWorld->portRect, &pictRect, srcXor, NULL);
}


