#include "Hydra.h"

///////////////////////////////////////////////////////////////
//	Hydra classes are Copyright 1997 by Rainer Brockerhoff.
//
//	Submitted as part of the paper :
//	"On Having Two (or More) Heads, or : Real-time Programming in Spite of
//		the MacOS", for MacHack 1997
//
//	You may use these classes in your application at no cost;
//	however, the application's "About" box should repeat my
//	copyright line as above. Be sure to write my name correctly ;-)
//	I'd also appreciate getting a demo copy of your application.
//	These classes have not been exhaustively tested in this form
//	and all use is at your own risk.
//
//	Contact me at <rainer@machome.com.br> or
//	http://www.machome.com.br/delta/
//
//	Stylistic note :
//	Astute older colleagues will no doubt deduce that in my misspent youth I:
//	1) learned FORTRAN	(short variable names; local variables called i,j,k)
//	2) learned PL/I		(using }; everywhere)
//	3) used punch cards (using {} even when not needed, to more easily insert
//		printf debug lines)
//	4) had too little contact with other programmers and so failed to learn
//		proper indentation, naming or bracing standards.
//	They're probably right on all counts... :-)
//
//	I'm assuming CodeWarrior 12 (or Pro 1) is being used.
//
//	This source file is overcommented, for pedagogical purposes only.

Hydra::Storage Hydra::sHydraStorage;		// this is my static storage,
											// used internally by all
											// Hydra classes.
											
///////////////////////////////////////////////////////////////
//	HydraApplication
//
//	This subclasses LApplication.
//
//	Although we really need to do this just to be able to override the
//	application's LMenuBar::MenuCommandSelection() function  and even this
//	might be handled by a tail patch to ::MenuSelect()  it's a convenient
//	way to hide Hydra's initialization stuff.
//	Notice that no LYieldAttachment is necessary (and might get in the way)!
//
/////////////////////////////
//	HydraApplication : Constructor
//
//	This does the necessary initialization for Hydra.
//
HydraApplication::HydraApplication():LApplication() {
	mEventMask = everyEvent;

	RegisterClass_(HydraWindow);	// this is the new registration macro, so
									// you don't have to register HydraWindow
									// in your main program.

	UScreenPort::Initialize();		// we'll use UScreenPort later, so might
									// as well do this now

	SetSleepTime(1);				// give as little time as possible to
									// background apps; but don't set it to 0,
									// or you will block faceless background
									// processes.

	Hydra::Initialize(this);		// this initializes the Hydra class itself
									// this MUST be called before using any
									// of the Hydra functions.
};

/////////////////////////////
//	HydraApplication : Destructor
//
//	This just tells Hydra to shut down.
//
HydraApplication::~HydraApplication() {
	Hydra::ShutDown();
};

/////////////////////////////
//	HydraApplication : ProcessNextEvent (overrides LApplication)
//
//	Here we try to regulate how often ::WaitNextEvent is called. If you have
//	Alessandro Levi Montalcini's excellent "MenuBall" control panel (which
//	is strongly recommended to any real-time application developer!) you'll
//	notice that the ball now will bounce slowly but regularly all the time;
//	none of that stopping or speeding-up you'll see with standard applications.
//	This regularity also applies to your cursor updates, idler and repeater
//	functions and those are now called even while you're inside a menu or
//	window drag!
//	Remember all UI processing must be done here or our algorithm will become
//	useless.
//
void HydraApplication::ProcessNextEvent() {
	EventRecord macEvent;
	UInt32 now;
	Boolean	gotEvent;
	now = Hydra::MilliSeconds();		// remember when we were called

	mEventMask = everyEvent;			// all events are accepted

	if ((Hydra::sHydraStorage.sMainTime<now)// check if it's that time again
		||::EventAvail(everyEvent,&macEvent)) {// or an event is enqueued

										// set to next time
		Hydra::sHydraStorage.sMainTime = now+Hydra::sHydraStorage.sDelay;

		if (IsOnDuty()) {				// if the application is on duty
										// (and it usually should be!)

			::OSEventAvail(0,&macEvent);// get mouse coordinates
			AdjustCursor(macEvent);		// and redraw the cursor if needed
		};

										// get an event and let background
										// applications run (a little)
		gotEvent = ::WaitNextEvent(everyEvent,&macEvent,mSleepTime,mMouseRgnH);

										// give attachments a shot at the
										// event first
		if (LAttachable::ExecuteAttachments(msg_Event,&macEvent)) {

										// ok, we can process the event

			if (gotEvent) {				// was it a null event?

				DispatchEvent(macEvent);// no, dispatch it
			} else {
				UseIdleTime(macEvent);	// yes, run idlers
			};
		};

		LPeriodical::DevoteTimeToRepeaters(macEvent);	// run repeaters

										// check if menus need to be updated
		if (IsOnDuty()&&GetUpdateCommandStatus()) {

			UpdateMenus();				// yes, do so
			SetUpdateCommandStatus(false);// invalidate immediately
		};
	};

	{									// actually we could just call
										// Hydra::Yield() here but this
										// saves some time

		Hydra saver;					// save the GrafPort on the stack

		LThread::Yield();				// yield to other threads
	};
};

/////////////////////////////
//	HydraApplication : TryToDispatch
//
//	This is called from Hydra::Yield() only, if it was called from somewhere
//	inside the main thread (not including ProcessNextEvent()).
//	This will happen if any event handler wishes to yield to other threads;
//	if you leave everything to PowerPlant this will happen only inside our
//	::WaitMouseUp() and ::StillDown() patches.
//
void HydraApplication::TryToDispatch() {
	EventRecord macEvent;
	UInt32 now;
	if (mEventMask!=everyEvent) {		// this will be true if we're called
										// by the event dispatcher; look at
										// DispatchEvent() for mask handling

		now = Hydra::MilliSeconds();	// remember when we were called

		if ((Hydra::sHydraStorage.sMainTime<now)// if it's that time again
			||::EventAvail(mEventMask,&macEvent)) {// or an event is enqueued

										// set to next time
			Hydra::sHydraStorage.sMainTime = now+Hydra::sHydraStorage.sDelay;

										// if it wasn't an null event...
			if (::WaitNextEvent(mEventMask,&macEvent,mSleepTime,mMouseRgnH)) {

				if (LAttachable::ExecuteAttachments(msg_Event,&macEvent)) {

					DispatchEvent(macEvent);// dispatch it
				};
			};
		};
	};
};

/////////////////////////////
//	HydraApplication : DispatchEvent (overrides LEventDispatcher)
//
//	We call the standard event dispatcher with a small difference : we
//	avoid dispatching the same event type more than once; otherwise we
//	may get a stack overflow (or mess up non-reentrant event handlers)
//	if many events are in the queue.
//	Some of this may be overzealous, but better safe than sorry!
//	If you call ::Alert() (or one of its variations), or the Standard
//	File Package, this may not work correctly. Try saving the event mask
//	and setting it to "everyEvent" before opening a standard dialog.
//
void HydraApplication::DispatchEvent(const EventRecord &inMacEvent) {

	EventMask om = mEventMask;			// save old mask

	mEventMask &= ~((1<<nullEvent)|		// always mask null event
		(inMacEvent.what==kHighLevelEvent?highLevelEventMask:// special case
		(1<<inMacEvent.what)));			// mask out this same event type

	LEventDispatcher::DispatchEvent(inMacEvent);// dispatch the event

	mEventMask = om;					// restore old mask
};

/////////////////////////////
//	HydraApplication : MakeMenuBar (overrides LApplication)
//
//	Just create our own standard menu bar.
//
void HydraApplication::MakeMenuBar() {
	new HydraMenuBar(MBAR_Initial);
};

///////////////////////////////////////////////////////////////
//	HydraApplication::HydraMenuBar
//
//	This subclasses LMenuBar.
//
//	We need to do this just to be able to call Hydra::RedrawUnderRegion()
//	immediately after the ::MenuSelect() call.
//
/////////////////////////////
//	HydraApplication::HydraMenuBar : Constructor
//
//	This does the necessary initialization for our local menu bar.
//
HydraApplication::HydraMenuBar::HydraMenuBar(ResIDT inMBARid)
	:LMenuBar(inMBARid){
};

/////////////////////////////
//	HydraApplication::HydraMenuBar : Destructor
//
//	Nothing special happens here.
//
HydraApplication::HydraMenuBar::~HydraMenuBar() {
};

/////////////////////////////
//	HydraApplication::HydraMenuBar : MenuCommandSelection (overrides LMenuBar)
//
//	This is the original body of LMenuBar::MenuCommandSelection() except for
//	one line where we redraw the screen region under the menus after the menu
//	snaps back up.
//	This region is calculated by the MBarHook procedure which we install in
//	Hydra::Initialize().
//	Unfortunately this just covers regular menus; hierarchical and pop-up
//	menus don't call the MBarHook procedure (they should!).
//	Note that keeping threads running while ::MenuSelect() is executing is
//	handled transparently by patching ::StillDown() and ::WaitMouseUp().
//
CommandT HydraApplication::HydraMenuBar::MenuCommandSelection
	(const EventRecord &inMouseEvent,Int32 &outMenuChoice) const {
	Int32 menuCmd = cmd_Nothing;
	outMenuChoice = ::MenuSelect(inMouseEvent.where);	// do menu selection
										// this will call our MBarHook!

	Hydra::RedrawUnderRegion();			// redraw under the menu

	if (HiWord(outMenuChoice)!=0) {		// user selected an item
										// find the command
		menuCmd = FindCommand(HiWord(outMenuChoice),LoWord(outMenuChoice));
	};
	return menuCmd;						// return the command code
};

///////////////////////////////////////////////////////////////
//	HydraWindow
//
//	This subclasses LWindow.
//
//	Here we implement live dragging and growing, and build a list
//	of our windows.
//
/////////////////////////////
//	HydraWindow : Static constructor
//
//	May be called to create a standard HydraWindow.
//
//	This is just copied over from LWindow::CreateWindow().
//
HydraWindow* HydraWindow::CreateWindow(ResIDT inWindowID,
	LCommander* inSuperCommander) {
	SetDefaultCommander(inSuperCommander);
	SetDefaultAttachable(nil);
	HydraWindow* theWindow = reinterpret_cast<HydraWindow*>
		(UReanimator::ReadObjects('PPob',inWindowID));
	theWindow->FinishCreate();
	if (theWindow->HasAttribute(windAttr_ShowNew)) {
		theWindow->Show();
	};
	return theWindow;
};

/////////////////////////////
//	HydraWindow : Constructors
//
//	These are just copied over from LWindow's standard constructors and
//	in fact call these before calling Hydra::RememberWindow(this) to
//	insert the window into our local window list.
//
HydraWindow::HydraWindow():LWindow() {
	Hydra::RememberWindow(this);
};

HydraWindow::HydraWindow(const SWindowInfo &inWindowInfo)
	:LWindow(inWindowInfo) {
	Hydra::RememberWindow(this);
};

HydraWindow::HydraWindow(ResIDT inWINDid,Uint32 inAttributes
	,LCommander* inSuper):LWindow(inWINDid,inAttributes,inSuper) {
	Hydra::RememberWindow(this);
};

HydraWindow::HydraWindow(LStream* inStream):LWindow(inStream) {
	Hydra::RememberWindow(this);
};

/////////////////////////////
//	HydraWindow : Destructor
//
//	This just removes the window from our local window list.
//
HydraWindow::~HydraWindow() {
	Hydra::ForgetWindow(this);
};

/////////////////////////////
//	HydraWindow : ClickInDrag (overrides LWindow)
//
//	This allows "live" window dragging. This is quite CPU-intensive!
//	Note that we don't send AppleEvents out here (for script recorders); this
//	probably will things down too much if you do it inside the dragging loop
//	and not be representative of what happens if you do it at the end. You're
//	free to re-insert this if you wish to, of course...
//
void HydraWindow::ClickInDrag(const EventRecord &inMacEvent) {
	GrafPtr wr;
	RgnHandle sr;
	Point op,np;
	short dh,dv;

	if (!(inMacEvent.modifiers&cmdKey)) {
		UDesktop::SelectDeskWindow(this);		// select the window only if
	};											// if the command key is up

	if (::WaitMouseUp()) {						// be sure the mouse is still
												// down; else the user has
												// just clicked the window's
												// title bar to select it
												
		wr = GetMacPort();						// this is both the window's
												// GrafPort and its WindowRec

		sr = ::NewRgn();						// create working region
		
		op = inMacEvent.where;					// original mouse-down location
												// in global coordinates

		while (::StillDown()) {					// this is the dragging loop

			::SetPort(UScreenPort::GetScreenPort());// this makes ::GetMouse()
												// return global coordinates 

			::GetMouse(&np);					// see where the mouse is now

			dh = np.h-op.h;						// calculate how much it moved
			dv = np.v-op.v;						// since last time around

			if ((dh!=0)||(dv!=0)) {				// if the mouse moved at all...
			
				::CopyRgn(((CWindowPeek)wr)->strucRgn,sr);// copy the
												// window's region before
												// moving it

				MoveWindowBy(dh,dv);			// move the window to follow
												// the mouse

												// copy the window's structure
												// region's boundary

				::DiffRgn(sr,((CWindowPeek)wr)->strucRgn,sr);// calculate
												// the region uncovered by
												// the window's move

				Hydra::RedrawUnderRegion(sr);	// redraw the uncovered
												// region immediately

				op = np;						// new location becomes old
				
			};
		};

		::DisposeRgn(sr);						// dispose of working region
	};
};

/////////////////////////////
//	HydraWindow : ClickInGrow (overrides LWindow)
//
//	This allows "live" window growing. This is very CPU-intensive!
//	Note that we don't send AppleEvents out here (for script recorders); this
//	probably will things down too much if you do it inside the dragging loop
//	and not be representative of what happens if you do it at the end. You're
//	free to re-insert this if you wish to, of course...
//
void HydraWindow::ClickInGrow(const EventRecord &inMacEvent) {
	WindowPtr wr;
	RgnHandle sr;
	Point op,np;
	short dh,dv,d;

	if (::WaitMouseUp()) {						// be sure the mouse is still
												// down; else the user has
												// just clicked the window's
												// grow box to select it

		wr = GetMacPort();						// this is both the window's
												// GrafPort and its WindowRec

		op = inMacEvent.where;					// original mouse-down location
												// in global coordinates

		sr = ::NewRgn();						// create working region used
												// inside the loop

		while (::StillDown()) {					// this is the dragging loop

			::SetPort(UScreenPort::GetScreenPort());// this makes ::GetMouse()
												// return global coordinates 

			::GetMouse(&np);					// see where the mouse is now

			dh = np.h-op.h;						// calculate how much it moved
			dv = np.v-op.v;						// since last time around

			d = mFrameSize.width+dh;			// calculate the new width

			if (d<mMinMaxSize.left) {			// pin to minimum width
				dh = mMinMaxSize.left-mFrameSize.width;
				np.h = op.h+dh;					// limit mouse movement
			};

			if (d>mMinMaxSize.right) {			// pin to maximum width
				dh = mMinMaxSize.right-mFrameSize.width;
				np.h = op.h+dh;					// limit mouse movement
			};

			d = mFrameSize.height+dv;			// calculate the new height

			if (d<mMinMaxSize.top) {			// pin to minimum height
				dv = mMinMaxSize.top-mFrameSize.height;
				np.v = op.v+dv;					// limit mouse movement
			};

			if (d>mMinMaxSize.bottom) {			// pin to maximum height
				dv = mMinMaxSize.bottom-mFrameSize.height;
				np.v = op.v+dv;					// limit mouse movement
			};

			if ((dh!=0)||(dv!=0)) {				// if the mouse moved at all...

				::CopyRgn(((CWindowPeek)wr)->strucRgn,sr);// copy the
												// window's region before
												// resizing it

				ResizeWindowBy(dh,dv);			// resize the window
				
												// copy the window's structure
												// region's boundary

				::DiffRgn(sr,((CWindowPeek)wr)->strucRgn,sr);	// calculate
												// the region uncovered by
												// the window's size change

				Hydra::RedrawUnderRegion(sr);	// redraw the uncovered
												// region immediately

				op = np;						// new location becomes old

			};
		};

		::DisposeRgn(sr);						// dispose of working region
	};
};

/////////////////////////////
//	HydraWindow : FocusDraw (overrides LWindow [actually LView])
//
//	This is needed if the window itself is drawn to - usually happens only
//	if you set the "Erase on Update" option.
//
Boolean HydraWindow::FocusDraw(LPane* inSubPane) {
	Boolean focused;
	focused = LWindow::FocusDraw(inSubPane);	// do actual focusing

	if (focused) {								// if a GrafPort is available...

		Hydra::ShieldMenu(this);				// protect menu from window
												// we pass "this" since the
												// window has no SuperView
	};
	return focused;
};

///////////////////////////////////////////////////////////////
//	Hydra
//
//	This class has two purposes :
//	1) It provides a stack-based GrafPort saver (like StGrafPortSaver) but
//	it additionally may clear the LView cache on destruction.
//	2) It provides static storage and utilities for Hydra functions.
//
/////////////////////////////
//	Hydra : Constructor
//
//	Use this as a stack-based object to save and restore the GrafPort.
//
Hydra::Hydra() {
	::GetPort(&mGrafPtr);			// save the current GrafPort
};

/////////////////////////////
//	Hydra : Destructor
//
//	Restores the GrafPort and optionally clears the LView cache.
//
Hydra::~Hydra() {
	Reset();
};

/////////////////////////////
//	Hydra : Reset
//
//	Restores the GrafPort and optionally clears the LView cache.
//	This may be called to temporarily restore the GrafPort saved on the stack.
//
void Hydra::Reset() {
	GrafPtr np;
	::GetPort(&np);					// get the current GrafPort

	if (np!=mGrafPtr) {				// check if it has changed since saving

		::SetPort(mGrafPtr);		// if yes, both restore the GrafPort
		LView::OutOfFocus(NULL);	// and clear the LView cache
	};
};

/////////////////////////////
//	Hydra : Initialize
//
//	This is called only once (by the HydraApplication constructor).
//	Here we create the main thread (as well as a dummy thread),
//	initialize our static storage, flags and so forth, and do the
//	necessary hooking and patching.
//
void Hydra::Initialize(HydraApplication* inApp) {
	{								// this looks very strange but we
									// need to get to PowerPlant's static
									// storage for putting in our own
									// drag tracker.

		Hydra::HydraDragAndDrop initialize;	// initialize drag and drop
	};								// destroy dummy object immediately

	new UMainThread;				// create the main thread
									// call LThread::GetMainThread() to
									// recover its address if necessary

	::Microseconds(reinterpret_cast<UnsignedWide*>	// store the current
		(&sHydraStorage.sStartedAt));	// time (used in MilliSeconds())

	sHydraStorage.sTheApp = inApp;	// point at the current application

	sHydraStorage.sMenuRegion = ::NewRgn();	// create the menu region

	SetWNEFrequency(10);			// default frequency for calling
									// ::WaitNextEvent()

	sHydraStorage.sMainTime = MilliSeconds();	// initialize the timer for
												// scheduling main thread

	sHydraStorage.sInWaitMouseUp = false;	// we're not in ::WaitMouseUp()

	sHydraStorage.sMBarHookUPP = NewMBarHookProc(MBarHook);	// create UPP
											// to out MBarHook procedure

	::LMSetMBarHook(sHydraStorage.sMBarHookUPP);// hook in to our procedure

									// store old ::StillDown() address
	sHydraStorage.sOldStillDown = ::GetToolTrapAddress(_StillDown);

									// patch in our ::StillDown() procedure
	::SetToolTrapAddress(NewRoutineDescriptor((ProcPtr)(StillDownPatch),
		kPascalStackBased|RESULT_SIZE(SIZE_CODE(sizeof(Boolean))),
		GetCurrentArchitecture()),_StillDown);

									// store old ::WaitMouseUp() address
	sHydraStorage.sOldWaitMouseUp = ::GetToolTrapAddress(_WaitMouseUp);

									// patch in our ::WaitMouseUp() procedure
::SetToolTrapAddress(NewRoutineDescriptor((ProcPtr)(StillDownPatch),
		kPascalStackBased|RESULT_SIZE(SIZE_CODE(sizeof(Boolean))),
		GetCurrentArchitecture()),_WaitMouseUp);

};

/////////////////////////////
//	Hydra : ShutDown
//
//	This is called only once (by the HydraApplication destructor).
//
void Hydra::ShutDown() {
	::LMSetMBarHook(NULL);			// clear the MBarHook (not really needed)
};

/////////////////////////////
//	Hydra : SetWNEFrequency
//
//	Call this to determine how many times per second you want ::WaitNextEvent()
//	to be called. Any value between 1 and 100 is acceptable; invalid values
//	cause the default of 10 Hz to be set.
//	This frequency is not strictly conformed to, but rather is converted to
//	the delay which we try to observe between ::WaitNextEvent() calls.
//
void Hydra::SetWNEFrequency(short perSecond) {
	sHydraStorage.sDelay = oneSecond/((perSecond<1)
		||(perSecond>100)?10:perSecond);
};

/////////////////////////////
//	Hydra : Yield
//
//	Call this anytime you wish to yield CPU time. Specify a parameter if
//	you wish to favor a specific thread.
//	Here we take the necessary steps to ensure the main thread is activated
//	often enough. If no thread is ready to run this just returns with
//	minimum overhead.
//	This function MUST be called instead of LThread::Yield().
//
void Hydra::Yield(const LThread* yieldTo) {
	Hydra saver;					// save the GrafPort on the stack
//	You may want to insert a custom scheduler here. Basically you should do it
//	like this :
//	if (yieldTo==NULL) {
//		...	loop over your threads in some way and select the one most needful
//			of CPU time...
//		yieldTo = thatThread;
//	};
//	This will have the side effect of not letting your main thread hog too much
//	time if you're dragging a window s-l-o-w-l-y around the screen, for instance.

	LThread::Yield(yieldTo);		// yield to the selected or main thread

	if (LThread::InMainThread()) {	// if called by main thread
		sHydraStorage.sTheApp->TryToDispatch();
	};
									// go on with our current thread
};

/////////////////////////////
//	Hydra : Sleep
//
//	Call this to put the calling thread to sleep for the specified time
//	(in milliseconds); the actual sleep time may be longer.
//	This function MUST be called instead of LThread::Sleep().
//
void Hydra::Sleep(Int32 milliSeconds) {
	if (milliSeconds<=0) {			// negative or zero sleep time??
		Hydra::Yield();				// if so, just blink (yield once)

	} else if (LThread::InMainThread()) {// if main thread wants to sleep
									// this is different; we don't let it
									// sleep, but just delay the next
									// ::WaitNextEvent() call
		sHydraStorage.sMainTime = MilliSeconds()+milliSeconds;

		do {
			Hydra::Yield();				// now let others run
		} while (MilliSeconds()<=sHydraStorage.sMainTime);

	} else {						// so we're not called by the main thread

		Hydra saver;				// save the GrafPort on the stack
									// while we're sleeping

		LThread::GetCurrentThread()->Sleep(milliSeconds);// let thread sleep
									// for the specified time
	};
									// go on with our current thread
};

/////////////////////////////
//	Hydra : MilliSeconds
//
//	Returns a millisecond counter. Zero is the time when Hydra::Initialize()
//	was called, so we return (roughly) the application's startup time.
//	We subtract the startup time because at some future time the value
//	might get too large for a UInt32.
//
UInt32 Hydra::MilliSeconds() {
	unsigned long long microsegs;
	::Microseconds(reinterpret_cast<UnsignedWide*>(&microsegs));
	return (microsegs-sHydraStorage.sStartedAt)/oneSecond;
};

/////////////////////////////
//	Hydra : RememberWindow
//
//	Called only by the HydraWindow constructors, to insert the window into
//	our private window list.
//
void Hydra::RememberWindow(HydraWindow* theWindow) {
	WindowPtr gp;
	gp = theWindow->GetMacPort();
	sHydraStorage.sHydraWindows.InsertItemsAt
		(1,LArray::index_Last,gp);
};

/////////////////////////////
//	Hydra : ForgetWindow
//
//	Called only by the HydraWindow destructors, to remove the window from
//	our private window list.
//
void Hydra::ForgetWindow(HydraWindow* theWindow) {
	WindowPtr gp;
	gp = theWindow->GetMacPort();
	sHydraStorage.sHydraWindows.Remove(gp);
};

/////////////////////////////
//	Hydra : GetMenuRegion
//
//	Call this if for some weird reason you need to access the menu region
//	directly - don't alter it, though.
//
RgnHandle Hydra::GetMenuRegion() {
	return sHydraStorage.sMenuRegion;
};

/////////////////////////////
//	Hydra : ShieldMenu
//
//	This function sets the view's clipping region to conform to the
//	current menu state. Notice that this may become invalid as soon as
//	you call Hydra::Yield() [or Hydra::Sleep()].
//	A list of shielded Grafports is maintained for later use
//	by RedrawUnderRegion().
//	For every pane that is updated by a real-time thread, you need to
//	call Hydra::ShieldMenu(mSuperView) before starting to draw to the pane.
//	You should do this at the start of the pane's DrawSelf() function, and
//	between FocusDraw() and Hydra::Yield() [or Hydra::Sleep()] from the
//	thread that updates the pane.
//	Alternatively you can override FocusDraw() for each pane; see
//	HydraWindow::FocusDraw() above for an example.
//	If you're calling this from a view, pass "this" instead of mSuperView.
//	We return true if a menu was down, just in case you'd like to know.
//
Boolean Hydra::ShieldMenu(LView* theView) {
	Rect rr;
	RgnHandle mr;
	WindowPtr tw;
	ArrayIndexT inlist;
	Boolean result=false;
	if (theView) {						// just make sure it's not NULL

		tw = theView->GetMacPort();		// now get the GrafPort (and window)

		if (tw) {						// more paranoid testing

			result = !::EmptyRgn(sHydraStorage.sMenuRegion);// check menu

			inlist = sHydraStorage.sWindows.FetchIndexOf(tw);// check if in list

			if (result) {							// if a menu is down...

				mr = ::NewRgn();					// make a copy of the menu
				::CopyRgn(sHydraStorage.sMenuRegion,mr);// region but in
				Point pt={0,0};						// local coordinates
				::LocalToGlobal(&pt);				// to match the clip region
				::OffsetRgn(mr,-pt.h,-pt.v);

				::DiffRgn(tw->clipRgn,mr,tw->clipRgn);// punch menu out from
													// clip region

				::DisposeRgn(mr);					// dispose of working region

				if (inlist==LArray::index_Bad) {	// if we're not in the list,
													// put us in
					sHydraStorage.sWindows.InsertItemsAt(1,LArray::index_Last,tw);
				};

			} else {								// no menu down;

				theView->GetRevealedRect(rr);		// so just clip to
				theView->PortToLocalPoint(topLeft(rr));// the view's revealed
				theView->PortToLocalPoint(botRight(rr));// rectangle
				::ClipRect(&rr);

				if (inlist!=LArray::index_Bad) {	// if we're in the list,
													// take us out
					sHydraStorage.sWindows.RemoveItemsAt(1,inlist);
				};
			};
		};
	};
	return result;							// tell caller if a menu was down
};

/////////////////////////////
//	Hydra : RedrawUnderRegion
//
//	Call this to immediately redraw Hydra windows under a region. The effect
//	is slightly different depending on whether the parameter region is the
//	menu region or not.
//	If the parameter is the menu region (or omitted or NULL), all windows that
//	were redrawn under a menu are updated, and the menu region is cleared;
//	this is a little obscure but prevents some code duplication elsewhere.
//	If the parameter is not the menu region all HydraWindows intersected by
//	the region are redrawn, and the region is not altered.
//	Note that the parameter region always refers to global coordinates.
//
void Hydra::RedrawUnderRegion(RgnHandle theRgn) {
	Hydra saver;					// save the GrafPort on the stack
	WindowPtr wp;
	LWindow* tw;
	RgnHandle lh;
	RgnHandle cr;
	RgnHandle rh;
	short pw;
	Boolean wasmr=false;
	if (theRgn==NULL) {
		theRgn = sHydraStorage.sMenuRegion;	// default is menu region
	};

	lh = theRgn;					// make a local copy of the region handle

	if (lh==sHydraStorage.sMenuRegion) {// are we redrawing under the menu?

		sHydraStorage.sMenuRegion = ::NewRgn();// make a new empty menu region

		::InsetRgn(lh,-3,-3);		// outset the menu region; this is needed
									// because the system saves a larger Rect
									// than that occupied by the menu, probably
									// because of the menu's shadow.

		wasmr = true;				// remember that the old menu region's
									// handle is held in lh; this is the last
									// known copy of the handle

	};
	if (!::EmptyRgn(lh)) {			// do the rest only if region non-empty

		cr = ::NewRgn();			// make a working region for the loop

									// iterate over the list of shielded
									// shielded GrafPorts (if menu region)
									// or over the list of HydraWindows (if not)
		TArrayIterator<WindowPtr> iterator
			(wasmr?sHydraStorage.sWindows:sHydraStorage.sHydraWindows);

		while (iterator.Next(wp)) {	// for each GrafPort, check and redraw

			tw = LWindow::FetchWindowObject(wp);// get PowerPlant window

			if (tw->IsVisible()) {	// do visible windows only, of course

				rh = ((CWindowPeek)wp)->strucRgn;// get the window's
									// structure region; remember
									// this includes the frame, title etc.

				::SectRgn(lh,rh,cr);// calculate the overlap between the
									// window's structure region and the
									// region to be redrawn

				if (!::EmptyRgn(cr)) {// if non-empty, redraw the window
									// since normal window updates don't
									// redraw the window's frame, we have
									// do it explicitly here

					pw = ::LMGetPaintWhite();	// save current value of
					::LMSetPaintWhite(0);		// PaintWhite (low-mem field)
									// and clear it to prevent erasure of
									// the window's contents by ::PaintOne()
									// this also prevents ::PaintOne() to add
									// the region to the window's update
									// region.

					::PaintOne(wp,rh);	// repaint the affected region of
										// the window (including frame, etc.)

					::LMSetPaintWhite(pw);// restore previous value of
										// PaintWhite; it's usually set

					::SetPort(wp);	// sets the window's port

					Point pt={0,0};				// now we need to relocate
					::LocalToGlobal(&pt);		// the affected region to
					::OffsetRgn(cr,-pt.h,-pt.v);// local coordinates

					tw->ValidPortRgn(cr);// subtract from the update region
									// since we won't need an update event

					tw->Draw(cr);	// redraw under the update region

				};

			};
		};
		::DisposeRgn(cr);			// dispose of working region
	};

	if (wasmr) {					// if we updated under the menu region
		::DisposeRgn(lh);			// dispose of its former handle
	};
};

/////////////////////////////
//	Hydra : MBarHook
//
//	This is our MBarHook procedure. It sets the menu region to the area
//	under the menu being drawn. If a previous menu's redraw was pending,
//	we do that first.
//	Remember this is called by ::MenuSelect() after the menu title is
//	highlighted, but before the menu is drawn. The parameter Rect is the
//	Rect the menu will be drawn in without considering the menu's frame or
//	its shadow.
//	Unfortunately the Menu Manager does not call this with an empty Rect
//	if the user drags to the empty space on the menu bar; in this case,
//	the old menu's region will not be updated at all until the user releases
//	the mouse or goes back to a menu title.
//	When Apple corrects this bug, we'll already be prepared for it.
//
pascal short Hydra::MBarHook(Rect* menuRect) {
	Rect wr;
	RgnHandle rgn;
	RedrawUnderRegion();			// if a previous menu was
									// on screen, this redraws under it

	rgn = ::NewRgn();				// make a working region

	wr = *menuRect;					// make a local copy of the menu Rect

	::RectRgn(rgn,&wr);				// make a region out of the Rect
	
	if (!::EmptyRgn(rgn)) {			// if the menu's Rect wasn't empty...

		::OffsetRgn(rgn,2,2);			// offset region 2 pixels to make
										// the shadow

		::InsetRect(&wr,-1,-1);			// grow the Rect by 1 pixel to include
										// the menu frame

		::RectRgn(sHydraStorage.sMenuRegion,&wr);// create the menu region
										// and add the shadow region to it
		::UnionRgn(rgn,sHydraStorage.sMenuRegion,sHydraStorage.sMenuRegion);
	};

	::DisposeRgn(rgn);				// dispose of working region

	return 0;						// must return 0, else ::MenuSelect()
									// will be aborted
};

/////////////////////////////
//	Hydra : StillDownPatch
//
//	This is our patch to ::StillDown().
//	We call the original procedure and then yield to other threads, unless
//	we were called from inside ::WaitMouseUp().
//	This test is necessary because otherwise we would yield twice, and the
//	yield inside this procedure would invalidate the event queue look-ahead
//	done inside ::WaitMouseUp(), with often fatal consequences.
//	Also see the comments for Hydra::WaitMouseUpPatch(), below.
//
pascal Boolean Hydra::StillDownPatch() {
	Boolean result;
									// call original procedure
	result = CallUniversalProc(sHydraStorage.sOldStillDown,
		kPascalStackBased|RESULT_SIZE(SIZE_CODE(sizeof(Boolean))));

	if (!sHydraStorage.sInWaitMouseUp) {// if not inside ::WaitMouseUp()
		Hydra::Yield();				// then yield to other threads
	};

	return result;					// return original result
};

/////////////////////////////
//	Hydra : WaitMouseUpPatch
//
//	This is our patch to ::WaitMouseUp().
//	We call the original procedure and then yield to other threads.
//	A flag is set around the original call for test by our ::StillDown()
//	patch; see the comments above.
//	Both patches are tail patches, which are considered non-kosher on the 68K
//	architecture; on the 68K we'd also need to save and set A4 with the
//	proper calls.
//	We have to use tail patches because both traps look ahead into the
//	event queue and then act on the information obtained; however,
//	Yield() also does this. If we used head patches, now and then the timing
//	might be such that Yield() could swallow an event from the queue after
//	the traps became sure there is such an event; when they'd act on the
//	false information afterwards, the system could hang or crash.
//
pascal Boolean Hydra::WaitMouseUpPatch() {
	Boolean result;
	sHydraStorage.sInWaitMouseUp = true;// set flag for StillDownPatch()

									// call original procedure
	result = CallUniversalProc(sHydraStorage.sOldWaitMouseUp,
		kPascalStackBased|RESULT_SIZE(SIZE_CODE(sizeof(Boolean))));

	sHydraStorage.sInWaitMouseUp = false;// clear flag for StillDownPatch()

	Hydra::Yield();					// yield to other threads

	return result;					// return original result
};

///////////////////////////////////////////////////////////////
//	Hydra::HydraDragAndDrop
//
//	This subclasses LDropArea.
//
//	This class transparently handles drag and drop.
//	We initialize PowerPlant's LDropArea stuff (that's why we have to
//	subclass LDropArea) and then install our own tracker (ditto).
//
/////////////////////////////
//	Hydra::HydraDragAndDrop : Constructor
//
//	This is called just once to install our custom tracking handler.
//
Hydra::HydraDragAndDrop::HydraDragAndDrop(WindowPtr theWindow):
	LDropArea(theWindow) {

	sHydraStorage.sOldDragTracking = sDragTrackingProc;	// save PowerPlant's handler

	sDragTrackingProc = NewDragTrackingHandlerProc(DragTracking);// point at ours

	::InstallTrackingHandler(sDragTrackingProc,NULL,NULL);// and install it
};

/////////////////////////////
//	Hydra::HydraDragAndDrop : Destructor
//
//	Boring...
//
Hydra::HydraDragAndDrop::~HydraDragAndDrop() {
};

/////////////////////////////
//	Hydra::HydraDragAndDrop::PointInDropArea (overrides LDropArea)
//
//	Dummy, just so LDropArea doesn't complain.
//
Boolean Hydra::HydraDragAndDrop::PointInDropArea(Point /**/) {
	return false;
};

/////////////////////////////
//	Hydra::HydraDragAndDrop::DragTracking
//
//	We just call PowerPlant's tracking procedure and then yield to others.
//	Unfortunately this fails if the user drags to another application's
//	windows, and there seems to be no easy way around it...
//
pascal OSErr Hydra::HydraDragAndDrop::DragTracking(DragTrackingMessage inMessage,WindowPtr inMacWindow,void* inRefCon,DragReference inDragRef) {
	OSErr err;

	err = CallUniversalProc(sHydraStorage.sOldDragTracking,// do tracking
		kPascalStackBased|RESULT_SIZE(SIZE_CODE(sizeof(OSErr)))
		|STACK_ROUTINE_PARAMETER(1,SIZE_CODE(sizeof(DragTrackingMessage)))
		|STACK_ROUTINE_PARAMETER(2,SIZE_CODE(sizeof(WindowPtr)))
		|STACK_ROUTINE_PARAMETER(3,SIZE_CODE(sizeof(void*)))
		|STACK_ROUTINE_PARAMETER(4,SIZE_CODE(sizeof(DragReference)))
		,inMessage,inMacWindow,inRefCon,inDragRef);

	Hydra::Yield();					// let others run during drag

	return err;						// return error
};

///////////////////////////////////////////////////////////////
//	end of Hydra.cp

