#include <LTextEditView.h>
#include <LActiveScroller.h>
#include "Hydra.h"
#include "HydraExample.h"

///////////////////////////////////////////////////////////////
//	Hydra classes are Copyright 1997 by Rainer Brockerhoff.
//
//	Submitted as part of the paper :
//	"On Having Two (or More) Heads, or : Real-time Programming in Spite of
//		the MacOS", for MacHack 1997
//
//	This is an example application for illustrating use of the Hydra classes.
//	It seems to run OK on any PowerMac under System 7.5 or later with standard
//	extensions and control panels.
//	It may crash if you open too many windows or have stuff installed which messes
//	with the same system resources that I do. See "Hydra.cp" and "Hydra.h" for
//	the full story.
//	This example uses the least error checking it can get away with.
//
//	You may use the Hydra classes in your application at no cost;
//	however, the application's "About" box should repeat my
//	copyright line as above. Be sure to write my name correctly ;-)
//	I'd also appreciate getting a demo copy of your application.
//	These classes have not been exhaustively tested in this form
//	and all use is at your own risk.
//
//	Contact me at <rainer@machome.com.br> or
//	http://www.machome.com.br/delta/
//
//	Stylistic note :
//	Astute older colleagues will no doubt deduce that in my misspent youth I:
//	1) learned FORTRAN	(short variable names; local variables called i,j,k)
//	2) learned PL/I		(using }; everywhere)
//	3) used punch cards (using {} even when not needed, to more easily insert
//		printf debug lines)
//	4) had too little contact with other programmers and so failed to learn
//		proper indentation, naming or bracing standards.
//	They're probably right on all counts... :-)
//
//	I'm assuming CodeWarrior 12 (or Pro 1) is being used.
//
//	This source file is overcommented, for pedagogical purposes only.

LWindow* about=NULL;						// global which points at the "About" window.
HydraWindow* paper=NULL;					// global which points at the "Paper" window.

/////////////////////////////
//	main
//
//	This is our modest but hard-working main() procedure.
//	Notice that, although this is a multi-threaded application, no special
//	initialization (especially of the main thread) needs to be done.
//	We don't need a LYieldAttachment, either.
//
void main(void) {
	SetDebugThrow_(debugAction_Alert);		// setup debugging stuff
	SetDebugSignal_(debugAction_Alert);
	InitializeHeap(3);						// standard heap initialization
	UQDGlobals::InitializeToolbox(&qd);		// standard toolbox initialization
	::LMSetRndSeed(::TickCount());			// randomizes the randomizer
	ExampleApp theApp;						// creates our application object
	theApp.Run();							// and runs it
};											// couldn't be simpler...

///////////////////////////////////////////////////////////////
//	ExampleApp
//
//	This subclasses HydraApplication.
//	We just add the necessary stuff for demonstrating the Hydra classes.
//
/////////////////////////////
//	ExampleApp : Constructor
//
//	All necessary stuff is done inside HydraApplication.
//	We just register the classes we use for the example... open "Hydra.ppob" for
//	details. Some resources have been stashed there to avoid having a separate
//	.rsrc file, so you may want to use both Constructor and ResEdit.
//	Notice the new-style RegisterClass_ macro, and also that the
//	HydraWindow class is registered inside HydraApplication.
//	
ExampleApp::ExampleApp():HydraApplication() {
	RegisterClass_(LWindow);
	RegisterClass_(LPicture);
	RegisterClass_(LDialogBox);
	RegisterClass_(LStdButton);
	RegisterClass_(LActiveScroller);
	RegisterClass_(LTextEditView);
	RegisterClass_(ExampleWindow);
};

/////////////////////////////
//	ExampleApp : Destructor
//
//	Nothing exciting happens here.
//
ExampleApp::~ExampleApp() {
};

/////////////////////////////
//	ExampleApp : Initialize (overrides LApplication)
//
//	This is called after the application is all set up, but just before
//	starting the main event loop.
//	Here we just create our windows.
//
void ExampleApp::Initialize() {
	paper = HydraWindow::CreateWindow(130,this);// create the "Paper" window
												// it's initially invisible

	about = LWindow::CreateWindow(129,this);	// create the "About" windows
												// it's initially visible

	UReanimator::LinkListenerToControls(this,about,129);// listen to its
												// "Continue" button

	ExampleWindow::CreateWindow(lines,this);	// create a "lines" example window

	ExampleWindow::CreateWindow(rects,this);	// create a "rects" example window
};

/////////////////////////////
//	ExampleApp : ListenToMessage (overrides LListener)
//
//	This just catches the message from the "About" window's "Continue" button.
//
void ExampleApp::ListenToMessage(MessageT inMessage,void* /*ioParam*/) {
	if (inMessage==1) {							// check for the button's ID

		about->Hide();							// hide "About" window

		Hydra::RedrawUnderRegion();				// immediately redraw screen below the window

		paper->Show();							// be sure to show the "Paper" window
		paper->Select();						// and put it on top
	};
};

/////////////////////////////
//	ExampleApp : ObeyCommand (overrides LApplication)
//
//	This is a plain-vanilla implementation of ObeyCommand().
//
Boolean ExampleApp::ObeyCommand(CommandT inCommand,void* ioParam) {
	Boolean cmdHandled=true;					// default is always true

	switch (inCommand) {						// handle specific commands

	case cmd_About:
		about->Show();							// show the "About" window
		break;

	case cmd_New:								// create a new random example window
		ExampleWindow::CreateWindow(randomInRange(lines,rects),this);	
		break;
	
	case cmd_Close:								// close our topmost window
		LWindow* tw;
		tw = UDesktop::FetchTopRegular();		// get topmost window
		if (tw) {
			tw->AttemptClose();					// if there is one, try to close it
		};
		break;

	default:									// otherwise do default handling
		cmdHandled = LApplication::ObeyCommand(inCommand,ioParam);
		break;
	};

	return cmdHandled;							// return and tell if we handled it or not
};

/////////////////////////////
//	ExampleApp : FindCommandStatus (overrides LApplication)
//
//	This is a plain-vanilla implementation of FindCommandStatus().
//
void ExampleApp::FindCommandStatus(CommandT inCommand,Boolean &outEnabled,
	Boolean &outUsesMark,Char16 &outMark,Str255 outName) {
	switch (inCommand) {						// handle specific commands

	case cmd_Close:
	case cmd_New:
		outEnabled = true;						// enable "Close" and "New" commands
		break;

	default:									// handle the rest by default
		LApplication::FindCommandStatus(inCommand,outEnabled,outUsesMark,outMark,outName);
		break;
	};
};

///////////////////////////////////////////////////////////////
//	ExampleWindow
//
//	This subclasses HydraWindow.
//	Here, too, we just need to do the special handling for updating the window.
//
/////////////////////////////
//	ExampleWindow : Static constructor
//
//	May be called to create an ExampleWindow.
//
//	This is just copied over from LWindow::CreateWindow(), except that the first
//	parameter is not the window's 'PPob' resource ID, but the window's type.
//
ExampleWindow* ExampleWindow::CreateWindow(short type,LCommander* inSuperCommander) {
	SetDefaultCommander(inSuperCommander);
	SetDefaultAttachable(nil);
	ExampleWindow* theWindow = reinterpret_cast<ExampleWindow*>
		(UReanimator::ReadObjects('PPob',128));		// <== note fixed ResID
	theWindow->SetPaneID(type);						// store the type in the pane ID
	theWindow->FinishCreate();
	if (theWindow->HasAttribute(windAttr_ShowNew)) {
		theWindow->Show();
	};
	return theWindow;
};

/////////////////////////////
//	ExampleWindow : Constructor
//
//	We just clear our local variables; the rest is handled by HydraWindow.
//
ExampleWindow::ExampleWindow(LStream* inStream):HydraWindow(inStream) {
	mThread = NULL;
	mGWorld = NULL;
};

/////////////////////////////
//	ExampleWindow : Destructor
//
//	We stop the window's thread and deallocate the local LGWorld;
//	the rest is handled by HydraWindow.
//
ExampleWindow::~ExampleWindow() {
	if (mThread) {								// just make sure we have a thread
		mThread->DeleteThread();				// and kill it
	};

	delete mGWorld;								// get rid of the LGWorld, too
};

/////////////////////////////
//	ExampleWindow : FinishCreateSelf (overrides HydraWindow [actually LWindow])
//
//	We setup our local variables, including the LGWorld and start the window's thread.
//
void ExampleWindow::FinishCreateSelf() {
	CalcLocalFrameRect(mFrame);					// store the window frame
												// notice it's not resizeable!

	mX = 0;										// set some reasonable default values
	mY = 0;										// and startup values
	mHSV.hue = 0;
	mHSV.saturation = 65535;
	mHSV.value = 32767;
	mState = 0;	
	mLastTime = Hydra::MilliSeconds();

	mGWorld = new LGWorld(mFrame,0);			// create a local LGWorld with the same
												// frame as our window, and the current
												// screen's bit depth

												// create a simple thread for updating
												// the window
	mThread = new LSimpleThread(&windowThread,this,false,32768);

	if (mThread) {								// if it was created, start running; 
		mThread->Resume();						// this will call windowThread()
	};
};

/////////////////////////////
//	ExampleWindow : DrawLines
//
//	Draws a colored line first directly on the screen and then into the LGWorld.
//	The line's color (and some coordinates) are passed implicitly in local variables.
//
void ExampleWindow::DrawLines(short x,short y) {
	RGBColor rgb;
	::HSV2RGB(&mHSV,&rgb);						// convert color to RGB

	Hydra::ShieldMenu(this);					// protect the menus
	// since we draw directly to the window, we don't need to call Hydra::ShieldMenu()

	::RGBForeColor(&rgb);						// draw line in this color

	::MoveTo(x,y);								// draw line from the given point
	::LineTo(mX,mY);							// to the center point

	mGWorld->BeginDrawing();					// now do the same inside the LGWorld

	::RGBForeColor(&rgb);						// draw line in this color

	::MoveTo(x,y);								// draw line from the given point
	::LineTo(mX,mY);							// to the center point

	mGWorld->EndDrawing();						// stop using the LGWorld; this restores
												// your previous GrafPort, too!

};

/////////////////////////////
//	ExampleWindow : DrawRects
//
//	Draws a colored rectangle first directly on the screen and then into the LGWorld.
//	The line's color is passed implicitly in local variables.
//
void ExampleWindow::DrawRects() {
	Rect fr;
	RGBColor rgb;
		
	::HSV2RGB(&mHSV,&rgb);						// convert color to RGB

												// generate a random rectangle
	fr.top = randomInRange(mFrame.top,mFrame.bottom);
	fr.bottom = randomInRange(fr.top,mFrame.bottom);
	fr.left = randomInRange(mFrame.left,mFrame.right);
	fr.right = randomInRange(fr.left,mFrame.right);

	Hydra::ShieldMenu(this);					// protect the menus

	::RGBBackColor(&rgb);						// draw rectangle in this color

	::EraseRect(&fr);							// draw the rectangle

	mGWorld->BeginDrawing();					// now do the same inside the LGWorld

	::RGBBackColor(&rgb);						// draw rectangle in this color

	::EraseRect(&fr);							// draw the rectangle

	mGWorld->EndDrawing();						// stop using the LGWorld
	
};

/////////////////////////////
//	ExampleWindow : DoUpdate
//
//	This is called repeatedly by the window's thread, to simulate a real-time
//	process, and calls the drawing routine appropriate to the window's type.
//	There's a simple state machine for the 'lines' window.
//
void ExampleWindow::DoUpdate() {
	UInt32 delta;
									// calculate how far we're behind
									// with updating
	delta = (Hydra::MilliSeconds()-mLastTime)/(1000/updatesPerSecond);

	if (delta>updatesPerSecond) {	// don't try to catch up if we're more
		delta = 2*updatesPerSecond;	// than two seconds behind
	};

	for (;delta>0;--delta) {		// loop over pending updates
	
		switch (GetPaneID()) {		// switch on the window's type

		case lines:					// this is a "lines" window

			switch (mState) {						// simple state machine
													// for 'lines' window
			case 0:
				mHSV.value = randomInRange(10000,65535);	// calculate a random color brightness
				mHueDelta = 1<<randomInRange(3,12);	// calculate a random exponential step
													// used later to increment the hue

				mStep = 1;							// fixed step;
//				mStep = randomInRange(1,8);			// try a random step later

													// calculate random center point
				mX = randomInRange(mFrame.left+10,mFrame.right-10);
				mY = randomInRange(mFrame.top+10,mFrame.bottom-10);
				mState = 1;							// setup for state 1
				mIndex = mFrame.left;
													// fall through into state 1
			case 1:
				DrawLines(mIndex,mFrame.top);		// draw one line

				mHSV.hue += mHueDelta;				// increment hue

				mIndex += mStep;					// step to next point

				if (mIndex>=mFrame.right) {			// if we're at the right
					mIndex = mFrame.top;			// edge, switch state
					mState = 2;
				};
				break;

			case 2:
				DrawLines(mFrame.right,mIndex);		// draw one line

				mHSV.hue += mHueDelta;				// increment hue

				mIndex += mStep;					// step to next point

				if (mIndex>=mFrame.bottom) {		// if we're at the bottom
					mIndex = mFrame.right-1;		// edge, switch state
					mState = 3;
				};
				break;

			case 3:
				DrawLines(mIndex,mFrame.bottom);	// draw one line

				mHSV.hue += mHueDelta;				// increment hue

				mIndex -= mStep;					// step to next point

				if (mIndex<mFrame.left) {			// if we're at the left
					mIndex = mFrame.bottom-1;		// edge, switch state
					mState = 4;
				};
				break;
			case 4:
				DrawLines(mFrame.left,mIndex);		// draw one line

				mHSV.hue += mHueDelta;				// increment hue

				mIndex -= mStep;					// step to next point

				if (mIndex<mFrame.top) {			// if we're at the top
					mState = 0;						// edge, switch state
				};
				break;
			};
			break;

		case rects:								// this is a "rects" window

			mHSV.value = randomInRange(10000,65535);// calculate random color brightness
			mHSV.hue = randomInRange(0,65535);		// calculate random hue

			DrawRects();							// draw one rectangle
			break;
		};
	};
};

/////////////////////////////
//	ExampleWindow : DrawSelf (overrides HydraWindow [actually LWindow])
//
//	This simply redraws the window from the image stored in the LGWorld, which is
//	always kept up-to-date by the window's thread.
//
void ExampleWindow::DrawSelf() {
	static RGBColor black={0,0,0};
	static RGBColor white={65535,65535,65535};
	::RGBForeColor(&black);						// set foreground/background
	::RGBBackColor(&white);						// so LGWorld::CopyImage() works correctly

												// copy the image to the screen
	mGWorld->CopyImage(GetMacPort(),mFrame,srcCopy,NULL);
};

/////////////////////////////
//	ExampleWindow : ClickSelf (overrides HydraWindow [actually LPane])
//
//	We show a quick&dirty way of dragging the window's contents to the desktop.
//	PowerPlant handles all of it.
//
void ExampleWindow::ClickSelf(const SMouseDownEvent& inMouseDown) {
	PicHandle ph;
	LDragTask* ldt;
	if (::WaitMouseMoved(inMouseDown.macEvent.where)) {// if mouse has been dragged...

		FocusDraw();							// set to our GrafPort

		ph = ::OpenPicture(&mFrame);			// open a picture (72 dpi)

		DrawSelf();								// draw the window image into the picture

		::ClosePicture();						// close and lock the picture
		::HLock((Handle)ph);

												// let PowerPlant do the drag&drop
		ldt = new LDragTask(inMouseDown.macEvent,mFrame,
			0,'PICT',*ph,::GetHandleSize((Handle)ph),0);
		delete ldt;

		::HUnlock((Handle)ph);					// get rid of the picture
		::KillPicture(ph);
	};
};

/////////////////////////////
//	windowThread (global function)
//
//	This loops forever calling the window's update procedure, to simulate
//	a real-time process; in the real world this would gather data from an
//	external stream or process.
//
void windowThread(LThread& /*thread*/,void* arg) {
	ExampleWindow* window;
	window = reinterpret_cast<ExampleWindow*>(arg);// our owner window

	window->FocusDraw();						// set up the window's GrafPort
												// you just need to do this once
												// this call (which was protected)
												// is published by HydraWindow

	while (true) {								// loop forever
		window->DoUpdate();						// updating the window
		Hydra::Yield();							// yield after each update
	};
};

/////////////////////////////
//	randomInRange (global function)
//
//	This is a "shim" function to generate random numbers inside a certain linear range.
//
long randomInRange(long from,long to) {
	return to<from?to:((::Random()+32768)*(to+1-from)>>16)+from;
};

///////////////////////////////////////////////////////////////
//	end of HydraExample.cp

