/****************************************************************************************
*	AndTheBandPlayedOn.c
*
*	Copyright  1997 United Software Express. All rights reserved.
*
*	Commenter	Date				Comment
*	---------	-----------------	-----------------------------------------------------
*	wolfie		Sun, May 25, 1997	Created.
*	
****************************************************************************************/

#include	"XThreads.h"
#include	"ThreadedIO.h"

/*
 *
 *	Funky Protos
 *
 */

void	Foo();
OSErr	SynchronousModemReset();
OSErr	AsynchronousModemReset();
OSErr	IdealThreadedModemReset();
OSErr	WolfieThreadedModemReset();

/**************************
 *
 *	main
 *
 */

void	main()
{
	OSErr	err = InitXThreads();
	
	if( !err )
		err = WolfieThreadedModemReset();
	if( err )
		SysBeep( 20 );
}


/**************************
 *
 *	Foo -- just testing.
 *
 */

void	Foo()
{
	;
}

/**************************
 *
 *	SynchronousModemReset
 *
 */

OSErr    SynchronousModemReset()
{
    Str255   resetCmd = "\pATZ\r";
    short    inRefNum = 0, outRefNum = 0;
    long     count = resetCmd[ 0 ];
    OSErr    err;
    
    /*    Attempt to open the modem serial port */
    err = OpenDriver( "\p.AOut", &outRefNum );
    if( !err )
        err = OpenDriver( "\p.AIn", &inRefNum );
    
    /*    Write the modem reset command using the synchronous model */
    if( !err )
        err = FSWrite( outRefNum, &count, resetCmd + 1 );
    
    /*    Call the test function */
    if( !err )
        Foo();
    
    /*    If we successfully opened the modem serial port, close it now */
    if( inRefNum ) {
        (void) CloseDriver( inRefNum );
        inRefNum = 0;
    }
    if( outRefNum ) {
        (void) CloseDriver( outRefNum );
        outRefNum = 0;
    }
    
    return( err );
}

/**************************
 *
 *	AsynchronousModemReset
 *
 */

OSErr    AsynchronousModemReset()
{
    Str255           resetCmd = "\pATZ\r";
    short            inRefNum = 0, outRefNum = 0;
    ParamBlockRec    pb;
    OSErr            err;
    
    /*    Attempt to open the modem serial port */
    err = OpenDriver( "\p.AOut", &outRefNum );
    if( !err )
        err = OpenDriver( "\p.AIn", &inRefNum );
    
    /*    Write the modem reset command using the asynchronous model */
    if( !err ) {
        pb.ioParam.ioCompletion = nil;
        pb.ioParam.ioRefNum = outRefNum;
        pb.ioParam.ioBuffer = (Ptr) resetCmd + 1;
        pb.ioParam.ioReqCount = resetCmd[ 0 ];
        pb.ioParam.ioPosMode = fsFromStart;
        pb.ioParam.ioPosOffset = 0;
        
        err = PBWriteAsync( &pb );
    }
    
    /*    Call the test function */
    if( !err )
        Foo();
    
    /*    Wait until the asynchronous job completes */
    if( !err )
        while( pb.ioParam.ioResult > noErr ) {}
    
    /*    If we successfully opened the modem serial port, close it now */
    if( inRefNum ) {
        (void) CloseDriver( inRefNum );
        inRefNum = 0;
    }
    if( outRefNum ) {
        (void) CloseDriver( outRefNum );
        outRefNum = 0;
    }
    
    return( err );
}

/**************************
 *
 *	IdealThreadedModemReset		WARNING	Don't use this function, it demonstrates
 *										the "window of death."
 */

OSErr    IdealThreadedModemReset()
{
    Str255           resetCmd = "\pATZ\r";
    short            inRefNum = 0, outRefNum = 0;
    ParamBlockRec    pb;
    OSErr            err;
    
    /*    Attempt to open the modem serial port */
    err = OpenDriver( "\p.AOut", &outRefNum );
    if( !err )
        err = OpenDriver( "\p.AIn", &inRefNum );
    
    /*    Write the modem reset command using the asynchronous model */
    if( !err ) {
        pb.ioParam.ioCompletion = nil;// NewIOCompletionProc( WakeUpCompletionRoutine );
        pb.ioParam.ioRefNum = outRefNum;
        pb.ioParam.ioBuffer = (Ptr) resetCmd + 1;
        pb.ioParam.ioReqCount = resetCmd[ 0 ];
        pb.ioParam.ioPosMode = fsFromStart;
        pb.ioParam.ioPosOffset = 0;
        
        err = PBWriteAsync( &pb );
    }
    
    /*    Sleep until WakeUpCompletionRoutine fires and wakes us up */
    if( !err )
        SetThreadState( kCurrentThreadID, kStoppedThreadState, kNoThreadID );
    
    /*    Call the test function */
    if( !err )
        Foo();
    
    /*    If we successfully opened the modem serial port, close it now */
    if( inRefNum ) {
        (void) CloseDriver( inRefNum );
        inRefNum = 0;
    }
    if( outRefNum ) {
        (void) CloseDriver( outRefNum );
        outRefNum = 0;
    }
    
    return( err );
}

/**************************
 *
 *	WolfieThreadedModemReset
 *
 */

OSErr	WolfieThreadedModemReset()
{
	Str255           resetCmd = "\pATZ\r";
    short            inRefNum = 0, outRefNum = 0;
    long             size = resetCmd[ 0 ];
    OSErr            err;
    
    /*    Attempt to open the modem serial port */
    err = OpenDriver( "\p.AOut", &outRefNum );
    if( !err )
        err = OpenDriver( "\p.AIn", &inRefNum );
    
    /*    Write the modem reset command using the threaded IO model */
    if( !err ) {
    	err = ThreadedWrite( outRefNum, resetCmd + 1, &size, 0, 5000 );
    }
    
    /*    If we successfully opened the modem serial port, close it now */
    if( inRefNum ) {
        (void) CloseDriver( inRefNum );
        inRefNum = 0;
    }
    if( outRefNum ) {
        (void) CloseDriver( outRefNum );
        outRefNum = 0;
    }
    
    return( err );
}