/****************************************************************************************
*	ThreadedIO.c
*
*	Copyright  1997 United Software Express. All rights reserved.
*
*	Commenter	Date				Comment
*	---------	-----------------	-----------------------------------------------------
*	wolfie		Sun, May 25, 1997	Created.
*	
****************************************************************************************/

#include	"ThreadedIO.h"
#include	"XThreads.h"
#include	"68kRegGlue.h"
#include	"Timer.h"

/****************************************************************************************
*	
*	Types
*	
****************************************************************************************/
#pragma mark	--Types--

typedef struct {
	QElemPtr						qLink;
	short							qType;
	TimerUPP						tmAddr;
	long							tmCount;
	long							tmWakeUp;
	long							tmReserved;
	long							tmStoredA5;
	short							tmRefNum;
}	XTM, *XTMPtr;

typedef struct {
	QElemPtr						qLink;
	short							qType;
	short							ioTrap;
	Ptr								ioCmdAddr;
	IOCompletionUPP					ioCompletion;
	OSErr							ioResult;
	StringPtr						ioNamePtr;
	short							ioVRefNum;
	short							ioRefNum;
	SInt8							ioVersNum;
	SInt8							ioPermssn;
	Ptr								ioMisc;
	Ptr								ioBuffer;
	long							ioReqCount;
	long							ioActCount;
	short							ioPosMode;
	long							ioPosOffset;
	XTM								tm;
	ThreadID						ioThread;
}	XIO, *XIOPtr;

typedef struct {
	QElemPtr						qLink;
	short							qType;
	short							ioTrap;
	Ptr								ioCmdAddr;
	IOCompletionUPP					ioCompletion;
	OSErr							ioResult;
	StringPtr						ioNamePtr;
	short							ioVRefNum;
	short							ioCRefNum;
	short							csCode;
	short							csParam[11];
	XTM								tm;
	ThreadID						ioThread;
}	XCS, *XCSPtr;

/****************************************************************************************
*	
*	Globals
*	
****************************************************************************************/
#pragma mark	--Globals--

IOCompletionUPP	gIOProc = nil;
TimerUPP		gTMProc = nil;

/****************************************************************************************
*	
*	Funky Protos
*	
****************************************************************************************/
#pragma mark	--Funky Protos--

OSErr	PrepIO( XIOPtr io, short refNum, void *buffer, long *size, long offset, long patience );
OSErr	PrepCS( XCSPtr cs, short refNum, short csCode, void *csParamPtr, short size, long patience );
pascal	void	IOProc();
pascal	void	TMProc();

/****************************************************************************************
*	
*	Public Functions
*	
****************************************************************************************/
#pragma mark	--Public Functions--

/**************************
 *
 *	ThreadedRead
 *
 */

OSErr	ThreadedRead( short refNum, void *buffer, long *size, long offset, long patience )
{
	XIO		io;
	OSErr	err;
	
	err = PrepIO( &io, refNum, buffer, size, offset, patience );
	
	if( !err )
		err = PBReadAsync( (ParmBlkPtr) &io );
	if( !err )
		err = YieldToAnyThread();
	/*	If we yield and we're the only thread then the scheduler will be ignored and
		we'll possibly regain control before IO is finished. We put a polling loop here
		just in case */
	if( !err )
		while( io.ioResult > noErr ) {}
	if( !err )
		*size = io.ioActCount;
	
	return( err ? err : io.ioResult );
}

/**************************
 *
 *	ThreadedWrite
 *
 */

OSErr	ThreadedWrite( short refNum, void *buffer, long *size, long offset, long patience )
{
	XIO		io;
	OSErr	err;
	
	err = PrepIO( &io, refNum, buffer, size, offset, patience );
	
	if( !err )
		err = PBWriteAsync( (ParmBlkPtr) &io );
	if( !err )
		err = YieldToAnyThread();
	/*	If we yield and we're the only thread then the scheduler will be ignored and
		we'll possibly regain control before IO is finished. We put a polling loop here
		just in case */
	if( !err )
		while( io.ioResult > noErr ) {}
	if( !err )
		*size = io.ioActCount;
	
	return( err ? err : io.ioResult );
}

/**************************
 *
 *	ThreadedControl
 *
 */

OSErr	ThreadedControl( short refNum, short csCode, void *csParamPtr, short size, long patience )
{
	XCS		cs;
	OSErr	err;
	
	err = PrepCS( &cs, refNum, csCode, csParamPtr, size, patience );
	
	if( !err )
		err = PBControlAsync( (ParmBlkPtr) &cs );
	if( !err )
		err = YieldToAnyThread();
	/*	If we yield and we're the only thread then the scheduler will be ignored and
		we'll possibly regain control before IO is finished. We put a polling loop here
		just in case */
	if( !err )
		while( cs.ioResult > noErr ) {}
	if( !err )
		BlockMoveData( &cs.csParam, csParamPtr, size );
	
	return( err ? err : cs.ioResult );
}

/**************************
 *
 *	ThreadedStatus
 *
 */

OSErr	ThreadedStatus( short refNum, short csCode, void *csParamPtr, short size, long patience )
{
	XCS		cs;
	OSErr	err;
	
	err = PrepCS( &cs, refNum, csCode, csParamPtr, size, patience );
	
	if( !err )
		err = PBControlAsync( (ParmBlkPtr) &cs );
	if( !err )
		err = YieldToAnyThread();
	/*	If we yield and we're the only thread then the scheduler will be ignored and
		we'll possibly regain control before IO is finished. We put a polling loop here
		just in case */
	if( !err )
		while( cs.ioResult > noErr ) {}
	if( !err )
		BlockMoveData( &cs.csParam, csParamPtr, size );
	
	return( err ? err : cs.ioResult );
}

/****************************************************************************************
*	
*	Private Functions
*	
****************************************************************************************/
#pragma mark	--Private Functions--

/**************************
 *
 *	PrepIO
 *
 */

OSErr	PrepIO( XIOPtr io, short refNum, void *buffer, long *size, long offset, long patience )
{
	OSErr	err = noErr;
	
	if( !gIOProc ) {
		gIOProc = NewIOCompletionProc( IOProc );
		if( !gIOProc )
			err = memFullErr;
	}
	if( !gTMProc ) {
		gTMProc = NewTimerProc( TMProc );
		if( !gTMProc )
			err = memFullErr;
	}
	
	io->ioCompletion = gIOProc;
	io->ioNamePtr = nil;
	io->ioVRefNum = 0;
	io->ioRefNum = io->tm.tmRefNum = refNum;
	io->ioBuffer = (Ptr) buffer;
	io->ioReqCount = *size;
	io->ioPosMode = fsFromStart;
	io->ioPosOffset = offset;
	io->tm.tmCount = io->tm.tmWakeUp = io->tm.tmReserved = 0;
	io->tm.tmStoredA5 = SetCurrentA5();
	if( !err )
		err = GetCurrentThread( &io->ioThread );
	if( !err )
		err = SetCurrentXThreadState( kXThreadIneligible );
	if( !err && patience ) {
		io->tm.tmAddr = gTMProc;
		InsTime( (QElemPtr) &io->tm );
		PrimeTime( (QElemPtr) &io->tm, patience );
	} else
		io->tm.tmAddr = nil;
	
	return( err );
}

/**************************
 *
 *	PrepCS
 *
 */

OSErr	PrepCS( XCSPtr cs, short refNum, short csCode, void *csParamPtr, short size, long patience )
{
	OSErr	err = noErr;
	
	if( !gIOProc ) {
		gIOProc = NewIOCompletionProc( IOProc );
		if( !gIOProc )
			err = memFullErr;
	}
	if( !gTMProc ) {
		gTMProc = NewTimerProc( TMProc );
		if( !gTMProc )
			err = memFullErr;
	}
	
	cs->ioCompletion = gIOProc;
	cs->ioNamePtr = nil;
	cs->ioVRefNum = 0;
	cs->ioCRefNum = cs->tm.tmRefNum = refNum;
	cs->csCode = csCode;
	BlockMoveData( csParamPtr, &cs->csParam, size );
	cs->tm.tmCount = cs->tm.tmWakeUp = cs->tm.tmReserved = 0;
	cs->tm.tmStoredA5 = SetCurrentA5();
	if( !err )
		err = GetCurrentThread( &cs->ioThread );
	if( !err )
		err = SetCurrentXThreadState( kXThreadIneligible );
	if( !err && patience ) {
		cs->tm.tmAddr = gTMProc;
		InsTime( (QElemPtr) &cs->tm );
		PrimeTime( (QElemPtr) &cs->tm, patience );
	} else
		cs->tm.tmAddr = nil;
	
	return( err );
}

/**************************
 *
 *	IOProc
 *
 */

pascal void	IOProc()
{
	XIOPtr	io = (XIOPtr) GetRegA0();
	long	oldA5 = SetA5( io->tm.tmStoredA5 );
	
	//	If we're impatient, kill the timer
	if( io->tm.tmAddr )
		RmvTime( (QElemPtr) &io->tm );		
	
	//	Our IO is completed -- prioritize the IO thread
	(void) SetXThreadState( io->ioThread, kXThreadPriority );
	
	//	Restore the old a5
	(void) SetA5( oldA5 );
}

/**************************
 *
 *	TMProc
 *
 */

pascal	void	TMProc()
{
	(void) KillIO( ((XTMPtr) GetRegA1())->tmRefNum );
}