//+++Header
/**********************************************************************************
	FILE: ...
	Description:...

	1994, CE Software, Inc.  All rights reserved.
	P.O. Box 65580
	West Des Moines, Iowa 50265 U.S.A.

***********************************************************************************/

//+++Inc - Includes for files not directly related to this project
#include <CeAssert.h>

//+++Inc - Includes for files directly related to this project
#include "OTEndPoint.h"
#include "Idler.h"
#include "Connection.h"

//+++ForwardDcl
typedef struct LookupNameReplyHeader
{
	unsigned short		muiAddrLen;
	unsigned short		muiNameLen;
} LookupNameReplyHeader;

//+++Constant
const	OTTimeout	kOTResolveTimeout		=	40;	// 40 seconds

//+++Enum

//+++Macro
#define	kOTResolveTimeOutResID			131

//+++Globals

//+++Implementation

// Constructors
OTEndPoint::OTEndPoint (void)
{
	mfCancelled = false;
	mfDataWaiting = false;
	mfNeedsDisconnecting = false;
	mulResolveTimeout = GetLimit (kOTResolveTimeOutResID, kOTResolveTimeout);
	mulResolveTimeout *= 1000L;		// Convert to milliseconds
	mlReceiveBufferSize = kDefaultTCPRcvSize;
	mpEndpoint = nil;
}

// Destructor
OTEndPoint::~OTEndPoint (void)
{
	OSErr		iErr = Close (nil);
	
	CeAssert (iErr == noErr);
}

// Operators

// Other methods

#pragma push
#pragma segment Main
// Returns true if the OTEndPoint::Cancel() has been called since the endpoint was opened, OR
// if the connection has been broken (by the other end disconnecting, for example)
// In keeping with the Connection class cancelling is an immediate, synchronous operation. Cancelling
// is expected to cause all further operations to fail immediately, thus eventually exiting out of the
// operation.
Boolean OTEndPoint::EndPointValid (void) const
{
	return (mpEndpoint != nil && !mfCancelled);
}


OSErr OTEndPoint::DestroyEndPoint (void)
{
	OSErr		iErr = noErr;
	
	if (mpEndpoint != nil)
	{
		(void) OTSetSynchronous (mpEndpoint);		// Guaranteed to succeed as per OT docs
		OTRemoveNotifier (mpEndpoint);
		iErr = OTCloseProvider(mpEndpoint);
		mpEndpoint = nil;
		mfDataWaiting = false;
	}
	return iErr;
}

void OTEndPoint::CheckOTError (OTResult lError)
{
	if (lError == kOTLookErr)
	{
		if (mpMapperRef != nil)
		{
			CeAssertDesc (false, "Unhandled OTLook() result (mpMapperRef) in OTEndPoint::CheckOTError()");
		}
		else if (mpEndpoint != nil)
		{
			OSErr			iDummy;
			OTResult		lResult;
			
			switch (OTLook (mpEndpoint))
			{
				case T_ORDREL:
					CeAssert (mfNeedsDisconnecting);
					lResult = OTRcvOrderlyDisconnect (mpEndpoint);
					CeAssert (lResult == kOTNoError);
					iDummy = DestroyEndPoint ();
					CeAssert (iDummy == noErr);
					break;
				case T_DISCONNECT:
					CeAssert (mfNeedsDisconnecting);
					lResult = OTRcvDisconnect (mpEndpoint, nil);
					CeAssert (lResult == kOTNoError);
					iDummy = DestroyEndPoint ();
					CeAssert (iDummy == noErr);
					break;
				default:
					CeAssertDesc (false, "Unhandled OTLook() result in OTEndPoint::CheckOTError()");
					break;
			}
		}
	}
	else if (lError != kOTNoError)
	{
		CeAssertDesc (false, "OT error occurred - state to follow");
		
		if (mpMapperRef != nil)
		{
			CeAssertDesc (false, "Mapper error");
		}
		else if (mpEndpoint != nil)
		{
			switch (OTGetEndpointState (mpEndpoint))
			{
				case T_UNINIT:
					CeAssertDesc (false, "T_UNINIT");
					break;
				case T_UNBND:
					CeAssertDesc (false, "T_UNBND");
					break;
				case T_IDLE:
					CeAssertDesc (false, "T_IDLE");
					break;
				case T_OUTCON:
					CeAssertDesc (false, "T_OUTCON");
					break;
				case T_INCON:
					CeAssertDesc (false, "T_INCON");
					break;
				case T_DATAXFER:
					CeAssertDesc (false, "T_DATAXFER");
					break;
				case T_OUTREL:
					CeAssertDesc (false, "T_OUTREL");
					break;
				case T_INREL:
					CeAssertDesc (false, "T_INREL");
					break;
				case kOTStateChangeErr:
					CeAssertDesc (false, "kOTStateChangeErr");
					break;
				default:
					CeAssertDesc (false, "Unknown state");
					break;
			}
		}
		else
		{
			CeAssertDesc (false, "nil endpoint and mapper");
		}
	}
}
#pragma pop


OSErr OTEndPoint::OpenEndpoint(const char *protocol)
{
	OSStatus			err;

	mfDataWaiting = false;
	
	CeAssert (!EndPointValid ());
	mpEndpoint = OTOpenEndpoint (OTCreateConfiguration(protocol), 0, nil, &err);
	CheckOTError (err);
	if (err != noErr)
		mpEndpoint = nil;
	return err;
}

// THIS COMMENT IS OBSOLETE (SEE USE_INET_RESOLVER BELOW) AND IS HERE ONLY FOR HISTORICAL PURPOSES
// We don't open the mapper asynchronously since I saw no example code that used OTAsyncOpenMapper() -
// all code that used async mapping used OTOpenMapper() followed by OTInstallNotifier()/OTSetAsynchronous()
// It could be that sporadic kOTBadAddressErr errors are returned due to OTAsyncOpenMapper() - I was getting
// such errors sporadically and changed the code to follow existing sample code. I do not know for certain
// that making this change actually helped the problem. jjm 11/2/96
//#define ASYNC_MAPPER_OPEN

// OTLookupName() was sporadically (very sporadically in some cases) returning kOTBadNameErr
// Using OTInetStringToAddress() cleared up these errors using MacPiper@att.worldnet.net
// I left the OTLookupName() code ifdef'd out since it will probably be useful in the future even if
// its usefullness is limited to indicating to someone reading the code that that approach had been tried
// and found to be problematic.
#define USE_INET_RESOLVER

OSErr OTEndPoint::ResolveAddress (Idler& oIdler, const char* pszHost, InetHost& lHostsIPAddress)
{
#ifdef USE_INET_RESOLVER
	OSErr				iResult;
	OSStatus			lErr;
	
	mpMapperRef = nil;

	lErr = OTInetStringToHost ((char*) pszHost, &lHostsIPAddress);
	// If host name wasn't an IP address (www.xxx.yyy.zzz) then we must resolve it
	if (lErr != noErr)
	{
		InetHostInfo	sHostInfo;
		
		miAsyncErr = noErr;
		meState = keOpeningMapper;
		// Could make this class faster and a much better overall OS citizen by opening the
		// mapper once and leaving it open until the object is destroyed. I realized this in
		// late beta and didn't want to change to be a better citizen.
		lErr = OTAsyncOpenInternetServices (kDefaultInternetServicesPath, 0, Notifier, this);
		CheckOTError (lErr);
		if (lErr == noErr)
		{
			do {
				oIdler.YieldTime ();
			} while (meState == keOpeningMapper && !mfCancelled);
			
			iResult = miAsyncErr;
			CheckOTError (iResult);
			if (iResult == noErr)
			{
				CeAssert (mpMapperRef != nil);
				
				miAsyncErr = noErr;
				meState = keResolving;
				iResult = OTInetStringToAddress ((InetSvcRef) mpMapperRef, (char*) pszHost, &sHostInfo);
				CheckOTError (iResult);
				if (iResult == noErr)
				{
					do {
						oIdler.YieldTime ();
					} while (meState == keResolving && !mfCancelled);
					
					iResult = miAsyncErr;
					if (iResult == noErr && !mfCancelled)
						lHostsIPAddress = sHostInfo.addrs [0];
				}
				if (iResult == noErr)
					iResult = OTCloseProvider (mpMapperRef);
				else
					(void) OTCloseProvider (mpMapperRef);
				mpMapperRef = nil;
			}
		}
		else
			iResult = lErr;
	}
	else
		iResult = noErr;
	return iResult;
#else
	OSErr		iErr;
	OSStatus	lResult;
	
	mpMapperRef = nil;
	miAsyncErr = noErr;
	meState = keOpeningMapper;
	oIdler.YieldTime ();
#ifdef ASYNC_MAPPER_OPEN
	lResult = OTAsyncOpenMapper (OTCreateConfiguration (kDNRName), 0, &Notifier, this);
#else
	mpMapperRef = OTOpenMapper (OTCreateConfiguration (kDNRName), 0, &lResult);
#endif // ASYNC_MAPPER_OPEN
	CheckOTError (lResult);
	if (lResult == kOTNoError)
	{
#ifdef ASYNC_MAPPER_OPEN
		do {
			oIdler.YieldTime ();
		} while (meState == keOpeningMapper && !mfCancelled);	// Note we aren't dealing with mpEndpoint
		iErr = miAsyncErr;
		CheckOTError (iErr);
#else
		iErr = OTInstallNotifier (mpMapperRef, &Notifier, this);
		{
			OSErr	iIgnore = OTSetAsynchronous (mpMapperRef);		// Guaranteed to succeed as per OT docs

			CeAssert (iIgnore == noErr);
		}
#endif // ASYNC_MAPPER_OPEN
		if (iErr == noErr && !mfCancelled)
		{
			OSErr					iCloseErr;
			short					iReplyBufferLen;
			char*					pReplyBuffer;
			
			CeAssert (mpMapperRef != nil);
			
			iReplyBufferLen = sizeof (LookupNameReplyHeader) + sizeof (InetAddress) + strlen (pszHost) + 5;
			pReplyBuffer = new char [iReplyBufferLen];
			if (pReplyBuffer != nil)
			{
				size_t				iiHostLen = strlen (pszHost);
				TLookupRequest		oRequest	=	{
															{	iiHostLen,	iiHostLen,	(UInt8*) pszHost	},
															{	0,	0,	nil	},
															1,							// Max # of addresses returned by resolver
															mulResolveTimeout
														};
				TLookupReply		oReply	=	{
															{	iReplyBufferLen,	iReplyBufferLen,	(UInt8*) pReplyBuffer	},
															1
														};
				
				miAsyncErr = noErr;
				meState = keResolving;
				oIdler.YieldTime ();
				lResult = OTLookupName (mpMapperRef, &oRequest, &oReply);
				CheckOTError (lResult);
				if (lResult == kOTNoError && !mfCancelled)	// Note we aren't dealing with mpEndpoint
				{
					do {
						oIdler.YieldTime ();
					} while (meState == keResolving && !mfCancelled);	// Note we aren't dealing with mpEndpoint
					iErr = miAsyncErr;
					CheckOTError (iErr);
					if (iErr == noErr && !mfCancelled)			// Note we aren't dealing with mpEndpoint
					{
						InetAddress*	psResolvedAddress;
						
						CeAssert (oReply.rspcount == 1);

						psResolvedAddress = (InetAddress*) (pReplyBuffer + sizeof (LookupNameReplyHeader));
						CeAssert (psResolvedAddress->fAddressType == AF_INET);
						lHostsIPAddress = psResolvedAddress->fHost;
					}
				}
				else
					iErr = lResult;
				delete [] pReplyBuffer;
			}
			else
				iErr = memFullErr;
			
			if (mpMapperRef != nil)
			{
				iCloseErr = OTCloseProvider (mpMapperRef);
				mpMapperRef = nil;
				CheckOTError (iCloseErr);
				if (iErr == noErr)
					iErr = iCloseErr;
				oIdler.YieldTime ();
			}
		}
		
		if (mfCancelled)
			iErr = Connection::keAborted;
	}
	else
		iErr = lResult;
	return iErr;
#endif // USE_INET_RESOLVER
}


// Put call back in main segment to avoid having the segment unloaded and causing typical callback problems
// if the segment is unloaded when the callback is called
#pragma push
#pragma segment Main

// *** Called at deferred task time. ***
pascal void OTEndPoint::Notifier(void* contextPtr, OTEventCode code, OTResult result, void* cookie)
{
	OTEndPoint*	thisPoint = (OTEndPoint*) contextPtr;
	
	thisPoint->HandleNotify (code, result, cookie);
}


// *** Called at deferred task time. ***
void OTEndPoint::HandleNotify( OTEventCode code, OTResult result, void* cookie)
{
	OTResult	lResult;
	
	CeAssertDesc (mpEndpoint != nil, "OTEndPoint::HandleNotify called with an invalid endpoint");
	
	switch (code)
	{
		case T_BINDCOMPLETE:
			miAsyncErr = result;
			meState = keBindingDone;
			break;
		case T_CONNECT:
			miAsyncErr = result;
			if (result == kOTNoError)
			{
				if (mpEndpoint != nil)
				{
					lResult = OTRcvConnect (mpEndpoint, nil);
					CheckOTError (lResult);
				}
				meState = keConnectingDone;
			}
			else
				meState = keBailingOut;
			break;
		case T_DISCONNECT:
			miAsyncErr = result;
			meState = keDisconnecting;
			// the endpoint may have been shut down by close call so check for nil
			if (mpEndpoint != nil)
			{
				lResult = OTRcvDisconnect (mpEndpoint, nil);
				CheckOTError (lResult);
				DestroyEndPoint ();
			}
			meState = keDisconnectingDone;
			break;
		case T_ORDREL:
			miAsyncErr = result;
			meState = keOrderlyDisconnecting;
			// the endpoint may have been shut down by close call so check for nil
			if (mpEndpoint != nil)
			{
				OSErr		iDummy;
				
				lResult = OTRcvOrderlyDisconnect (mpEndpoint);
				CheckOTError (lResult);
				iDummy = DestroyEndPoint ();
				CeAssert (iDummy == noErr);
			}
			meState = keOrderlyDisconnectingDone;
			break;
		case T_DATA:
			mfDataWaiting = true;
			break;
		case T_DNRSTRINGTOADDRCOMPLETE:
		case T_LKUPNAMECOMPLETE:
			meState = keResolvingDone;
			miAsyncErr = result;
			break;
		case T_OPENCOMPLETE:
			meState = keOpeningMapperDone;
			miAsyncErr = result;
			if (result == kOTNoError)
				mpMapperRef = (MapperRef) cookie;
			break;
	}
}

#pragma pop


OSErr OTEndPoint::Open (Idler& oIdler, const char *address, short port)
{
	OSErr		err;
	
	CeAssert (mpEndpoint == nil);
	
	mfNeedsDisconnecting = false;
	mfCancelled = false;
	err = OpenEndpoint (kTCPName);
	if (err == noErr && EndPointValid ())
	{
		err = OTInstallNotifier (mpEndpoint, &Notifier, this);
		CheckOTError (err);
		// make it asynchronous
		{
			OSErr		iIgnored = OTSetAsynchronous (mpEndpoint);		// Guaranteed to succeed as per OT docs
			
			CeAssert (iIgnored == noErr);
		}
		if (err == noErr && EndPointValid ())
		{
			mfDestAddress.inet.fAddressType = AF_INET;
			mfDestAddress.inet.fPort = port;
			err = ResolveAddress (oIdler, address, mfDestAddress.inet.fHost);
			if (err == noErr && EndPointValid ())
			{
				InetAddress	inAddress;
				TBind			inBind = { { sizeof(InetAddress), sizeof(InetAddress), (UInt8*) &inAddress }, 0 };

				inAddress.fAddressType = AF_INET;
				inAddress.fPort = 0;
				inAddress.fHost = 0;
				miAsyncErr = noErr;
				meState = keBinding;
				err = OTBind (mpEndpoint, &inBind, nil);
				CheckOTError (err);
				if (err == kOTNoError)
				{
					// Use "do" loop so YieldTime is called no matter what
					do {
						oIdler.YieldTime ();
					} while (meState == keBinding && EndPointValid ());
					if (!EndPointValid ())
						err = Connection::keAborted;
					else
					{
						err = miAsyncErr;
						CheckOTError (err);
					}
					
					if (err == noErr && EndPointValid ())
					{
						TCall	sConnectionParms	=	{
															{ sizeof(mfDestAddress), sizeof(mfDestAddress), (UInt8*) &mfDestAddress },
															{ 0, 0, NULL },
															{ 0, 0, NULL },
															};

						miAsyncErr = noErr;
						meState = keConnecting;
						err = OTConnect (mpEndpoint, &sConnectionParms, nil);
						if (err == kOTNoError || err == kOTNoDataErr)
						{
							// Use "do" loop so YieldTime is called no matter what
							do {
								oIdler.YieldTime ();
							} while (meState == keConnecting && EndPointValid ());
							if (!EndPointValid ())
								err = Connection::keAborted;
							else
							{
								err = miAsyncErr;
								if (err == noErr)
									mfNeedsDisconnecting = true;
								else
									CheckOTError (err);
							}
							// OTRcvConnect() is done in HandleNotify()
						}
					}
				}
			}
		}
		
		// If the open fails we must close the endpoint that we know at this point was successfully opened
		// If endpoint is not valid (mpEndPoint is nil or endpoint has been cancelled) it's benign to call Close()
		if (err != noErr || !EndPointValid ())
		{
			OSErr	iDummy = Close (&oIdler);
			
			CeAssert (iDummy == noErr);
		}
	}
		
	return err;
}


OSErr OTEndPoint::Close (Idler* poIdler)
{
	OSErr		iResult = noErr;
	
	// Need to close the endpoint if Cancel() was called so we check for mpEndpoint != nil instead of EndPointValid ()
	if (mpEndpoint != nil)
	{
		OSErr		iDummy;
		
		if (mfNeedsDisconnecting)
		{
			miAsyncErr = noErr;
			meState = keOrderlyDisconnecting;
			iResult = OTSndOrderlyDisconnect (mpEndpoint);
			CheckOTError (iResult);
/*
	There were cases where OT would not call HandleNotify() with T_ORDREL so this loop would turn infinite
			if (iResult == kOTNoError)
			{
				// Use "do" loop so YieldTime is called no matter what
				do {
					if (poIdler != nil)
						poIdler->YieldTime ();
				} while (meState == keOrderlyDisconnecting && EndPointValid ());
			}
			// OTRcvOrderlyDisconnect() is done in HandleNotify()
*/
		}
		
		iDummy = DestroyEndPoint ();
		CeAssert (iDummy == noErr);

		if (poIdler != nil)
			poIdler->YieldTime ();
	}
	return iResult;
}


void OTEndPoint::Cancel (void)
{
	mfCancelled = true;
}


void OTEndPoint::SetReceiverBufferSize (long lBufferSize)
{
	mlReceiveBufferSize = lBufferSize;
}

OSErr OTEndPoint::SendData (Idler& oIdler, const void *pData, long lSize)
{
	long		lBytesSent = 0;
	OSStatus	lResult = noErr;
	
	while (EndPointValid ())
	{
		lResult = OTSnd (mpEndpoint, (void*) (((UInt8*) pData) + lBytesSent), lSize, 0);
		
		if (lResult >= 0)
		{
			lBytesSent += lResult;
			if (lBytesSent >= lSize)
			{
				lResult = kOTNoError;
				break;
			}
		}
		else if (lResult != kOTFlowErr)
		{
			CheckOTError (lResult);
			break;
		}
		// else no bytes were sent - we'll just try sending again
		oIdler.YieldTime ();
	}
	
	if (!EndPointValid ())
		lResult = Connection::keAborted;

	return (OSErr) lResult;
}


OSErr OTEndPoint::GetData (Idler& oIdler, const void *pData, unsigned short& uiBytesReceived)
{
	OSStatus	lResult = noErr;
//	static Boolean	giCallCount = 0;
	
	if (!EndPointValid ())
		return Connection::keAborted;
	
	do {
		oIdler.YieldTime ();
	} while (!mfDataWaiting && EndPointValid ());
	
//	giCallCount++;
	
	uiBytesReceived = 0;
	while (EndPointValid ())
	{
		OTFlags		flags;
		
		// The notify function may set mfDataWaiting to true at interrupt time. Thus we set mfDataWaiting to false
		// here and then only set it to true if OT tells us that there is more data to be consumed (T_MORE is set)
		mfDataWaiting = false;
		
/*
		// Useful code for testing receives of small amounts of data instead of large chunks which can't be relied upon in general
		if (giCallCount % 3 != 0)
		{
			OTResult		theResult = OTRcv (mpEndpoint, (void*) (((char*) pData)), 4, &flags);
			
			uiBytesReceived += theResult;
			lResult = noErr;
			if ((flags & T_MORE) != 0)
				mfDataWaiting = true;
			CeAssert (uiBytesReceived != 0);
			break;
		}
*/

		OTResult		theResult = OTRcv (mpEndpoint, (void*) (((char*) pData) + uiBytesReceived), mlReceiveBufferSize - uiBytesReceived, &flags);
		
		if (theResult >= 0)
		{
			uiBytesReceived += theResult;
			if ((flags & T_MORE) != 0)
				mfDataWaiting = true;
			if (uiBytesReceived >= mlReceiveBufferSize || (flags & T_MORE) == 0)
			{
				CeAssert (uiBytesReceived == mlReceiveBufferSize);
				lResult = noErr;
				CeAssert (uiBytesReceived != 0);
				break;
			}
		}
		else if (theResult != kOTNoDataErr)
		{
			CheckOTError (theResult);
			lResult = theResult;
			break;
		}
		else
		{
			// else handle kOTNoDataErr
			lResult = noErr;
			break;
		}
		// OT docs say to enter a loop until a T_DATA event is encountered by the notification proc but
		// there doesn't appear to be any reason for entering such a state since we need to appear synchronous
		// to the caller anyway. Perhaps there are some performance gains to be had by doing so but I
		// haven't noticed any problems in just looping on OTRcv().

		oIdler.YieldTime ();
	}

	return (OSErr) lResult;
}

OSErr OTEndPoint::ReceiveOutstandingData (Idler& oIdler)
{
	if (mfDataWaiting)
	{
		unsigned short		uiIgnored;
		char*					pbBuffer = new char [kDefaultTCPRcvSize];
		
		for ( ; mfDataWaiting; )
		{
			if (GetData (oIdler, pbBuffer, uiIgnored) != noErr)
				break;
		}
		delete [] pbBuffer;
	}
	return noErr;
}
