//+++Header
/**********************************************************************************
	FILE: IPLSBROTEndPoint.cp
	Description: IPLS Browser Endpoint, the Open Transport code that opens a
		connection to a server and sends and receives.

	1997, CE Software, Inc.  All rights reserved.
	P.O. Box 65580
	West Des Moines, Iowa 50265 U.S.A.

***********************************************************************************/

#define kIPLSBufferRcvSize 4096

#include <stdio.h>



//+++Inc - Includes for files directly related to this project
#include "IPLSBROTEndPoint.h"
#include "Idler.h"

//stolen from Connection so we don't have to link it all in
long GetLimit (short iResID, long lDefault)
{
	long	lResult = lDefault;
	Handle	h = GetResource ('LIMT', iResID);
	
	if (h != nil)
		lResult = *(long*) (*h);
	return lResult;
};

// We 're taking the QM OTEndpoint and just adding a readastring item
// Constructors
IPLSBROTEndPoint::IPLSBROTEndPoint (void)
	: OTEndPoint()
{
	mfBytesReceived = 0;
	mfBufferSize = kIPLSBufferRcvSize;
	mfInComingData = new char [kIPLSBufferRcvSize];
}

// We do need to dispose of our data
IPLSBROTEndPoint::~IPLSBROTEndPoint (void)
{
	if (mfInComingData)
		delete mfInComingData;
}

//
// Read a pascal string in.  There may already be characters from a previous read in our buffer,
// so we can't just wait for mfData to be true;
//
// We do need an Idler if the server hasn't responded yet.
//
OSErr IPLSBROTEndPoint::ReadAString (Idler& oIdler, Str255 thestr )
{
	unsigned short		numreceived;
	OSErr		theerr;
	
	short	index;
	short	numcharsinstring;
	
	if (!EndPointValid ())
		return -1;
	if (!mfInComingData)
		return -1;
	
	while (EndPointValid())
	{
		if (mfBytesReceived>0)	//see if we already had some bytes in our stuff
		{
			numcharsinstring = 0;	// see if we've got a string
			
			if (mfBytesReceived>=255)	//Have we hit our maximum size?
			{
				numcharsinstring = 255;
			}
			else
			{
				for (index = 0; index < mfBytesReceived; index++)
				{
					if (mfInComingData[index]==0x0A)	//found end of line, line feed of cr/lf
					{
						index++;	//convert the index into the size
						numcharsinstring = index;
						break;
					}
				}
			}
			if (numcharsinstring)
			{
				thestr[0] = numcharsinstring;
				BlockMove(mfInComingData, &thestr[1], numcharsinstring);
				BlockMove(&mfInComingData[numcharsinstring], &mfInComingData[0], mfBytesReceived-numcharsinstring);
				mfBytesReceived -= numcharsinstring;
				return noErr;
			}
		}
		//  We need some more characters.  Get data from our endpoint then loop back and look for a string
		SetReceiverBufferSize(mfBufferSize - mfBytesReceived);
		theerr = GetData(oIdler, (void*) (&mfInComingData[mfBytesReceived]), numreceived);
		if (theerr!=noErr)
		{
			return theerr;
		}
		else mfBytesReceived+=numreceived;
	}
	return -1;
}

//Registering needs to get my URL from time to time
OSErr IPLSBROTEndPoint::GetDomainName( char *dest)
{
	InetInterfaceInfo my_info;
	OSErr myErr = OTInetGetInterfaceInfo( &my_info, 0 );
	
	if( myErr != noErr )
	{
		// Could not get IP address. We can probably do just fine by using domain.
		strcpy( dest, "domain" );
	}
	else
	{
		unsigned char*my_ip = (unsigned char*)&my_info.fAddress;
		sprintf( dest, "%i.%i.%i.%i", my_ip[0], my_ip[1], my_ip[2], my_ip[3] );
	}
	return myErr;
}
