#ifndef OTHANDLERENDPOINT_HPP
#define OTHANDLERENDPOINT_HPP

//+++Header
/**********************************************************************************
	FILE: OTENDPOINT_HPP
	Description:  Implementation of class OTEndPoint

	1994, CE Software, Inc.  All rights reserved.
	P.O. Box 65580
	West Des Moines, Iowa 50265 U.S.A.
	
	Revised by Donald Brown to handle server with multiple sessions handling it,
	waiting for connections, and using a polling instead of idler routines

***********************************************************************************/

//+++Inc - Includes for files not directly related to this project
#include <OpenTransport.h>
#include <OpenTptInternet.h>
#include <Types.h>

//+++Inc - Includes for files directly related to this project

//+++ForwardDcl
class OTServerEndPoint;
//+++Types

//+++Constant

//+++Enum

//+++Macro

//+++Class
class OTHandlerEndPoint
{
	//+++Friends

	public:
		//+++Enums

	protected:
		Boolean			mfCancelled;
		Boolean			mfDataWaiting;
		Boolean			mfNeedsDisconnecting;
		Boolean			mfNeedsUnbinding;
		Boolean			mfJustConnected;
		Boolean			mfDataInIsText;
		
		long			mlReceiveBufferSize;
		volatile EndpointRef		mpEndpoint;
		TEndpointInfo			mfEndpointInfo;		// Endpoint information
		TCall 					mfCallInfo;
		TDiscon					mfDisconInfo;
		UInt8*					mfData;
		
		Ptr		mfOutGoingData;		//data being sent out
		long	mfSizeOfData;
		long	mfBytesSent;
		
		char	mfInComingData[4096];		//data being received
		short	mfBytesReceived;
		
	
		OTServerEndPoint* mfServer;
		short			mfHandlerID;
		
		enum	OutpointState	{	keBinding, keBindingDone,
										keConnecting, keConnectingDone,
										keDisconnecting, keDisconnectingDone,
										keOrderlyDisconnecting, keOrderlyDisconnectingDone,
										
										keOpeningMapper, keOpeningMapperDone,
										keResolving, keResolvingDone,
										
										keOpeningListener, keOpenListenerDone,
										keListening, keAccepting, keAcceptingDone,
										
										keBailingOut	};
		
		OSErr				miAsyncErr;
		OutpointState	meState;
		union {
			InetAddress	inet;
		}					mfDestAddress;
 	
	protected:
						OTHandlerEndPoint		(const OTHandlerEndPoint& oRHS);
		OTHandlerEndPoint&	operator=		(const OTHandlerEndPoint& oRHS);

		Boolean			EndPointValid	(void) const;
		OSErr			DestroyEndPoint(void);
		void			CheckOTError	(OTResult lError);
		
		static pascal void		Notifier			(void* contextPtr, OTEventCode code,
															OTResult result, void* cookie);
		void		 	HandleNotify	(OTEventCode code, OTResult result, void* cookie);
		
//
// If we need to send some MOD or other text on connection, override this
//
		virtual void	HandleStartup	(void);
//
// This is the workhorse
//
		virtual void	HandleData (char* thestring, short numchars);
//
// If we need to tell someone we've closed, override this
//
		virtual void	HandleShutdown	(void);

	public:
						OTHandlerEndPoint		(OTServerEndPoint* theserver, short handlerID);
		virtual			~OTHandlerEndPoint		(void);
		
		void			PassOff(EndpointRef theref);
		
		virtual void	DoIdle(void);
		
		virtual OSErr	Close				(void);

		void			Cancel			(void);
		void			SetCancelFlag 	(Boolean fCancel) { mfCancelled = fCancel; }
		
		EndpointRef		GetEndPointRef 	(void) { return mpEndpoint; }
		
		void			SetReceiverBufferSize	(long lBufferSize);
		
		OSErr			SendData			( const void *pData, long lSize);
		OSErr			SendMore	(void);
		
		OSErr			GetData			( void );
		
};

#endif		//OTENDPOINT_HPP
