#ifndef OTPOLLENDPOINT_HPP
#define OTPOLLENDPOINT_HPP

//+++Header
/**********************************************************************************
	FILE: OTPollENDPOINT_HPP
	Description:  Implementation of class OTEndPoint
		Modification using polling instead of idler for reading

	1994, CE Software, Inc.  All rights reserved.
	P.O. Box 65580
	West Des Moines, Iowa 50265 U.S.A.

***********************************************************************************/

//+++Inc - Includes for files not directly related to this project
#include <OpenTransport.h>
#include <OpenTptInternet.h>
#include <Types.h>

#define keOTPEPaborted -1
//+++Inc - Includes for files directly related to this project

//+++ForwardDcl
class Idler;

//+++Types

//+++Constant

//+++Enum

//+++Macro

//+++Class
class OTPollEndPoint
{
	//+++Friends

	public:
		//+++Enums

	protected:
		Boolean			mfCancelled;
		Boolean			mfDataWaiting;
		Boolean			mfLostConnection;
		Boolean			mfNeedsDisconnecting;
		Boolean			mfDataInIsText;

		char	mfInComingData[4096];		//data being received
		short	mfBytesReceived;


		OTTimeout		mulResolveTimeout;
		volatile EndpointRef		mpEndpoint;
		MapperRef		mpMapperRef;

		enum	OutpointState	{	keBinding, keBindingDone,
										keConnecting, keConnectingDone,
										keDisconnecting, keDisconnectingDone,
										keOrderlyDisconnecting, keOrderlyDisconnectingDone,
										
										keOpeningMapper, keOpeningMapperDone,
										keResolving, keResolvingDone,
										
										keBailingOut	};
		
		OSErr				miAsyncErr;
		OutpointState	meState;
		union {
			InetAddress	inet;
		}					mfDestAddress;
 	
	protected:
						OTPollEndPoint		(const OTPollEndPoint& oRHS);
		OTPollEndPoint&	operator=		(const OTPollEndPoint& oRHS);

		// Returns true if the endpoint has been cancelled. Also disconnects and destroys
		// the endpoint if it has been cancelled.
		Boolean		CheckCancelled	(Idler& oIdler);
		
		Boolean		EndPointValid	(void) const;
		OSErr			DestroyEndPoint(void);
		void			CheckOTError	(OTResult lError);
		
		OSErr			OpenEndpoint	(const char *protocol);
		OSErr			ResolveAddress	(Idler& oIdler, const char* pszHost, InetHost& lHostsIPAddress);

		static pascal void		Notifier			(void* contextPtr, OTEventCode code,
															OTResult result, void* cookie);
		void		 					HandleNotify	(OTEventCode code, OTResult result, void* cookie);

		virtual OSErr	GetData ( void );
//
// This is the workhorse
//
		virtual void	HandleData			(char* thestring, short numchars);
//
// If we need to tell someone we've closed, override this
//
		virtual void	HandleShutdown	(void);
		
	public:
						OTPollEndPoint		(void);
		virtual		~OTPollEndPoint		(void);
		
		OSErr			Open				(Idler& oIdler, const char *address, short port);
		OSErr			Close				(Idler* poIdler);

		void			Cancel			(void);
		void			SetCancelFlag 	(Boolean fCancel) { mfCancelled = fCancel; }

		virtual OSErr	SendData			(Idler& oIdler, const void *pData, long lSize);

		virtual void	DoIdle				(void);


};

// Streams

#endif		//OTPollEndPoint_HPP
