/**********************************************************************************
	FILE: WindowHandler.h
	Description: A window handler.  Sorta kinda like another framework, something
		I'm sure the whole programming world agrees we need
		
		We put the pointer to the class in the window handle of a window, and the only
		windows we have are this.  (I know, dangerous assumption).
		
		It is never called directly, it's called to be derived from

	1997, CE Software, Inc.  All rights reserved.
	P.O. Box 65580
	West Des Moines, Iowa 50265 U.S.A.

***********************************************************************************/

#include "WindowHandler.h"

//
// We put this in the windowrefcon of our windows
//
struct WHMaker {
	OSType			TheMarker;			//must be 'WinH'
	WindowHandler*	TheHandler;
	unsigned short rpOurPort;
};

Boolean	IsMagicWindow(WindowPtr thewind)	// Is this a windowhandler window?
{
	WHMaker* themarker;
	
	themarker = (WHMaker*)GetWRefCon (thewind);
	if (!themarker)
		return false;
	return (themarker->TheMarker=='WinH');
}

Boolean DoHandleEvent(EventRecord* MacEvent, WindowPtr thewind)
{
	WHMaker* themarker;
	
	if (!thewind)
		return false;
	themarker = (WHMaker*)GetWRefCon (thewind);
	if (!themarker)
		return false;
	if (themarker->TheMarker!='WinH')
		return false;
	return themarker->TheHandler->HandleEvent(MacEvent);
}

Boolean DoHandleMenu(short menuID, short itemID, WindowPtr thewind)
{
	WHMaker* themarker;
	
	if (!thewind)
		return false;
	themarker = (WHMaker*)GetWRefCon (thewind);
	if (!themarker)
		return false;
	if (themarker->TheMarker!='WinH')
		return false;
	return themarker->TheHandler->HandleMenu(menuID, itemID);

}

void DoSetupMenus(void )
{
	WindowPtr thewind = FrontWindow();
	WHMaker* themarker;
	
	if (!thewind)
		return;
	themarker = (WHMaker*)GetWRefCon (thewind);
	if (!themarker)
		return;
	if (themarker->TheMarker!='WinH')
		return;
	themarker->TheHandler->SetupMenus();

}
void IdleWindow(WindowPtr thewind)
{
	WHMaker* themarker;
	
	if (thewind)
	{
		themarker = (WHMaker*)GetWRefCon (thewind);
		if (themarker)
		{
			if (themarker->TheMarker=='WinH')
				themarker->TheHandler->DoIdle();
		}
	}
}

Boolean TryClose(WindowPtr thewind)
{
	WHMaker* themarker;
	
	if (!thewind)
		return false;
	themarker = (WHMaker*)GetWRefCon (thewind);
	if (!themarker)
		return false;
	if (themarker->TheMarker!='WinH')
		return false;
	if (!themarker->TheHandler->OKToClose())
		return false;
	delete themarker->TheHandler;	//and the marker will get magicly deleted below us
	return true;
}

//
// Our constructor.  It creates a new window, remembers it, and sets the pointer to ourselves
//
WindowHandler::WindowHandler(short theResID)
{
	WHMaker* themarker = (WHMaker*)(NewPtr(sizeof(WHMaker))); //hook ourselves to it

	mfTheWind = GetNewWindow (theResID, nil, (WindowPtr)-1 ); //Create the stuff
	themarker->TheMarker = 'WinH';
	themarker->TheHandler = this;
	SetWRefCon (mfTheWind, (long)themarker);
}

//
// Our Destructor.  Delete our pointer, then close the window
//
WindowHandler::~WindowHandler(void)
{
	WHMaker* themarker = (WHMaker*)GetWRefCon (mfTheWind);
	DisposePtr( (Ptr)themarker);
}

//
// Override this if you need to be saved before you can close, or want to cancel a close
//
Boolean WindowHandler::OKToClose(void)
{
	return true;	// It's OK to close us
}

//
// Override this to handle all the events.  Including draw, whatever
//
Boolean WindowHandler::HandleEvent(EventRecord*	MacEvent)
{

	WindowPtr	theWindow;
	short		theaction;
	Rect		boundsRect;
	

	switch ( MacEvent->what ) {
		case mouseDown:
			theaction = FindWindow (MacEvent->where, &theWindow);
			switch (theaction)
			{
				case inDrag:
					SetRect(&boundsRect, -32767, -32767, 32767, 32767);
					DragWindow(theWindow, MacEvent->where, &boundsRect);
					return true;
					break;
				case inGrow:
				case inContent:
					HandleContentClick(MacEvent);
					return true;
					break;
				case inGoAway:
					if (TrackGoAway(theWindow,MacEvent->where))
					{
						if (OKToClose())
						{
							delete this;
						}
					}
				default:
					return false;	//we didn't handle it
			}
			break;
		case keyDown:
		case autoKey:
			if (!(MacEvent->modifiers & cmdKey ))		//Don't handle command keys
				return HandleKeystroke(MacEvent);
			break;
		case updateEvt:
			HandleUpdateEvent();
			return true;
			break;
	}
	return false;

}

//
// Override this if we want to do anything on an idle
//
void WindowHandler::DoIdle(void)
{

}

// The specific window handlers, to be overridden

void WindowHandler::HandleContentClick(EventRecord*	/*MacEvent*/)
{
}
Boolean WindowHandler::HandleKeystroke(EventRecord*	/*MacEvent*/)
{
	return false;
}
void WindowHandler::HandleUpdateEvent(void)
{
	BeginUpdate(mfTheWind);
	HandleDraw();
	EndUpdate(mfTheWind);
}
void WindowHandler::HandleDraw(void)
{
}
void WindowHandler::SetupMenus(void)
{
}
Boolean WindowHandler::HandleMenu(short /*menuID*/, short /*itemID*/)
{
	return false;
}
