/**********************************************************************************
	FILE: IPLS.c
	Description: Base application handler.  An old shell I've used for years.

	1997, CE Software, Inc.  All rights reserved.
	P.O. Box 65580
	West Des Moines, Iowa 50265 U.S.A.

***********************************************************************************/

#include "IPLS.h"
#include "IPLSServer.h"

/* The "g" prefix is used to emphasize that a variable is global. */

/* GInBackground is maintained by our osEvent handling routines. Any part of
   the program can check it to find out if it is currently in the background. */
Boolean		gInBackground;		/* maintained by Initialize and DoEvent */
Boolean		gDoneFlag = true;

/* Here are declarations for all of the C routines. In MPW 3.0 and later we can use
   actual prototypes for parameter type checking. */
void EventLoop( void );
void DoEvent( EventRecord *event );
void DoMenuCommand( long menuResult );
void Initialize( void );

static pascal OSErr IPLSHandleQuit(const AppleEvent* /*theAppleEvent*/, 
	const AppleEvent* /*reply*/, long /*theblock*/)
{
	gDoneFlag = false;
	return noErr;
}

static void InstallHandlers( void )
{
	AEInstallEventHandler( kCoreEventClass, kAEQuitApplication,(AEEventHandlerUPP)IPLSHandleQuit, 0, false ) ;
}

#pragma segment Main
main()
{
	SetApplLimit(GetApplLimit()-32768); //give an additional 32K to stack
	MaxApplZone();					/* expand the heap so code segments load at the top */

	Initialize();					/* initialize the program */
	
	EventLoop();					/* call the main event loop */
	
	ServerShutdown();				// shut things down properly
}


/*	Get events forever, and handle them by calling DoEvent.
	Get the events by calling WaitNextEvent. */

#pragma segment Main
void EventLoop()
{
	Boolean		gotEvent;
	EventRecord	event;
	
	do {
		gotEvent = WaitNextEvent(everyEvent, &event, 1, nil);
		if ( gotEvent ) {
			DoEvent(&event);
		}
		ServerIdle();
	} while ( gDoneFlag );	/* loop forever; we quit via ExitToShell */
	
	gotEvent = true;
} /*EventLoop*/


/* Do the right thing for an event. Determine what kind of event it is, and call
 the appropriate routines. */

#pragma segment Main

void DoEvent(EventRecord	*event)
{

	short		part, err;
	WindowPtr	window;
	char		key;

	switch ( event->what ) {
		case kHighLevelEvent:
			err = AEProcessAppleEvent( event ) ;
			break;
		case mouseDown:
			part = FindWindow(event->where, &window);
			switch ( part ) {
				case inMenuBar:				/* process a mouse menu command (if any) */
					DoMenuCommand(MenuSelect(event->where));
					break;
				case inSysWindow:			/* let the system handle the mouseDown */
					SystemClick(event, window);
					break;
			}
			break;
		case keyDown:
		case autoKey:						/* check for menukey equivalents */
			key = event->message & charCodeMask;
			if ( event->modifiers & cmdKey )			/* Command key down */
				if ( event->what == keyDown ) {
					DoMenuCommand(MenuKey(key));
				}
			break;
		case kOSEvent:
			switch ((event->message >> 24) & 0x0FF) {		/* high byte of message */
				case kSuspendResumeMessage:		/* suspend/resume is also an activate/deactivate */
					gInBackground = (event->message & kResumeMask) == 0;
					break;
			}
			break;
	}
} /*DoEvent*/

/*	This is called when an item is chosen from the menu bar (after calling
	MenuSelect or MenuKey). It performs the right operation for each command.
	It is good to have both the result of MenuSelect and MenuKey go to
	one routine like this to keep everything organized. */

#pragma segment Main
void DoMenuCommand(long		menuResult)
{
	short		menuID;				/* the resource ID of the selected menu */
	short		menuItem;			/* the item number of the selected menu */
	short		itemHit;
	Str255		daName;
	short		daRefNum;
	Boolean		handledByDA;

	menuID = HiWord(menuResult);	/* use macros for efficiency to... */
	menuItem = LoWord(menuResult);	/* get menu item number and menu number */
	switch ( menuID ) {
		case mApple:
			switch ( menuItem ) {
				case iAbout:		/* bring up alert for About */
					itemHit = Alert(rAboutAlert, nil);
					break;
				default:			/* all non-About items in this menu are DAs */
					GetItem(GetMHandle(mApple), menuItem, daName);
					daRefNum = OpenDeskAcc(daName);
					break;
			}
			break;
		case mFile:
			switch ( menuItem ) {
				case iConfigure:
					ServerConfigure();
					break;
				case iQuit:
					gDoneFlag = false;
					break;
			}
			break;
		case mEdit:					/* call SystemEdit for DA editing & MultiFinder */
			handledByDA = SystemEdit(menuItem-1);	/* since we dont do any Editing */
			break;
	}
	HiliteMenu(0);					/* unhighlight what MenuSelect (or MenuKey) hilited */
} /*DoMenuCommand*/


#pragma segment Initialize
void Initialize()
{
	Handle		menuBar;
	EventRecord event;
	short		count;
	OSErr		err;

	gInBackground = false;
	
	InitGraf((Ptr) &qd.thePort);
	InitFonts();
	InitWindows();
	InitMenus();
	TEInit();
	InitDialogs(nil);
	InitCursor();
	
	//InstallAEHandlers();
	menuBar = GetNewMBar(rMenuBar);			/* read menus into menu bar */
	if ( menuBar == nil ) ExitToShell();
	SetMenuBar(menuBar);					/* install menus */
	DisposHandle(menuBar);
	AddResMenu(GetMHandle(mApple), 'DRVR');	/* add DA names to Apple menu */
	DrawMenuBar();
	
	InstallHandlers();
	
	ServerInit();	//Start up our server
	
	for (count = 1; count <= 3; count++)
	{
		if (WaitNextEvent(everyEvent, &event, 5, nil))
		{
			if (event.what==kHighLevelEvent)
			{
			err = AEProcessAppleEvent( &event ) ;
			}

		}
	}
	
} /*Initialize*/

