/**********************************************************************************
	FILE: IPLSEndPoint.cp
	Description: This is the overrides of the generic server classes, to
		handle the requests/registration calls.  The generic server/handler
		classes make this very simple.

	1997, CE Software, Inc.  All rights reserved.
	P.O. Box 65580
	West Des Moines, Iowa 50265 U.S.A.

***********************************************************************************/

#include "IPLSEndPoint.h"
#include "OTHandlerEndPoint.h"
#include "IPLSServer.h"
#include <string.h>

//
// Here is the class definition of our Handler EndPoint.  We don't need
// to make it public since it is only called by our server class
//
class IPLSHandlerEndPoint: public OTHandlerEndPoint
{
	public:
	
		IPLSHandlerEndPoint( OTServerEndPoint* theserver, short handlerID );
		virtual	~IPLSHandlerEndPoint	( void ) { }
	
	protected:	
		virtual void HandleData (char* thestring, short numchars);
};

//
// The server constructor does nothing, just here to call the base constructor
//
IPLSServer::IPLSServer( unsigned short port ): OTServerEndPoint(port)
{
}

//
// We just override the CreateHandler method so we've got
// IPLSHandlerEndPoints, not plain old OTHandlerEndPoints
//
OTHandlerEndPoint* IPLSServer::CreateHandler (short HandlerID)
{
	return new IPLSHandlerEndPoint(this, HandlerID);
}


// and here's our handler!

//
// The handler constructor does nothing, just here to call the base constructor
//
IPLSHandlerEndPoint::IPLSHandlerEndPoint(OTServerEndPoint* theserver, short handlerID)
	: OTHandlerEndPoint(theserver, handlerID)
{
}

static void AddAChar(char c, Str255 tempstr)
{
	tempstr[0]++; tempstr[tempstr[0]]=c;
}

static void AddANum(short num, Str255 tempstr)
{
	AddAChar(num/10 + '0', tempstr);
	AddAChar(num%10 + '0', tempstr);
}

// Separate routine to create the starting time routine
static void FormLastStartDate(Str255 tempstr)
{
	DateTimeRec	thetime;
	
	SecondsToDate (gStartupDate, &thetime);
	
	PLstrcpy(tempstr, "\p181 ");			//181 yy-mm-dd-hh:mm LAST STARTED<cr><lf>
	AddANum(thetime.year%100, tempstr); AddAChar('-', tempstr);
	AddANum(thetime.month, tempstr); AddAChar('-', tempstr);
	AddANum(thetime.day, tempstr); AddAChar('-', tempstr);
	AddANum(thetime.hour, tempstr); AddAChar(':', tempstr);
	AddANum(thetime.minute, tempstr);
	
	PLstrcat(tempstr, "\p LAST STARTED\r\n");
}

// Separate routine to get the list of services.  We only list 15 services
#define	MaxServices	15
static void FormServiceList(Handle& thestuff)
{
	OSType	SavedService[MaxServices];
	short	index;
	short	index2;
	Boolean	foundit;
	short	numsaved = 0;
	Str255	tempstr;
	
	for (index = 0; index < MaxServices; index++)	//clear the array
		SavedService[index] = 0;
	
	thestuff = NewHandle(0);
	
	// Get the list of services we've already got
	for (index=0; ((index<gServerPrefs.ipspNumEntries)&&(numsaved<MaxServices)); index++)
	{
		if (gServerData[index].iplsRegisterType)
		{
			// See if this service type is already in the list.  If not, add it.
			foundit = false;
			for (index2 = 0; index2<numsaved; index2++)
			{
				if (SavedService[index2]==gServerData[index].iplsRegisterType)
				{
					foundit = true;
				}
			}
			if (!foundit)
			{
				//put this service in the list so we don't add it again
				SavedService[numsaved] = gServerData[index].iplsRegisterType;
				numsaved++;
				
				PLstrcpy(tempstr, "\p120 xxxx ");
				BlockMove(&gServerData[index].iplsRegisterType, &tempstr[5], 4);
				switch (gServerData[index].iplsRegisterType)
				{
					case 'http':
						PLstrcat(tempstr, "\pWeb Page");
						break;
					case 'ftp_':
						PLstrcat(tempstr, "\pFTP file transfer server");
						break;
					default:
						PLstrcat(tempstr, "\pUnknown service");
				}
				tempstr[0]++; tempstr[tempstr[0]]= 0x0D; //return
				tempstr[0]++; tempstr[tempstr[0]]= 0x0A; //line feed
				PtrAndHand (&tempstr[1], thestuff, tempstr[0]);
				
			}
		}
	}
	PLstrcpy(tempstr, "\p190 EOF\r\n");
	PtrAndHand (&tempstr[1], thestuff, tempstr[0]);
}

//
// Here is the meat of the server.  This gets called every time a string
// (ended by <cr><lf>) has been received.  For us, we get our command,
// get our parameters, and send back the response
//
// Implementation note--thestring points into a buffer and there may
// be characters after this line already in the buffer, so we must
// not zero terminate the string by writing a zero after our string
//
void IPLSHandlerEndPoint::HandleData (char* thestring, short numchars)
{
	char	outstr[2048];
	OSType	typetosearch;
	Handle	thestuff;
	Str15	thepass;
	Str31	thename;
	Str255	tempstr;
	short	index;
	IPLSEntry temprecord;
	Boolean	foundIP;
	
	//This handles our input strings
	switch (thestring[0])
	{
		case 'V':	//Get version number and info and such
			
			thestuff = NewHandle(0);
			
			PLstrcpy(tempstr, "\p180 1.0 Version\r\n");			//version 1.0
			PtrAndHand (&tempstr[1], thestuff, tempstr[0]);
			
			FormLastStartDate(tempstr);
			PtrAndHand (&tempstr[1], thestuff, tempstr[0]);
			
			NumToString(recordsInUse, tempstr); tempstr[tempstr[0]+1] = 0;
			strcpy(outstr, "182 ");
			strcat(outstr, (char*)&tempstr[1]);
			strcat(outstr, " RECORDS STORED\r\n");
			PtrAndHand (outstr, thestuff, strlen(outstr));
			
			PLstrcpy(tempstr, "\p190 EOF\r\n");
			PtrAndHand (&tempstr[1], thestuff, tempstr[0]);
			
			HLock(thestuff);
			SendData(*thestuff, GetHandleSize(thestuff));
			DisposeHandle(thestuff);
			break;
			
		case 'R':	//Register/unregister
			BlockMove(&thestring[3], &temprecord.iplsRegisterType, 4);
			
			thestring += 8;	//skip past the "RE<tab>xxxx<tab>"
			numchars -= 8;
			
			temprecord.iplsUserName[0] = 0;				//copy the name over
			for ( ; (*thestring!= 0x09)&&(numchars>0); thestring++)
			{
				if (temprecord.iplsUserName[0]<31)
				{
					temprecord.iplsUserName[0]++;
					temprecord.iplsUserName[temprecord.iplsUserName[0]]=*thestring;
				}
				numchars--;
			}
			thestring++; numchars --;	//skip past the tab
			
			temprecord.iplsURL[0] = 0;				//copy the URL over
			for ( ; (*thestring!= 0x09)&&(numchars>0); thestring++)
			{
				if (temprecord.iplsURL[0]<63)
				{
					temprecord.iplsURL[0]++;
					temprecord.iplsURL[temprecord.iplsURL[0]]=*thestring;
				}
				numchars--;
			}
			thestring++; numchars --;	//skip past the tab
			
			temprecord.iplsPersonalPassword[0] = 0;				//copy the personal password over
			for ( ; (*thestring!= 0x09)&&(*thestring!= 0x0D)&&(numchars>0); thestring++)
			{
				if (temprecord.iplsPersonalPassword[0]<15)
				{
					temprecord.iplsPersonalPassword[0]++;
					temprecord.iplsPersonalPassword[temprecord.iplsPersonalPassword[0]]=*thestring;
				}
				numchars--;
			}
			temprecord.iplsLastEntered = 0;
			thestring++; numchars --;	//skip past the tab
			
			thepass[0] = 0;				//copy the registration password over
			for ( ; (*thestring!= 0x0D)&&(*thestring!= 0x09)&&(numchars>0); thestring++)
			{
				if (thepass[0]<15)
				{
					thepass[0]++;
					thepass[thepass[0]]=*thestring;
				}
				numchars--;
			}
			// Check on the password matching, if any.
			if ((gServerPrefs.ipspRegisterPassword[0]>0)&&
				(!EqualString (thepass, gServerPrefs.ipspRegisterPassword, false, false)))
			{
				strcpy(outstr, "999 BAD PASSWORD\r\n");
				SendData(outstr, strlen(outstr));
			}
			else if (AddRecord(&temprecord))
				strcpy (outstr, "150 Successful Registration\r\n");
				else strcpy (outstr, "993 Server full\r\n");
			SendData(outstr, strlen(outstr));
			break;
		case 'S':	//list services
			FormServiceList(thestuff);
			HLock(thestuff);
			SendData(*thestuff, GetHandleSize(thestuff));
			DisposeHandle(thestuff);
			break;
		case 'L':	//list persons
			if (numchars<9)
			{
				strcpy(outstr, "991 BAD FORMAT\r\n");
				SendData(outstr, strlen(outstr));
			}
			else
			{
				BlockMove(&thestring[3], &typetosearch, 4);
				
				thepass[0] = 0;
				for (index=8; index<numchars-2; index++)
				{
					if (thepass[0]<15)
					{
						thepass[0]++; thepass[thepass[0]]=thestring[index];
					}
				}
				// Check on the password matching, if any.
				if ((gServerPrefs.ipspLookupPassword[0]>0)&&
					(!EqualString (thepass, gServerPrefs.ipspLookupPassword, false, false)))
				{
					strcpy(outstr, "999 BAD PASSWORD\r\n");
					SendData(outstr, strlen(outstr));
				}
				else
				{
					//OK, really send the list
					thestuff = NewHandle(0);
					
					for (index=0; index<gServerPrefs.ipspNumEntries; index++)
					{
						if (gServerData[index].iplsRegisterType==typetosearch)
						{
							PLstrcpy(tempstr, "\p100 ");
							PLstrcat(tempstr, gServerData[index].iplsUserName);
							tempstr[0]++; tempstr[tempstr[0]]= 0x0D; //return
							tempstr[0]++; tempstr[tempstr[0]]= 0x0A; //line feed
							PtrAndHand (&tempstr[1], thestuff, tempstr[0]);
						}
					}
					PLstrcpy(tempstr, "\p190 EOF\r\n");
					PtrAndHand (&tempstr[1], thestuff, tempstr[0]);
					
					HLock(thestuff);
					SendData(*thestuff, GetHandleSize(thestuff));
					DisposeHandle(thestuff);
				}
			}
			break;
		case 'I':	//Give URL
			foundIP = false;
			if (numchars<12)
			{
				strcpy(outstr, "991 BAD FORMAT\r\n");
				SendData(outstr, strlen(outstr));
			}
			else
			{
				BlockMove(&thestring[3], &typetosearch, 4);
				
				//Get password and username
				thepass[0] = 0;
				for (index=8; (index<numchars-2)&&(thestring[index]!=0x09 /*tab*/); index++)
				{
					if (thepass[0]<15)
					{
						thepass[0]++; thepass[thepass[0]]=thestring[index];
					}
				}
				index++;	//skip over the dash between password and name;
				thename[0] = 0;
				for (; (index<numchars-2); index++)
				{
					if (thename[0]<31)
					{
						thename[0]++; thename[thename[0]]=thestring[index];
					}
				}
				
				for (index=0; index<gServerPrefs.ipspNumEntries; index++)
				{
					if ((gServerData[index].iplsRegisterType==typetosearch)
						&&(EqualString (thename, gServerData[index].iplsUserName, false, false)))
					{
						foundIP = true;
						
						// Check on the password matching, if any.
						if (gServerData[index].iplsPersonalPassword[0]>0) // personal password
						{
							if (!EqualString (thepass, gServerData[index].iplsPersonalPassword, false, false))
							{
								strcpy(outstr, "999 BAD PASSWORD\r\n");
								SendData(outstr, strlen(outstr));
								return;
							}
						}
						else if (gServerPrefs.ipspLookupPassword[0]>0) // main password
						{
							if (!EqualString (thepass, gServerPrefs.ipspLookupPassword, false, false))
							{
								strcpy(outstr, "999 BAD PASSWORD\r\n");
								SendData(outstr, strlen(outstr));
								return;
							}
						}
						
						PLstrcpy(tempstr, "\p110 ");
						PLstrcat(tempstr, gServerData[index].iplsURL);
						tempstr[0]++; tempstr[tempstr[0]]= 0x0D; //return
						tempstr[0]++; tempstr[tempstr[0]]= 0x0A; //line feed
						SendData((char*)&tempstr[1], tempstr[0]);
					}
				}
				if (!foundIP)
				{
					strcpy(outstr, "992 NO SUCH ENTRY\r\n");
					SendData(outstr, strlen(outstr));
				}
			}
			break;
		case 'Q':	//quit
			Close();
			break;
		default:
			strcpy(outstr, "992 UNRECOGNIZED COMMAND\r\n");
			SendData(outstr, strlen(outstr));
			break;
	}
}
