/**********************************************************************************
	FILE: IPLSServer.c
	Description: This handles the server actions.  It sets up and mainipulates
		the array of data, the prefs, and lets the server class run.

	1997, CE Software, Inc.  All rights reserved.
	P.O. Box 65580
	West Des Moines, Iowa 50265 U.S.A.

***********************************************************************************/

#include <OpenTransport.h>
#include <OpenTptInternet.h>
#include <PLStringFuncs.h>		/* some special string handling stuff */
#include "IPLSServer.h"
#include "IPLSEndPoint.h"

IPLSPrefs	gServerPrefs;	//the prefs used by the system
IPLSEntry*	gServerData = nil;	//the array of entries

FSSpec		gPrefsSpec;	//the file to put our prefs in

IPLSServer* theserver = nil;
long	recordsInUse = 0;
unsigned long gStartupDate = 0;
//
// This inserts an entry into the array.  It looks for a match on the
// name and type, and either updates the entry or adds a new one if no match.
//
// If the URL is empty, it's a delete not an add
//
Boolean	AddRecord(IPLSEntry* therecord)
{
	short	index;
	short	emptyrec = -1;
	
	GetDateTime (&therecord->iplsLastEntered);
	for (index=0; index<gServerPrefs.ipspNumEntries; index++)
	{
		if ((emptyrec == -1)&&(gServerData[index].iplsUserName[0] == 0))
		{
			emptyrec = index;
		}
		else if ((gServerData[index].iplsRegisterType==therecord->iplsRegisterType)
			&&(EqualString (therecord->iplsUserName, gServerData[index].iplsUserName, false, false)))
		{
			if (therecord->iplsURL[0])
			{
				//update the record
				gServerData[index] = *therecord;
			}
			else
			{
				//delete the record
				gServerData[index].iplsUserName[0]=0;
				gServerData[index].iplsRegisterType = 0;
				gServerData[index].iplsURL[0] = 0;
			}
			return true;
		}
	}
	if (emptyrec != -1)	//is there an empty rec?
	{
		if (therecord->iplsURL[0])	//and we were trying to add
			return false;
	}
	if (therecord->iplsURL[0])	//Was there a place to add it?
	{
		//Add it
		gServerData[emptyrec] = *therecord;
		recordsInUse++;
	}
	return true;
}

//
// Some routines to fill the server with fake data for testing
//
static void AddOneEntry(Str31 name, OSType regtype, Str63 URL)
{
	IPLSEntry temprecord;
	
	PLstrcpy( temprecord.iplsUserName, name);
	temprecord.iplsRegisterType = regtype;
	PLstrcpy( temprecord.iplsURL, URL);
	temprecord.iplsLastEntered = 0;
	temprecord.iplsPersonalPassword[0] = 0;
	AddRecord(&temprecord);
}
static void FillData( void )
{
	AddOneEntry("\pBrown|Donald", 'QTlk', "\pDonTalker");
	AddOneEntry("\pBrown|Donald", 'http', "\phttp://www.cesoft.com");
	AddOneEntry("\pBrown|Donald", 'ftp_', "\pftp://123.123.123.123:152");
	AddOneEntry("\pCavanaugh|Tom", 'QTlk', "\pTomTalker");
	AddOneEntry("\pCavanaugh|Tom", 'http', "\phttp://design.cesoft.com");
	AddOneEntry("\pCavanaugh|Tom", 'ftp_', "\pftp://123.123.123.123:Tom");
	AddOneEntry("\pSkeie|Richard", 'QTlk', "\pRichardTalker");
	AddOneEntry("\pSkeie|Richard", 'ftp_', "\pftp://123.123.123.123:Richard");
}

//
//  On startup, we create the data, read in the prefs, and start the server
//
void ServerInit(void)	//read in the prefs (configure if necessary), open the endpoints, etc.
{
	OSErr	theerr;
	short	vref;
	long	dirID;
	short	refnum;
	long	count;
	
	theerr = FindFolder(kOnSystemDisk, kPreferencesFolderType, false, &vref, &dirID);
	theerr = FSMakeFSSpec(vref, dirID, "\pIPLS Server.prefs", &gPrefsSpec);
	theerr = FSpOpenDF(&gPrefsSpec, 0, &refnum);
	if (theerr==noErr)
	{
		count = sizeof(gServerPrefs);
		theerr = FSRead(refnum, &count, &gServerPrefs);
		theerr = FSClose(refnum);
	}
	else
	{
		gServerPrefs.ipspLookupPort = 1026;
		gServerPrefs.ipspRegisterPassword[0] = 0;	//default to no passwords
		gServerPrefs.ipspLookupPassword[0] = 0;
		gServerPrefs.ipspNumEntries = 100; //This may be read from a resource or whatever
		
		if (!ServerConfigure())
		{
			ExitToShell();	//if the user cancelled setup, quit
		}
	}
	
	gServerData = (IPLSEntry*) NewPtrClear( gServerPrefs.ipspNumEntries*sizeof(IPLSEntry));
	if (!gServerData)
	{
		DebugStr("\pUnable to allocate data storage");
		ExitToShell();
	}
	
	//FillData();	//Fill with fake data, not to be in the final server
	
	GetDateTime (&gStartupDate);	//remember when we started
	
	if (InitOpenTransport()!=kOTNoError)
	{
		InitCursor();
		StopAlert(140, nil);
		ExitToShell();
	}
	
	theserver = new IPLSServer(gServerPrefs.ipspLookupPort);	// Open TCP/IP listening port
	theserver->StartHandlers();
	
}

static void BlinkButton(DialogPtr theDialog,
				short btnNumber)
{
	Handle				itemHandle;
	short				itemType;
	Rect				itemRect;
	long				ignoreLong;
	
	GetDItem(theDialog, btnNumber, &itemType, &itemHandle, &itemRect);
	HiliteControl((ControlHandle)itemHandle, true);
	Delay(7, &ignoreLong);
	HiliteControl((ControlHandle)itemHandle, false);
}
	
static pascal void OutlineDialogItem(DialogPtr thedialog, short itemnum)
{
	short	dtype;
	Handle	ditem;
	Rect	rview;

	GetDialogItem(thedialog,itemnum,&dtype,&ditem,&rview);
	PenSize(2,2);
	FrameRoundRect(&rview, 16, 16);
}

static pascal Boolean ConfigDialogEventFilter(DialogPtr pDialog,
									EventRecord *macEvent,
									short *itemHit)
	{
	#define kDelay			7
	
	short	itemType;
	ControlHandle	itemHandle;
	Rect	itemRect;

	switch (macEvent->what)
		{
		case keyDown:
		case autoKey:
			switch (macEvent->message & charCodeMask)
				{
				case 0x03:		// enter key
				case 0x0D:		// return key
					GetDItem(pDialog, 1, &itemType, &((Handle)itemHandle), &itemRect);
					if ((*itemHandle)->contrlHilite)
					{
						macEvent->what = 0;
						return	false;
					}
					else
					{
						BlinkButton(pDialog, 1);
						*itemHit = 1;
						return true;
					}
				case 0x1B:		// escape key
					BlinkButton(pDialog, 2);
					*itemHit = 2;
					return true;
				case '.':		//command-period
					if (macEvent->modifiers & cmdKey)
					{
						BlinkButton(pDialog, 2);
						*itemHit = 2;
						return true;
					}
				default:
					return false;
				}
			break; //of keydown
		default:
			return false;
		}
	}

Boolean ServerConfigure(void)	//Have user enter our complex configurations
{
	DialogPtr mydialog;
	short	dtype;
	Handle	ditem;
	Rect	rview;
	short	itemhit;
	
	IPLSPrefs	tempprefs;
	
	Str255	tempstr;
	long	templong;

	OSErr	theerr;
	short	refnum;
	long	count;
	
	
	ModalFilterUPP	OurFilterUPP;
	UserItemUPP	OurItemUPP;
	
	BlockMove(&gServerPrefs, &tempprefs, sizeof(IPLSPrefs));
	
	mydialog=GetNewDialog(129,nil,(WindowPtr)-1);
	
	OurItemUPP = NewUserItemProc(OutlineDialogItem);	//outline the OK button
	GetDialogItem(mydialog,12,&dtype,&ditem,&rview);
	SetDialogItem(mydialog,12,dtype,(Handle)OurItemUPP,&rview);
	
	GetDialogItem(mydialog,3,&dtype,&ditem,&rview);	//registration password
	SetDialogItemText (ditem, tempprefs.ipspRegisterPassword);
	
	GetDialogItem(mydialog,4,&dtype,&ditem,&rview);	//lookup password
	SetDialogItemText (ditem, tempprefs.ipspLookupPassword);
	
	GetDialogItem(mydialog,5,&dtype,&ditem,&rview);	//lookup port
	NumToString(tempprefs.ipspLookupPort, tempstr);
	SetDialogItemText (ditem, tempstr);
	
	GetDialogItem(mydialog,6,&dtype,&ditem,&rview);	//max entries
	NumToString(tempprefs.ipspNumEntries, tempstr);
	SetDialogItemText (ditem, tempstr);
	
	// Just call ModalDialog with a simple filter for escape, command period, etc.
	OurFilterUPP = NewModalFilterProc(ConfigDialogEventFilter);
	InitCursor();
	do
	{
		ModalDialog(OurFilterUPP, &itemhit);
	}
	while (itemhit>2);
	DisposeRoutineDescriptor(OurFilterUPP);
	
	if (itemhit==ok)
	{
		GetDialogItem(mydialog,3,&dtype,&ditem,&rview);	//registration password
		GetDialogItemText (ditem, tempstr);
		if (tempstr[0]>15) tempstr[0] = 15;
		PLstrcpy(tempprefs.ipspRegisterPassword, tempstr);
		
		GetDialogItem(mydialog,4,&dtype,&ditem,&rview);	//lookup password
		GetDialogItemText (ditem, tempstr);
		if (tempstr[0]>15) tempstr[0] = 15;
		PLstrcpy(tempprefs.ipspLookupPassword, tempstr);
		
		GetDialogItem(mydialog,5,&dtype,&ditem,&rview);	//lookup port
		GetDialogItemText (ditem, tempstr);
		StringToNum(tempstr, &templong);
		tempprefs.ipspLookupPort = templong;
		
		if (!gServerData)	//we can only change the number of entries if we haven't allocated yet
		{
			GetDialogItem(mydialog,6,&dtype,&ditem,&rview);	//max entries
			GetDialogItemText (ditem, tempstr);
			StringToNum(tempstr, &templong);
			tempprefs.ipspNumEntries = templong;
		}
		
		BlockMove(&tempprefs, &gServerPrefs, sizeof(IPLSPrefs));
	
		theerr = FSpCreate(&gPrefsSpec, 'IPLS', 'DATA', smSystemScript);
		theerr = FSpOpenDF(&gPrefsSpec, 0, &refnum);
		if (theerr==noErr)
		{
			count = sizeof(gServerPrefs);
			theerr = FSWrite(refnum, &count, &gServerPrefs);
			theerr = FSClose(refnum);
		}
	}
	
	DisposeRoutineDescriptor(OurItemUPP);
	DisposeDialog(mydialog);
	
	return (itemhit==ok);

}

void ServerIdle(void)	//this is called to check on the packets, handle connections, etc.
{
	theserver->DoIdle();
}

void ServerShutdown(void)	//close the endpoints, etc.
{
	delete theserver;
	theserver = nil;
	
	CloseOpenTransport();
}
