#include "ClassicPane.h"
#include "Appearance.h"
#include "AppearanceHelpers.h"
#include "UDialogUtils.h"

#define MIN( a, b )		( ( (a) < (b) ) ? (a) : (b) )
#define MAX( a, b )		( ( (a) > (b) ) ? (a) : (b) )

enum
{
	kEngageButton		 	= 1,
	kMakeItSoButton			= 2,
	kEjectCoreButton		= 3,
	kRaiseShieldsButton		= 4,
	kUserItem				= 6,
	kVerticalScrollBar		= 7,
	kHorizontalScrollBar	= 8,
	kTuneUpCheckBox 		= 9,
	kConstrictorsCheckBox	= 10,
	kIntermixCheckBox		= 11,
	kDessertRadio			= 12,
	kFloorWaxRadio			= 13,
	kBothButton				= 14
};

enum
{
	kConstrictorsMask		= 1,
	kIntermixMask			= 2,
	kAllCheckMask			= 3
};

ControlActionUPP	ClassicPane::fScrollBarProc = NewControlActionProc( ClassicPane::ScrollingFeedbackProc );
ControlUserPaneDrawUPP	ClassicPane::fDrawProc = NewControlActionProc( ClassicPane::DrawPictureProc );

ClassicPane::ClassicPane( DialogPtr dialog, SInt16 items ) : MegaPane( dialog, items )
{
	ControlHandle		control;
	
	AppendDialogItemList( dialog, 6003, overlayDITL );

	GetDialogItemAsControl( dialog, kTuneUpCheckBox + items, &control );
	SetControlMaximum( control, 2 );

	GetDialogItemAsControl( dialog, kMakeItSoButton + items, &control );
	SetPushButtonDefaultState( control, true );
	
	GetDialogItemAsControl( dialog, kEjectCoreButton + items, &control );
	DeactivateControl( control );

	GetDialogItemAsControl( dialog, items + kHorizontalScrollBar, &fHorizontalScrollBar );
	GetDialogItemAsControl( dialog, items + kVerticalScrollBar, &fVerticalScrollBar );
	
	SetControlReference( fHorizontalScrollBar, (long)this );
	SetControlReference( fVerticalScrollBar, (long)this );
	
	fPictOffset.h = 0;
	fPictOffset.v = 0;
	
	fPicture = GetPicture( 6003 );
	fPictWidth = (**fPicture).picFrame.right - (**fPicture).picFrame.left;
	fPictHeight = (**fPicture).picFrame.bottom - (**fPicture).picFrame.top;
	
	GetDialogItemAsControl( dialog, items + kUserItem, &fUserItem );
	SetControlReference( fUserItem, (long)this );
	SetControlData( fUserItem, 0, kControlUserPaneDrawProcTag, sizeof( fDrawProc ), (Ptr)&fDrawProc );

	fUserItemHeight = (**fUserItem).contrlRect.bottom - (**fUserItem).contrlRect.top;
	fUserItemWidth = (**fUserItem).contrlRect.right - (**fUserItem).contrlRect.left;
	
	SetControlMaximum( fHorizontalScrollBar, fPictWidth - fUserItemWidth );
	SetControlMaximum( fVerticalScrollBar, fPictHeight - fUserItemHeight );
	
	SetControlAction( fHorizontalScrollBar, fScrollBarProc );
	SetControlAction( fVerticalScrollBar, fScrollBarProc );
	
	fGroup = new RadioGroup;
	GetDialogItemAsControl( dialog, fOrigItems + kDessertRadio, &control );
	SetControlMaximum( control, kControlCheckboxMixedValue );
	fGroup->AddControl( control, kDessertRadio );
	GetDialogItemAsControl( dialog, fOrigItems + kFloorWaxRadio, &control );
	SetControlMaximum( control, kControlCheckboxMixedValue );
	fGroup->AddControl( control, kFloorWaxRadio );
	
	fGroup->SetValue( kDessertRadio );
}

ClassicPane::~ClassicPane()
{
	ShortenDITL( fDialog, CountDITL( fDialog ) - fOrigItems );
	
	delete fGroup;
}

void
ClassicPane::ItemHit( SInt16 item )
{
	SInt16			localItem;
	ControlHandle	control;
	SInt16			value;
	Boolean			syncCheck = false;
	SInt16			checkValues = 0;
	
	localItem = item - fOrigItems;
	
	switch ( localItem )
	{
		case kTuneUpCheckBox:
			GetDialogItemAsControl( fDialog, item, &control );
			value = GetControlValue( control );
			
			if ( value == kControlCheckboxUncheckedValue )
			{
				SetControlValue( control, kControlCheckboxCheckedValue );
				GetDialogItemAsControl( fDialog, fOrigItems + kConstrictorsCheckBox, &control );
				SetControlValue( control, kControlCheckboxCheckedValue );
				GetDialogItemAsControl( fDialog, fOrigItems + kIntermixCheckBox, &control );
				SetControlValue( control, kControlCheckboxCheckedValue );
			}
			else if ( value == kControlCheckboxCheckedValue )
			{
				SetControlValue( control, kControlCheckboxUncheckedValue );
				GetDialogItemAsControl( fDialog, fOrigItems + kConstrictorsCheckBox, &control );
				SetControlValue( control, kControlCheckboxUncheckedValue );
				GetDialogItemAsControl( fDialog, fOrigItems + kIntermixCheckBox, &control );
				SetControlValue( control, kControlCheckboxUncheckedValue );
			}
			else if ( value == kControlCheckboxMixedValue )
			{
				SetControlValue( control, kControlCheckboxCheckedValue );
				GetDialogItemAsControl( fDialog, fOrigItems + kConstrictorsCheckBox, &control );
				SetControlValue( control, kControlCheckboxCheckedValue );
				GetDialogItemAsControl( fDialog, fOrigItems + kIntermixCheckBox, &control );
				SetControlValue( control, kControlCheckboxCheckedValue );
			}
			break;
			
		case kConstrictorsCheckBox:
		case kIntermixCheckBox:
			UDialogUtils::ToggleCheckBox( fDialog, item );
			GetDialogItemAsControl( fDialog, fOrigItems + kConstrictorsCheckBox, &control );

			if ( GetControlValue( control ) )
				checkValues |= kConstrictorsMask;
			else
				checkValues &= ~kConstrictorsMask;
			
			GetDialogItemAsControl( fDialog, fOrigItems + kIntermixCheckBox, &control );

			if ( GetControlValue( control ) )
				checkValues |= kIntermixMask;
			else
				checkValues &= ~kIntermixMask;
			
			syncCheck = true;
			break;
		
		case kDessertRadio:
		case kFloorWaxRadio:
			fGroup->SetValue( localItem );
			break;
		
		case kBothButton:
			fGroup->SetMixed( kDessertRadio );
			fGroup->SetMixed( kFloorWaxRadio );
			break;
	}
	
	if ( syncCheck )
	{
		GetDialogItemAsControl( fDialog, fOrigItems + kTuneUpCheckBox, &control );

		if ( checkValues == 0 )
			SetControlValue( control, kControlCheckboxUncheckedValue );
		else if ( checkValues == kAllCheckMask )
			SetControlValue( control, kControlCheckboxCheckedValue );
		else
			SetControlValue( control, kControlCheckboxMixedValue );
	}
}

pascal void
ClassicPane::DrawPictureProc( ControlHandle control, SInt16 part )
{
	#pragma unused( part )
	
	Rect			bounds;
	RgnHandle		saveClip;
	ClassicPane*	pane;
	
	pane = (ClassicPane*)GetControlReference( control );
	bounds = (**control).contrlRect;
	
	FrameRect( &bounds );
	InsetRect( &bounds, 1, 1 );
	
	saveClip = NewRgn();
	GetClip( saveClip );
	
	ClipRect( &bounds );

	bounds.top -= pane->fPictOffset.v;
	bounds.left -= pane->fPictOffset.h;
	bounds.bottom = bounds.top + pane->fPictHeight;
	bounds.right = bounds.left + pane->fPictWidth;
	
	DrawPicture( pane->fPicture, &bounds );
	
	SetClip( saveClip );
	DisposeRgn( saveClip );
}

pascal void
ClassicPane::ScrollingFeedbackProc( ControlHandle control, SInt16 part )
{
	SInt16			startValue, delta, min, max;
	ClassicPane*	pane;
	
	pane = (ClassicPane*)GetControlReference( control );

	startValue = GetControlValue( control );
	min = GetControlMinimum( control );
	max = GetControlMaximum( control );
	
	delta = 0;
	
	switch ( part )
	{
		case kControlUpButtonPart:
			if ( startValue > min )
				delta = MAX( -5, min - startValue );
			break;
		
		case kControlDownButtonPart:
			if ( startValue < max )
				delta = MIN( 5, max - startValue );
			break;
		
		case kControlPageUpPart:
			if ( startValue > min )
				if ( control == pane->fHorizontalScrollBar )
					delta = MAX( -(pane->fUserItemWidth - 1), min - startValue );
				else
					delta = MAX( -(pane->fUserItemHeight - 1), min - startValue );
			break;
		
		case kControlPageDownPart:
			if ( startValue < max )
				if ( control == pane->fHorizontalScrollBar )
					delta = MIN( pane->fUserItemHeight - 1, max - startValue );
				else
					delta = MAX( -(pane->fUserItemHeight - 1), min - startValue );
			break;
	}

	if ( delta )
	{
		SetControlValue( control, startValue + delta );
		if ( control == pane->fHorizontalScrollBar )
			pane->fPictOffset.h += delta;
		if ( control == pane->fVerticalScrollBar )
			pane->fPictOffset.v += delta;
		
			// pretty inefficient scrolling here, but you get the point.
			
		DrawOneControl( pane->fUserItem );
	}
	else if ( part == kControlIndicatorPart )
	{
		if ( control == pane->fHorizontalScrollBar )
			pane->fPictOffset.h = startValue - min;
			
		if ( control == pane->fVerticalScrollBar )
			pane->fPictOffset.v = startValue - min;

		DrawOneControl( pane->fUserItem );
	}
}
