/*
	File:		ColorUtils.c

	Contains:	Macros to help drawing in color.

	Version:	Appearance 1.0

	Author:		Edward Voas

	Copyright:	 1996-1997 by Apple Computer, Inc., all rights reserved.
*/

#include "ColorUtils.h"

//
//	 SwitchToWindowManagerPort
//
//	Switches into the window manager port to draw the control. It saves previous state
//	information for later restoring in the data record ptr passed in.
//
void
SwitchToWindowManagerPort( PortSwitchData* data )
{
	GrafPtr			colorPort;
	Point			anchor = { 0, 0 };
	RgnHandle		tempRgn;
	ColorPenState	state;
	FontState		fontState;
	
	data->clipRgn = NewRgn();
	
	GetPort( &data->port );
	GetColorAndPenState( &state, true );
	SaveFontState( &fontState );
	
	LocalToGlobal( &anchor );
	tempRgn = NewRgn();
	SectRgn( data->port->visRgn, data->port->clipRgn, tempRgn );
	GetCWMgrPort( (CGrafPtr*)&colorPort );
	SetPort( colorPort );
	GetClip( data->clipRgn );
	SetOrigin( -anchor.h, -anchor.v );
	SetClip( tempRgn );
	DisposeRgn( tempRgn );
//	GetColorAndPenState( &data->colorState, true );
	SaveFontState( &data->fontState );
//	SetColorAndPenState( &state, true );
	RestoreFontState( &fontState );
}

//
//	 SwitchOutOfWindowManagerPort
//
//	Using the data in the data parameter, we restore some window manager properties and
//	restore our port after successfully calling SwitchToWindowManagerPort and drawing.
//
void
SwitchOutOfWindowManagerPort( PortSwitchData* data )
{
	RestoreFontState( &data->fontState );
//	SetColorAndPenState( &data->colorState, true );
	
	SetClip( data->clipRgn );
	SetOrigin( 0, 0 );
	SetPort( data->port );
	
	DisposeRgn( data->clipRgn );
}

