#include "ProgressPane.h"
#include "Appearance.h"
#include "AppearanceHelpers.h"

enum
{
	kProgress1 = 1,
	kProgress2 = 2,
	kMovie		= 4
};

static OSErr		GetMyFileSpec( FSSpecPtr file );

ProgressPane::ProgressPane( DialogPtr dialog, SInt16 items ) : MegaPane( dialog, items )
{
	AppendDialogItemList( dialog, 6001, overlayDITL );
	
	fProgressValue = 0;
	fIsAscending = true;
	
	GetDialogItemAsControl( dialog, items + kProgress1, &fProgress1 );
	GetDialogItemAsControl( dialog, items + kProgress2, &fProgress2 );
	
	SetProgressIndicatorState( fProgress2, false );

#if MOVIE_HACK	
	fMovie = GetMovie();
	if ( fMovie )
	{
		ControlHandle		control;
		
		GetDialogItemAsControl( dialog, items + kMovie, &control );
		SetControlData( control, 0, 'moov', sizeof( fMovie ), (Ptr)&fMovie );
	}
#endif
}

ProgressPane::~ProgressPane()
{
	ShortenDITL( fDialog, CountDITL( fDialog ) - fOrigItems );
}

void
ProgressPane::Idle()
{
	IdleControls( fDialog );
	
	if ( fIsAscending )
	{
		fProgressValue += 2;
		if ( fProgressValue > 100 )
		{
			fProgressValue = 98;
			fIsAscending = false;
		}
	}
	else
	{
		fProgressValue -= 2;
		if ( fProgressValue < 0 )
		{
			fProgressValue = 2;
			fIsAscending = true;
		}
	}
	SetControlValue( fProgress1, fProgressValue );
}

Movie
ProgressPane::GetMovie()
{
	FSSpec		file;
	OSErr		err;
	SInt16		movieFile;
	Movie		movie;
	Str255		movieName;
	Boolean		wasChanged;
	SInt16		movieResID = 0;
	
	GetMyFileSpec( &file );
	err = FSMakeFSSpec( file.vRefNum, file.parID, "\pMovie Sample", &file );
	if ( err ) return nil;
	
	err = OpenMovieFile( &file, &movieFile, fsRdPerm );
	if ( err ) return nil;
	
	err = NewMovieFromFile( &movie, movieFile, &movieResID, movieName, newMovieActive, &wasChanged );
	CloseMovieFile( movieFile );
	
	return movie;
}

static OSErr
GetMyFileSpec( FSSpecPtr file )
{
	FCBPBRec	pb;
	Str255		ourName;
	OSErr		err;
	
	pb.ioVRefNum	= -1;
	pb.ioFCBIndx	= 0;
	pb.ioNamePtr	= ourName;
	pb.ioRefNum		= CurResFile();
	
	err = PBGetFCBInfoSync( &pb );
	if ( err ) return err;
	
	err = FSMakeFSSpec( pb.ioFCBVRefNum, pb.ioFCBParID, "\pPane Dump", file );
	return err;
}
