//
//	TextPane.cp
//
//	by Edward Voas, Apple Computer, Inc.
//
//	Copyright  1997 Edward Voas, All rights reserved.
//
//	Our text pane definition.
//

#include "TextPane.h"
#include "Appearance.h"
#include "AppearanceHelpers.h"
#include <string.h>

enum
{
	kStaticText			= 1,
	kDisabledStaticText	= 2,
	kEditText 			= 3,
	kPasswordEditText	= 4,
	kEditFilterText		= 5,
	kClock				= 6,
	kListBox			= 7,
	kShowPasswordButton	= 8,
	kPasswordStaticText	= 10
};

enum
{
	kLeftArrow		= 0x1C,
	kRightArrow		= 0x1D,
	kUpArrow		= 0x1E,
	kDownArrow		= 0x1F,
	kBackspace		= 0x08
};

#define kNumStrings		8

static Str255	gStrings[ kNumStrings ] =
{
	"\pBeam Up",
	"\pSite-to-site transport",
	"\pHold in pattern buffer",
	"\pMolecular resolution transport",
	"\pMolecular dispersal (ouch!)",
	"\pNear warp transport",
	"\pTransport at warp",
	"\pBiofilter scan"
};

ControlKeyFilterUPP TextPane::fFilterProc = NewControlKeyFilterProc( TextPane::NumericFilter );

TextPane::TextPane( DialogPtr dialog, SInt16 items ) : MegaPane( dialog, items )
{
	ControlHandle		control;
	ListHandle			list;
	OSErr				err;
	
	AppendDialogItemList( dialog, 6004, overlayDITL );
	
	GetDialogItemAsControl( dialog, items + kDisabledStaticText, &control );
	DeactivateControl( control );
	
	GetDialogItemAsControl( dialog, items + kListBox, &control );
	err = GetListBoxListHandle( control, &list );
	if ( err == noErr )
	{
		Cell		cell;
		int			i;
		
		cell.h = 0;
		
		LDoDraw( false, list );
		for ( i = 0; i < kNumStrings; i++ )
		{
			LAddRow( 1, (**list).dataBounds.bottom, list );
			cell.v = (**list).dataBounds.bottom - 1;
			LSetCell( (Ptr)&gStrings[i] + 1, gStrings[i][0], cell, list );
		}
		LDoDraw( true, list );
	}
	
	GetDialogItemAsControl( dialog, items + kEditFilterText, &control );
	SetEditTextKeyFilter( control, fFilterProc );
}

TextPane::~TextPane()
{
	ShortenDITL( fDialog, CountDITL( fDialog ) - fOrigItems );
}

void
TextPane::Idle()
{
	IdleControls( fDialog );
}

void
TextPane::ItemHit( SInt16 itemHit )
{
	SInt16			localItem = itemHit - fOrigItems;
	ControlHandle	control;
	Str255			text;
	
	switch ( localItem )
	{
		case kShowPasswordButton:
			GetDialogItemAsControl( fDialog, fOrigItems + kPasswordEditText, &control );
			GetEditTextPasswordText( control, text );
			GetDialogItemAsControl( fDialog, fOrigItems + kPasswordStaticText, &control );
			SetStaticTextText( control, text, true );
			break;
	}
}

pascal ControlKeyFilterResult
TextPane::NumericFilter( ControlHandle control, SInt16* keyCode, SInt16* charCode, SInt16* modifiers)
{
	#pragma unused( control, keyCode, modifiers )

	if ( ((char)*charCode >= '0') && ((char)*charCode <= '9') )
		return kControlKeyFilterPassKey;
	
	switch ( *charCode )
	{
		case '-':
		case kLeftArrow:
		case kRightArrow:
		case kUpArrow:
		case kDownArrow:
		case kBackspace:
			return kControlKeyFilterPassKey;

		default:
			SysBeep( 10 );
			return kControlKeyFilterBlockKey;
	}
}

