/*
	GEQDWin.H

	Graphic Elements Windows compatibility module.

	Copyright 1995 by Al Evans. All rights reserved.

	Updated for GE for Windows 95 1.0, 11/29/95

    NOTE:
		
		This version of Graphic Elements for Windows 95 uses the Microsoft
		DirectDraw graphics interface.

		In terms of this interface,  the Mac GDHandle (handle to a graphics device)
		will be approximated by a DirectDraw object	with its cooperative level set
		to DDSCL_NORMAL, so that multiple GEWorlds can be used in multiple windows.

		The offscreen GWorlds used by Graphic Elements are approximated by 
		DirectDrawSurface objects, with the gEWPort of the GEWorld represented
		by the "primary DirectDraw" surface. The stageGWorld and elements are 
		represented by regular "DDSCAPS_OFFSCREENPLAIN" offscreen surfaces. 

*/



#ifndef GEQDWIN

#define GEQDWIN



#ifdef __cplusplus

	extern "C" {

#endif

// Windows only -- DirectDraw object must be created before any other calls to Graphic Elements

HRESULT	InitializeDirectDraw( HWND theWindow );

void TerminateDirectDraw();

LPDIRECTDRAW GetDirectDrawObject();

// 

// Analogs of Mac QuickDraw entities and calls for Windows 95 version


typedef RECT Rect;
typedef HRGN RgnHandle;

// Define a Point which can be accessed using either Mac (h, v) or
// Windows 95 (x, y) naming conventions.

typedef struct { union { POINT WinPoint;
                         struct { long h; long v; };
                       };
               } Point;

typedef enum {v, h} VHSelect;

// Can't use equivalent Windows SRCCOPY etc. because we count on copy modes being < 14 bits

#define srcCopy     0
#define srcOr       1
#define transparent 2

// Mac-compatible text styles

#define normal		0
#define bold		1
#define italic		2
#define underline	4
#define outline		8
#define shadow		16
#define condense	32
#define extend		64

typedef TEXTMETRIC FontInfo;

typedef struct IDirectDrawSurface FAR *BitMap;

typedef struct GWorld {
	Rect		portRect;
	HWND		portWindow;	// nil except for geWorld->gEWPort
	BitMap		portBits;
} GWorld, *GWorldPtr;

// As used in Graphic Elements, these are all equivalent
typedef GWorldPtr GrafPtr;
typedef GWorldPtr CGrafPtr;

// Ugly, but usage is the same
typedef BitMap PixMapHandle;


// Mac-compatible RGBColor
typedef struct RGBColor {
	unsigned short red;
	unsigned short green;
	unsigned short blue;
} RGBColor;

typedef FOURCC OSType;
typedef LPDIRECTDRAW GDHandle;
typedef LPDIRECTDRAWPALETTE CTabHandle;
typedef long Fixed;

typedef short OSErr;

OSErr NewGWorld( GWorldPtr *offscreenGWorld,
                 short  offscrnDepth,
                 Rect *boundsRect,
                 CTabHandle worldColors, GDHandle aGDevice, unsigned long flags);

void DisposeGWorld(GWorldPtr world);
void DisposCTable(CTabHandle cTable);

void GetPort (GrafPtr * savePort);


// Get rid of #define in WINSPOOL.H

#ifdef SetPort

#undef SetPort

#endif



void SetPort (GrafPtr newPort);
void SetOrigin(short h, short v);
void InvalRect(Rect * invalRect); // causes repaint


#define NewRgn() CreateRectRgn(0,0,1,1)

void RectRgn(RgnHandle region, Rect *newRect);
void DisposeRgn(RgnHandle rgn);

// Get rid of definition in WINDOWSX.H
#ifdef CopyRgn
#undef CopyRgn
#endif

void CopyRgn(RgnHandle source, RgnHandle dest);

#define OffsetRgn(r,dh,dv)  OffsetRgn((HRGN)r,dh,dv)

void SectRgn(RgnHandle a, RgnHandle b, RgnHandle result);
void DiffRgn(RgnHandle a, RgnHandle b, RgnHandle result);

// Get rid of definition in WINDOWSX.H
#ifdef UnionRgn
#undef UnionRgn
#endif

void UnionRgn(RgnHandle a, RgnHandle b, RgnHandle result);

#define PtInRgn(pt,rgn) PtInRegion((HRGN)rgn,(pt).h,(pt).v)
#define PtInRect(pt,rect)  PtInRect(rect,(pt).WinPoint)

void GetGWorld(CGrafPtr *port, GDHandle *gdh);
void SetGWorld(CGrafPtr port, GDHandle gdh);
PixMapHandle GetGWorldPixMap(GWorldPtr world);


// Used only in TextGE
void ClipRect(Rect *);


void GlobalToLocal(Point * p);

// Irrelevant for Windows 95 version
#define LockPixels(PixMapHandle)			((Boolean) true)
#define NoPurgePixels(PixMapHandle)
#define UnlockPixels

void EraseRect(Rect * r);
void InsetRect( Rect*, short h, short v );


GE_CALLBACK(void,CopyBits)(const BitMap *srcBits, const BitMap *dstBits,
                        const Rect *srcRect, const Rect *dstRect, short mode, RgnHandle maskRgn);

// Not (yet) using custom blitters in Windows version
#define FastCopyBits CopyBits


// Declare QD calls used on Mac for syntax-check compile under Windows

#if defined(TEST_COMPILE) && defined(TARGET_IS_MACOS)

#define srcCopy     0
#define srcOr       1
#define transparent 2

typedef struct {short dummy;} BitMap;
typedef struct {short dummy;} RGBColor;

typedef struct {Ptr baseAddr; Rect bounds;
                short rowBytes; short pmVersion;} * PixMapPtr;
typedef PixMapPtr * PixMapHandle;

typedef struct {Rect portRect; BitMap portBits;
                short txFont; short txFace; short txSize;} * GrafPtr;
typedef struct {Rect picFrame;} ** PicHandle;
typedef struct {Rect portRect; } * GWorldPtr;
typedef GWorldPtr CGrafPtr;
typedef long OSType;
typedef struct {short dummy;} * GDHandle;
typedef struct {short dummy;} * CTabHandle;
typedef long Fixed;

typedef short OSErr;

OSErr NewGWorld( GWorldPtr *newWorld,
                 long  offscrnDepth,
                 Rect *wRectCpy,
                 CTabHandle worldColors, long fred, long bill);

GDHandle GetGWorldDevice(GWorldPtr);

RgnHandle NewRgn();

void RectRgn(RgnHandle region, Rect * NewValue);
void DisposeGWorld(GWorldPtr);
void DisposeRgn(RgnHandle);
void GetPort (GrafPtr * savePort);
void SetPort (GrafPtr newPort);
void SetOrigin(short h, short v);
void CopyRgn(RgnHandle source, RgnHandle dest);

void OffsetRgn(RgnHandle r, short deltaH, short deltaV);

void SectRgn(RgnHandle a, RgnHandle b, RgnHandle result);
void DiffRgn(RgnHandle a, RgnHandle b, RgnHandle result);
void UnionRgn(RgnHandle a, RgnHandle b, RgnHandle result);
void InvalRect(Rect * invalRect); // causes repaint

Boolean PtInRgn (Point a, RgnHandle b);
Boolean PtInRect (Point a, Rect * b);

void GetGWorld(CGrafPtr * port, GDHandle * world);
void SetGWorld(CGrafPtr gworld, GDHandle world);
PixMapHandle GetGWorldPixMap(CGrafPtr world);
Ptr GetToolTrapAddress(short trapNum);  // Used to get copybits address.
void GetClip(RgnHandle);
void SetClip(RgnHandle);
void GlobalToLocal(Point * p);

PicHandle GetPicture(short resourceNum);

void LoadResource(Handle a);

void ReleaseResource(Handle);
void HLock(Handle);
void LockPixels(PixMapHandle);
void NoPurgePixels(PixMapHandle);
void EraseRect(Rect * r);
void DrawPicture(PicHandle h, Rect * r);

PAS_FUNC(void,CopyBits)(BitMap *srcBits, BitMap *dstBits,
                        Rect *srcRect, Rect *dstRect, short mode, RgnHandle maskRgn);


#endif



#ifdef __cplusplus

	}

#endif



#endif



