/*
	GELoaders.h

	Default element acquisition routines

	For Mac: Load one or more sequential PICT resources into offscreen GWorld

	For Windows: Load one or more sequential .BMP files named "Pn.BMP",
	where "n" is an integer representing the position in the sequence, into an 
	offscreen DIRECTDRAWSURFACE
	
	For BeOS: Load one or more sequential .CBBM files named "Pn.CBBM",
	where "n" is an integer representing the position in the sequence, into a
	BBitMap.
	

*/

#ifndef GELOADERS
#define GELOADERS

#include "DispCtrl.h"

#ifdef __cplusplus
	extern "C" {
#endif

/*
	GraphicLoadFunc which does nothing except initialize the appropriate fields.
	For Graphic Elements which have no permanent offscreen representations, e.g.
	strings or fills.
*/

GE_CALLBACK(Boolean,NoLoader)(GEWorldPtr world, GrafElPtr element,
										short startResNum, short nResources);


#if defined(TARGET_IS_MACOS)

/*
	GraphicLoadFunc which:
	
	1)	Searches to see if the required graphics are already available in this world, 
		and copies their parameters if so,
	2) 	Otherwise, creates a new offscreen GWorld of an appropriate size, and
		loads nResources PICTs from an open resource file into it, starting
		at the PICT with resource number startResNum
		
	This is the default loaderFunc of a newly created GEWorld.
*/

GE_CALLBACK(Boolean,LoadPICTElement)(GEWorldPtr world, GrafElPtr element,
										short startResNum, short nResources);



#endif

#if defined(TARGET_IS_WIN95)

GE_CALLBACK(Boolean,LoadBMPElement)(GEWorldPtr world, GrafElPtr element,
										short startResNum, short nResources);


#endif

#if defined(TARGET_IS_BEOS)

GE_CALLBACK(Boolean,LoadCBBMElement)(GEWorldPtr world, GrafElPtr element,
										short startResNum, short nResources);


#endif

#ifdef __cplusplus
	}
#endif

#endif
