
#include "common.h"

#include <Files.h>
#include <Resources.h>
#include <stdio.h>
#include <QDOffscreen.h>
#include <Script.h>
#include <Errors.h>
#include <assert.h>
#include <Memory.h>

#ifndef bool
	#define bool int
	#define false 0
	#define true 1
#endif

#if 0
#pragma mark ---------------------------------
#pragma mark |  univ pic capture
#pragma mark ---------------------------------
#endif

// convert in place
static void convert_pic( Handle h, CTabHandle ctab )
{
	// setup picture
	Rect bounds = (**(PicHandle) h).picFrame;
	OffsetRect( &bounds, -bounds.left, -bounds.top );
	
	// create offscreen
	GWorldPtr gw;
	OSErr err = NewGWorld( &gw, 8, &bounds, ctab, nil, 0L );
	assert( gw && !err );
	PixMapHandle pm = GetGWorldPixMap( gw );
	bool locked = LockPixels( pm );
	assert( locked );
	
	// draw picture into it
	CGrafPtr save_port;
	GDHandle save_dev;
	GetGWorld( &save_port, &save_dev );
	SetGWorld( gw, nil );
	EraseRect( &bounds ); // in case picture has transparent areas
	DrawPicture( (PicHandle) h, &bounds );
	SetGWorld( save_port, save_dev );
	
	// grab pixels
	int row_bytes = (**pm).rowBytes & 0x3FFF;
	SetHandleSize( h, (long) bounds.right * row_bytes );
	assert( !MemError() );
	char* src = (**pm).baseAddr;
	char* dest = *h;
	*(int16*) dest = bounds.right;
	dest += 2;
	*(int16*) dest = bounds.bottom;
	dest += 2;
	for ( int v = bounds.bottom; --v >= 0; )
	{
		BlockMoveData( src, dest, bounds.right );
		dest += bounds.right;
		src += row_bytes;
	}
	
	DisposeGWorld( gw );
}

// insure bytes are written to file successfully
static void fwrite_chk( const void* p, size_t size, FILE* file )
{
	if ( size == 0 ) return;
	
	int nwritten = fwrite( p, size, 1, file );
	assert( nwritten == 1 );
}

// an entry in the table of contents
struct resfile_flat_item
{
	int32 type;
	int16 id;
	int16 pad;
	int32 size;
	int32 data_offset;
};

assert_anywhere( sizeof (resfile_flat_item) == 16 );

int main()
{
	InitGraf( &qd.thePort );
	
	FSSpec spec;
	OSErr err;
	
	// open source file
	err = FSMakeFSSpec( 0, 0, "\pmac.rsrc", &spec );
	assert( !err );
	short src_file = FSpOpenResFile( &spec, fsRdPerm );
	assert( src_file != -1 );
	UseResFile( src_file );
	
	// load color table
	CTabHandle ctab = GetCTable( 128 );
	
	int num_items = 0;
	Handle toc_h = NewHandle( 0 );
	assert( toc_h );
	Handle data_h = NewHandle( 0 );
	assert( data_h );
	uint16 num_resources = 0;
	
	// convert resources and convert picts
	for ( int i = Count1Types() + 1; --i; )
	{
		UInt32 type;
		Get1IndType( &type, i );
		for ( int i = Count1Resources( type ) + 1; --i; )
		{
			// get resource
			Handle h = Get1IndResource( type, i );
			assert( h && *h );
			HNoPurge( h );
			
			// get resource info
			short id;
			Str255 str;
			GetResInfo( h, &id, &type, str );
			long new_type = type;
			
			// pict - convert
			if ( type == 'PICT' )
			{
				convert_pic( h, ctab );
				new_type = 'IMAG';
			}
			
			// add toc entry
			resfile_flat_item entry;
			entry.type = new_type;
			entry.id = id;
			entry.size = GetHandleSize( h );
			entry.data_offset = GetHandleSize( data_h );
			err = PtrAndHand( &entry, toc_h, sizeof entry );
			assert( !err );
			
			// add resource data
			err = HandAndHand( h, data_h );
			assert( !err );
			
			ReleaseResource( h );
			
			++num_resources;
		}
	}
	
	// write data to file
	FILE* const file = fopen( "data.flat", "wb" );
	assert( file );
	long toc_size = GetHandleSize( toc_h );
	uint32 data_offset = 6 + toc_size;
	fwrite_chk( &data_offset, 4, file );
	fwrite_chk( &num_resources, 2, file );
	fwrite_chk( *toc_h, toc_size, file );
	fwrite_chk( *data_h, GetHandleSize( data_h ), file );
	fclose( file );
	
	CloseResFile( src_file );
	
	return 0;
}

