/*
	DemoGraphicTypes.cpp
	
	Graphic Types for MacHack 97 demo
	
	Al Evans
	
	6/12/97
	
*/

#include <string.h>
#include "DemoGraphicTypes.h"
#include "DemoConversions.h"
#include "DemoNetAccess.h"
#include "Rects.h"
#include "reschain.h"
#include "byteorder.h"

#if TARGET_IS_WIN95
#include "GEMouse.h"
#include "GEMem.h"
#endif

#if 0
#pragma mark ---------------------------------
#pragma mark |  Background
#pragma mark ---------------------------------
#endif

GrafElPtr MakeNewBackground(GEWorldPtr world, int bkgResID)
{
	return NewBasicPICT(world, 'BKG ', 0, bkgResID, srcCopy, 0, 0);
}

#if 0
#pragma mark ---------------------------------
#pragma mark | Bouncer
#pragma mark ---------------------------------
#endif

static GEDirection CheckLimits(Rect *objRect, Rect *limitRect)
{
	if (objRect->right >= limitRect->right)
		return right;
	if (objRect->left <= limitRect->left)
		return left;
	if (objRect->top <= limitRect->top)
		return up;
	if (objRect->bottom >= limitRect->bottom)
		return down;
	return none;
}

static void BounceObj(GEDirection direction, short *xMove, short *yMove)
{
	switch (direction) {
		case right:
			if (*xMove > 0)
				*xMove = -*xMove;
			break;
		case left:
			if (*xMove < 0)
				*xMove = -*xMove;
			break;
		case up:
			if (*yMove < 0)
				*yMove = -*yMove;
			break;
		case down:
			if (*yMove > 0)
				*yMove = -*yMove;
			break;
	}
}

GE_CALLBACK(void, DoBallRandom)(GEWorldPtr world, GrafElPtr obj)
{
	GEDirection collisionDir;

	register short *currXMove = (short *) &obj->changeData + 1;
	register short *currYMove = (short *) &obj->changeData;
	
	collisionDir = CheckLimits(&obj->animationRect, &world->animationRect);
	if (collisionDir != none)
		BounceObj(collisionDir, currXMove, currYMove);
	PtrMoveElement(world, obj, *currXMove, *currYMove);
}

GrafElPtr MakeNewBouncer(GEWorldPtr world, int bouncerResID)
{
	GrafElPtr aBall = NewBasicPICT(world, 'BALL', 20, bouncerResID, transparent, 20, 20);
	
	if (aBall) {
		// Autochange proc	
		SetAutoChange(world, aBall->objectID, DoBallRandom, nil, 16);
		// Movement speed kept in changeData
		*((short *) &aBall->changeData) = 4;
		*((short *) &aBall->changeData + 1) = 3;
	}
	return aBall;
}

#if 0
#pragma mark ---------------------------------
#pragma mark | Button
#pragma mark ---------------------------------
#endif

static void	ButtonChangeState(GEWorldPtr world, GrafElPtr theButton)
{
	if (theButton->sensorState & 1)
		RectOffset(&theButton->graphRect, -RectWidth(&theButton->animationRect), 0);
	else
		RectOffset(&theButton->graphRect, RectWidth(&theButton->animationRect), 0);
	theButton->sensorState ^= 1;
	ChangedRect(world, &theButton->animationRect);
}

GE_CALLBACK(Boolean, ButtonTracker)(GEWorldPtr world, GrafElPtr button)
{
	Point	localPt = { button->animationRect.top + 1, button->animationRect.left + 1 };
	Boolean on = false;
	
	do {
		if (PtInRect(localPt, &button->animationRect)) {
			//Button goes on if it was off
			if (!on) {
				ButtonChangeState(world, button);
				on = true;
			}
		}
		else {
			//Button goes off it was on
			if (on) {
				ButtonChangeState(world, button);
				on = false;
			}
		}
		DoWorldUpdate(world, false);
		GetMouse(&localPt);
	} while (StillDown());
	if (on) {
		if (button->actionProc)
			(button->actionProc)(world, button, true);
		ButtonChangeState(world, button);
	}
	return true;
}

GE_CALLBACK(void, OpenWebSite)(GEWorldPtr world, GrafElPtr button, short ignored)
{
	DemoWebConnection connection;
	
	if (connection.CanAccessWeb())
		connection.OpenSite((char *) button->changeData);
}

GE_CALLBACK(void, DisposeURL)(GEWorldPtr world, GrafElPtr button)
{
	if (button->changeData)
		DisposePtr(button->changeData);
}

GrafElPtr MakeNewBtn(GEWorldPtr world, int btnResID)
{
	const OSType	btnID = 'BTN ';
	ButtonSpec		spec;
	GrafElPtr		button = nil;
	
	void* specres = GetResourcePtr('Bnfo', btnResID);
	if (!specres)
		return nil;

	spec = *(ButtonSpec *) specres;
	ReleaseResourcePtr( specres );
	
	SwapIfRequired(&spec);
		
	button = NewBasicPICT(world, btnID, 30, (short) spec.btnPicResNum, 
					transparent, 0, 0);
	if (button) {
		Rect tmpRect = button->graphRect;
		tmpRect.right = tmpRect.left + (RectWidth(&tmpRect) / 2);
		button->graphRect = tmpRect;
		RectOffset(&tmpRect, spec.btnTopLeft.h, spec.btnTopLeft.v);
		button->animationRect = tmpRect;
		button->trackingProc = ButtonTracker;
		AddSensorToList(world, btnID, &button->animationRect);
		
		// Get URL called by button
		void* urlres = GetResourcePtr('URLs', spec.btnURLResNum);
		if (urlres) {
			button->changeData = NewPtrClear(strlen((char *) urlres) + 1);
			if (button->changeData)
				strcpy((char *) button->changeData, (char *) urlres);
			SetCleanupProc(world, btnID, DisposeURL);
			SetSensorAction(world, btnID, OpenWebSite);
		}
		ReleaseResourcePtr( urlres );
		
	}
					
		
	return button;
}



