/*
	CDemoWindow.cpp

	Demo Window for MacHack 97

	Windows 95 version -- uses MFC

	Al Evans

	6/18/97

*/


#include "..\resource.h"
#include "WinPalette.h"
#include "CDemoWindow.h"
#include "CGEView.h"

const CString	demoWndClassName = "DEMO_WINDOW";

CDemoWindow::CDemoWindow()
{
	mCurrView = nil;
	mPalette = nil;
	LoadAccelTable(MAKEINTRESOURCE(IDR_MAINFRAME));
	mMainMenu.LoadMenu(IDR_MAINFRAME);

}

CDemoWindow::~CDemoWindow()
{
}

BOOL CDemoWindow::Create(CString fileName)
{
	 const int width = 521;
	 const int height = 393;

	if(!CreateEx(0, demoWndClassName, "HelloBothWorlds", 
					WS_POPUP | WS_CAPTION | WS_SYSMENU | WS_MINIMIZEBOX | WS_BORDER,
					0, 0, width, height, NULL, mMainMenu.m_hMenu, NULL))
		return FALSE;

	// Center window on screen

	CRect window_rect;				// rectangle of entire window, including frame, title bar
	GetWindowRect( window_rect );
	CRect content_rect;				// rectangle of content region, excluding menu bar
	GetClientRect( content_rect );
	ClientToScreen( &content_rect );
	int const window_frame_height = window_rect.Height() - content_rect.Height();
	int const window_frame_width = window_rect.Width() - content_rect.Width();
	int const window_frame_bottom = window_rect.bottom - content_rect.bottom;
	int const screen_width = GetSystemMetrics(SM_CXSCREEN);
	int const screen_height = GetSystemMetrics(SM_CYSCREEN);
	int window_top = screen_height - height - window_frame_height + window_frame_bottom;
	int const window_left = (screen_width - width - window_frame_width) / 2;

	// if screen is big enough vertically, center
	if ( window_top > 0 )
		window_top /= 2;

	SetWindowPos( NULL, window_left, window_top,
			width + window_frame_width, height + window_frame_height, SWP_NOZORDER );

	
	mPalette = MakePaletteFromResource(128);
	InitGETimer();
	mCurrView = new CGEView(this, 128);

	return TRUE;
}

Rect CDemoWindow::GetWindRect()
{
	CRect r;
	GetClientRect(r);
	return r;
}

HPALETTE CDemoWindow::GetPalette()
{
	return mPalette;
}

HWND CDemoWindow::GetSystemWindow()
{
	return GetSafeHwnd();
}

BEGIN_MESSAGE_MAP( CDemoWindow, CFrameWnd )
	//{{AFX_MSG_MAP( CDemoWindow )
	ON_WM_DESTROY()
	ON_WM_PAINT()
	ON_WM_SHOWWINDOW()
	ON_WM_LBUTTONDOWN()
	ON_MESSAGE(UM_SPENDTIME, OnSpendTime)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

void CDemoWindow::OnDestroy()
{
	if (mCurrView)
		delete mCurrView;
	ShutdownGETimer();
	CFrameWnd ::OnDestroy();	
}

void CDemoWindow::OnPaint()
{
	CPaintDC dc(this);
	if (mCurrView)
		mCurrView->Draw();
}

void CDemoWindow::OnShowWindow( BOOL bShow, UINT nStatus )
{
	if (mCurrView) {
		mCurrView->Activate(bShow);
	}

	CFrameWnd::OnShowWindow(bShow, nStatus);
}


LRESULT CDemoWindow::OnSpendTime(WPARAM wParam, LPARAM lParam)
{
	if (mCurrView)
		mCurrView->Idle();
	return (LRESULT) 0;
}

void CDemoWindow::OnLButtonDown(UINT nFlags, CPoint point)
{
	if (mCurrView) {
		Point pt;

		pt.h = point.x;
		pt.v = point.y;
		mCurrView->HandleMouseDown(pt);
	}
	CFrameWnd::OnLButtonDown(nFlags, point);
}




