#include "Subwoofer.h"

#include <Types.h>
#include <Files.h>
#include <Folders.h>
#include <Dialogs.h>
#include <QuickDraw.h>
#include <Fonts.h>
#include <Windows.h>
#include <Menus.h>
#include <TextEdit.h>
#include <String.h>

void main();

Boolean GetURL(char* url, DialogPtr theDialog, short *displayProgress);
pascal Boolean MyDialogFilter(DialogPtr dialog, EventRecord *event, short *itemHit);
void ShowError(DialogPtr theDialog, OSErr err);

void main()
{
	OSErr err = noErr;

	FSSpec fileSpec;
	short vRefNum;
	long dirID;
	
	DialogPtr theDialog;
	char url[255];
	short displayProgress = 1;
	
	MaxApplZone();

	// initialize stuff
	InitGraf(&qd);
	InitFonts();
	InitWindows();
	InitMenus();
	TEInit();
	InitDialogs(nil);

	// get the saved url
	theDialog = GetNewDialog(128, nil, (WindowPtr) -1);
	ShowError(theDialog, noErr);
	DrawDialog(theDialog);

	url[0] = 0;
		
	// create the FSSpec for the file on the desktop
	err =  FindFolder(kOnSystemDisk, kDesktopFolderType, false, &vRefNum, &dirID);
	if ( err != noErr )
	{
		debugstr("FindFolder failed");
		return;
	}

	// call the dialog and download until cancelled
	while ( GetURL(url, theDialog, &displayProgress) )
	{
		(void) FSMakeFSSpec(vRefNum, dirID, "\p testfilename", &fileSpec);
		fileSpec.name[0] = 0; // let URLDownload give the name
		
		err = URLDownload(	url,
							&fileSpec,
							kURLDisplayProgressFlag * displayProgress +
							kURLDisplayAuthFlag + kURLExpandFileFlag );
		ShowError(theDialog, err);
	}

	DisposeDialog(theDialog);
	
	return;
}

Boolean GetURL(char* url, DialogPtr theDialog, short *displayProgress)
{
	Boolean result = false;
	ModalFilterUPP MyDialogFilterProcUPP = NewModalFilterProc(MyDialogFilter);
	
	Handle textItem, checkBox;
	short theItem = 0;
		
	Str255 theURL;
	
	Boolean continueEvents = true;

	// set the default check box value
	GetDialogItem(theDialog, 3, nil, &checkBox, nil);
	SetControlValue((ControlHandle) checkBox, *displayProgress);

	// show the dialog
	ShowWindow(theDialog);
	
	// set theURL
	theURL[0] = strlen(url);
	if ( theURL[0] )
		BlockMoveData(url, theURL + 1, theURL[0]);
	
	// set the dialog text item to the previous url
	GetDialogItem(theDialog, 4, nil, &textItem, nil);
	SetDialogItemText(textItem, theURL);
	SelectDialogItemText(theDialog, 4, 0, 32767);
	
	while ( continueEvents )
	{
		ModalDialog(MyDialogFilterProcUPP, &theItem);
		
		switch ( theItem )
		{
		case 1:
			result = true;
			continueEvents = false;
			break;
			
		case 2:
			result = false;
			continueEvents = false;
			break;
		
		case 3:
			*displayProgress = 1 - *displayProgress;
			GetDialogItem(theDialog, 3, nil, &checkBox, nil);
			SetControlValue((ControlHandle) checkBox, *displayProgress);
			theItem = 0;
			break;
			
		default:
			break;
		}
	}
	
	// get the url
	GetDialogItemText(textItem, theURL);
	
	// set the url string
	BlockMoveData(theURL + 1, url, theURL[0]);
	url[theURL[0]] = 0;
			
	// hide the dialog
	HideWindow(theDialog);

	return result;
}

pascal Boolean MyDialogFilter(DialogPtr dialog, EventRecord *event, short *itemHit)
{
	#pragma unused ( dialog )
	
	Boolean handled = false;
	
	char ch;
	Boolean isOK = false;
	Boolean isCancel = false;
	
	*itemHit = 0;
	
	switch ( event->what )
	{
	case autoKey:
	case keyDown:
		ch = (char) event->message & charCodeMask;
		
		isOK = (ch == '\r' || ch == '\015' || ch == '\003');
		isCancel = (ch == '\033' || (ch == '.' && (event->modifiers & cmdKey) ));
		
		if ( event->modifiers & cmdKey )
		{
			// Command
			switch ( ch )
			{
				case '.':
					// Cancel: pass through
					break;

				default:
					break;
			}
		}
		break;
	}
	
	if ( isCancel )
	{
		*itemHit = 2;
		handled = true;
	}
	if ( isOK )
	{
		*itemHit = 1;
		handled = true;
	}

	return handled;
}

void ShowError(DialogPtr theDialog, OSErr err)
{
	Str63 theString;
	Handle theItem;
	
	switch ( err )
	{
	case noErr:
		BlockMoveData("\p", theString, 1);
		break;
		
	case kURLInvalidURLReferenceError:
		BlockMoveData("\p kURLInvalidURLReferenceError", theString, 40);
		break;
	case kURLInvalidParameterError:
		BlockMoveData("\p kURLInvalidParameterError", theString, 40);
		break;
	case kURLProgressAlreadyDisplayedError:
		BlockMoveData("\p kURLProgressAlreadyDisplayedError", theString, 40);
		break;
	case kURLFileExistsError:
		BlockMoveData("\p kURLFileExistsError", theString, 40);
		break;
	case kURLInvalidURLError:
		BlockMoveData("\p kURLInvalidURLError", theString, 40);
		break;
	case kURLUnsupportedSchemeError:
		BlockMoveData("\p kURLUnsupportedSchemeError", theString, 40);
		break;
	case kURLServerBusyError:
		BlockMoveData("\p kURLServerBusyError", theString, 40);
		break;
	case kURLAuthenticationError:
		BlockMoveData("\p kURLAuthenticationError", theString, 40);
		break;
	case kURLUnknownPropertyError:
		BlockMoveData("\p kURLUnkownPropertyError", theString, 40);
		break;
	case kURLPropertyBufferToSmallError:
		BlockMoveData("\p kURLPropertyBufferToSmallError", theString, 40);
		break;
	case kURLUnsetablePropertyError:
		BlockMoveData("\p kURLUnsetablePropertyError", theString, 40);
		break;
	case kURLInvalidCallError:
		BlockMoveData("\p kURLInvalidCallError", theString, 40);
		break;
	case kURLNoInternetConfigError:
		BlockMoveData("\p kURLNoInternetConfigError", theString, 40);
		break;
	case kURLUserCanceledError:
		BlockMoveData("\p kURLUserCancelledError", theString, 40);
		break;
	case kURLFileEmptyError:
		BlockMoveData("\p kURLFileEmptyError", theString, 40);
		break;
	case kURLResourceNotFoundError:
		BlockMoveData("\p kURLResourceNotFoundError", theString, 40);
		break;
	case kURLExtensionFailureError:
		BlockMoveData("\p kURLExtensionFailureError", theString, 40);
		break;

	default :
		NumToString(err, theString);
		break;
	}
	
	GetDialogItem(theDialog, 7, nil, &theItem, nil);
	SetDialogItemText(theItem, theString);
}