/*
	File:		StuffItEngineLib.h

	Contains:	<contents>

	Written by:	Frederic Artru

	Copyright:	<copyright>

	Change History (most recent first):

        <0+>      3/3/97    FA      first checked in
*/


/*
	File:		StuffItEngineLib.h

	Contains:	<contents>

	Written by:	Frederic Artru

	Copyright:	<copyright>

	Change History (most recent first):

        <0+>      3/3/97    FA      first checked in
*/


/******************************************************************************
**
**  Project Name:	StuffItLib
**     File Name:	StuffItEngineLib.h
**
**   Description:	Module containing the description of the library routines
**
**   Copyright 1992-1996 Aladdin Systems, inc.
**
*******************************************************************************
**                       A U T H O R   I D E N T I T Y
*******************************************************************************
**
**	Initials	Name
**	--------	-----------------------------------------------
**	LDR			Leonard Rosenthol
**
******************************************************************************/

#ifndef __StuffItLib__
#define __StuffItLib__

#ifdef __cplusplus
extern "C" {
#endif

/*
	this is the version of the library you know about
	the top byte is the version, the bottom is the revision.
	See documentation for how to use this information
	
	0x0100		First version
	0x0101		4.0d2: changes for new enum values for creating folders & text conversion
*/
#define kCurrentLibraryVersion	0x0101
#define kFirstSupportedEngine	0x0400

enum {
	USIT_conflict_prompt = 0,
	USIT_conflict_replaceExisting = 1,
	USIT_conflict_autoRename = 2,
	USIT_conflict_skipItem = 3
} /* unstuffConflictActions */;

enum {
	SIT_conflict_prompt = 0,
	SIT_conflict_replaceExisting = 1,
	SIT_conflict_autoRename = 2,
	SIT_conflict_useExisting = 3
} /* stuffConflictActions */;

enum {
	createFolder_Never = 0,
	createFolder_Smart = 1,
	createFolder_Always = 2
} /* createFolderOptions */;

enum {
	textConvert_Never = 0,
	textConvert_Smart = 1,
	textConvert_Always = 2
} /* textConvertOptions */;



// set of defines for having more easily readable code
#define kPromptForDestination		true
#define kDontPromptForDestination	false
#define	kDeleteOriginal				true
#define	kDontDeleteOriginal			false
#define kSaveComments				true
#define	kDontSaveComments			false
#define	kStopOnAbort				true
#define kDontStopOnAbort			false
#define kCompressOriginal			true
#define kDontCompressOriginal		false
#define	kEncryptOriginal			true
#define kDontEncryptOriginal		false
#define	kResolveAliases				true
#define kDontResolveAliases			false
#define	kRecompressCompresed		false
#define kDontRecompressCompresed	true
#define	kAddLineFeeds				true
#define kDontAddLineFeeds			false
#define kDefaultHQXCreator			'BnHq'

enum {
	ftype_unknown = 0,
	ftype_StuffIt = 1,
	ftype_BinHex = 2,
	ftype_CompactPro = 3,
	ftype_ALinkPKG = 4,
	ftype_ZIP = 5,
	ftype_ARC = 6,
	ftype_UnixCompress = 7,
	ftype_UUEncoded = 8,
	ftype_MacBinary = 9,
	ftype_TAR = 10,
	ftype_MIME = 11
} /* engineFileTypes */;


// these are the ways you can open the StuffIt Engine
enum {
	kUseExternalEngine = 0,
	kUseBuiltInEngine = 1
};

// Possible error codes
enum {
	engineAbortErr			= -1,
	versionNotAvailableErr	= -2,
	notValidFileTypeErr		= -211
};


typedef FSSpecArrayPtr *FSSpecArrayHandle;       /* handle to array of FSSpecs */


/*
	These routines are used to determine what version of the library &
	StuffIt Engine you are using.
*/
pascal	short	GetSITLibraryVersion(void);
pascal	short	GetSITEngineVersion(long magicCookie);




/*
	These routines help you open and close the engine for use.  These calls are for
	use both with the "extension" called StuffIt Engine, or for directly linked
	"engines" in your application.  Use the location option to specify which one.
	
	Use OpenSITEngine() to attempt to open it.  
		If your product doesn't have access to a resource fork, then you should use OpenSITEngineByName passing in the appropriate string ("StuffIt Engine")
	When all done call CloseSITEngine().
*/
extern pascal	OSErr	OpenSITEngine(short location, long *magicCookie);
extern pascal	OSErr	OpenSITEngineByName(short location, unsigned char *engineName, long *magicCookie);
extern pascal	void	CloseSITEngine(long magicCookie);


/*
	These routines are used for working with StuffIt archives.

	They all assume that you've already opened the engine (via the above calls)
*/
extern pascal	OSErr UnStuffFSSpec(long magicCookie, FSSpecPtr archiveSpec, 
									FSSpecPtr destSpec, FSSpec *resultSpec, 
									Boolean promptForDestination, Boolean deleteOriginal, short createFolder,
									Boolean saveComments, Boolean stopOnAbort, short conflictAction);

extern pascal	OSErr StuffFSSList(long magicCookie, FSSpecArrayHandle fileList, 
								   FSSpecPtr archiveSpec, FSSpec *resultFSSpec,
				   		   		   Boolean compressOriginal, Boolean deleteOriginal, Boolean encryptOriginal, 
				   		   		   Boolean resolveAliases, Boolean noRecompression, 
				   		   		   short conflictAction);

extern pascal	OSErr ConvertSIT2SEA(long magicCookie, FSSpecPtr archive);
extern pascal	OSErr ConvertSEA2SIT(long magicCookie, FSSpecPtr archive);
extern pascal Boolean DetermineSelfExtracting(FSSpecPtr theSpec);

/*
	These routines are used for working with the non-StuffIt formats 
	supported by the StuffIt Engine.

	They all assume that you've already opened the engine (via the above calls)
*/
extern pascal short DetermineFileType(FSSpecPtr theSpec);
extern pascal OSErr ExpandFSSpec(long magicCookie, short ftype, 
								 FSSpecPtr srcSpec, FSSpecPtr destSpec, FSSpec *expandedSpec,
								 short createFolder, Boolean deleteOriginal, short convertTextFiles);
extern pascal OSErr HQXEncodeFSSpec(long magicCookie, FSSpecPtr srcFSSpec, FSSpecPtr destFSSpec, 
					   		 		Boolean deleteOriginal, Boolean addLineFeeds, OSType hqxCreator);



/*
	These routines are helpful in building up a list of items to Stuff
*/
extern pascal	FSSpecArrayHandle	NewFSSpecList(void);
extern pascal	OSErr 				AddToFSSpecList(FSSpec *fSpec, FSSpecArrayHandle fileList);
extern pascal 	long 				CountFSSpecList(FSSpecArrayHandle fsList);
extern pascal	void 				DisposeFSSpecList(FSSpecArrayHandle fsList);

/*
	and finally some general purpose routines
*/
pascal void 	GetNewArchiveFSSpec(FSSpecArrayHandle fspH, Boolean asSEA, FSSpec *arcSpec);

#ifdef __cplusplus
}
#endif


#endif // __StuffItLib__
