/*
	File:		StuffItEngineLib_LL.h

	Contains:	<contents>

	Written by:	Frederic Artru

		Copyright:	 1997 by Apple Computer, Inc., all rights reserved.

     You may incorporate this sample code into your applications without
    restriction, though the sample code has been provided "AS IS" and the
    responsibility for its operation is 100% yours.  However, what you are
    not permitted to do is to redistribute the source as "DSC Sample Code"
    after having made changes. If you're going to re-distribute the source,
    we require that you make it clear in the source that the code was
    descended from Apple Sample Code, but that you've made changes.

	Change History (most recent first):

        <0+>      3/3/97    FA      first checked in
*/


/******************************************************************************
**
**  Project Name:	StuffItLib
**     File Name:	StuffItEngineLib_LL.h
**
**   Description:	Module containing the description of the LOW LEVEL library routines
**
**   Copyright 1992-1995 Aladdin Systems, inc.
**
*******************************************************************************
**                       A U T H O R   I D E N T I T Y
*******************************************************************************
**
**	Initials	Name
**	--------	-----------------------------------------------
**	LDR			Leonard Rosenthol
**
******************************************************************************/

#ifndef __StuffItLowLevelLib__
#define __StuffItLowLevelLib__

#ifdef __cplusplus
extern "C" {
#endif

// Some additional definitions for new functions
#define kNoCallback					NULL
#define kShowStandardProgress		false
#define kShowNoProgress				true
#define	kNoPassword					NULL


#define kArchiveRootLevel	(archiveObject *)NULL
enum {	// new low level errors
	unsupportedArchiveFormat = 1,	// you'll get this if you try to open anything other than StuffIt
	unknownArchiveVersionErr = 2,
	noObjectAtThatLocation = 3	// returned when navigating and nothing else in that direction
};

// archive versions returned in archiveInfo
enum {
	kOriginalStuffItVersion = 1,
	kStuffItDeluxeVersion = 2
};

// special values used in the low level stuff
enum {
	kInvalidFSLocation = -1	// this is the value of fileLocation or folderLocation when not stored!
};


/*
	This structure contains information about an open archive.
*/
#if defined(powerc) || defined (__powerc)
#pragma options align=mac68k
#endif
typedef struct {
	long			archiveReference;	// magic cookie to find the archive later on
	FSSpec			archiveFSSpec;		// and a standard way to find it...
	short			archiveType;		// currently only knows about ftype_StuffIt
	unsigned char	archiveVersion;
	unsigned short	numberOfFiles;
	// special magic values
	long			reserved[5];	
} archiveInfo, *archiveInfoPtr;
#if defined(powerc) || defined(__powerc)
#pragma options align=reset
#endif

/*
	This structure contains information about a given item in an archive
*/
#if defined(powerc) || defined (__powerc)
#pragma options align=mac68k
#endif
typedef struct {
	long			objectReference;	// magic cookie to find the object in the archive
	unsigned char	objectName[32];	
	Boolean			objectIsCompressed;
	Boolean 		objectIsEncrypted;
	Boolean 		objectIsFolder;
	Boolean			objectIsLocked;		// represents bit 7 of ioFlAttrib
	short			objectFinderFlags;	// either FInfo.fdFlags or DInfo.frFlags
	unsigned long	objectCreationDate;
	unsigned long	objectModificationDate;
	unsigned long	objectCompressedSize;
	unsigned long	objectExpandedSize;
	// valid only for files
	OSType			fileType;
	OSType			fileCreator;
	Point			fileLocation;		// equivalent to FInfo.fdLocation
	// valid only for folders
	short			numberOfFiles;
	Rect			folderRectangle;	// equivalent to DInfo.frRect
	short			folderView;			// equivalent to DInfo.frView
	Point			folderLocation;		// equivalent to DInfo.frLocation
	Point			folderScroll;		// equivalent to DXInfo.frScroll
	// special magic values
	long			reserved[5];
}archiveObject, *arcObjectPtr, **arcObjectHandle;
typedef arcObjectHandle arcObjectArrayHandle;		/* handle to array of arcObjects */
#if defined(powerc) || defined(__powerc)
#pragma options align=reset
#endif

/*
	Here is the declarations for the StuffIt Engine callbacks for progress, alerts & status.
	Both CFM and non-CFM declarations are present.
	
	SITAlertCallback is passed what type of error happend (OSErr) and an internal cookie about
	where it happened.  You should ALWAYS return 1 from this function!
	
	SITStatusCallback is passed a Ptr to the structure defined below with all the info about
	where things during an expansion/stuffing so that you cna display your own progress.  Return
	false unless the user aborted, in which case return true.
	
	SITPeriodicCallback should return false unless the user aborted, in which case return true.
*/
#if defined(powerc) || defined (__powerc)
#pragma options align=mac68k
#endif
typedef struct {	
	unsigned char	selector;		/* function */

	short			itemsDone;		/* number of items processed. */
	short			itemsTotal;		/* total items to process */
	long			overallDone;	/* size of data already processed */
	long			overallTotal;	/* total size of data to process */
	
	unsigned char 	*currentFileName;
	
	unsigned char 	*operationName;		/* You don't HAVE to use these, but they're provided... */
	unsigned char 	*actionName;
	unsigned char 	*remainMsg;
} seStatusRec, *seStatusRecPtr;
#if defined(powerc) || defined(__powerc)
#pragma options align=reset
#endif


/* selectors */
enum {	
	seStatus_CreateWindow = 0,			/* Create the window. */
	seStatus_PeriodicCall,				/* a periodic call to check for cmd-., etc.  All
											other fields invalid.  Other stuff should
											check for cmd-. as well!  */
	seStatus_DrawAll,					/* draw everything. */
	seStatus_DrawProgress,				/* draw bar graph */
	seStatus_UpdateProgress,			/* update bar graph (i.e. new progress is
												in advance of last progress so you
												don't need to erase the bar as in
												DrawProgress */
	seStatus_DrawName,					/* draw current file name */
	seStatus_DrawOperation,				/* draw current operation (what'd normally be
													in the title bar) */
	seStatus_DrawAction,				/* draw current action (what'd normally
													preceed the file name.)  */
	seStatus_DrawRemainCount,			/* draw items remaining (itemsTotal-itemsDone) */
	seStatus_DrawRemainMsg,				/* draw msg. preceding items remaining */
	seStatus_DisposeWindow				/* all other fields invalid */
};

typedef pascal short (*SITAlertCallbackProcPtr)(short errorCode, long misc);
typedef pascal short (*SITStatusCallbackProcPtr)(seStatusRecPtr pb);
typedef pascal short (*SITPeriodicCallbackProcPtr)(void);

#if GENERATINGCFM
typedef UniversalProcPtr SITAlertCallbackUPP;
typedef UniversalProcPtr SITStatusCallbackUPP;
typedef UniversalProcPtr SITPeriodicCallbackUPP;
#else
typedef SITAlertCallbackProcPtr SITAlertCallbackUPP;
typedef SITStatusCallbackProcPtr SITStatusCallbackUPP;
typedef SITPeriodicCallbackProcPtr SITPeriodicCallbackUPP;
#endif

enum {
	uppSITAlertCallbackProcInfo = kPascalStackBased | 
						RESULT_SIZE(SIZE_CODE(sizeof(short))) |
						STACK_ROUTINE_PARAMETER(1, SIZE_CODE(sizeof(short))) |
						STACK_ROUTINE_PARAMETER(2, SIZE_CODE(sizeof(long))),
	uppSITStatusCallbackProcInfo = kPascalStackBased | 
						RESULT_SIZE(SIZE_CODE(sizeof(short))) |
						STACK_ROUTINE_PARAMETER(1, SIZE_CODE(sizeof(seStatusRecPtr))),
	uppSITPeriodicCallbackProcInfo = kPascalStackBased | 
						RESULT_SIZE(SIZE_CODE(sizeof(short)))
};

#if GENERATINGCFM
#define NewSITAlertCallback(userRoutine)		\
		(SITAlertCallbackUPP) NewRoutineDescriptor((ProcPtr)(userRoutine), uppSITAlertCallbackProcInfo, GetCurrentArchitecture())
#define NewSITStatusCallback(userRoutine)		\
		(SITStatusCallbackUPP) NewRoutineDescriptor((ProcPtr)(userRoutine), uppSITStatusCallbackProcInfo, GetCurrentArchitecture())
#define NewSITPeriodicCallback(userRoutine)		\
		(SITPeriodicCallbackUPP) NewRoutineDescriptor((ProcPtr)(userRoutine), uppSITPeriodicCallbackProcInfo, GetCurrentArchitecture())
#else
#define NewSITAlertCallback(userRoutine)		\
		((SITAlertCallbackUPP) (userRoutine))
#define NewSITStatusCallback(userRoutine)		\
		((SITStatusCallbackUPP) (userRoutine))
#define NewSITPeriodicCallback(userRoutine)		\
		((SITPeriodicCallbackUPP) (userRoutine))
#endif


/*
	These are the routines for low level access to the engine!
*/
extern pascal OSErr OpenArchive(FSSpecPtr archiveSpec, SignedByte permission, archiveInfo *arcInfo);
extern pascal OSErr CloseArchive(archiveInfo *arcInfo);
extern pascal OSErr NewArchive(FSSpecPtr archiveSpec, short archiveType);

extern pascal OSErr	GetNextArchiveObject(archiveInfo *arcInfo, archiveObject *arcObj, archiveObject *nextObj);
extern pascal OSErr GetFirstChildArchiveObject(archiveInfo *arcInfo, archiveObject *arcObj, archiveObject *childObj);
extern pascal OSErr GetParentArchiveObject(archiveInfo *arcInfo, archiveObject *arcObj, archiveObject *parentObj);
extern pascal OSErr SetArchiveObjectInfo(archiveInfo *arcInfo, archiveObject *arcObj);

extern pascal OSErr UpdateArchiveObjectInfo(archiveInfo *arcInfo, archiveObject *arcObj);
extern pascal OSErr UpdateArchiveInfo(archiveInfo *arcInfo);


pascal OSErr AddNewFolderToArchive(long magicCookie, archiveInfo *arcInfo, 
								   archiveObject *parentObj, unsigned char *folderName,
								   archiveObject *newFolder);
pascal OSErr GetArchiveComments(archiveInfo *arcInfo, Handle *commentText, StScrpHandle *commentStyl);
pascal OSErr SetArchiveComments(archiveInfo *arcInfo, Handle commentText, StScrpHandle commentStyl);
								   

/*
	Low level archiving functions for individual item operations
*/
extern pascal OSErr AddToArchive(long magicCookie, archiveInfo *arcInfo, 
								 archiveObject *parentObj,
						  		 FSSpecArrayHandle fileList,
				  		  		 Boolean compressOriginal, Boolean deleteOriginal, Boolean encryptOriginal, 
				   		  		 Boolean resolveAliases, Boolean noRecompression,
				   		  		 short conflictAction,
						  		 unsigned char *password, Boolean showNoProgress, 
						  		 SITAlertCallbackUPP alertCBUPP, 
					      		 SITStatusCallbackUPP	statusCBUPP,
					      		 SITPeriodicCallbackUPP periodicCBUPP);

extern pascal OSErr ExpandFromArchive(long magicCookie, archiveInfo *arcInfo, 
							   		  arcObjectArrayHandle objects,
							   		  FSSpecPtr destSpec, FSSpec *resultSpec, 
						   	  		  Boolean promptForDestination, 
						   	  		  short createFolder,
						   	  		  Boolean stopOnAbort,
						   	   		  short conflictAction,
						   	   		  unsigned char *password,
							   		  Boolean noProgress, 
							   		  SITAlertCallbackUPP alertCBUPP, 
							   		  SITStatusCallbackUPP statusCBUPP,
							   		  SITPeriodicCallbackUPP periodicCBUPP);

/*
	New "enhanced" versions of the UnStuff & Stuff functions with opertions for progress & callbacks

	If you simply don't want any progress (like in an FBA), 
	then pass kShowNoProgress for noProgress & pass kNoCallback for the three callbacks.  
	You should only need to use your own callbacks in VERY special circumstatnces
*/
pascal OSErr UnStuffFSSpec_Enhanced(long magicCookie, FSSpecPtr archiveSpec, FSSpecPtr destSpec, FSSpec *resultSpec, 
						   			Boolean promptForDestination, Boolean deleteOriginal, 
						   			short createFolder, Boolean saveComments, 
						   			Boolean stopOnAbort, short conflictAction,
								    unsigned char *password, Boolean noProgress, 
								    SITAlertCallbackUPP alertCBUPP, 
								    SITStatusCallbackUPP statusCBUPP,
								    SITPeriodicCallbackUPP periodicCBUPP);

pascal OSErr StuffFSSList_Enhanced(long magicCookie, FSSpecArrayHandle fileList, 
						  		   FSSpecPtr archiveSpec, FSSpec *resultFSSpec,
				  		  		   Boolean compressOriginal, Boolean deleteOriginal, Boolean encryptOriginal, 
				   		  		   Boolean resolveAliases, Boolean noRecompression,
				   		  		   short conflictAction,
						  		   unsigned char *password, Boolean showNoProgress, 
						  		   SITAlertCallbackUPP alertCBUPP, 
					      		   SITStatusCallbackUPP	statusCBUPP,
					      		   SITPeriodicCallbackUPP periodicCBUPP);


/*
	These routines are helpful in building up a list of items to UnStuff
*/
extern pascal	arcObjectArrayHandle	NewArcObjectList(void);
extern pascal	OSErr 					AddToArcObjectList(arcObjectPtr arcObjPtr, arcObjectArrayHandle arcObjList);
extern pascal 	long 					CountArcObjectList(arcObjectArrayHandle arcObjList);
extern pascal	void 					DisposeArcObjectList(arcObjectArrayHandle arcObjList);

#ifdef __cplusplus
}
#endif


#endif // __StuffItLowLevelLib__
